@@ -261,13 +261,11 @@ function get_count_integration_files() {
         "^hadoop-ozone/dev-support/checks/integration.sh"
         "^hadoop-ozone/dev-support/checks/junit.sh"
         "^hadoop-ozone/integration-test"
+        "^hadoop-ozone/mini-cluster"
         "^hadoop-ozone/fault-injection-test/mini-chaos-tests"
         "src/test/java"
         "src/test/resources"
     )
-    local ignore_array=(
-        $(grep -Flr 'org.apache.ozone.test.tag.Native' hadoop-ozone/integration-test)
-    )
     filter_changed_files true
     COUNT_INTEGRATION_CHANGED_FILES=${match_count}
     readonly COUNT_INTEGRATION_CHANGED_FILES
@@ -439,40 +437,6 @@ function check_needs_pmd() {
     start_end::group_end
 }
 
-function check_needs_native() {
-    start_end::group_start "Check if native is needed"
-    local pattern_array=(
-        "^hadoop-ozone/dev-support/checks/native.sh"
-        "^hadoop-hdds/rocks-native"
-        # include tests tagged as @Native in any module
-        $(grep -Flr 'org.apache.ozone.test.tag.Native' hadoop-*/*/src/test/java)
-    )
-    filter_changed_files true
-
-    if [[ ${match_count} != "0" ]]; then
-        add_basic_check native
-    else
-        local pattern_array=(
-            "^hadoop-ozone/dev-support/checks/junit.sh"
-            # dependencies
-            "^hadoop-hdds/annotations"
-            "^hadoop-hdds/common"
-            "^hadoop-hdds/config"
-            "^hadoop-hdds/hadoop-dependency-client"
-            "^hadoop-hdds/managed-rocksdb"
-            "^hadoop-hdds/test-utils"
-            "^pom.xml"
-        )
-        filter_changed_files
-
-        if [[ ${match_count} != "0" ]]; then
-            add_basic_check native
-        fi
-    fi
-
-    start_end::group_end
-}
-
 # Counts other files which do not need to trigger any functional test
 # (i.e. no compose/integration/kubernetes)
 function get_count_misc_files() {
@@ -608,6 +572,5 @@ check_needs_checkstyle
 check_needs_docs
 check_needs_findbugs
 check_needs_pmd
-check_needs_native
 calculate_test_types_to_run
 set_outputs
