@@ -17,8 +17,6 @@
 
 package org.apache.hadoop.hdds.scm.storage;
 
-import static org.apache.hadoop.hdds.client.ReplicationConfig.getLegacyFactor;
-
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import java.io.EOFException;
@@ -32,7 +30,6 @@ import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.Function;
 import org.apache.hadoop.hdds.client.BlockID;
-import org.apache.hadoop.hdds.client.StandaloneReplicationConfig;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.BlockData;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ChunkInfo;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerCommandResponseProto;
@@ -62,7 +59,7 @@ import org.slf4j.LoggerFactory;
  */
 public class BlockInputStream extends BlockExtendedInputStream {
 
-  public static final Logger LOG = LoggerFactory.getLogger(BlockInputStream.class);
+  private static final Logger LOG = LoggerFactory.getLogger(BlockInputStream.class);
 
   private static final List<Validator> VALIDATORS =
       ContainerProtocolCalls.toValidatorList((request, response) -> validate(response));
@@ -114,6 +111,8 @@ public class BlockInputStream extends BlockExtendedInputStream {
 
   private final Function<BlockID, BlockLocationInfo> refreshFunction;
 
+  private BlockData blockData;
+
   public BlockInputStream(
       BlockLocationInfo blockInfo,
       Pipeline pipeline,
@@ -155,7 +154,6 @@ public class BlockInputStream extends BlockExtendedInputStream {
       return;
     }
 
-    BlockData blockData = null;
     List<ChunkInfo> chunks = null;
     IOException catchEx = null;
     do {
@@ -295,10 +293,7 @@ public class BlockInputStream extends BlockExtendedInputStream {
     boolean okForRead =
         pipeline.getType() == HddsProtos.ReplicationType.STAND_ALONE
             || pipeline.getType() == HddsProtos.ReplicationType.EC;
-    Pipeline readPipeline = okForRead ? pipeline : Pipeline.newBuilder(pipeline)
-        .setReplicationConfig(StandaloneReplicationConfig.getInstance(
-            getLegacyFactor(pipeline.getReplicationConfig())))
-        .build();
+    Pipeline readPipeline = okForRead ? pipeline : pipeline.copyForRead();
     pipelineRef.set(readPipeline);
   }
 
@@ -552,8 +547,7 @@ public class BlockInputStream extends BlockExtendedInputStream {
     return length;
   }
 
-  @VisibleForTesting
-  synchronized int getChunkIndex() {
+  public synchronized int getChunkIndex() {
     return chunkIndex;
   }
 
@@ -616,9 +610,12 @@ public class BlockInputStream extends BlockExtendedInputStream {
     refreshBlockInfo(cause);
   }
 
-  @VisibleForTesting
   public synchronized List<ChunkInputStream> getChunkStreams() {
     return chunkStreams;
   }
 
+  public BlockData getStreamBlockData() {
+    return blockData;
+  }
+
 }
