@@ -462,8 +462,6 @@ public class ChunkInputStream extends InputStream
 
     ReadChunkResponseProto readChunkResponse = response.getReadChunk();
     List<ByteString> byteStrings;
-    boolean isV0 = false;
-
     if (readChunkResponse.hasData()) {
       ByteString byteString = readChunkResponse.getData();
       if (byteString.size() != reqChunkInfo.getLen()) {
@@ -475,7 +473,6 @@ public class ChunkInputStream extends InputStream
       }
       byteStrings = new ArrayList<>();
       byteStrings.add(byteString);
-      isV0 = true;
     } else {
       byteStrings = readChunkResponse.getDataBuffers().getBuffersList();
       long buffersLen = BufferUtils.getBuffersLen(byteStrings);
@@ -500,8 +497,7 @@ public class ChunkInputStream extends InputStream
           chunkInfo.getOffset();
       int bytesPerChecksum = checksumData.getBytesPerChecksum();
       int startIndex = (int) (relativeOffset / bytesPerChecksum);
-      Checksum.verifyChecksum(byteStrings, checksumData, startIndex,
-          isV0);
+      Checksum.verifyChecksum(byteStrings, checksumData, startIndex);
     }
   }
 
@@ -675,7 +671,6 @@ public class ChunkInputStream extends InputStream
     }
   }
 
-
   /**
    * Release the buffers upto the given index.
    * @param releaseUptoBufferIndex bufferIndex (inclusive) upto which the
@@ -747,4 +742,8 @@ public class ChunkInputStream extends InputStream
   public ByteBuffer[] getCachedBuffers() {
     return BufferUtils.getReadOnlyByteBuffers(buffers);
   }
+
+  public ChunkInfo getChunkInfo() {
+    return chunkInfo;
+  }
 }
