@@ -1,22 +1,30 @@
-/**
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License.  You may obtain a copy of the License at
- * <p>
- * http://www.apache.org/licenses/LICENSE-2.0
- * <p>
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 package org.apache.hadoop.ozone.client.io;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertInstanceOf;
+import static org.mockito.ArgumentMatchers.any;
+
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.Map;
 import org.apache.hadoop.hdds.client.BlockID;
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.client.RatisReplicationConfig;
@@ -32,16 +40,8 @@ import org.apache.hadoop.hdds.scm.storage.BlockExtendedInputStream;
 import org.apache.hadoop.hdds.scm.storage.BlockInputStream;
 import org.apache.hadoop.hdds.scm.storage.BlockLocationInfo;
 import org.junit.jupiter.api.Test;
-import org.junit.jupiter.api.Assertions;
 import org.mockito.Mockito;
 
-import java.io.IOException;
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.Map;
-
-import static org.mockito.ArgumentMatchers.any;
-
 /**
  * Tests for BlockInputStreamFactoryImpl.
  */
@@ -66,9 +66,9 @@ public class TestBlockInputStreamFactoryImpl {
         factory.create(repConfig, blockInfo, blockInfo.getPipeline(),
             blockInfo.getToken(), null, null,
             clientConfig);
-    Assertions.assertTrue(stream instanceof BlockInputStream);
-    Assertions.assertEquals(stream.getBlockID(), blockInfo.getBlockID());
-    Assertions.assertEquals(stream.getLength(), blockInfo.getLength());
+    assertInstanceOf(BlockInputStream.class, stream);
+    assertEquals(stream.getBlockID(), blockInfo.getBlockID());
+    assertEquals(stream.getLength(), blockInfo.getLength());
   }
 
   @Test
@@ -86,9 +86,9 @@ public class TestBlockInputStreamFactoryImpl {
         factory.create(repConfig, blockInfo, blockInfo.getPipeline(),
             blockInfo.getToken(), null, null,
             clientConfig);
-    Assertions.assertTrue(stream instanceof ECBlockInputStreamProxy);
-    Assertions.assertEquals(stream.getBlockID(), blockInfo.getBlockID());
-    Assertions.assertEquals(stream.getLength(), blockInfo.getLength());
+    assertInstanceOf(ECBlockInputStreamProxy.class, stream);
+    assertEquals(stream.getBlockID(), blockInfo.getBlockID());
+    assertEquals(stream.getLength(), blockInfo.getLength());
   }
 
   private BlockLocationInfo createKeyLocationInfo(ReplicationConfig repConf,
