@@ -17,22 +17,21 @@
 
 package org.apache.hadoop.hdds;
 
-import com.google.common.base.Preconditions;
 import java.nio.ByteBuffer;
 import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import org.apache.ratis.thirdparty.io.netty.buffer.Unpooled;
+import org.apache.ratis.util.Preconditions;
 
 /**
  * Simple utility class to collection string conversion methods.
  */
 public final class StringUtils {
+  private static final Charset UTF8 = StandardCharsets.UTF_8;
 
   private StringUtils() {
   }
 
-  private static final Charset UTF8 = StandardCharsets.UTF_8;
-
   /**
    * Decode a specific range of bytes of the given byte array to a string
    * using UTF8.
@@ -55,14 +54,22 @@ public final class StringUtils {
   }
 
   public static String bytes2Hex(ByteBuffer buffer, int max) {
+    Preconditions.assertTrue(max > 0, () -> "max = " + max + " <= 0");
     buffer = buffer.asReadOnlyBuffer();
     final int remaining = buffer.remaining();
-    final int n = Math.min(max, remaining);
-    final StringBuilder builder = new StringBuilder(3 * n);
-    for (int i = 0; i < n; i++) {
-      builder.append(String.format("%02X ", buffer.get()));
+    final boolean overflow = max < remaining;
+    final int n = overflow ? max : remaining;
+    final StringBuilder builder = new StringBuilder(3 * n + (overflow ? 3 : 0));
+    if (n > 0) {
+      for (int i = 0; i < n; i++) {
+        builder.append(String.format("%02X ", buffer.get()));
+      }
+      builder.setLength(builder.length() - 1);
     }
-    return builder + (remaining > max ? "..." : "");
+    if (overflow) {
+      builder.append("...");
+    }
+    return builder.toString();
   }
 
   public static String bytes2Hex(ByteBuffer buffer) {
@@ -91,8 +98,29 @@ public final class StringUtils {
     return str.getBytes(UTF8);
   }
 
-  public static String appendIfNotPresent(String str, char c) {
-    Preconditions.checkNotNull(str, "Input string is null");
-    return str.isEmpty() || str.charAt(str.length() - 1) != c ? str + c : str;
+  public static String getLexicographicallyLowerString(String val) {
+    if (val == null || val.isEmpty()) {
+      throw new IllegalArgumentException("Input string must not be null or empty");
+    }
+    char[] charVal = val.toCharArray();
+    int lastIdx = charVal.length - 1;
+    if (charVal[lastIdx] == Character.MIN_VALUE) {
+      throw new IllegalArgumentException("Cannot decrement character below Character.MIN_VALUE");
+    }
+    charVal[lastIdx] -= 1;
+    return String.valueOf(charVal);
+  }
+
+  public static String getLexicographicallyHigherString(String val) {
+    if (val == null || val.isEmpty()) {
+      throw new IllegalArgumentException("Input string must not be null or empty");
+    }
+    char[] charVal = val.toCharArray();
+    int lastIdx = charVal.length - 1;
+    if (charVal[lastIdx] == Character.MAX_VALUE) {
+      throw new IllegalArgumentException("Cannot increment character above Character.MAX_VALUE");
+    }
+    charVal[lastIdx] += 1;
+    return String.valueOf(charVal);
   }
 }
