@@ -34,6 +34,20 @@ public class ECReplicationConfig implements ReplicationConfig {
 
   public static final String EC_REPLICATION_PARAMS_DELIMITER = "-";
 
+  // Acceptable patterns are like:
+  //   rs-3-2-1024k
+  //   RS-3-2-2048
+  //   XOR-10-4-4096K
+  private static final Pattern STRING_FORMAT = Pattern.compile("([a-zA-Z]+)-(\\d+)-(\\d+)-(\\d+)([kK])?");
+
+  private final int data;
+
+  private final int parity;
+
+  private final int ecChunkSize;
+
+  private final EcCodec codec;
+
   /**
    * Enum defining the allowed list of ECCodecs.
    */
@@ -53,21 +67,6 @@ public class ECReplicationConfig implements ReplicationConfig {
     }
   }
 
-  // Acceptable patterns are like:
-  //   rs-3-2-1024k
-  //   RS-3-2-2048
-  //   XOR-10-4-4096K
-  private static final Pattern STRING_FORMAT
-      = Pattern.compile("([a-zA-Z]+)-(\\d+)-(\\d+)-(\\d+)([kK])?");
-
-  private final int data;
-
-  private final int parity;
-
-  private final int ecChunkSize;
-
-  private final EcCodec codec;
-
   public ECReplicationConfig(int data, int parity) {
     this(data, parity, EcCodec.RS, 1024 * 1024);
   }
