@@ -23,7 +23,6 @@ import static org.apache.hadoop.hdds.ratis.RatisHelper.HDDS_DATANODE_RATIS_PREFI
 import static org.apache.hadoop.ozone.OzoneConfigKeys.OZONE_CONTAINER_COPY_WORKDIR;
 
 import com.google.common.base.Preconditions;
-import java.io.IOException;
 import java.net.URL;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -37,6 +36,7 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.SortedSet;
 import java.util.TreeSet;
+import java.util.concurrent.TimeUnit;
 import java.util.function.Consumer;
 import java.util.stream.Collectors;
 import javax.xml.bind.JAXBContext;
@@ -53,13 +53,13 @@ import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.utils.LegacyHadoopConfigurationSource;
 import org.apache.hadoop.ozone.OzoneConfigKeys;
 import org.apache.ratis.server.RaftServerConfigKeys;
+import org.slf4j.Logger;
 
 /**
  * Configuration for ozone.
  */
 @InterfaceAudience.Private
-public class OzoneConfiguration extends Configuration
-    implements MutableConfigurationSource {
+public class OzoneConfiguration extends Configuration implements MutableConfigurationSource {
 
   public static final SortedSet<String> TAGS = unmodifiableSortedSet(
       Arrays.stream(ConfigTag.values())
@@ -72,6 +72,8 @@ public class OzoneConfiguration extends Configuration
     activate();
   }
 
+  private Properties delegatingProps;
+
   public static OzoneConfiguration of(ConfigurationSource source) {
     if (source instanceof LegacyHadoopConfigurationSource) {
       return new OzoneConfiguration(((LegacyHadoopConfigurationSource) source)
@@ -102,42 +104,10 @@ public class OzoneConfiguration extends Configuration
   }
 
   public OzoneConfiguration() {
-    OzoneConfiguration.activate();
-    loadDefaults();
   }
 
   public OzoneConfiguration(Configuration conf) {
     super(conf);
-    //load the configuration from the classloader of the original conf.
-    setClassLoader(conf.getClassLoader());
-    if (!(conf instanceof OzoneConfiguration)) {
-      loadDefaults();
-      addResource(conf);
-    }
-  }
-
-  private void loadDefaults() {
-    try {
-      //there could be multiple ozone-default-generated.xml files on the
-      // classpath, which are generated by the annotation processor.
-      // Here we add all of them to the list of the available configuration.
-      Enumeration<URL> generatedDefaults =
-          OzoneConfiguration.class.getClassLoader().getResources(
-              "ozone-default-generated.xml");
-      while (generatedDefaults.hasMoreElements()) {
-        addResource(generatedDefaults.nextElement());
-      }
-    } catch (IOException e) {
-      e.printStackTrace();
-    }
-    addResource("ozone-default.xml");
-    // Adding core-site here because properties from core-site are
-    // distributed to executors by spark driver. Ozone properties which are
-    // added to core-site, will be overridden by properties from adding Resource
-    // ozone-default.xml. So, adding core-site again will help to resolve
-    // this override issue.
-    addResource("core-site.xml");
-    addResource("ozone-site.xml");
   }
 
   public List<Property> readPropertyFromXml(URL url) throws JAXBException {
@@ -239,10 +209,31 @@ public class OzoneConfiguration extends Configuration
     }
   }
 
+  /** Add default resources. */
   public static void activate() {
-    // adds the default resources
-    Configuration.addDefaultResource("hdfs-default.xml");
-    Configuration.addDefaultResource("hdfs-site.xml");
+    // core-default and core-site are added by parent class
+    addDefaultResource("hdfs-default.xml");
+    addDefaultResource("hdfs-site.xml");
+
+    // Modules with @Config annotations.  If new one is introduced, add it to this list.
+    String[] modules = new String[] {
+        "hdds-common",
+        "hdds-client",
+        "hdds-container-service",
+        "hdds-server-framework",
+        "hdds-server-scm",
+        "ozone-common",
+        "ozone-csi",
+        "ozone-manager",
+        "ozone-recon",
+    };
+    for (String module : modules) {
+      addDefaultResource(module + "-default.xml");
+    }
+
+    // Non-generated configs
+    addDefaultResource("ozone-default.xml");
+    addDefaultResource("ozone-site.xml");
   }
 
   /**
@@ -358,8 +349,6 @@ public class OzoneConfiguration extends Configuration
             ScmConfigKeys.HDDS_CONTAINER_RATIS_NUM_CONTAINER_OP_EXECUTORS_KEY),
         new DeprecationDelta("dfs.container.ratis.num.write.chunk.threads.per.volume",
             ScmConfigKeys.HDDS_CONTAINER_RATIS_NUM_WRITE_CHUNK_THREADS_PER_VOLUME),
-        new DeprecationDelta("dfs.container.ratis.replication.level",
-            ScmConfigKeys.HDDS_CONTAINER_RATIS_REPLICATION_LEVEL_KEY),
         new DeprecationDelta("dfs.container.ratis.rpc.type",
             ScmConfigKeys.HDDS_CONTAINER_RATIS_RPC_TYPE_KEY),
         new DeprecationDelta("dfs.container.ratis.segment.preallocated.size",
@@ -408,6 +397,8 @@ public class OzoneConfiguration extends Configuration
             HddsConfigKeys.HDDS_DATANODE_KERBEROS_KEYTAB_FILE_KEY),
         new DeprecationDelta("dfs.metrics.percentiles.intervals",
             HddsConfigKeys.HDDS_METRICS_PERCENTILES_INTERVALS_KEY),
+        new DeprecationDelta("hdds.recon.heartbeat.interval",
+            HddsConfigKeys.HDDS_RECON_HEARTBEAT_INTERVAL),
     });
   }
 
@@ -434,8 +425,6 @@ public class OzoneConfiguration extends Configuration
     return Integer.parseInt(value);
   }
 
-  private Properties delegatingProps;
-
   @Override
   public synchronized void reloadConfiguration() {
     super.reloadConfiguration();
@@ -476,6 +465,36 @@ public class OzoneConfiguration extends Configuration
     }
   }
 
+  /**
+   * Get a duration value from the configuration, and default to the given value if it's invalid.
+   * @param logger the logger to use
+   * @param key the key to get the value from
+   * @param defaultValue the default value to use if the key is not set
+   * @param unit the unit of the duration
+   * @return the duration value
+   */
+  public long getOrFixDuration(Logger logger, String key, String defaultValue, TimeUnit unit) {
+    maybeFixInvalidDuration(logger, key, defaultValue, unit);
+    return getTimeDuration(key, defaultValue, unit);
+  }
+
+  private boolean maybeFixInvalidDuration(Logger logger, String key, String defaultValue, TimeUnit unit) {
+    boolean fixed = maybeFixInvalidDuration(key, defaultValue, unit);
+    if (fixed) {
+      logger.warn("{} must be greater than zero, defaulting to {}", key, defaultValue);
+    }
+    return fixed;
+  }
+
+  private boolean maybeFixInvalidDuration(String key, String defaultValue, TimeUnit unit) {
+    long duration = getTimeDuration(key, defaultValue, unit);
+    if (duration <= 0) {
+      set(key, defaultValue);
+      return true;
+    }
+    return false;
+  }
+
   @Override
   public Iterator<Map.Entry<String, String>> iterator() {
     DelegatingProperties properties = (DelegatingProperties) getProps();
