@@ -27,6 +27,8 @@ import com.google.common.base.Preconditions;
 import com.google.common.base.Strings;
 import com.google.common.collect.ImmutableSet;
 import com.google.protobuf.ByteString;
+import java.net.InetAddress;
+import java.net.UnknownHostException;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.EnumSet;
@@ -34,6 +36,7 @@ import java.util.List;
 import java.util.Objects;
 import java.util.Set;
 import java.util.UUID;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.hadoop.hdds.DatanodeVersion;
 import org.apache.hadoop.hdds.HddsUtils;
 import org.apache.hadoop.hdds.annotation.InterfaceAudience;
@@ -64,11 +67,9 @@ import org.slf4j.LoggerFactory;
  */
 @InterfaceAudience.Private
 @InterfaceStability.Evolving
-public class DatanodeDetails extends NodeImpl implements
-    Comparable<DatanodeDetails> {
+public class DatanodeDetails extends NodeImpl implements Comparable<DatanodeDetails> {
 
-  private static final Logger LOG =
-      LoggerFactory.getLogger(DatanodeDetails.class);
+  private static final Logger LOG = LoggerFactory.getLogger(DatanodeDetails.class);
 
   private static final Codec<DatanodeDetails> CODEC = new DelegatedCodec<>(
       Proto2Codec.get(ExtendedDatanodeDetailsProto.getDefaultInstance()),
@@ -76,10 +77,6 @@ public class DatanodeDetails extends NodeImpl implements
       DatanodeDetails::getExtendedProtoBufMessage,
       DatanodeDetails.class);
 
-  public static Codec<DatanodeDetails> getCodec() {
-    return CODEC;
-  }
-
   /**
    * DataNode's unique identifier in the cluster.
    */
@@ -142,6 +139,10 @@ public class DatanodeDetails extends NodeImpl implements
     this.currentVersion = datanodeDetails.getCurrentVersion();
   }
 
+  public static Codec<DatanodeDetails> getCodec() {
+    return CODEC;
+  }
+
   public DatanodeID getID() {
     return id;
   }
@@ -175,13 +176,41 @@ public class DatanodeDetails extends NodeImpl implements
     this.ipAddress = StringWithByteString.valueOf(ip);
   }
 
+  /**
+   * Resolves and validates the IP address of the datanode based on its hostname.
+   * If the resolved IP address differs from the current IP address,
+   * it updates the IP address to the newly resolved value.
+   */
+  public void validateDatanodeIpAddress() {
+    final String oldIP = getIpAddress();
+    final String hostname = getHostName();
+    if (StringUtils.isBlank(hostname)) {
+      LOG.warn("Could not resolve IP address of datanode '{}'", this);
+      return;
+    }
+
+    try {
+      final String newIP = InetAddress.getByName(hostname).getHostAddress();
+      if (StringUtils.isBlank(newIP)) {
+        throw new UnknownHostException("New IP address is invalid: " + newIP);
+      }
+
+      if (!newIP.equals(oldIP)) {
+        LOG.info("Updating IP address of datanode {} to {}", this, newIP);
+        setIpAddress(newIP);
+      }
+    } catch (UnknownHostException e) {
+      LOG.warn("Could not resolve IP address of datanode '{}'", this, e);
+    }
+  }
+
   /**
    * Returns IP address of DataNode.
    *
    * @return IP address
    */
   public String getIpAddress() {
-    return ipAddress.getString();
+    return ipAddress == null ? null : ipAddress.getString();
   }
 
   /**
@@ -208,7 +237,7 @@ public class DatanodeDetails extends NodeImpl implements
    * @return Hostname
    */
   public String getHostName() {
-    return hostName.getString();
+    return hostName == null ? null : hostName.getString();
   }
 
   /**
@@ -383,7 +412,6 @@ public class DatanodeDetails extends NodeImpl implements
     return getPort(Name.STANDALONE);
   }
 
-
   /**
    * Starts building a new DatanodeDetails from the protobuf input.
    *
@@ -650,7 +678,6 @@ public class DatanodeDetails extends NodeImpl implements
         id.equals(((DatanodeDetails) obj).id);
   }
 
-
   /**
    * Checks hostname, ipAddress and port of the 2 nodes are the same.
    * @param datanodeDetails dnDetails object to compare with.
@@ -987,6 +1014,8 @@ public class DatanodeDetails extends NodeImpl implements
    * Container to hold DataNode Port details.
    */
   public static final class Port {
+    private final Name name;
+    private final Integer value;
 
     /**
      * Ports that are supported in DataNode.
@@ -1011,9 +1040,6 @@ public class DatanodeDetails extends NodeImpl implements
           EnumSet.of(STANDALONE, RATIS, RATIS_DATASTREAM));
     }
 
-    private final Name name;
-    private final Integer value;
-
     /**
      * Private constructor for constructing Port object. Use
      * DatanodeDetails#newPort to create a new Port object.
