@@ -30,12 +30,12 @@ import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 import java.util.UUID;
+import java.util.concurrent.TimeUnit;
 import java.util.function.BiFunction;
 import java.util.function.BooleanSupplier;
 import java.util.stream.Collectors;
 import javax.net.ssl.TrustManager;
 import org.apache.hadoop.hdds.HddsConfigKeys;
-import org.apache.hadoop.hdds.StringUtils;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
@@ -66,6 +66,7 @@ import org.apache.ratis.retry.RetryPolicies;
 import org.apache.ratis.retry.RetryPolicy;
 import org.apache.ratis.rpc.RpcType;
 import org.apache.ratis.rpc.SupportedRpcType;
+import org.apache.ratis.server.RaftServerConfigKeys;
 import org.apache.ratis.thirdparty.com.google.protobuf.ByteString;
 import org.apache.ratis.thirdparty.io.netty.buffer.ByteBuf;
 import org.apache.ratis.util.JavaUtils;
@@ -411,11 +412,9 @@ public final class RatisHelper {
     });
   }
 
-
   private static Map<String, String> getDatanodeRatisPrefixProps(
       ConfigurationSource configuration) {
-    return configuration.getPropsMatchPrefixAndTrimPrefix(
-        StringUtils.appendIfNotPresent(HDDS_DATANODE_RATIS_PREFIX_KEY, '.'));
+    return configuration.getPropsMatchPrefixAndTrimPrefix(HDDS_DATANODE_RATIS_PREFIX_KEY + '.');
   }
 
   // For External gRPC client to server with gRPC TLS.
@@ -529,7 +528,6 @@ public final class RatisHelper {
     return RaftPeer.newBuilder(peer).setPriority(priority).build();
   }
 
-
   /**
    * Use raft client to send admin request, transfer the leadership.
    * 1. Set priority and send setConfiguration request
@@ -546,6 +544,12 @@ public final class RatisHelper {
           + group.getPeers().stream().map(RaftPeer::getId)
               .collect(Collectors.toList()) + ".");
     }
+    if (!group.getPeer(targetPeerId).getStartupRole().equals(RaftProtos.RaftPeerRole.FOLLOWER)) {
+      throw new IOException("Target " + targetPeerId + " not in FOLLOWER role. "
+          + group.getPeers().stream()
+          .map(p -> p.getId() + ":" + p.getStartupRole())
+          .collect(Collectors.toList()) + ".");
+    }
 
     LOG.info("Start transferring leadership to {}", targetPeerId);
     try (RaftClient client = newRaftClient(SupportedRpcType.GRPC, null,
@@ -568,13 +572,19 @@ public final class RatisHelper {
       RaftClientReply setConf = null;
       try {
         // Set priority
-        final List<RaftPeer> peersWithNewPriorities = group.getPeers().stream()
+        final List<RaftPeer> followerWithNewPriorities = group.getPeers().stream()
+            .filter(peer -> peer.getStartupRole().equals(RaftProtos.RaftPeerRole.FOLLOWER))
+            .map(peer -> newRaftPeer(peer, targetPeerId))
+            .collect(Collectors.toList());
+        final List<RaftPeer> listenerWithNewPriorities = group.getPeers().stream()
+            .filter(peer -> peer.getStartupRole().equals(RaftProtos.RaftPeerRole.LISTENER))
             .map(peer -> newRaftPeer(peer, targetPeerId))
             .collect(Collectors.toList());
         // Set new configuration
-        setConf = client.admin().setConfiguration(peersWithNewPriorities);
+        setConf = client.admin().setConfiguration(followerWithNewPriorities, listenerWithNewPriorities);
         if (setConf.isSuccess()) {
-          LOG.info("Successfully set priority: {}", peersWithNewPriorities);
+          LOG.info("Successfully set priority: Follower: {}, Listener: {}", followerWithNewPriorities,
+              listenerWithNewPriorities);
         } else {
           throw new IOException("Failed to set priority.",
               setConf.getException());
@@ -600,13 +610,19 @@ public final class RatisHelper {
   }
 
   private static void resetPriorities(RaftGroup original, RaftClient client) {
-    final List<RaftPeer> resetPeers = original.getPeers().stream()
+    final List<RaftPeer> resetFollower = original.getPeers().stream()
+        .filter(peer -> peer.getStartupRole().equals(RaftProtos.RaftPeerRole.FOLLOWER))
         .map(originalPeer -> RaftPeer.newBuilder(originalPeer)
             .setPriority(NEUTRAL_PRIORITY).build())
         .collect(Collectors.toList());
-    LOG.info("Resetting Raft peers priorities to {}", resetPeers);
+    final List<RaftPeer> resetListener = original.getPeers().stream()
+        .filter(peer -> peer.getStartupRole().equals(RaftProtos.RaftPeerRole.LISTENER))
+        .map(originalPeer -> RaftPeer.newBuilder(originalPeer)
+            .setPriority(NEUTRAL_PRIORITY).build())
+        .collect(Collectors.toList());
+    LOG.info("Resetting Raft peers priorities to Follower: {}, Listener: {}", resetFollower, resetListener);
     try {
-      RaftClientReply reply = client.admin().setConfiguration(resetPeers);
+      RaftClientReply reply = client.admin().setConfiguration(resetFollower, resetListener);
       if (reply.isSuccess()) {
         LOG.info("Successfully reset priorities: {}", original);
       } else {
@@ -645,4 +661,15 @@ public final class RatisHelper {
     return (int) (max / interval);
   }
 
+  public static void setFirstElectionTimeoutDuration(
+      ConfigurationSource conf, RaftProperties properties, String configKey) {
+    long firstElectionTimeout = conf.getTimeDuration(configKey, -1, TimeUnit.MILLISECONDS);
+    if (firstElectionTimeout > 0) {
+      RaftServerConfigKeys.Rpc.setFirstElectionTimeoutMin(
+          properties,  TimeDuration.valueOf(firstElectionTimeout, TimeUnit.MILLISECONDS));
+      RaftServerConfigKeys.Rpc.setFirstElectionTimeoutMax(
+          properties,  TimeDuration.valueOf(firstElectionTimeout + 200, TimeUnit.MILLISECONDS));
+    }
+  }
+
 }
