@@ -22,23 +22,20 @@ package org.apache.hadoop.hdds.scm.container;
  */
 public class ContainerNotFoundException extends ContainerException {
 
-  /**
-   * Constructs an {@code ContainerNotFoundException} with {@code null}
-   * as its error detail message.
-   */
   public ContainerNotFoundException() {
-    super(ResultCodes.CONTAINER_NOT_FOUND);
+    this("Container not found for unknown id");
   }
 
-  /**
-   * Constructs an {@code ContainerNotFoundException} with the specified
-   * detail message.
-   *
-   * @param message
-   *        The detail message (which is saved for later retrieval
-   *        by the {@link #getMessage()} method)
-   */
+  /** Required by {@link org.apache.hadoop.ipc.RemoteException#unwrapRemoteException()}. */
   public ContainerNotFoundException(String message) {
     super(message, ResultCodes.CONTAINER_NOT_FOUND);
   }
+
+  public ContainerNotFoundException(ContainerID containerID) {
+    this("Container " +  containerID + " not found");
+  }
+
+  public static ContainerNotFoundException newInstanceForTesting() {
+    return new ContainerNotFoundException("For testing");
+  }
 }
