@@ -17,9 +17,11 @@
 
 package org.apache.hadoop.hdds.scm.container;
 
+import com.fasterxml.jackson.databind.annotation.JsonSerialize;
 import java.util.UUID;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
+import org.apache.hadoop.hdds.server.JsonUtils;
 
 /**
  * Class which stores ContainerReplica details on the client.
@@ -34,6 +36,8 @@ public final class ContainerReplicaInfo {
   private long keyCount;
   private long bytesUsed;
   private int replicaIndex = -1;
+  @JsonSerialize(using = JsonUtils.ChecksumSerializer.class)
+  private long dataChecksum;
 
   public static ContainerReplicaInfo fromProto(
       HddsProtos.SCMContainerReplicaProto proto) {
@@ -47,7 +51,8 @@ public final class ContainerReplicaInfo {
         .setKeyCount(proto.getKeyCount())
         .setBytesUsed(proto.getBytesUsed())
         .setReplicaIndex(
-            proto.hasReplicaIndex() ? (int)proto.getReplicaIndex() : -1);
+            proto.hasReplicaIndex() ? (int)proto.getReplicaIndex() : -1)
+        .setDataChecksum(proto.getDataChecksum());
     return builder.build();
   }
 
@@ -86,6 +91,10 @@ public final class ContainerReplicaInfo {
     return replicaIndex;
   }
 
+  public long getDataChecksum() {
+    return dataChecksum;
+  }
+
   /**
    * Builder for ContainerReplicaInfo class.
    */
@@ -133,6 +142,11 @@ public final class ContainerReplicaInfo {
       return this;
     }
 
+    public Builder setDataChecksum(long dataChecksum) {
+      subject.dataChecksum = dataChecksum;
+      return this;
+    }
+
     public ContainerReplicaInfo build() {
       return subject;
     }
