@@ -1,13 +1,12 @@
-/**
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License.  You may obtain a copy of the License at
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -15,8 +14,12 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 package org.apache.hadoop.hdds.scm.net;
 
+import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
+
 /**
  * The interface defines a node in a network topology.
  * A node may be a leave representing a data node or an inner
@@ -126,4 +129,21 @@ public interface Node {
    * @return true if this node is under a specific scope
    */
   boolean isDescendant(String nodePath);
+
+  default HddsProtos.NetworkNode toProtobuf(
+      int clientVersion) {
+    return null;
+  }
+
+  static Node fromProtobuf(
+      HddsProtos.NetworkNode networkNode) {
+    if (networkNode.hasDatanodeDetails()) {
+      return DatanodeDetails.getFromProtoBuf(
+          networkNode.getDatanodeDetails());
+    } else if (networkNode.hasInnerNode()) {
+      return InnerNode.fromProtobuf(networkNode.getInnerNode());
+    } else {
+      return null;
+    }
+  }
 }
