@@ -18,7 +18,6 @@
 package org.apache.hadoop.hdds.scm.pipeline;
 
 import com.fasterxml.jackson.annotation.JsonIgnore;
-import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 import java.io.IOException;
@@ -31,17 +30,20 @@ import java.util.LinkedHashMap;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Optional;
 import java.util.Set;
-import java.util.UUID;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.commons.lang3.builder.EqualsBuilder;
 import org.apache.commons.lang3.builder.HashCodeBuilder;
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
+import org.apache.hadoop.hdds.client.ReplicatedReplicationConfig;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
+import org.apache.hadoop.hdds.client.StandaloneReplicationConfig;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.protocol.DatanodeID;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.DatanodeDetailsProto;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType;
@@ -49,6 +51,7 @@ import org.apache.hadoop.hdds.utils.db.Codec;
 import org.apache.hadoop.hdds.utils.db.DelegatedCodec;
 import org.apache.hadoop.hdds.utils.db.Proto2Codec;
 import org.apache.hadoop.ozone.ClientVersion;
+import org.apache.ratis.util.Preconditions;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -68,10 +71,6 @@ public final class Pipeline {
       Pipeline.class,
       DelegatedCodec.CopyType.UNSUPPORTED);
 
-  public static Codec<Pipeline> getCodec() {
-    return CODEC;
-  }
-
   private static final Logger LOG = LoggerFactory.getLogger(Pipeline.class);
   private final PipelineID id;
   private final ReplicationConfig replicationConfig;
@@ -82,11 +81,11 @@ public final class Pipeline {
   // nodes with ordered distance to client
   private final ImmutableList<DatanodeDetails> nodesInOrder;
   // Current reported Leader for the pipeline
-  private UUID leaderId;
+  private DatanodeID leaderId;
   // Timestamp for pipeline upon creation
   private Instant creationTimestamp;
   // suggested leader id with high priority
-  private final UUID suggestedLeaderId;
+  private final DatanodeID suggestedLeaderId;
 
   private final Instant stateEnterTime;
 
@@ -117,6 +116,10 @@ public final class Pipeline {
     stateEnterTime = Instant.now();
   }
 
+  public static Codec<Pipeline> getCodec() {
+    return CODEC;
+  }
+
   /**
    * Returns the ID of this pipeline.
    *
@@ -163,7 +166,7 @@ public final class Pipeline {
    *
    * @return Suggested LeaderId
    */
-  public UUID getSuggestedLeaderId() {
+  public DatanodeID getSuggestedLeaderId() {
     return suggestedLeaderId;
   }
 
@@ -175,18 +178,18 @@ public final class Pipeline {
   }
 
   /**
-   * Return the pipeline leader's UUID.
+   * Return the pipeline leader's DatanodeID.
    *
-   * @return DatanodeDetails.UUID.
+   * @return DatanodeDetails.DatanodeID.
    */
-  public UUID getLeaderId() {
+  public DatanodeID getLeaderId() {
     return leaderId;
   }
 
   /**
    * Pipeline object, outside of letting leader id to be set, is immutable.
    */
-  void setLeaderId(UUID leaderId) {
+  void setLeaderId(DatanodeID leaderId) {
     this.leaderId = leaderId;
   }
 
@@ -223,7 +226,6 @@ public final class Pipeline {
     return getNodeSet().equals(pipeline.getNodeSet());
   }
 
-
   /**
    * Return the replica index of the specific datanode in the datanode set.
    * <p>
@@ -257,7 +259,7 @@ public final class Pipeline {
     }
     Optional<DatanodeDetails> datanodeDetails =
         nodeStatus.keySet().stream().filter(d ->
-            d.getUuid().equals(leaderId)).findFirst();
+            d.getID().equals(leaderId)).findFirst();
     if (datanodeDetails.isPresent()) {
       return datanodeDetails.get();
     } else {
@@ -361,14 +363,11 @@ public final class Pipeline {
     return replicationConfig;
   }
 
-  public HddsProtos.Pipeline getProtobufMessage(int clientVersion)
-      throws UnknownPipelineStateException {
+  public HddsProtos.Pipeline getProtobufMessage(int clientVersion) {
     return getProtobufMessage(clientVersion, Collections.emptySet());
   }
 
-  public HddsProtos.Pipeline getProtobufMessage(int clientVersion, Set<DatanodeDetails.Port.Name> filterPorts)
-      throws UnknownPipelineStateException {
-
+  public HddsProtos.Pipeline getProtobufMessage(int clientVersion, Set<DatanodeDetails.Port.Name> filterPorts) {
     List<HddsProtos.DatanodeDetailsProto> members = new ArrayList<>();
     List<Integer> memberReplicaIndexes = new ArrayList<>();
 
@@ -393,19 +392,11 @@ public final class Pipeline {
       builder.setFactor(ReplicationConfig.getLegacyFactor(replicationConfig));
     }
     if (leaderId != null) {
-      HddsProtos.UUID uuid128 = HddsProtos.UUID.newBuilder()
-          .setMostSigBits(leaderId.getMostSignificantBits())
-          .setLeastSigBits(leaderId.getLeastSignificantBits())
-          .build();
-      builder.setLeaderID128(uuid128);
+      builder.setLeaderDatanodeID(leaderId.toProto());
     }
 
     if (suggestedLeaderId != null) {
-      HddsProtos.UUID uuid128 = HddsProtos.UUID.newBuilder()
-          .setMostSigBits(suggestedLeaderId.getMostSignificantBits())
-          .setLeastSigBits(suggestedLeaderId.getLeastSignificantBits())
-          .build();
-      builder.setSuggestedLeaderID(uuid128);
+      builder.setSuggestedLeaderDatanodeID(suggestedLeaderId.toProto());
     }
 
     // To save the message size on wire, only transfer the node order based on
@@ -427,24 +418,46 @@ public final class Pipeline {
     return builder.build();
   }
 
-  private static Pipeline getFromProtobufSetCreationTimestamp(
-      HddsProtos.Pipeline proto) throws UnknownPipelineStateException {
+  private static Pipeline getFromProtobufSetCreationTimestamp(HddsProtos.Pipeline proto) {
     return toBuilder(proto)
         .setCreateTimestamp(Instant.now())
         .build();
   }
 
-  public Pipeline copyWithNodesInOrder(List<DatanodeDetails> nodes) {
+  public Pipeline copyWithNodesInOrder(List<? extends DatanodeDetails> nodes) {
     return toBuilder().setNodesInOrder(nodes).build();
   }
 
-  public Builder toBuilder() {
-    return newBuilder(this);
+  public Pipeline copyForRead() {
+    if (replicationConfig.getReplicationType() == ReplicationType.STAND_ALONE) {
+      return this;
+    }
+
+    HddsProtos.ReplicationFactor factor = replicationConfig instanceof ReplicatedReplicationConfig
+        ? ((ReplicatedReplicationConfig) replicationConfig).getReplicationFactor()
+        : HddsProtos.ReplicationFactor.ONE;
+
+    return toBuilder()
+        .setReplicationConfig(StandaloneReplicationConfig.getInstance(factor))
+        .build();
   }
 
-  public static Builder toBuilder(HddsProtos.Pipeline pipeline)
-      throws UnknownPipelineStateException {
-    Preconditions.checkNotNull(pipeline, "Pipeline is null");
+  public Pipeline copyForReadFromNode(DatanodeDetails node) {
+    Preconditions.assertTrue(nodeStatus.containsKey(node), () -> node + " is not part of the pipeline " + id.getId());
+
+    return toBuilder()
+        .setNodes(Collections.singletonList(node))
+        .setReplicaIndexes(Collections.singletonMap(node, getReplicaIndex(node)))
+        .setReplicationConfig(StandaloneReplicationConfig.getInstance(HddsProtos.ReplicationFactor.ONE))
+        .build();
+  }
+
+  public Builder toBuilder() {
+    return new Builder(this);
+  }
+
+  public static Builder toBuilder(HddsProtos.Pipeline pipeline) {
+    Objects.requireNonNull(pipeline, "pipeline == null");
 
     Map<DatanodeDetails, Integer> nodes = new LinkedHashMap<>();
     int index = 0;
@@ -457,26 +470,30 @@ public final class Pipeline {
       nodes.put(DatanodeDetails.getFromProtoBuf(member), repIndex);
       index++;
     }
-    UUID leaderId = null;
-    if (pipeline.hasLeaderID128()) {
+    DatanodeID leaderId = null;
+    if (pipeline.hasLeaderDatanodeID()) {
+      leaderId = DatanodeID.of(pipeline.getLeaderDatanodeID().getUuid());
+    } else if (pipeline.hasLeaderID128()) {
       HddsProtos.UUID uuid = pipeline.getLeaderID128();
-      leaderId = new UUID(uuid.getMostSigBits(), uuid.getLeastSigBits());
+      leaderId = DatanodeID.of(uuid);
     } else if (pipeline.hasLeaderID() &&
         StringUtils.isNotEmpty(pipeline.getLeaderID())) {
-      leaderId = UUID.fromString(pipeline.getLeaderID());
+      leaderId = DatanodeID.fromUuidString(pipeline.getLeaderID());
     }
 
-    UUID suggestedLeaderId = null;
-    if (pipeline.hasSuggestedLeaderID()) {
+    DatanodeID suggestedLeaderId = null;
+    if (pipeline.hasSuggestedLeaderDatanodeID()) {
+      suggestedLeaderId = DatanodeID.of(pipeline.getSuggestedLeaderDatanodeID().getUuid());
+    } else if (pipeline.hasSuggestedLeaderID()) {
       HddsProtos.UUID uuid = pipeline.getSuggestedLeaderID();
-      suggestedLeaderId =
-          new UUID(uuid.getMostSigBits(), uuid.getLeastSigBits());
+      suggestedLeaderId = DatanodeID.of(uuid);
     }
 
     final ReplicationConfig config = ReplicationConfig
         .fromProto(pipeline.getType(), pipeline.getFactor(),
             pipeline.getEcReplicationConfig());
-    return new Builder().setId(PipelineID.getFromProtobuf(pipeline.getId()))
+    return newBuilder()
+        .setId(PipelineID.getFromProtobuf(pipeline.getId()))
         .setReplicationConfig(config)
         .setState(PipelineState.fromProtobuf(pipeline.getState()))
         .setNodes(new ArrayList<>(nodes.keySet()))
@@ -487,8 +504,7 @@ public final class Pipeline {
         .setCreateTimestamp(pipeline.getCreationTimeStamp());
   }
 
-  public static Pipeline getFromProtobuf(HddsProtos.Pipeline pipeline)
-      throws UnknownPipelineStateException {
+  public static Pipeline getFromProtobuf(HddsProtos.Pipeline pipeline) {
     return toBuilder(pipeline).build();
   }
 
@@ -522,20 +538,20 @@ public final class Pipeline {
   @Override
   public String toString() {
     final StringBuilder b =
-        new StringBuilder(getClass().getSimpleName()).append("{");
+        new StringBuilder(getClass().getSimpleName()).append('{');
     b.append(" Id: ").append(id.getId());
     b.append(", Nodes: [");
     for (DatanodeDetails datanodeDetails : nodeStatus.keySet()) {
       b.append(" {").append(datanodeDetails);
       b.append(", ReplicaIndex: ").append(this.getReplicaIndex(datanodeDetails)).append("},");
     }
-    b.append("]");
+    b.append(']');
     b.append(", ReplicationConfig: ").append(replicationConfig);
     b.append(", State:").append(getPipelineState());
     b.append(", leaderId:").append(leaderId != null ? leaderId.toString() : "");
     b.append(", CreationTimestamp").append(getCreationTimestamp()
         .atZone(ZoneId.systemDefault()));
-    b.append("}");
+    b.append('}');
     return b.toString();
   }
 
@@ -543,28 +559,24 @@ public final class Pipeline {
     return new Builder();
   }
 
-  public static Builder newBuilder(Pipeline pipeline) {
-    return new Builder(pipeline);
-  }
-
   /**
    * Builder class for Pipeline.
    */
-  public static class Builder {
+  public static final class Builder {
     private PipelineID id = null;
     private ReplicationConfig replicationConfig = null;
     private PipelineState state = null;
     private Map<DatanodeDetails, Long> nodeStatus = null;
     private List<Integer> nodeOrder = null;
     private List<DatanodeDetails> nodesInOrder = null;
-    private UUID leaderId = null;
+    private DatanodeID leaderId = null;
     private Instant creationTimestamp = null;
-    private UUID suggestedLeaderId = null;
+    private DatanodeID suggestedLeaderId = null;
     private Map<DatanodeDetails, Integer> replicaIndexes = ImmutableMap.of();
 
-    public Builder() { }
+    private Builder() { }
 
-    public Builder(Pipeline pipeline) {
+    private Builder(Pipeline pipeline) {
       this.id = pipeline.id;
       this.replicationConfig = pipeline.replicationConfig;
       this.state = pipeline.state;
@@ -585,6 +597,11 @@ public final class Pipeline {
       }
     }
 
+    public Builder setId(DatanodeID datanodeID) {
+      this.id = datanodeID.toPipelineID();
+      return this;
+    }
+
     public Builder setId(PipelineID id1) {
       this.id = id1;
       return this;
@@ -600,13 +617,26 @@ public final class Pipeline {
       return this;
     }
 
-    public Builder setLeaderId(UUID leaderId1) {
+    public Builder setLeaderId(DatanodeID leaderId1) {
       this.leaderId = leaderId1;
       return this;
     }
+
     public Builder setNodes(List<DatanodeDetails> nodes) {
-      this.nodeStatus = new LinkedHashMap<>();
-      nodes.forEach(node -> nodeStatus.put(node, -1L));
+      Map<DatanodeDetails, Long> newNodeStatus = new LinkedHashMap<>();
+      nodes.forEach(node -> newNodeStatus.put(node, -1L));
+
+      // replace pipeline ID if nodes are not the same
+      if (nodeStatus != null && !nodeStatus.keySet().equals(newNodeStatus.keySet())) {
+        if (nodes.size() == 1) {
+          setId(nodes.iterator().next().getID());
+        } else {
+          setId(PipelineID.randomId());
+        }
+      }
+
+      nodeStatus = newNodeStatus;
+
       if (nodesInOrder != null) {
         // nodesInOrder may belong to another pipeline, avoid overwriting it
         nodesInOrder = new LinkedList<>(nodesInOrder);
@@ -622,7 +652,7 @@ public final class Pipeline {
       return this;
     }
 
-    public Builder setNodesInOrder(List<DatanodeDetails> nodes) {
+    public Builder setNodesInOrder(List<? extends DatanodeDetails> nodes) {
       this.nodesInOrder = new LinkedList<>(nodes);
       return this;
     }
@@ -637,22 +667,21 @@ public final class Pipeline {
       return this;
     }
 
-    public Builder setSuggestedLeaderId(UUID uuid) {
-      this.suggestedLeaderId = uuid;
+    public Builder setSuggestedLeaderId(DatanodeID dnId) {
+      this.suggestedLeaderId = dnId;
       return this;
     }
 
-
     public Builder setReplicaIndexes(Map<DatanodeDetails, Integer> indexes) {
       this.replicaIndexes = indexes == null ? ImmutableMap.of() : ImmutableMap.copyOf(indexes);
       return this;
     }
 
     public Pipeline build() {
-      Preconditions.checkNotNull(id);
-      Preconditions.checkNotNull(replicationConfig);
-      Preconditions.checkNotNull(state);
-      Preconditions.checkNotNull(nodeStatus);
+      Objects.requireNonNull(id, "id == null");
+      Objects.requireNonNull(replicationConfig, "replicationConfig == null");
+      Objects.requireNonNull(state, "state == null");
+      Objects.requireNonNull(nodeStatus, "nodeStatus == null");
 
       if (nodeOrder != null && !nodeOrder.isEmpty()) {
         List<DatanodeDetails> nodesWithOrder = new ArrayList<>();
@@ -684,31 +713,29 @@ public final class Pipeline {
   public enum PipelineState {
     ALLOCATED, OPEN, DORMANT, CLOSED;
 
-    public static PipelineState fromProtobuf(HddsProtos.PipelineState state)
-        throws UnknownPipelineStateException {
-      Preconditions.checkNotNull(state, "Pipeline state is null");
+    public static PipelineState fromProtobuf(HddsProtos.PipelineState state) {
+      Objects.requireNonNull(state, "state == null");
       switch (state) {
       case PIPELINE_ALLOCATED: return ALLOCATED;
       case PIPELINE_OPEN: return OPEN;
       case PIPELINE_DORMANT: return DORMANT;
       case PIPELINE_CLOSED: return CLOSED;
       default:
-        throw new UnknownPipelineStateException(
-            "Pipeline state: " + state + " is not recognized.");
+        throw new IllegalArgumentException("Unexpected value " + state
+            + " from " + state.getClass());
       }
     }
 
-    public static HddsProtos.PipelineState getProtobuf(PipelineState state)
-        throws UnknownPipelineStateException {
-      Preconditions.checkNotNull(state, "Pipeline state is null");
+    public static HddsProtos.PipelineState getProtobuf(PipelineState state) {
+      Objects.requireNonNull(state, "state == null");
       switch (state) {
       case ALLOCATED: return HddsProtos.PipelineState.PIPELINE_ALLOCATED;
       case OPEN: return HddsProtos.PipelineState.PIPELINE_OPEN;
       case DORMANT: return HddsProtos.PipelineState.PIPELINE_DORMANT;
       case CLOSED: return HddsProtos.PipelineState.PIPELINE_CLOSED;
       default:
-        throw new UnknownPipelineStateException(
-            "Pipeline state: " + state + " is not recognized.");
+        throw new IllegalArgumentException("Unexpected value " + state
+            + " from " + state.getClass());
       }
     }
   }
