@@ -37,18 +37,20 @@ import org.slf4j.LoggerFactory;
  */
 public abstract class BackgroundService {
 
-  @VisibleForTesting
-  public static final Logger LOG =
+  protected static final Logger LOG =
       LoggerFactory.getLogger(BackgroundService.class);
 
   // Executor to launch child tasks
-  private final ScheduledThreadPoolExecutor exec;
-  private final ThreadGroup threadGroup;
+  private ScheduledThreadPoolExecutor exec;
+  private ThreadGroup threadGroup;
   private final String serviceName;
-  private final long interval;
-  private final long serviceTimeoutInNanos;
-  private final TimeUnit unit;
+  private long interval;
+  private volatile long serviceTimeoutInNanos;
+  private TimeUnit unit;
+  private final int threadPoolSize;
+  private final String threadNamePrefix;
   private final PeriodicalTask service;
+  private CompletableFuture<Void> future;
 
   public BackgroundService(String serviceName, long interval,
       TimeUnit unit, int threadPoolSize, long serviceTimeout) {
@@ -63,23 +65,23 @@ public abstract class BackgroundService {
     this.serviceName = serviceName;
     this.serviceTimeoutInNanos = TimeDuration.valueOf(serviceTimeout, unit)
             .toLong(TimeUnit.NANOSECONDS);
-    threadGroup = new ThreadGroup(serviceName);
-    ThreadFactory threadFactory = new ThreadFactoryBuilder()
-        .setThreadFactory(r -> new Thread(threadGroup, r))
-        .setDaemon(true)
-        .setNameFormat(threadNamePrefix + serviceName + "#%d")
-        .build();
-    exec = (ScheduledThreadPoolExecutor) Executors.newScheduledThreadPool(
-        threadPoolSize, threadFactory);
+    this.threadPoolSize = threadPoolSize;
+    this.threadNamePrefix = threadNamePrefix;
+    initExecutorAndThreadGroup();
     service = new PeriodicalTask();
+    this.future = CompletableFuture.completedFuture(null);
+  }
+
+  protected CompletableFuture<Void> getFuture() {
+    return future;
   }
 
   @VisibleForTesting
-  public ExecutorService getExecutorService() {
+  public synchronized ExecutorService getExecutorService() {
     return this.exec;
   }
 
-  public void setPoolSize(int size) {
+  public synchronized void setPoolSize(int size) {
     if (size <= 0) {
       throw new IllegalArgumentException("Pool size must be positive.");
     }
@@ -90,6 +92,11 @@ public abstract class BackgroundService {
     exec.setCorePoolSize(size);
   }
 
+  public synchronized void setServiceTimeoutInNanos(long newTimeout) {
+    LOG.info("{} timeout is set to {} {}", serviceName, newTimeout, TimeUnit.NANOSECONDS.name().toLowerCase());
+    this.serviceTimeoutInNanos = newTimeout;
+  }
+
   @VisibleForTesting
   public int getThreadCount() {
     return threadGroup.activeCount();
@@ -101,23 +108,48 @@ public abstract class BackgroundService {
     while (!tasks.isEmpty()) {
       tasks.poll().call();
     }
+    execTaskCompletion();
   }
 
-
   // start service
-  public void start() {
+  public synchronized void start() {
+    if (exec == null || exec.isShutdown() || exec.isTerminated()) {
+      initExecutorAndThreadGroup();
+    }
+    LOG.info("Starting service {} with interval {} {}", serviceName,
+        interval, unit.name().toLowerCase());
     exec.scheduleWithFixedDelay(service, 0, interval, unit);
   }
 
+  protected synchronized void setInterval(long newInterval, TimeUnit newUnit) {
+    this.interval = newInterval;
+    this.unit = newUnit;
+  }
+
+  protected synchronized long getIntervalMillis() {
+    return this.unit.toMillis(interval);
+  }
+
   public abstract BackgroundTaskQueue getTasks();
 
+  protected void execTaskCompletion() { }
+
   /**
    * Run one or more background tasks concurrently.
    * Wait until all tasks to return the result.
    */
   public class PeriodicalTask implements Runnable {
     @Override
-    public synchronized void run() {
+    public void run() {
+      // wait for previous set of tasks to complete
+      try {
+        future.join();
+      } catch (RuntimeException e) {
+        LOG.error("Background service execution failed.", e);
+      } finally {
+        execTaskCompletion();
+      }
+
       if (LOG.isDebugEnabled()) {
         LOG.debug("Running background service : {}", serviceName);
       }
@@ -130,35 +162,36 @@ public abstract class BackgroundService {
       if (LOG.isDebugEnabled()) {
         LOG.debug("Number of background tasks to execute : {}", tasks.size());
       }
-
-      while (!tasks.isEmpty()) {
-        BackgroundTask task = tasks.poll();
-        CompletableFuture.runAsync(() -> {
-          long startTime = System.nanoTime();
-          try {
-            BackgroundTaskResult result = task.call();
-            if (LOG.isDebugEnabled()) {
-              LOG.debug("task execution result size {}", result.getSize());
+      synchronized (BackgroundService.this) {
+        while (!tasks.isEmpty()) {
+          BackgroundTask task = tasks.poll();
+          future = future.thenCombine(CompletableFuture.runAsync(() -> {
+            long startTime = System.nanoTime();
+            try {
+              BackgroundTaskResult result = task.call();
+              if (LOG.isDebugEnabled()) {
+                LOG.debug("task execution result size {}", result.getSize());
+              }
+            } catch (Throwable e) {
+              LOG.error("Background task execution failed", e);
+              if (e instanceof Error) {
+                throw (Error) e;
+              }
+            } finally {
+              long endTime = System.nanoTime();
+              if (endTime - startTime > serviceTimeoutInNanos) {
+                LOG.warn("{} Background task execution took {}ns > {}ns(timeout)",
+                    serviceName, endTime - startTime, serviceTimeoutInNanos);
+              }
             }
-          } catch (Throwable e) {
-            LOG.error("Background task execution failed", e);
-            if (e instanceof Error) {
-              throw (Error) e;
-            }
-          } finally {
-            long endTime = System.nanoTime();
-            if (endTime - startTime > serviceTimeoutInNanos) {
-              LOG.warn("{} Background task execution took {}ns > {}ns(timeout)",
-                  serviceName, endTime - startTime, serviceTimeoutInNanos);
-            }
-          }
-        }, exec);
+          }, exec).exceptionally(e -> null), (Void1, Void) -> null);
+        }
       }
     }
   }
 
   // shutdown and make sure all threads are properly released.
-  public void shutdown() {
+  public synchronized void shutdown() {
     LOG.info("Shutting down service {}", this.serviceName);
     exec.shutdown();
     try {
@@ -174,4 +207,14 @@ public abstract class BackgroundService {
       threadGroup.destroy();
     }
   }
+
+  private void initExecutorAndThreadGroup() {
+    threadGroup = new ThreadGroup(serviceName);
+    ThreadFactory threadFactory = new ThreadFactoryBuilder()
+        .setThreadFactory(r -> new Thread(threadGroup, r))
+        .setDaemon(true)
+        .setNameFormat(threadNamePrefix + serviceName + "#%d")
+        .build();
+    exec = (ScheduledThreadPoolExecutor) Executors.newScheduledThreadPool(threadPoolSize, threadFactory);
+  }
 }
