@@ -51,12 +51,35 @@ import org.slf4j.LoggerFactory;
  * for supporting RocksDB direct {@link ByteBuffer} APIs.
  */
 public class CodecBuffer implements UncheckedAutoCloseable {
-  public static final Logger LOG = LoggerFactory.getLogger(CodecBuffer.class);
+  private static final Logger LOG = LoggerFactory.getLogger(CodecBuffer.class);
+
+  private static final ByteBufAllocator POOL = PooledByteBufAllocator.DEFAULT;
+
+  private static final IntFunction<ByteBuf> POOL_DIRECT = c -> c >= 0
+      ? POOL.directBuffer(c, c) // allocate exact size
+      : POOL.directBuffer(-c);  // allocate a resizable buffer
+
+  private static final IntFunction<ByteBuf> POOL_HEAP = c -> c >= 0
+      ? POOL.heapBuffer(c, c)   // allocate exact size
+      : POOL.heapBuffer(-c);    // allocate a resizable buffer
+
+  private static final CodecBuffer EMPTY_BUFFER = new CodecBuffer(new EmptyByteBuf(POOL), null);
+
+  private static final AtomicInteger LEAK_COUNT = new AtomicInteger();
+
+  private final StackTraceElement[] elements;
+
+  private final ByteBuf buf;
+
+  private final Object wrapped;
+
+  private final CompletableFuture<Void> released = new CompletableFuture<>();
 
   /** To create {@link CodecBuffer} instances. */
   private static class Factory {
     private static volatile BiFunction<ByteBuf, Object, CodecBuffer> constructor
         = CodecBuffer::new;
+
     static void set(BiFunction<ByteBuf, Object, CodecBuffer> f, String name) {
       constructor = f;
       LOG.info("Successfully set constructor to {}: {}", name, f);
@@ -123,18 +146,6 @@ public class CodecBuffer implements UncheckedAutoCloseable {
     }
   }
 
-  private static final ByteBufAllocator POOL
-      = PooledByteBufAllocator.DEFAULT;
-  private static final IntFunction<ByteBuf> POOL_DIRECT = c -> c >= 0
-      ? POOL.directBuffer(c, c) // allocate exact size
-      : POOL.directBuffer(-c);  // allocate a resizable buffer
-  private static final IntFunction<ByteBuf> POOL_HEAP = c -> c >= 0
-      ? POOL.heapBuffer(c, c)   // allocate exact size
-      : POOL.heapBuffer(-c);    // allocate a resizable buffer
-
-  private static final CodecBuffer EMPTY_BUFFER = new CodecBuffer(
-      new EmptyByteBuf(POOL), null);
-
   public static CodecBuffer getEmptyBuffer() {
     return EMPTY_BUFFER;
   }
@@ -153,10 +164,6 @@ public class CodecBuffer implements UncheckedAutoCloseable {
       }
     };
 
-    static Allocator getDirect() {
-      return DIRECT;
-    }
-
     Allocator HEAP = new Allocator() {
       @Override
       public CodecBuffer apply(int capacity) {
@@ -169,6 +176,10 @@ public class CodecBuffer implements UncheckedAutoCloseable {
       }
     };
 
+    static Allocator getDirect() {
+      return DIRECT;
+    }
+
     static Allocator getHeap() {
       return HEAP;
     }
@@ -177,8 +188,6 @@ public class CodecBuffer implements UncheckedAutoCloseable {
     boolean isDirect();
   }
 
-  private final StackTraceElement[] elements;
-
   /**
    * Allocate a buffer using the given allocator.
    *
@@ -222,8 +231,6 @@ public class CodecBuffer implements UncheckedAutoCloseable {
         Unpooled.wrappedBuffer(bytes.asReadOnlyByteBuffer()), bytes);
   }
 
-  private static final AtomicInteger LEAK_COUNT = new AtomicInteger();
-
   /** Assert the number of leak detected is zero. */
   public static void assertNoLeaks() {
     final long leak = LEAK_COUNT.get();
@@ -232,10 +239,6 @@ public class CodecBuffer implements UncheckedAutoCloseable {
     }
   }
 
-  private final ByteBuf buf;
-  private final Object wrapped;
-  private final CompletableFuture<Void> released = new CompletableFuture<>();
-
   private CodecBuffer(ByteBuf buf, Object wrapped) {
     this.buf = buf;
     this.wrapped = wrapped;
@@ -459,16 +462,16 @@ public class CodecBuffer implements UncheckedAutoCloseable {
    * @param source put bytes to an {@link OutputStream} and return the size.
    *               The returned size must be non-null and non-negative.
    * @return this object.
-   * @throws IOException in case the source throws an {@link IOException}.
+   * @throws CodecException in case the source throws an {@link IOException}.
    */
-  public CodecBuffer put(
-      CheckedFunction<OutputStream, Integer, IOException> source)
-      throws IOException {
+  public CodecBuffer put(CheckedFunction<OutputStream, Integer, IOException> source) throws CodecException {
     assertRefCnt(1);
     final int w = buf.writerIndex();
     final int size;
     try (ByteBufOutputStream out = new ByteBufOutputStream(buf)) {
       size = source.apply(out);
+    } catch (IOException e) {
+      throw new CodecException("Failed to apply source to " + this + ", " + source, e);
     }
     final ByteBuf returned = buf.setIndex(buf.readerIndex(), w + size);
     Preconditions.assertSame(buf, returned, "buf");
