@@ -29,8 +29,6 @@ import org.apache.hadoop.hdds.annotation.InterfaceAudience;
  */
 @InterfaceAudience.Private
 public final class OzoneConsts {
-
-
   public static final String STORAGE_DIR = "scm";
   public static final String SCM_ID = "scmUuid";
   public static final String SCM_HA = "scmHA";
@@ -67,7 +65,7 @@ public final class OzoneConsts {
   public static final String OZONE_ACL_CREATE = "c";
   public static final String OZONE_ACL_READ_ACL = "x";
   public static final String OZONE_ACL_WRITE_ACL = "y";
-
+  public static final String OZONE_ACL_ASSUME_ROLE = "m";
 
   public static final String OZONE_DATE_FORMAT =
       "EEE, dd MMM yyyy HH:mm:ss zzz";
@@ -90,8 +88,8 @@ public final class OzoneConsts {
   public static final String OZONE_ROOT = OZONE_URI_DELIMITER;
   public static final Path ROOT_PATH = Paths.get(OZONE_ROOT);
 
-
   public static final String CONTAINER_EXTENSION = ".container";
+  public static final String CONTAINER_DATA_CHECKSUM_EXTENSION = ".tree";
   public static final String CONTAINER_META_PATH = "metadata";
   public static final String CONTAINER_TEMPORARY_CHUNK_PREFIX = "tmp";
   public static final String CONTAINER_CHUNK_NAME_DELIMITER = ".";
@@ -145,6 +143,7 @@ public final class OzoneConsts {
   public static final String CONTAINER_BYTES_USED = "#BYTESUSED";
   public static final String PENDING_DELETE_BLOCK_COUNT =
       "#PENDINGDELETEBLOCKCOUNT";
+  public static final String CONTAINER_DATA_CHECKSUM = "#DATACHECKSUM";
 
   /**
    * OM LevelDB prefixes.
@@ -170,6 +169,7 @@ public final class OzoneConsts {
    */
 
   public static final String OM_KEY_PREFIX = "/";
+  public static final String DOUBLE_SLASH_OM_KEY_PREFIX  = "//";
   public static final String OM_USER_PREFIX = "$";
   public static final String OM_S3_PREFIX = "S3:";
   public static final String OM_S3_CALLER_CONTEXT_PREFIX = "S3Auth:S3G|";
@@ -181,18 +181,12 @@ public final class OzoneConsts {
    */
   public static final int OZONE_SCM_CHUNK_MAX_SIZE = 32 * 1024 * 1024;
 
-
   /**
    * Quota RESET default is -1, which means quota is not set.
    */
   public static final long QUOTA_RESET = -1;
   public static final long OLD_QUOTA_DEFAULT = -2;
 
-  /**
-   * Quota Units.
-   */
-  public enum Units { TB, GB, MB, KB, B }
-
   /**
    * Object ID to identify reclaimable uncommitted blocks.
    */
@@ -211,9 +205,21 @@ public final class OzoneConsts {
 
   public static final String SCM_CONTEXT_ATTRIBUTE = "ozone.scm";
 
-  private OzoneConsts() {
-    // Never Constructed
-  }
+  // YAML field constants for OmSnapshotLocalData (thus the OM_SLD_ prefix) YAML files
+  public static final String OM_SLD_VERSION = "version";
+  public static final String OM_SLD_CHECKSUM = "checksum";
+  public static final String OM_SLD_IS_SST_FILTERED = "isSSTFiltered";
+  public static final String OM_SLD_LAST_DEFRAG_TIME = "lastDefragTime";
+  public static final String OM_SLD_NEEDS_DEFRAG = "needsDefrag";
+  public static final String OM_SLD_VERSION_SST_FILE_INFO = "versionSstFileInfos";
+  public static final String OM_SLD_SNAP_ID = "snapshotId";
+  public static final String OM_SLD_PREV_SNAP_ID = "previousSnapshotId";
+  public static final String OM_SLD_VERSION_META_SST_FILES = "sstFiles";
+  public static final String OM_SLD_VERSION_META_PREV_SNAP_VERSION = "previousSnapshotVersion";
+  public static final String OM_SST_FILE_INFO_FILE_NAME = "fileName";
+  public static final String OM_SST_FILE_INFO_START_KEY = "startKey";
+  public static final String OM_SST_FILE_INFO_END_KEY = "endKey";
+  public static final String OM_SST_FILE_INFO_COL_FAMILY = "columnFamily";
 
   // YAML fields for .container files
   public static final String CONTAINER_ID = "containerID";
@@ -256,6 +262,8 @@ public final class OzoneConsts {
   public static final String DST_KEY = "dstKey";
   public static final String USED_BYTES = "usedBytes";
   public static final String USED_NAMESPACE = "usedNamespace";
+  public static final String SNAPSHOT_USED_BYTES = "snapshotUsedBytes";
+  public static final String SNAPSHOT_USED_NAMESPACE = "snapshotUsedNamespace";
   public static final String QUOTA_IN_BYTES = "quotaInBytes";
   public static final String QUOTA_IN_NAMESPACE = "quotaInNamespace";
   public static final String OBJECT_ID = "objectID";
@@ -303,6 +311,13 @@ public final class OzoneConsts {
   public static final String TENANT = "tenant";
   public static final String USER_PREFIX = "userPrefix";
   public static final String REWRITE_GENERATION = "rewriteGeneration";
+  public static final String FROM_SNAPSHOT = "fromSnapshot";
+  public static final String TO_SNAPSHOT = "toSnapshot";
+  public static final String TOKEN = "token";
+  public static final String PAGE_SIZE = "pageSize";
+  public static final String FORCE_FULL_DIFF = "forceFullDiff";
+  public static final String DISABLE_NATIVE_DIFF = "disableNativeDiff";
+  public static final String JOB_STATUS = "jobStatus";
 
   // For multi-tenancy
   public static final String TENANT_ID_USERNAME_DELIMITER = "$";
@@ -504,6 +519,7 @@ public final class OzoneConsts {
   public static final String OM_SNAPSHOT_DIR = "db.snapshots";
   public static final String OM_SNAPSHOT_CHECKPOINT_DIR = OM_SNAPSHOT_DIR
       + OM_KEY_PREFIX + "checkpointState";
+  public static final String OM_SNAPSHOT_CHECKPOINT_DEFRAGGED_DIR = "checkpointStateDefragged";
   public static final String OM_SNAPSHOT_DIFF_DIR = OM_SNAPSHOT_DIR
       + OM_KEY_PREFIX + "diffState";
 
@@ -538,4 +554,19 @@ public final class OzoneConsts {
    * S3G multipart upload request's ETag header key.
    */
   public static final String ETAG = "ETag";
+
+  /**
+   * A constant string used as a separator in various contexts within
+   * the OMDBCheckpoint functions.
+   */
+  public static final String OM_SNAPSHOT_SEPARATOR = "-";
+
+  private OzoneConsts() {
+    // Never Constructed
+  }
+
+  /**
+   * Quota Units.
+   */
+  public enum Units { TB, GB, MB, KB, B }
 }
