@@ -1,29 +1,29 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License.  You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * Some portions of this file Copyright (c) 2004-2006 Intel Corportation
- * and licensed under the BSD license.
  */
-package org.apache.hadoop.ozone.common;
 
-import org.apache.ratis.util.Preconditions;
+/*
+ * Some portions of this file Copyright (c) 2004-2006 Intel Corporation and licensed under the BSD license.
+ */
+
+package org.apache.hadoop.ozone.common;
 
 import java.nio.ByteBuffer;
 import java.util.zip.Checksum;
+import org.apache.ratis.util.Preconditions;
 
 /**
  * A sub-interface of {@link Checksum}
@@ -35,7 +35,12 @@ public interface ChecksumByteBuffer extends Checksum {
    * Upon return, the buffer's position will be equal to its limit.
    *
    * @param buffer the bytes to update the checksum with
+   *
+   * @apiNote {@link Override} annotation is missing since {@link Checksum#update(ByteBuffer)} introduced only in Java9.
+   * TODO: Remove when Java 1.8 support is dropped.
+   * TODO: <a href="https://issues.apache.org/jira/browse/HDDS-12366">HDDS-12366</a>
    */
+  @SuppressWarnings("PMD.MissingOverride")
   void update(ByteBuffer buffer);
 
   @Override
