@@ -1,20 +1,20 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- *  with the License.  You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
- *  Unless required by applicable law or agreed to in writing, software
- *  distributed under the License is distributed on an "AS IS" BASIS,
- *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- *  See the License for the specific language governing permissions and
- *  limitations under the License.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
+
 package org.apache.hadoop.ozone.common;
 
 import java.io.IOException;
@@ -22,34 +22,31 @@ import java.nio.ByteBuffer;
 import java.nio.channels.GatheringByteChannel;
 import java.util.List;
 import java.util.Objects;
-import java.util.function.Function;
-import java.util.function.Supplier;
-
-import org.apache.hadoop.hdds.scm.ByteStringConversion;
-
+import org.apache.hadoop.hdds.utils.db.CodecBuffer;
 import org.apache.ratis.thirdparty.com.google.protobuf.ByteString;
+import org.apache.ratis.util.UncheckedAutoCloseable;
 
 /** Buffer for a block chunk. */
-public interface ChunkBuffer {
+public interface ChunkBuffer extends ChunkBufferToByteString, UncheckedAutoCloseable {
 
   /** Similar to {@link ByteBuffer#allocate(int)}. */
   static ChunkBuffer allocate(int capacity) {
     return allocate(capacity, 0);
   }
 
-  /**
-   * Similar to {@link ByteBuffer#allocate(int)}
+  /** Similar to {@link ByteBuffer#allocate(int)}
    * except that it can specify the increment.
    *
    * @param increment
    *   the increment size so that this buffer is allocated incrementally.
-   *   When increment <= 0, entire buffer is allocated in the beginning.
+   *   When increment {@literal <= 0}, entire buffer is allocated in the beginning.
    */
   static ChunkBuffer allocate(int capacity, int increment) {
     if (increment > 0 && increment < capacity) {
       return new IncrementalChunkBuffer(capacity, increment, false);
     }
-    return new ChunkBufferImplWithByteBuffer(ByteBuffer.allocate(capacity));
+    CodecBuffer codecBuffer = CodecBuffer.allocateDirect(capacity);
+    return new ChunkBufferImplWithByteBuffer(codecBuffer.asWritableByteBuffer(), codecBuffer);
   }
 
   /** Wrap the given {@link ByteBuffer} as a {@link ChunkBuffer}. */
@@ -57,7 +54,8 @@ public interface ChunkBuffer {
     return new ChunkBufferImplWithByteBuffer(buffer);
   }
 
-  /** Wrap the given list of {@link ByteBuffer}s as a {@link ChunkBuffer}. */
+  /** Wrap the given list of {@link ByteBuffer}s as a {@link ChunkBuffer},
+   * with a function called when buffers are released.*/
   static ChunkBuffer wrap(List<ByteBuffer> buffers) {
     Objects.requireNonNull(buffers, "buffers == null");
     if (buffers.size() == 1) {
@@ -86,6 +84,10 @@ public interface ChunkBuffer {
   /** Similar to {@link ByteBuffer#clear()}. */
   ChunkBuffer clear();
 
+  @Override
+  default void close() {
+  }
+
   /** Similar to {@link ByteBuffer#put(ByteBuffer)}. */
   ChunkBuffer put(ByteBuffer b);
 
@@ -136,53 +138,4 @@ public interface ChunkBuffer {
    * @return The number of bytes written, possibly zero
    */
   long writeTo(GatheringByteChannel channel) throws IOException;
-
-  /**
-   * Convert this buffer to a {@link ByteString}.
-   * The position and limit of this {@link ChunkBuffer} remains unchanged.
-   * The given function must preserve the position and limit
-   * of the input {@link ByteBuffer}.
-   */
-  default ByteString toByteString(Function<ByteBuffer, ByteString> function) {
-    return toByteStringImpl(b -> applyAndAssertFunction(b, function, this));
-  }
-
-  /**
-   * Convert this buffer(s) to a list of {@link ByteString}.
-   * The position and limit of this {@link ChunkBuffer} remains unchanged.
-   * The given function must preserve the position and limit
-   * of the input {@link ByteBuffer}.
-   */
-  default List<ByteString> toByteStringList(
-      Function<ByteBuffer, ByteString> function) {
-    return toByteStringListImpl(b -> applyAndAssertFunction(b, function, this));
-  }
-
-  // for testing
-  default ByteString toByteString() {
-    return toByteString(ByteStringConversion::safeWrap);
-  }
-
-  ByteString toByteStringImpl(Function<ByteBuffer, ByteString> function);
-
-  List<ByteString> toByteStringListImpl(
-      Function<ByteBuffer, ByteString> function);
-
-  static void assertInt(int expected, int computed, Supplier<String> prefix) {
-    if (expected != computed) {
-      throw new IllegalStateException(prefix.get()
-          + ": expected = " + expected + " but computed = " + computed);
-    }
-  }
-
-  /** Apply the function and assert if it preserves position and limit. */
-  static ByteString applyAndAssertFunction(ByteBuffer buffer,
-      Function<ByteBuffer, ByteString> function, Object name) {
-    final int pos = buffer.position();
-    final int lim = buffer.limit();
-    final ByteString bytes = function.apply(buffer);
-    assertInt(pos, buffer.position(), () -> name + ": Unexpected position");
-    assertInt(lim, buffer.limit(), () -> name + ": Unexpected limit");
-    return bytes;
-  }
 }
