@@ -17,6 +17,7 @@
 
 package org.apache.hadoop.ozone.util;
 
+import com.google.protobuf.CodedOutputStream;
 import java.util.UUID;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 
@@ -37,4 +38,16 @@ public final class ProtobufUtils {
   public static UUID fromProtobuf(HddsProtos.UUID proto) {
     return new UUID(proto.getMostSigBits(), proto.getLeastSigBits());
   }
+
+  /**
+   * Computes the serialized size of a string in a repeated string field.
+   * Wraps protobuf's computeStringSizeNoTag for safer use.
+   */
+  public static int computeRepeatedStringSize(String value) {
+    return CodedOutputStream.computeStringSizeNoTag(value);
+  }
+
+  public static int computeLongSizeWithTag(int fieldNumber, long value) {
+    return CodedOutputStream.computeInt64Size(fieldNumber, value);
+  }
 }
