@@ -23,13 +23,7 @@ import java.security.SecureRandom;
  * Helper methods to deal with random UUIDs.
  */
 public final class UUIDUtil {
-
-  private UUIDUtil() {
-
-  }
-
-  private static final ThreadLocal<SecureRandom> GENERATOR =
-      ThreadLocal.withInitial(SecureRandom::new);
+  private static final ThreadLocal<SecureRandom> GENERATOR = ThreadLocal.withInitial(SecureRandom::new);
 
   public static byte[] randomUUIDBytes() {
     final byte[] bytes = new byte[16];
@@ -42,4 +36,6 @@ public final class UUIDUtil {
     return bytes;
   }
 
+  private UUIDUtil() {
+  }
 }
