@@ -20,7 +20,6 @@ package org.apache.hadoop.hdds.protocol;
 import static org.apache.hadoop.hdds.protocol.DatanodeDetails.Port.Name.ALL_PORTS;
 
 import java.util.Random;
-import java.util.UUID;
 import java.util.concurrent.ThreadLocalRandom;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.ozone.test.GenericTestUtils;
@@ -31,16 +30,16 @@ import org.apache.ozone.test.GenericTestUtils;
 public final class MockDatanodeDetails {
 
   /**
-   * Creates DatanodeDetails with random UUID and random IP address.
+   * Creates DatanodeDetails with random ID and random IP address.
    *
    * @return DatanodeDetails
    */
   public static DatanodeDetails randomDatanodeDetails() {
-    return createDatanodeDetails(UUID.randomUUID());
+    return createDatanodeDetails(DatanodeID.randomID());
   }
 
   /**
-   * Creates DatanodeDetails with random UUID, specific hostname and network
+   * Creates DatanodeDetails with random DatanodeID, specific hostname and network
    * location.
    *
    * @return DatanodeDetails
@@ -52,46 +51,46 @@ public final class MockDatanodeDetails {
         + "." + random.nextInt(256)
         + "." + random.nextInt(256)
         + "." + random.nextInt(256);
-    return createDatanodeDetails(UUID.randomUUID().toString(), hostname,
+    return createDatanodeDetails(DatanodeID.randomID(), hostname,
         ipAddress, loc);
   }
 
   /**
-   * Creates DatanodeDetails using the given UUID.
+   * Creates DatanodeDetails using the given DatanodeID.
    *
-   * @param uuid Datanode's UUID
+   * @param id Datanode's ID
    *
    * @return DatanodeDetails
    */
-  public static DatanodeDetails createDatanodeDetails(UUID uuid) {
+  public static DatanodeDetails createDatanodeDetails(DatanodeID id) {
     Random random = ThreadLocalRandom.current();
     String ipAddress = random.nextInt(256)
         + "." + random.nextInt(256)
         + "." + random.nextInt(256)
         + "." + random.nextInt(256);
-    return createDatanodeDetails(uuid.toString(), "localhost" + "-" + ipAddress,
+    return createDatanodeDetails(id, "localhost" + "-" + ipAddress,
         ipAddress, null);
   }
 
   /**
    * Creates DatanodeDetails with the given information.
    *
-   * @param uuid      Datanode's UUID
+   * @param id      Datanode's ID
    * @param hostname  hostname of Datanode
    * @param ipAddress ip address of Datanode
    *
    * @return DatanodeDetails
    */
-  public static DatanodeDetails createDatanodeDetails(String uuid,
+  public static DatanodeDetails createDatanodeDetails(DatanodeID id,
       String hostname, String ipAddress, String networkLocation) {
-    return createDatanodeDetails(uuid, hostname, ipAddress, networkLocation, 0);
+    return createDatanodeDetails(id, hostname, ipAddress, networkLocation, 0);
   }
 
-  public static DatanodeDetails createDatanodeDetails(String uuid,
+  public static DatanodeDetails createDatanodeDetails(DatanodeID id,
       String hostname, String ipAddress, String networkLocation, int port) {
 
     DatanodeDetails.Builder dn = DatanodeDetails.newBuilder()
-        .setUuid(UUID.fromString(uuid))
+        .setID(id)
         .setHostName(hostname)
         .setIpAddress(ipAddress)
         .setNetworkLocation(networkLocation)
@@ -106,12 +105,12 @@ public final class MockDatanodeDetails {
   }
 
   /**
-   * Creates DatanodeDetails with random UUID and valid local address and port.
+   * Creates DatanodeDetails with random ID and valid local address and port.
    *
    * @return DatanodeDetails
    */
   public static DatanodeDetails randomLocalDatanodeDetails() {
-    return createDatanodeDetails(UUID.randomUUID().toString(),
+    return createDatanodeDetails(DatanodeID.randomID(),
         GenericTestUtils.PortAllocator.HOSTNAME,
         GenericTestUtils.PortAllocator.HOST_ADDRESS, null,
         GenericTestUtils.PortAllocator.getFreePort());
