@@ -19,6 +19,7 @@ package org.apache.hadoop.hdds.utils.db;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.assertj.core.api.Assertions.assertThat;
+import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 
 import com.google.protobuf.InvalidProtocolBufferException;
@@ -31,11 +32,13 @@ public abstract class Proto2CodecTestBase<T> {
   public abstract Codec<T> getCodec();
 
   @Test
-  public void testInvalidProtocolBuffer() throws Exception {
-    InvalidProtocolBufferException exception =
-        assertThrows(InvalidProtocolBufferException.class,
+  public void testInvalidProtocolBuffer() {
+    final CodecException exception =
+        assertThrows(CodecException.class,
             () -> getCodec().fromPersistedFormat("random".getBytes(UTF_8)));
-    assertThat(exception.getMessage())
+    final InvalidProtocolBufferException cause = assertInstanceOf(
+        InvalidProtocolBufferException.class, exception.getCause());
+    assertThat(cause.getMessage())
         .contains("the input ended unexpectedly");
   }
 
