@@ -35,14 +35,14 @@ import org.junit.jupiter.api.Test;
  */
 public class TestChecksumImplsComputeSameValues {
 
-  private int dataSize = 1024 * 1024 * 64;
-  private ByteBuffer data = ByteBuffer.allocate(dataSize);
+  private static final int DATA_SIZE = 1024 * 1024 * 64;
+  private ByteBuffer data = ByteBuffer.allocate(DATA_SIZE);
   private int[] bytesPerChecksum = {512, 1024, 2048, 4096, 32768, 1048576};
 
   @Test
   public void testCRC32ImplsMatch() {
     data.clear();
-    data.put(RandomUtils.nextBytes(data.remaining()));
+    data.put(RandomUtils.secure().randomBytes(data.remaining()));
     for (int bpc : bytesPerChecksum) {
       List<ChecksumByteBuffer> impls = new ArrayList<>();
       impls.add(new PureJavaCrc32ByteBuffer());
@@ -58,7 +58,7 @@ public class TestChecksumImplsComputeSameValues {
   @Test
   public void testCRC32CImplsMatch() {
     data.clear();
-    data.put(RandomUtils.nextBytes(data.remaining()));
+    data.put(RandomUtils.secure().randomBytes(data.remaining()));
     for (int bpc : bytesPerChecksum) {
       List<ChecksumByteBuffer> impls = new ArrayList<>();
       impls.add(new PureJavaCrc32CByteBuffer());
