@@ -56,19 +56,25 @@ public class ContainerMetrics implements Closeable {
   @Metric private MutableCounterLong containerForceDelete;
   @Metric private MutableCounterLong numReadStateMachine;
   @Metric private MutableCounterLong bytesReadStateMachine;
-
+  @Metric private MutableCounterLong numContainerReconciledWithoutChanges;
+  @Metric private MutableCounterLong numContainerReconciledWithChanges;
 
   private final EnumMap<ContainerProtos.Type, MutableCounterLong> numOpsArray;
   private final EnumMap<ContainerProtos.Type, MutableCounterLong> opsBytesArray;
+  private final EnumMap<ContainerProtos.Type, MutableCounterLong> opsForClosedContainer;
   private final EnumMap<ContainerProtos.Type, MutableRate> opsLatency;
   private final EnumMap<ContainerProtos.Type, MutableQuantiles[]> opsLatQuantiles;
-  private MetricsRegistry registry = null;
+
+  // TODO: https://issues.apache.org/jira/browse/HDDS-13555
+  @SuppressWarnings("PMD.SingularField")
+  private MetricsRegistry registry;
 
   public ContainerMetrics(int[] intervals) {
     final int len = intervals.length;
     MutableQuantiles[] latQuantiles = new MutableQuantiles[len];
     this.numOpsArray = new EnumMap<>(ContainerProtos.Type.class);
     this.opsBytesArray = new EnumMap<>(ContainerProtos.Type.class);
+    this.opsForClosedContainer = new EnumMap<>(ContainerProtos.Type.class);
     this.opsLatency = new EnumMap<>(ContainerProtos.Type.class);
     this.opsLatQuantiles = new EnumMap<>(ContainerProtos.Type.class);
     this.registry = new MetricsRegistry("StorageContainerMetrics");
@@ -77,7 +83,9 @@ public class ContainerMetrics implements Closeable {
       numOpsArray.put(type, registry.newCounter(
           "num" + type, "number of " + type + " ops", (long) 0));
       opsBytesArray.put(type, registry.newCounter(
-          "bytes" + type, "bytes used by " + type + "op", (long) 0));
+          "bytes" + type, "bytes used by " + type + " op", (long) 0));
+      opsForClosedContainer.put(type, registry.newCounter("bytesForClosedContainer" + type,
+          "bytes used by " + type + " for closed container op", (long) 0));
       opsLatency.put(type, registry.newRate("latencyNs" + type, type + " op"));
 
       for (int j = 0; j < len; j++) {
@@ -126,9 +134,14 @@ public class ContainerMetrics implements Closeable {
     opsBytesArray.get(type).incr(bytes);
   }
 
+  public void incClosedContainerBytesStats(ContainerProtos.Type type, long bytes) {
+    opsForClosedContainer.get(type).incr(bytes);
+  }
+
   public void incContainerDeleteFailedBlockCountNotZero() {
     containerDeleteFailedBlockCountNotZero.incr();
   }
+
   public void incContainerDeleteFailedNonEmpty() {
     containerDeleteFailedNonEmpty.incr();
   }
@@ -164,4 +177,12 @@ public class ContainerMetrics implements Closeable {
   public long getBytesReadStateMachine() {
     return bytesReadStateMachine.value();
   }
+
+  public void incContainerReconciledWithoutChanges() {
+    numContainerReconciledWithoutChanges.incr();
+  }
+
+  public void incContainerReconciledWithChanges() {
+    numContainerReconciledWithChanges.incr();
+  }
 }
