@@ -18,11 +18,13 @@
 package org.apache.hadoop.ozone.container.common.impl;
 
 import static org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerDataProto.State.RECOVERING;
+import static org.apache.hadoop.ozone.container.metadata.ContainerCreateInfo.INVALID_REPLICA_INDEX;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableMap;
 import com.google.protobuf.Message;
+import jakarta.annotation.Nullable;
 import java.io.IOException;
 import java.time.Clock;
 import java.util.ArrayList;
@@ -37,15 +39,20 @@ import java.util.concurrent.ConcurrentSkipListMap;
 import java.util.concurrent.ConcurrentSkipListSet;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
+import java.util.function.ToLongFunction;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerDataProto.State;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerReportsProto;
+import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.common.helpers.StorageContainerException;
 import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.ozone.container.common.interfaces.Container;
 import org.apache.hadoop.ozone.container.common.statemachine.StateContext;
 import org.apache.hadoop.ozone.container.common.utils.ContainerLogger;
 import org.apache.hadoop.ozone.container.common.volume.HddsVolume;
+import org.apache.hadoop.ozone.container.metadata.ContainerCreateInfo;
+import org.apache.hadoop.ozone.container.metadata.WitnessedContainerMetadataStore;
+import org.apache.hadoop.ozone.container.ozoneimpl.OnDemandContainerScanner;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -56,15 +63,6 @@ public class ContainerSet implements Iterable<Container<?>> {
 
   private static final Logger LOG = LoggerFactory.getLogger(ContainerSet.class);
 
-  public static ContainerSet newReadOnlyContainerSet(long recoveringTimeout) {
-    return new ContainerSet(null, recoveringTimeout);
-  }
-
-  public static ContainerSet newRwContainerSet(Table<Long, String> containerIdsTable, long recoveringTimeout) {
-    Objects.requireNonNull(containerIdsTable, "containerIdsTable == null");
-    return new ContainerSet(containerIdsTable, recoveringTimeout);
-  }
-
   private final ConcurrentSkipListMap<Long, Container<?>> containerMap = new
       ConcurrentSkipListMap<>();
   private final ConcurrentSkipListSet<Long> missingContainerSet =
@@ -73,15 +71,28 @@ public class ContainerSet implements Iterable<Container<?>> {
       new ConcurrentSkipListMap<>();
   private final Clock clock;
   private long recoveringTimeout;
-  private final Table<Long, String> containerIdsTable;
+  @Nullable
+  private final WitnessedContainerMetadataStore containerMetadataStore;
+  // Handler that will be invoked when a scan of a container in this set is requested.
+  private OnDemandContainerScanner containerScanner;
 
-  private ContainerSet(Table<Long, String> continerIdsTable, long recoveringTimeout) {
-    this(continerIdsTable, recoveringTimeout, null);
+  public static ContainerSet newReadOnlyContainerSet(long recoveringTimeout) {
+    return new ContainerSet(null, recoveringTimeout);
   }
 
-  ContainerSet(Table<Long, String> continerIdsTable, long recoveringTimeout, Clock clock) {
+  public static ContainerSet newRwContainerSet(
+      WitnessedContainerMetadataStore metadataStore, long recoveringTimeout) {
+    Objects.requireNonNull(metadataStore, "WitnessedContainerMetadataStore == null");
+    return new ContainerSet(metadataStore, recoveringTimeout);
+  }
+
+  private ContainerSet(WitnessedContainerMetadataStore containerMetadataStore, long recoveringTimeout) {
+    this(containerMetadataStore, recoveringTimeout, null);
+  }
+
+  ContainerSet(WitnessedContainerMetadataStore containerMetadataStore, long recoveringTimeout, Clock clock) {
     this.clock = clock != null ? clock : Clock.systemUTC();
-    this.containerIdsTable = continerIdsTable;
+    this.containerMetadataStore = containerMetadataStore;
     this.recoveringTimeout = recoveringTimeout;
   }
 
@@ -89,6 +100,11 @@ public class ContainerSet implements Iterable<Container<?>> {
     return clock.millis();
   }
 
+  @Nullable
+  public WitnessedContainerMetadataStore getContainerMetadataStore() {
+    return containerMetadataStore;
+  }
+
   @VisibleForTesting
   public void setRecoveringTimeout(long recoveringTimeout) {
     this.recoveringTimeout = recoveringTimeout;
@@ -123,6 +139,45 @@ public class ContainerSet implements Iterable<Container<?>> {
     }
   }
 
+  /**
+   * @param scanner The scanner instance will be invoked when a scan of a container in this set is requested.
+   */
+  public void registerOnDemandScanner(OnDemandContainerScanner scanner) {
+    this.containerScanner = scanner;
+  }
+
+  /**
+   * Triggers a scan of a container in this set. This is a no-op if no scanner is registered or the container does not
+   * exist in the set.
+   * @param containerID The container in this set to scan.
+   */
+  public void scanContainer(long containerID, String reasonForScan) {
+    if (containerScanner != null) {
+      Container<?> container = getContainer(containerID);
+      if (container != null) {
+        containerScanner.scanContainer(container, reasonForScan);
+      } else {
+        LOG.warn("Request to scan container {} which was not found in the container set", containerID);
+      }
+    }
+  }
+
+  /**
+   * Triggers a scan of a container in this set regardless of whether it was recently scanned.
+   * This is a no-op if no scanner is registered or the container does not exist in the set.
+   * @param containerID The container in this set to scan.
+   */
+  public void scanContainerWithoutGap(long containerID, String reasonForScan) {
+    if (containerScanner != null) {
+      Container<?> container = getContainer(containerID);
+      if (container != null) {
+        containerScanner.scanContainerWithoutGap(container, reasonForScan);
+      } else {
+        LOG.warn("Request to scan container {} which was not found in the container set", containerID);
+      }
+    }
+  }
+
   /**
    * Add Container to container map.
    * @param container container to be added
@@ -144,16 +199,8 @@ public class ContainerSet implements Iterable<Container<?>> {
         LOG.debug("Container with container Id {} is added to containerMap",
             containerId);
       }
-      try {
-        if (containerIdsTable != null) {
-          containerIdsTable.put(containerId, containerState.toString());
-        }
-      } catch (IOException e) {
-        throw new StorageContainerException(e, ContainerProtos.Result.IO_EXCEPTION);
-      }
+      updateContainerIdTable(containerId, container.getContainerData());
       missingContainerSet.remove(containerId);
-      // wish we could have done this from ContainerData.setState
-      container.getContainerData().commitSpace();
       if (container.getContainerData().getState() == RECOVERING) {
         recoveringContainerMap.put(
             clock.millis() + recoveringTimeout, containerId);
@@ -167,6 +214,23 @@ public class ContainerSet implements Iterable<Container<?>> {
     }
   }
 
+  private void updateContainerIdTable(long containerId, ContainerData containerData) throws StorageContainerException {
+    if (null != containerMetadataStore) {
+      try {
+        ContainerID containerIdObj = ContainerID.valueOf(containerId);
+        Table<ContainerID, ContainerCreateInfo> containerCreateInfoTable =
+            containerMetadataStore.getContainerCreateInfoTable();
+        ContainerCreateInfo containerCreateInfo = containerCreateInfoTable.get(containerIdObj);
+        if (containerCreateInfo == null || containerCreateInfo.getReplicaIndex() == INVALID_REPLICA_INDEX) {
+          containerCreateInfoTable.put(containerIdObj,
+              ContainerCreateInfo.valueOf(containerData.getState(), containerData.getReplicaIndex()));
+        }
+      } catch (IOException e) {
+        throw new StorageContainerException(e, ContainerProtos.Result.IO_EXCEPTION);
+      }
+    }
+  }
+
   /**
    * Returns the Container with specified containerId.
    * @param containerId ID of the container to get
@@ -228,13 +292,7 @@ public class ContainerSet implements Iterable<Container<?>> {
     }
     Container<?> removed = containerMap.remove(containerId);
     if (removeFromDB) {
-      try {
-        if (containerIdsTable != null) {
-          containerIdsTable.delete(containerId);
-        }
-      } catch (IOException e) {
-        throw new StorageContainerException(e, ContainerProtos.Result.IO_EXCEPTION);
-      }
+      deleteFromContainerTable(containerId);
     }
     if (removed == null) {
       LOG.debug("Container with containerId {} is not present in " +
@@ -247,6 +305,16 @@ public class ContainerSet implements Iterable<Container<?>> {
     }
   }
 
+  private void deleteFromContainerTable(long containerId) throws StorageContainerException {
+    if (null != containerMetadataStore) {
+      try {
+        containerMetadataStore.getContainerCreateInfoTable().delete(ContainerID.valueOf(containerId));
+      } catch (IOException e) {
+        throw new StorageContainerException(e, ContainerProtos.Result.IO_EXCEPTION);
+      }
+    }
+  }
+
   /**
    * Removes the Recovering Container matching with specified containerId.
    * @param containerId ID of the container to remove.
@@ -461,10 +529,6 @@ public class ContainerSet implements Iterable<Container<?>> {
     return missingContainerSet;
   }
 
-  public Table<Long, String> getContainerIdsTable() {
-    return containerIdsTable;
-  }
-
   /**
    * Builds the missing container set by taking a diff between total no
    * containers actually found and number of containers which actually
@@ -475,10 +539,9 @@ public class ContainerSet implements Iterable<Container<?>> {
    * @param container2BCSIDMap Map of containerId to BCSID persisted in the
    *                           Ratis snapshot
    */
-  public void buildMissingContainerSetAndValidate(
-      Map<Long, Long> container2BCSIDMap) {
+  public <T> void buildMissingContainerSetAndValidate(Map<T, Long> container2BCSIDMap, ToLongFunction<T> getId) {
     container2BCSIDMap.entrySet().parallelStream().forEach((mapEntry) -> {
-      long id = mapEntry.getKey();
+      final long id = getId.applyAsLong(mapEntry.getKey());
       if (!containerMap.containsKey(id)) {
         LOG.warn("Adding container {} to missing container set.", id);
         missingContainerSet.add(id);
