@@ -18,8 +18,8 @@
 package org.apache.hadoop.ozone.container.common.impl;
 
 import java.io.IOException;
+import net.jcip.annotations.Immutable;
 import org.apache.hadoop.fs.StorageType;
-import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.StorageTypeProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.MetadataStorageReportProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.StorageReportProto;
@@ -30,8 +30,8 @@ import org.apache.hadoop.ozone.container.common.volume.VolumeUsage;
  * Storage location stats of datanodes that provide back store for containers.
  *
  */
-public final class StorageLocationReport implements
-    StorageLocationReportMXBean {
+@Immutable
+public final class StorageLocationReport implements StorageLocationReportMXBean {
 
   private final String id;
   private final boolean failed;
@@ -43,19 +43,20 @@ public final class StorageLocationReport implements
   private final StorageType storageType;
   private final String storageLocation;
 
-  @SuppressWarnings("checkstyle:parameternumber")
-  private StorageLocationReport(String id, boolean failed, long capacity,
-      long scmUsed, long remaining, long committed, long freeSpaceToSpare,
-      StorageType storageType, String storageLocation) {
-    this.id = id;
-    this.failed = failed;
-    this.capacity = capacity;
-    this.scmUsed = scmUsed;
-    this.remaining = remaining;
-    this.committed = committed;
-    this.freeSpaceToSpare = freeSpaceToSpare;
-    this.storageType = storageType;
-    this.storageLocation = storageLocation;
+  private StorageLocationReport(Builder builder) {
+    this.id = builder.id;
+    this.failed = builder.failed;
+    this.capacity = builder.capacity;
+    this.scmUsed = builder.scmUsed;
+    this.remaining = builder.remaining;
+    this.committed = builder.committed;
+    this.freeSpaceToSpare = builder.freeSpaceToSpare;
+    this.storageType = builder.storageType;
+    this.storageLocation = builder.storageLocation;
+  }
+
+  public long getUsableSpace() {
+    return VolumeUsage.getUsableSpace(this);
   }
 
   @Override
@@ -168,11 +169,6 @@ public final class StorageLocationReport implements
    * @throws IOException In case, the storage type specified is invalid.
    */
   public StorageReportProto getProtoBufMessage() throws IOException {
-    return getProtoBufMessage(null);
-  }
-
-  public StorageReportProto getProtoBufMessage(ConfigurationSource conf)
-      throws IOException {
     StorageReportProto.Builder srb = StorageReportProto.newBuilder();
     return srb.setStorageUuid(getId())
         .setCapacity(getCapacity())
@@ -182,8 +178,7 @@ public final class StorageLocationReport implements
         .setStorageType(getStorageTypeProto())
         .setStorageLocation(getStorageLocation())
         .setFailed(isFailed())
-        .setFreeSpaceToSpare(conf != null ?
-            new VolumeUsage.MinFreeSpaceCalculator(conf).get(getCapacity()) : 0)
+        .setFreeSpaceToSpare(getFreeSpaceToSpare())
         .build();
   }
 
@@ -237,6 +232,27 @@ public final class StorageLocationReport implements
     return builder.build();
   }
 
+  @Override
+  public String toString() {
+    StringBuilder sb = new StringBuilder(128)
+        .append('{')
+        .append(" id=").append(id)
+        .append(" dir=").append(storageLocation)
+        .append(" type=").append(storageType);
+
+    if (failed) {
+      sb.append(" failed");
+    } else {
+      sb.append(" capacity=").append(capacity)
+          .append(" used=").append(scmUsed)
+          .append(" available=").append(remaining)
+          .append(" minFree=").append(freeSpaceToSpare)
+          .append(" committed=").append(committed);
+    }
+
+    return sb.append(" }").toString();
+  }
+
   /**
    * Returns StorageLocation.Builder instance.
    *
@@ -282,6 +298,10 @@ public final class StorageLocationReport implements
       return this;
     }
 
+    public boolean isFailed() {
+      return failed;
+    }
+
     /**
      * Sets the capacity of volume.
      *
@@ -292,6 +312,11 @@ public final class StorageLocationReport implements
       this.capacity = capacityValue;
       return this;
     }
+
+    public long getCapacity() {
+      return capacity;
+    }
+
     /**
      * Sets the scmUsed Value.
      *
@@ -365,8 +390,7 @@ public final class StorageLocationReport implements
      * @return StorageLocationReport
      */
     public StorageLocationReport build() {
-      return new StorageLocationReport(id, failed, capacity, scmUsed,
-          remaining, committed, freeSpaceToSpare, storageType, storageLocation);
+      return new StorageLocationReport(this);
     }
 
   }
