@@ -18,7 +18,11 @@
 package org.apache.hadoop.ozone.container.common.statemachine;
 
 import static java.util.concurrent.TimeUnit.MICROSECONDS;
+import static org.apache.hadoop.hdds.conf.ConfigTag.CONTAINER;
 import static org.apache.hadoop.hdds.conf.ConfigTag.DATANODE;
+import static org.apache.hadoop.hdds.conf.ConfigTag.MANAGEMENT;
+import static org.apache.hadoop.hdds.conf.ConfigTag.OZONE;
+import static org.apache.hadoop.hdds.conf.ConfigTag.STORAGE;
 import static org.apache.hadoop.ozone.container.common.statemachine.DatanodeConfiguration.CONFIG_PREFIX;
 
 import java.time.Duration;
@@ -28,6 +32,7 @@ import org.apache.hadoop.hdds.conf.ConfigTag;
 import org.apache.hadoop.hdds.conf.ConfigType;
 import org.apache.hadoop.hdds.conf.PostConstruct;
 import org.apache.hadoop.hdds.conf.ReconfigurableConfig;
+import org.apache.hadoop.hdds.conf.StorageSize;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -38,41 +43,43 @@ import org.slf4j.LoggerFactory;
 public class DatanodeConfiguration extends ReconfigurableConfig {
   public static final String CONFIG_PREFIX = "hdds.datanode";
 
-  private static final String BLOCK_DELETE_THREAD_MAX
-      = "block.delete.threads.max";
+  private static final String BLOCK_DELETE_THREAD_MAX = "block.delete.threads.max";
+
   public static final String HDDS_DATANODE_BLOCK_DELETE_THREAD_MAX =
       CONFIG_PREFIX + "." + BLOCK_DELETE_THREAD_MAX;
 
-  private static final Logger LOG =
-      LoggerFactory.getLogger(DatanodeConfiguration.class);
+  private static final Logger LOG = LoggerFactory.getLogger(DatanodeConfiguration.class);
 
-  static final String CONTAINER_DELETE_THREADS_MAX_KEY =
-      "hdds.datanode.container.delete.threads.max";
-  static final String CONTAINER_CLOSE_THREADS_MAX_KEY =
-      "hdds.datanode.container.close.threads.max";
-  static final String PERIODIC_DISK_CHECK_INTERVAL_MINUTES_KEY =
-      "hdds.datanode.periodic.disk.check.interval.minutes";
-  public static final String DISK_CHECK_FILE_SIZE_KEY =
-      "hdds.datanode.disk.check.io.file.size";
-  public static final String DISK_CHECK_IO_TEST_COUNT_KEY =
-      "hdds.datanode.disk.check.io.test.count";
-  public static final String DISK_CHECK_IO_FAILURES_TOLERATED_KEY =
-      "hdds.datanode.disk.check.io.failures.tolerated";
-  public static final String FAILED_DATA_VOLUMES_TOLERATED_KEY =
-      "hdds.datanode.failed.data.volumes.tolerated";
-  public static final String FAILED_METADATA_VOLUMES_TOLERATED_KEY =
-      "hdds.datanode.failed.metadata.volumes.tolerated";
-  public static final String FAILED_DB_VOLUMES_TOLERATED_KEY =
-      "hdds.datanode.failed.db.volumes.tolerated";
-  public static final String DISK_CHECK_MIN_GAP_KEY =
-      "hdds.datanode.disk.check.min.gap";
-  public static final String DISK_CHECK_TIMEOUT_KEY =
-      "hdds.datanode.disk.check.timeout";
+  static final String CONTAINER_DELETE_THREADS_MAX_KEY = "hdds.datanode.container.delete.threads.max";
+  static final String CONTAINER_CLOSE_THREADS_MAX_KEY = "hdds.datanode.container.close.threads.max";
+  static final String PERIODIC_DISK_CHECK_INTERVAL_MINUTES_KEY = "hdds.datanode.periodic.disk.check.interval.minutes";
+  public static final String DISK_CHECK_FILE_SIZE_KEY = "hdds.datanode.disk.check.io.file.size";
+  public static final String DISK_CHECK_IO_TEST_COUNT_KEY = "hdds.datanode.disk.check.io.test.count";
+  public static final String DISK_CHECK_IO_FAILURES_TOLERATED_KEY = "hdds.datanode.disk.check.io.failures.tolerated";
+  public static final String FAILED_DATA_VOLUMES_TOLERATED_KEY = "hdds.datanode.failed.data.volumes.tolerated";
+  public static final String FAILED_METADATA_VOLUMES_TOLERATED_KEY = "hdds.datanode.failed.metadata.volumes.tolerated";
+  public static final String FAILED_DB_VOLUMES_TOLERATED_KEY = "hdds.datanode.failed.db.volumes.tolerated";
+  public static final String DISK_CHECK_MIN_GAP_KEY = "hdds.datanode.disk.check.min.gap";
+  public static final String DISK_CHECK_TIMEOUT_KEY = "hdds.datanode.disk.check.timeout";
 
-  public static final String WAIT_ON_ALL_FOLLOWERS =
-      "hdds.datanode.wait.on.all.followers";
-  public static final String CONTAINER_SCHEMA_V3_ENABLED =
-      "hdds.datanode.container.schema.v3.enabled";
+  // Minimum space should be left on volume.
+  // Ex: If volume has 1000GB and minFreeSpace is configured as 10GB,
+  // In this case when availableSpace is 10GB or below, volume is assumed as full
+  public static final String HDDS_DATANODE_VOLUME_MIN_FREE_SPACE = "hdds.datanode.volume.min.free.space";
+  public static final String HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_DEFAULT = "20GB";
+  // Minimum percent of space should be left on volume.
+  // Ex: If volume has 1000GB and minFreeSpacePercent is configured as 2%,
+  // In this case when availableSpace is 20GB(2% of 1000) or below, volume is assumed as full
+  public static final String HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_PERCENT =
+      "hdds.datanode.volume.min.free.space.percent";
+  public static final float HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_PERCENT_DEFAULT = 0.02f;
+
+  public static final String WAIT_ON_ALL_FOLLOWERS = "hdds.datanode.wait.on.all.followers";
+  public static final String CONTAINER_SCHEMA_V3_ENABLED = "hdds.datanode.container.schema.v3.enabled";
+  public static final String CONTAINER_CHECKSUM_LOCK_STRIPES_KEY = "hdds.datanode.container.checksum.lock.stripes";
+  public static final String CONTAINER_CLIENT_CACHE_SIZE = "hdds.datanode.container.client.cache.size";
+  public static final String CONTAINER_CLIENT_CACHE_STALE_THRESHOLD =
+      "hdds.datanode.container.client.cache.stale.threshold";
 
   static final boolean CHUNK_DATA_VALIDATION_CHECK_DEFAULT = false;
 
@@ -96,23 +103,29 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
   static final long ROCKSDB_LOG_MAX_FILE_SIZE_BYTES_DEFAULT = 32 * 1024 * 1024;
   static final int ROCKSDB_LOG_MAX_FILE_NUM_DEFAULT = 64;
   // one hour
-  static final long ROCKSDB_DELETE_OBSOLETE_FILES_PERIOD_MICRO_SECONDS_DEFAULT =
-      1L * 60 * 60 * 1000 * 1000;
+  static final long ROCKSDB_DELETE_OBSOLETE_FILES_PERIOD_MICRO_SECONDS_DEFAULT = 1L * 60 * 60 * 1000 * 1000;
   static final int ROCKSDB_MAX_OPEN_FILES_DEFAULT = 1024;
-  public static final String ROCKSDB_LOG_MAX_FILE_SIZE_BYTES_KEY =
-      "hdds.datanode.rocksdb.log.max-file-size";
-  public static final String ROCKSDB_LOG_MAX_FILE_NUM_KEY =
-      "hdds.datanode.rocksdb.log.max-file-num";
-  public static final String
-      ROCKSDB_DELETE_OBSOLETE_FILES_PERIOD_MICRO_SECONDS_KEY =
+  public static final String ROCKSDB_LOG_MAX_FILE_SIZE_BYTES_KEY = "hdds.datanode.rocksdb.log.max-file-size";
+  public static final String ROCKSDB_LOG_MAX_FILE_NUM_KEY = "hdds.datanode.rocksdb.log.max-file-num";
+  public static final String ROCKSDB_DELETE_OBSOLETE_FILES_PERIOD_MICRO_SECONDS_KEY =
       "hdds.datanode.rocksdb.delete_obsolete_files_period";
   public static final Boolean
       OZONE_DATANODE_CHECK_EMPTY_CONTAINER_DIR_ON_DELETE_DEFAULT = false;
+  public static final int CONTAINER_CHECKSUM_LOCK_STRIPES_DEFAULT = 127;
+  public static final int CONTAINER_CLIENT_CACHE_SIZE_DEFAULT = 100;
+  public static final int
+      CONTAINER_CLIENT_CACHE_STALE_THRESHOLD_MILLISECONDS_DEFAULT = 10000;
 
-  private static final long
-      AUTO_COMPACTION_SMALL_SST_FILE_INTERVAL_MINUTES_DEFAULT = 120;
-  private static final int
-      AUTO_COMPACTION_SMALL_SST_FILE_THREADS_DEFAULT = 1;
+  private static final long AUTO_COMPACTION_SMALL_SST_FILE_INTERVAL_MINUTES_DEFAULT = 120;
+  private static final int AUTO_COMPACTION_SMALL_SST_FILE_THREADS_DEFAULT = 1;
+
+  static final int CONTAINER_DELETE_THREADS_DEFAULT = 2;
+  static final int CONTAINER_CLOSE_THREADS_DEFAULT = 3;
+  static final int BLOCK_DELETE_THREADS_DEFAULT = 5;
+
+  public static final String BLOCK_DELETE_COMMAND_WORKER_INTERVAL =
+      "hdds.datanode.block.delete.command.worker.interval";
+  public static final Duration BLOCK_DELETE_COMMAND_WORKER_INTERVAL_DEFAULT = Duration.ofSeconds(2);
 
   /**
    * Number of threads per volume that Datanode will use for chunk read.
@@ -121,20 +134,10 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       type = ConfigType.INT,
       defaultValue = "10",
       tags = {DATANODE},
-      description = "Number of threads per volume that Datanode will use for " +
-          "reading replicated chunks."
+      description = "Number of threads per volume that Datanode will use for reading replicated chunks."
   )
   private int numReadThreadPerVolume = 10;
 
-  static final int CONTAINER_DELETE_THREADS_DEFAULT = 2;
-  static final int CONTAINER_CLOSE_THREADS_DEFAULT = 3;
-  static final int BLOCK_DELETE_THREADS_DEFAULT = 5;
-
-  public static final String BLOCK_DELETE_COMMAND_WORKER_INTERVAL =
-      "hdds.datanode.block.delete.command.worker.interval";
-  public static final Duration BLOCK_DELETE_COMMAND_WORKER_INTERVAL_DEFAULT =
-      Duration.ofSeconds(2);
-
   /**
    * The maximum number of threads used to delete containers on a datanode
    * simultaneously.
@@ -143,8 +146,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       type = ConfigType.INT,
       defaultValue = "2",
       tags = {DATANODE},
-      description = "The maximum number of threads used to delete containers " +
-          "on a datanode"
+      description = "The maximum number of threads used to delete containers on a datanode"
   )
   private int containerDeleteThreads = CONTAINER_DELETE_THREADS_DEFAULT;
 
@@ -156,8 +158,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       type = ConfigType.INT,
       defaultValue = "3",
       tags = {DATANODE},
-      description = "The maximum number of threads used to close containers " +
-          "on a datanode"
+      description = "The maximum number of threads used to close containers on a datanode"
   )
   private int containerCloseThreads = CONTAINER_CLOSE_THREADS_DEFAULT;
 
@@ -172,8 +173,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       type = ConfigType.INT,
       defaultValue = "5",
       tags = {DATANODE},
-      description = "The maximum number of threads used to handle delete " +
-          " blocks on a datanode"
+      description = "The maximum number of threads used to handle delete blocks on a datanode"
   )
   private int blockDeleteThreads = BLOCK_DELETE_THREADS_DEFAULT;
 
@@ -200,8 +200,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       type = ConfigType.TIME,
       defaultValue = "2s",
       tags = {DATANODE},
-      description = "The interval between DeleteCmdWorker execution of " +
-          "delete commands."
+      description = "The interval between DeleteCmdWorker execution of delete commands."
   )
   private Duration blockDeleteCommandWorkerInterval =
       BLOCK_DELETE_COMMAND_WORKER_INTERVAL_DEFAULT;
@@ -214,8 +213,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       type = ConfigType.INT,
       defaultValue = "5000",
       tags = {DATANODE},
-      description = "The default maximum number of commands in the queue " +
-          "and command type's sub-queue on a datanode"
+      description = "The default maximum number of commands in the queue and command type's sub-queue on a datanode"
   )
   private int cmdQueueLimit = 5000;
 
@@ -255,41 +253,18 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       defaultValue = "100ms",
       type = ConfigType.TIME,
       tags = { DATANODE, ConfigTag.DELETION},
-      description = "Timeout for the thread used to process the delete" +
-          " block command to wait for the container lock."
+      description = "Timeout for the thread used to process the delete block command to wait for the container lock."
   )
-  private long blockDeleteMaxLockWaitTimeoutMs =
-      Duration.ofMillis(100).toMillis();
-
-  public Duration getBlockDeletionInterval() {
-    return blockDeletionInterval;
-  }
-
-  public void setRecoveringContainerScrubInterval(Duration duration) {
-    recoveringContainerScrubInterval = duration;
-  }
-
-  public Duration getRecoveringContainerScrubInterval() {
-    return recoveringContainerScrubInterval;
-  }
-
-  public void setBlockDeletionInterval(Duration duration) {
-    blockDeletionInterval = duration;
-  }
+  private long blockDeleteMaxLockWaitTimeoutMs = Duration.ofMillis(100).toMillis();
 
   @Config(key = "block.deleting.limit.per.interval",
-      defaultValue = "5000",
+      defaultValue = "20000",
       reconfigurable = true,
       type = ConfigType.INT,
-      tags = { ConfigTag.SCM, ConfigTag.DELETION },
-      description =
-          "Number of blocks to be deleted in an interval."
+      tags = { ConfigTag.SCM, ConfigTag.DELETION, DATANODE },
+      description = "Number of blocks to be deleted in an interval."
   )
-  private int blockLimitPerInterval = 5000;
-
-  public int getBlockDeletionLimit() {
-    return blockLimitPerInterval;
-  }
+  private int blockLimitPerInterval = 20000;
 
   @Config(key = "block.deleting.max.lock.holding.time",
       defaultValue = "1s",
@@ -304,20 +279,31 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
           + "which means the actual execution time may exceed this limit. "
           + "Unit could be defined with postfix (ns,ms,s,m,h,d). "
   )
-  private long blockDeletingMaxLockHoldingTime =
-      Duration.ofSeconds(1).toMillis();
+  private long blockDeletingMaxLockHoldingTime = Duration.ofSeconds(1).toMillis();
 
-  public Duration getBlockDeletingMaxLockHoldingTime() {
-    return Duration.ofMillis(blockDeletingMaxLockHoldingTime);
-  }
+  @Config(key = "hdds.datanode.volume.min.free.space",
+      defaultValue = "-1",
+      type = ConfigType.SIZE,
+      tags = { OZONE, CONTAINER, STORAGE, MANAGEMENT },
+      description = "This determines the free space to be used for closing containers" +
+          " When the difference between volume capacity and used reaches this number," +
+          " containers that reside on this volume will be closed and no new containers" +
+          " would be allocated on this volume." +
+          " Max of min.free.space and min.free.space.percent will be used as final value."
+  )
+  private long minFreeSpace = getDefaultFreeSpace();
 
-  public void setBlockDeletingMaxLockHoldingTime(Duration maxLockHoldingTime) {
-    blockDeletingMaxLockHoldingTime = maxLockHoldingTime.toMillis();
-  }
-
-  public void setBlockDeletionLimit(int limit) {
-    this.blockLimitPerInterval = limit;
-  }
+  @Config(key = "hdds.datanode.volume.min.free.space.percent",
+      defaultValue = "0.02", // match HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_PERCENT_DEFAULT
+      type = ConfigType.FLOAT,
+      tags = { OZONE, CONTAINER, STORAGE, MANAGEMENT },
+      description = "This determines the free space percent to be used for closing containers" +
+          " When the difference between volume capacity and used reaches (free.space.percent of volume capacity)," +
+          " containers that reside on this volume will be closed and no new containers" +
+          " would be allocated on this volume." +
+          " Max of min.free.space or min.free.space.percent will be used as final value."
+  )
+  private float minFreeSpaceRatio = HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_PERCENT_DEFAULT;
 
   @Config(key = "periodic.disk.check.interval.minutes",
       defaultValue = "60",
@@ -422,8 +408,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       defaultValue = "false",
       type = ConfigType.BOOLEAN,
       tags = { DATANODE },
-      description = "Enable safety checks such as checksum validation"
-          + " for Ratis calls."
+      description = "Enable safety checks such as checksum validation for Ratis calls."
   )
   private boolean isChunkDataValidationCheck =
       CHUNK_DATA_VALIDATION_CHECK_DEFAULT;
@@ -439,14 +424,6 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
 
   private boolean waitOnAllFollowers = WAIT_ON_ALL_FOLLOWERS_DEFAULT;
 
-  public boolean waitOnAllFollowers() {
-    return waitOnAllFollowers;
-  }
-
-  public void setWaitOnAllFollowers(boolean val) {
-    this.waitOnAllFollowers = val;
-  }
-
   @Config(key = "container.schema.v3.enabled",
       defaultValue = "true",
       type = ConfigType.BOOLEAN,
@@ -460,8 +437,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       defaultValue = "|",
       type = ConfigType.STRING,
       tags = { DATANODE },
-      description = "The default separator between Container ID and container" +
-           " meta key name."
+      description = "The default separator between Container ID and container meta key name."
   )
   private String containerSchemaV3KeySeparator = "|";
 
@@ -469,8 +445,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       defaultValue = "INFO",
       type = ConfigType.STRING,
       tags = { DATANODE },
-      description =
-          "The user log level of RocksDB(DEBUG/INFO/WARN/ERROR/FATAL))"
+      description = "The user log level of RocksDB(DEBUG/INFO/WARN/ERROR/FATAL))"
   )
   private String rocksdbLogLevel = "INFO";
 
@@ -478,8 +453,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       defaultValue = "32MB",
       type = ConfigType.SIZE,
       tags = { DATANODE },
-      description = "The max size of each user log file of RocksDB. " +
-          "O means no size limit."
+      description = "The max size of each user log file of RocksDB. O means no size limit."
   )
   private long rocksdbLogMaxFileSize = ROCKSDB_LOG_MAX_FILE_SIZE_BYTES_DEFAULT;
 
@@ -498,8 +472,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       defaultValue = "1h", timeUnit = MICROSECONDS,
       type = ConfigType.TIME,
       tags = { DATANODE },
-      description = "Periodicity when obsolete files get deleted. " +
-          "Default is 1h."
+      description = "Periodicity when obsolete files get deleted. Default is 1h."
   )
   private long rocksdbDeleteObsoleteFilesPeriod =
       ROCKSDB_DELETE_OBSOLETE_FILES_PERIOD_MICRO_SECONDS_DEFAULT;
@@ -526,8 +499,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       defaultValue = "1MB",
       type = ConfigType.SIZE,
       tags = { DATANODE },
-      description = "SST files smaller than this configuration will be " +
-          "auto compacted."
+      description = "SST files smaller than this configuration will be auto compacted."
   )
   private long autoCompactionSmallSstFileSize = 1024 * 1024;
 
@@ -535,8 +507,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       defaultValue = "512",
       type = ConfigType.INT,
       tags = { DATANODE },
-      description = "Auto compaction will happen if the number of small SST " +
-          " files exceeds this threshold."
+      description = "Auto compaction will happen if the number of small SST files exceeds this threshold."
   )
   private int autoCompactionSmallSstFileNum = 512;
 
@@ -566,12 +537,44 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       type = ConfigType.BOOLEAN,
       defaultValue = "false",
       tags = { DATANODE },
-      description = "Boolean Flag to decide whether to check container " +
-          "directory or not to determine container is empty"
+      description = "Boolean Flag to decide whether to check container directory or not to determine container is empty"
   )
   private boolean bCheckEmptyContainerDir =
       OZONE_DATANODE_CHECK_EMPTY_CONTAINER_DIR_ON_DELETE_DEFAULT;
 
+  /**
+   * Whether to check container directory or not to determine
+   * container is empty.
+   */
+  @Config(key = "container.checksum.lock.stripes",
+      type = ConfigType.INT,
+      defaultValue = "127",
+      tags = { DATANODE },
+      description = "The number of lock stripes used to coordinate modifications to container checksum information. " +
+          "This information is only updated after a container is closed and does not affect the data read or write" +
+          " path. Each container in the datanode will be mapped to one lock which will only be held while its " +
+          "checksum information is updated."
+  )
+  private int containerChecksumLockStripes = CONTAINER_CHECKSUM_LOCK_STRIPES_DEFAULT;
+
+  @Config(key = "container.client.cache.size",
+      type = ConfigType.INT,
+      defaultValue = "100",
+      tags = { DATANODE },
+      description = "The maximum number of clients to be cached by the datanode client manager"
+  )
+  private int containerClientCacheSize = CONTAINER_CLIENT_CACHE_SIZE_DEFAULT;
+
+  @Config(key = "container.client.cache.stale.threshold",
+      type = ConfigType.INT,
+      defaultValue = "10000",
+      tags = { DATANODE },
+      description = "The stale threshold in ms for a client in cache. After this threshold the client " +
+          "is evicted from cache."
+  )
+  private int containerClientCacheStaleThreshold =
+      CONTAINER_CLIENT_CACHE_STALE_THRESHOLD_MILLISECONDS_DEFAULT;
+
   @Config(key = "delete.container.timeout",
       type = ConfigType.TIME,
       defaultValue = "60s",
@@ -582,10 +585,7 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
   )
   private long deleteContainerTimeoutMs = Duration.ofSeconds(60).toMillis();
 
-  public long getDeleteContainerTimeoutMs() {
-    return deleteContainerTimeoutMs;
-  }
-
+  @SuppressWarnings("checkstyle:MethodLength")
   @PostConstruct
   public void validate() {
     if (containerDeleteThreads < 1) {
@@ -719,30 +719,123 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
       rocksdbDeleteObsoleteFilesPeriod =
           ROCKSDB_DELETE_OBSOLETE_FILES_PERIOD_MICRO_SECONDS_DEFAULT;
     }
+
+    if (containerChecksumLockStripes < 1) {
+      LOG.warn("{} must be at least 1. Defaulting to {}", CONTAINER_CHECKSUM_LOCK_STRIPES_KEY,
+          CONTAINER_CHECKSUM_LOCK_STRIPES_DEFAULT);
+      containerChecksumLockStripes = CONTAINER_CHECKSUM_LOCK_STRIPES_DEFAULT;
+    }
+
+    if (containerClientCacheSize < 1) {
+      LOG.warn("{} must be at least 1. Defaulting to {}", CONTAINER_CLIENT_CACHE_SIZE,
+          CONTAINER_CLIENT_CACHE_SIZE_DEFAULT);
+      containerClientCacheSize = CONTAINER_CLIENT_CACHE_SIZE_DEFAULT;
+    }
+
+    if (containerClientCacheStaleThreshold < 1) {
+      LOG.warn("{} must be at least 1. Defaulting to {}", CONTAINER_CLIENT_CACHE_STALE_THRESHOLD,
+          CONTAINER_CLIENT_CACHE_STALE_THRESHOLD_MILLISECONDS_DEFAULT);
+      containerClientCacheStaleThreshold =
+          CONTAINER_CLIENT_CACHE_STALE_THRESHOLD_MILLISECONDS_DEFAULT;
+    }
+
+    validateMinFreeSpace();
   }
 
-  public void setContainerDeleteThreads(int containerDeleteThreads) {
-    this.containerDeleteThreads = containerDeleteThreads;
+  /**
+   * validate value of 'hdds.datanode.volume.min.free.space' and 'hdds.datanode.volume.min.free.space.percent'
+   * and update with default value if not within range.
+   */
+  private void validateMinFreeSpace() {
+    if (minFreeSpaceRatio > 1 || minFreeSpaceRatio < 0) {
+      LOG.warn("{} = {} is invalid, should be between 0 and 1",
+          HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_PERCENT,
+          minFreeSpaceRatio);
+      minFreeSpaceRatio = HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_PERCENT_DEFAULT;
+    }
+
+    if (minFreeSpace < 0) {
+      minFreeSpace = getDefaultFreeSpace();
+    }
+  }
+
+  public Duration getBlockDeletionInterval() {
+    return blockDeletionInterval;
+  }
+
+  public void setBlockDeletionInterval(Duration duration) {
+    blockDeletionInterval = duration;
+  }
+
+  public Duration getRecoveringContainerScrubInterval() {
+    return recoveringContainerScrubInterval;
+  }
+
+  public void setRecoveringContainerScrubInterval(Duration duration) {
+    recoveringContainerScrubInterval = duration;
+  }
+
+  public int getBlockDeletionLimit() {
+    return blockLimitPerInterval;
+  }
+
+  public void setBlockDeletionLimit(int limit) {
+    this.blockLimitPerInterval = limit;
+  }
+
+  public long getDeleteContainerTimeoutMs() {
+    return deleteContainerTimeoutMs;
+  }
+
+  public Duration getBlockDeletingMaxLockHoldingTime() {
+    return Duration.ofMillis(blockDeletingMaxLockHoldingTime);
+  }
+
+  public void setBlockDeletingMaxLockHoldingTime(Duration maxLockHoldingTime) {
+    blockDeletingMaxLockHoldingTime = maxLockHoldingTime.toMillis();
+  }
+
+  public boolean waitOnAllFollowers() {
+    return waitOnAllFollowers;
+  }
+
+  public void setWaitOnAllFollowers(boolean val) {
+    this.waitOnAllFollowers = val;
   }
 
   public int getContainerDeleteThreads() {
     return containerDeleteThreads;
   }
 
-  public void setContainerCloseThreads(int containerCloseThreads) {
-    this.containerCloseThreads = containerCloseThreads;
+  public void setContainerDeleteThreads(int containerDeleteThreads) {
+    this.containerDeleteThreads = containerDeleteThreads;
   }
 
   public int getContainerCloseThreads() {
     return containerCloseThreads;
   }
 
+  public void setContainerCloseThreads(int containerCloseThreads) {
+    this.containerCloseThreads = containerCloseThreads;
+  }
+
+  public long getMinFreeSpace(long capacity) {
+    return Math.max((long) (capacity * minFreeSpaceRatio), minFreeSpace);
+  }
+
+  public long getMinFreeSpace() {
+    return minFreeSpace;
+  }
+
+  public float getMinFreeSpaceRatio() {
+    return minFreeSpaceRatio;
+  }
+
   public long getPeriodicDiskCheckIntervalMinutes() {
     return periodicDiskCheckIntervalMinutes;
   }
 
-  public void setPeriodicDiskCheckIntervalMinutes(
-      long periodicDiskCheckIntervalMinutes) {
+  public void setPeriodicDiskCheckIntervalMinutes(long periodicDiskCheckIntervalMinutes) {
     this.periodicDiskCheckIntervalMinutes = periodicDiskCheckIntervalMinutes;
   }
 
@@ -826,14 +919,14 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
     return blockDeleteQueueLimit;
   }
 
-  public long getBlockDeleteMaxLockWaitTimeoutMs() {
-    return blockDeleteMaxLockWaitTimeoutMs;
-  }
-
   public void setBlockDeleteQueueLimit(int queueLimit) {
     this.blockDeleteQueueLimit = queueLimit;
   }
 
+  public long getBlockDeleteMaxLockWaitTimeoutMs() {
+    return blockDeleteMaxLockWaitTimeoutMs;
+  }
+
   public Duration getBlockDeleteCommandWorkerInterval() {
     return blockDeleteCommandWorkerInterval;
   }
@@ -859,14 +952,14 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
     isChunkDataValidationCheck = writeChunkValidationCheck;
   }
 
-  public void setNumReadThreadPerVolume(int threads) {
-    this.numReadThreadPerVolume = threads;
-  }
-
   public int getNumReadThreadPerVolume() {
     return numReadThreadPerVolume;
   }
 
+  public void setNumReadThreadPerVolume(int threads) {
+    this.numReadThreadPerVolume = threads;
+  }
+
   public boolean getContainerSchemaV3Enabled() {
     return this.containerSchemaV3Enabled;
   }
@@ -947,12 +1040,23 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
     this.autoCompactionSmallSstFileNum = num;
   }
 
+  public int getContainerChecksumLockStripes() {
+    return containerChecksumLockStripes;
+  }
+
+  public int getContainerClientCacheSize() {
+    return containerClientCacheSize;
+  }
+
+  public int getContainerClientCacheStaleThreshold() {
+    return containerClientCacheStaleThreshold;
+  }
+
   public long getAutoCompactionSmallSstFileIntervalMinutes() {
     return autoCompactionSmallSstFileIntervalMinutes;
   }
 
-  public void setAutoCompactionSmallSstFileIntervalMinutes(
-      long autoCompactionSmallSstFileIntervalMinutes) {
+  public void setAutoCompactionSmallSstFileIntervalMinutes(long autoCompactionSmallSstFileIntervalMinutes) {
     this.autoCompactionSmallSstFileIntervalMinutes =
         autoCompactionSmallSstFileIntervalMinutes;
   }
@@ -961,9 +1065,13 @@ public class DatanodeConfiguration extends ReconfigurableConfig {
     return autoCompactionSmallSstFileThreads;
   }
 
-  public void setAutoCompactionSmallSstFileThreads(
-      int autoCompactionSmallSstFileThreads) {
+  public void setAutoCompactionSmallSstFileThreads(int autoCompactionSmallSstFileThreads) {
     this.autoCompactionSmallSstFileThreads =
         autoCompactionSmallSstFileThreads;
   }
+
+  static long getDefaultFreeSpace() {
+    final StorageSize measure = StorageSize.parse(HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_DEFAULT);
+    return Math.round(measure.getUnit().toBytes(measure.getValue()));
+  }
 }
