@@ -19,8 +19,12 @@ package org.apache.hadoop.ozone.container.common.statemachine.commandhandler;
 
 import java.io.IOException;
 import java.util.Collection;
+import java.util.Set;
+import java.util.UUID;
 import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.Executor;
+import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.function.BiFunction;
@@ -62,6 +66,7 @@ public class ClosePipelineCommandHandler implements CommandHandler {
   private final Executor executor;
   private final BiFunction<RaftPeer, GrpcTlsConfig, RaftClient> newRaftClient;
   private final MutableRate opsLatencyMs;
+  private final Set<UUID> pipelinesInProgress;
 
   /**
    * Constructs a closePipelineCommand handler.
@@ -82,6 +87,16 @@ public class ClosePipelineCommandHandler implements CommandHandler {
     MetricsRegistry registry = new MetricsRegistry(
         ClosePipelineCommandHandler.class.getSimpleName());
     this.opsLatencyMs = registry.newRate(SCMCommandProto.Type.closePipelineCommand + "Ms");
+    this.pipelinesInProgress = ConcurrentHashMap.newKeySet();
+  }
+
+  /**
+   * Returns true if pipeline close is in progress, else false.
+   *
+   * @return boolean
+   */
+  public boolean isPipelineCloseInProgress(UUID pipelineID) {
+    return pipelinesInProgress.contains(pipelineID);
   }
 
   /**
@@ -95,71 +110,88 @@ public class ClosePipelineCommandHandler implements CommandHandler {
   @Override
   public void handle(SCMCommand<?> command, OzoneContainer ozoneContainer,
       StateContext context, SCMConnectionManager connectionManager) {
-    queuedCount.incrementAndGet();
-    CompletableFuture.runAsync(() -> {
-      invocationCount.incrementAndGet();
-      final long startTime = Time.monotonicNow();
-      final DatanodeDetails dn = context.getParent().getDatanodeDetails();
-      ClosePipelineCommand closePipelineCommand =
-          (ClosePipelineCommand) command;
-      final PipelineID pipelineID = closePipelineCommand.getPipelineID();
-      final HddsProtos.PipelineID pipelineIdProto = pipelineID.getProtobuf();
+    final ClosePipelineCommand closePipelineCommand = (ClosePipelineCommand) command;
+    final PipelineID pipelineID = closePipelineCommand.getPipelineID();
+    final UUID pipelineUUID = pipelineID.getId();
+    
+    // Check if this pipeline is already being processed
+    if (!pipelinesInProgress.add(pipelineUUID)) {
+      LOG.debug("Close Pipeline command for pipeline {} is already in progress, " +
+          "skipping duplicate command.", pipelineID);
+      return;
+    }
+    
+    try {
+      queuedCount.incrementAndGet();
+      CompletableFuture.runAsync(() -> {
+        invocationCount.incrementAndGet();
+        final long startTime = Time.monotonicNow();
+        final DatanodeDetails dn = context.getParent().getDatanodeDetails();
+        final HddsProtos.PipelineID pipelineIdProto = pipelineID.getProtobuf();
 
-      try {
-        XceiverServerSpi server = ozoneContainer.getWriteChannel();
-        if (server.isExist(pipelineIdProto)) {
-          if (server instanceof XceiverServerRatis) {
-            // TODO: Refactor Ratis logic to XceiverServerRatis
-            // Propagate the group remove to the other Raft peers in the pipeline
-            XceiverServerRatis ratisServer = (XceiverServerRatis) server;
-            final RaftGroupId raftGroupId = RaftGroupId.valueOf(pipelineID.getId());
-            final boolean shouldDeleteRatisLogDirectory = ratisServer.getShouldDeleteRatisLogDirectory();
-            // This might throw GroupMismatchException if the Ratis group has been closed by other datanodes
-            final Collection<RaftPeer> peers = ratisServer.getRaftPeersInPipeline(pipelineID);
-            // Try to send remove group for the other datanodes first, ignoring GroupMismatchException
-            // if the Ratis group has been closed in the other datanodes
-            peers.stream()
-                .filter(peer -> !peer.getId().equals(ratisServer.getServer().getId()))
-                .forEach(peer -> {
-                  try (RaftClient client = newRaftClient.apply(peer, ozoneContainer.getTlsClientConfig())) {
-                    client.getGroupManagementApi(peer.getId())
-                        .remove(raftGroupId, shouldDeleteRatisLogDirectory, !shouldDeleteRatisLogDirectory);
-                  } catch (GroupMismatchException ae) {
-                    // ignore silently since this means that the group has been closed by earlier close pipeline
-                    // command in another datanode
-                    LOG.debug("Failed to remove group {} for pipeline {} on peer {} since the group has " +
-                        "been removed by earlier close pipeline command handled in another datanode", raftGroupId,
-                        pipelineID, peer.getId());
-                  } catch (IOException ioe) {
-                    LOG.warn("Failed to remove group {} of pipeline {} on peer {}",
-                        raftGroupId, pipelineID, peer.getId(), ioe);
-                  }
-                });
+        try {
+          XceiverServerSpi server = ozoneContainer.getWriteChannel();
+          if (server.isExist(pipelineIdProto)) {
+            if (server instanceof XceiverServerRatis) {
+              // TODO: Refactor Ratis logic to XceiverServerRatis
+              // Propagate the group remove to the other Raft peers in the pipeline
+              XceiverServerRatis ratisServer = (XceiverServerRatis) server;
+              final RaftGroupId raftGroupId = RaftGroupId.valueOf(pipelineID.getId());
+              final boolean shouldDeleteRatisLogDirectory = ratisServer.getShouldDeleteRatisLogDirectory();
+              // This might throw GroupMismatchException if the Ratis group has been closed by other datanodes
+              final Collection<RaftPeer> peers = ratisServer.getRaftPeersInPipeline(pipelineID);
+              // Try to send remove group for the other datanodes first, ignoring GroupMismatchException
+              // if the Ratis group has been closed in the other datanodes
+              peers.stream()
+                  .filter(peer -> !peer.getId().equals(ratisServer.getServer().getId()))
+                  .forEach(peer -> {
+                    try (RaftClient client = newRaftClient.apply(peer, ozoneContainer.getTlsClientConfig())) {
+                      client.getGroupManagementApi(peer.getId())
+                          .remove(raftGroupId, shouldDeleteRatisLogDirectory, !shouldDeleteRatisLogDirectory);
+                    } catch (GroupMismatchException ae) {
+                      // ignore silently since this means that the group has been closed by earlier close pipeline
+                      // command in another datanode
+                      LOG.debug("Failed to remove group {} for pipeline {} on peer {} since the group has " +
+                          "been removed by earlier close pipeline command handled in another datanode", raftGroupId,
+                          pipelineID, peer.getId());
+                    } catch (IOException ioe) {
+                      LOG.warn("Failed to remove group {} of pipeline {} on peer {}",
+                          raftGroupId, pipelineID, peer.getId(), ioe);
+                    }
+                  });
+            }
+            // Remove the Ratis group from the current datanode pipeline, might throw GroupMismatchException as
+            // well. It is a no-op for XceiverServerSpi implementations (e.g. XceiverServerGrpc)
+            server.removeGroup(pipelineIdProto);
+            LOG.info("Close Pipeline {} command on datanode {}.", pipelineID, dn);
+          } else {
+            LOG.debug("Ignoring close pipeline command for pipeline {} on datanode {} " +
+                "as it does not exist", pipelineID, dn);
           }
-          // Remove the Ratis group from the current datanode pipeline, might throw GroupMismatchException as
-          // well. It is a no-op for XceiverServerSpi implementations (e.g. XceiverServerGrpc)
-          server.removeGroup(pipelineIdProto);
-          LOG.info("Close Pipeline {} command on datanode {}.", pipelineID,
-              dn.getUuidString());
-        } else {
-          LOG.debug("Ignoring close pipeline command for pipeline {} on datanode {} " +
-              "as it does not exist", pipelineID, dn.getUuidString());
+        } catch (IOException e) {
+          Throwable gme = HddsClientUtils.containsException(e, GroupMismatchException.class);
+          if (gme != null) {
+            // ignore silently since this means that the group has been closed by earlier close pipeline
+            // command in another datanode
+            LOG.debug("The group for pipeline {} on datanode {} has been removed by earlier close " +
+                "pipeline command handled in another datanode", pipelineID, dn);
+          } else {
+            LOG.error("Can't close pipeline {}", pipelineID, e);
+          }
+        } finally {
+          long endTime = Time.monotonicNow();
+          this.opsLatencyMs.add(endTime - startTime);
         }
-      } catch (IOException e) {
-        Throwable gme = HddsClientUtils.containsException(e, GroupMismatchException.class);
-        if (gme != null) {
-          // ignore silently since this means that the group has been closed by earlier close pipeline
-          // command in another datanode
-          LOG.debug("The group for pipeline {} on datanode {} has been removed by earlier close " +
-              "pipeline command handled in another datanode", pipelineID, dn.getUuidString());
-        } else {
-          LOG.error("Can't close pipeline {}", pipelineID, e);
-        }
-      } finally {
-        long endTime = Time.monotonicNow();
-        this.opsLatencyMs.add(endTime - startTime);
-      }
-    }, executor).whenComplete((v, e) -> queuedCount.decrementAndGet());
+      }, executor).whenComplete((v, e) -> {
+        queuedCount.decrementAndGet();
+        pipelinesInProgress.remove(pipelineUUID);
+      });
+    } catch (RejectedExecutionException ex) {
+      queuedCount.decrementAndGet();
+      pipelinesInProgress.remove(pipelineUUID);
+      LOG.warn("Close Pipeline command for pipeline {} is rejected as " +
+          "command queue has reached max size.", pipelineID);
+    }
   }
 
   /**
