@@ -138,8 +138,36 @@ import org.slf4j.LoggerFactory;
  *    Then, WriteChunk commit and CreateContainer will be executed in the same order.
  */
 public class ContainerStateMachine extends BaseStateMachine {
-  static final Logger LOG =
-      LoggerFactory.getLogger(ContainerStateMachine.class);
+  static final Logger LOG = LoggerFactory.getLogger(ContainerStateMachine.class);
+
+  private final SimpleStateMachineStorage storage = new SimpleStateMachineStorage();
+  private final ContainerDispatcher dispatcher;
+  private final ContainerController containerController;
+  private final XceiverServerRatis ratisServer;
+  private final NavigableMap<Long, WriteFutures> writeChunkFutureMap;
+
+  private final long writeChunkWaitMaxNs;
+
+  // keeps track of the containers created per pipeline
+  private final Map<Long, Long> container2BCSIDMap;
+  private final TaskQueueMap containerTaskQueues = new TaskQueueMap();
+  private final ExecutorService executor;
+  private final List<ThreadPoolExecutor> chunkExecutors;
+  private final Map<Long, Long> applyTransactionCompletionMap;
+  private final Set<Long> unhealthyContainers;
+  private final Cache<Long, ByteString> stateMachineDataCache;
+  private final AtomicBoolean stateMachineHealthy;
+
+  private final Semaphore applyTransactionSemaphore;
+  private final boolean waitOnBothFollowers;
+  private final HddsDatanodeService datanodeService;
+  private static Semaphore semaphore = new Semaphore(1);
+  private final AtomicBoolean peersValidated;
+
+  /**
+   * CSM metrics.
+   */
+  private final CSMMetrics metrics;
 
   static class TaskQueueMap {
     private final Map<Long, TaskQueue> map = new HashMap<>();
@@ -217,35 +245,6 @@ public class ContainerStateMachine extends BaseStateMachine {
     }
   }
 
-  private final SimpleStateMachineStorage storage =
-      new SimpleStateMachineStorage();
-  private final ContainerDispatcher dispatcher;
-  private final ContainerController containerController;
-  private final XceiverServerRatis ratisServer;
-  private final NavigableMap<Long, WriteFutures> writeChunkFutureMap;
-  private final long writeChunkWaitMaxNs;
-
-  // keeps track of the containers created per pipeline
-  private final Map<Long, Long> container2BCSIDMap;
-  private final TaskQueueMap containerTaskQueues = new TaskQueueMap();
-  private final ExecutorService executor;
-  private final List<ThreadPoolExecutor> chunkExecutors;
-  private final Map<Long, Long> applyTransactionCompletionMap;
-  private final Set<Long> unhealthyContainers;
-  private final Cache<Long, ByteString> stateMachineDataCache;
-  private final AtomicBoolean stateMachineHealthy;
-
-  private final Semaphore applyTransactionSemaphore;
-  private final boolean waitOnBothFollowers;
-  private final HddsDatanodeService datanodeService;
-  private static Semaphore semaphore = new Semaphore(1);
-  private final AtomicBoolean peersValidated;
-
-  /**
-   * CSM metrics.
-   */
-  private final CSMMetrics metrics;
-
   @SuppressWarnings("parameternumber")
   public ContainerStateMachine(HddsDatanodeService hddsDatanodeService, RaftGroupId gid,
       ContainerDispatcher dispatcher,
@@ -378,6 +377,7 @@ public class ContainerStateMachine extends BaseStateMachine {
       dispatcher.buildMissingContainerSetAndValidate(container2BCSIDMap);
     }
   }
+
   /**
    * As a part of taking snapshot with Ratis StateMachine, it will persist
    * the existing container set in the snapshotFile.
@@ -1265,8 +1265,10 @@ public class ContainerStateMachine extends BaseStateMachine {
 
   @Override
   public void notifyLogFailed(Throwable t, LogEntryProto failedEntry) {
-    LOG.error("{}: {} {}", getGroupId(), TermIndex.valueOf(failedEntry),
-        toStateMachineLogEntryString(failedEntry.getStateMachineLogEntry()), t);
+    String stateMachineLogEntry = failedEntry == null
+        ? "null"
+        : toStateMachineLogEntryString(failedEntry.getStateMachineLogEntry());
+    LOG.error("{}: {} {}", getGroupId(), TermIndex.valueOf(failedEntry), stateMachineLogEntry, t);
     ratisServer.handleNodeLogFailure(getGroupId(), t);
   }
 
