@@ -17,10 +17,12 @@
 
 package org.apache.hadoop.ozone.container.common.utils;
 
-import static org.apache.hadoop.ozone.container.common.interfaces.Container.ScanResult;
+import static org.apache.hadoop.hdds.HddsUtils.checksumToString;
 
 import com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.ozone.container.common.impl.ContainerData;
+import org.apache.hadoop.ozone.container.common.interfaces.ScanResult;
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
@@ -91,10 +93,7 @@ public final class ContainerLogger {
    */
   public static void logUnhealthy(ContainerData containerData,
       ScanResult reason) {
-    String message = reason.getFailureType() + " for file " +
-        reason.getUnhealthyFile() +
-        ". Message: " + reason.getException().getMessage();
-    LOG.error(getMessage(containerData, message));
+    LOG.error(getMessage(containerData, reason.toString()));
   }
 
   /**
@@ -128,7 +127,7 @@ public final class ContainerLogger {
    * @param containerData The container that was imported to this datanode.
    */
   public static void logImported(ContainerData containerData) {
-    LOG.info(getMessage(containerData));
+    LOG.info(getMessage(containerData, "Container imported"));
   }
 
   /**
@@ -137,7 +136,7 @@ public final class ContainerLogger {
    * @param containerData The container that was exported from this datanode.
    */
   public static void logExported(ContainerData containerData) {
-    LOG.info(getMessage(containerData));
+    LOG.info(getMessage(containerData, "Container exported"));
   }
 
   /**
@@ -149,6 +148,35 @@ public final class ContainerLogger {
     LOG.info(getMessage(containerData));
   }
 
+  /**
+   * Logged when a container's checksum is updated.
+   *
+   * @param containerData The container which has the updated data checksum.
+   * @param oldDataChecksum The old data checksum.
+   */
+  public static void logChecksumUpdated(ContainerData containerData, long oldDataChecksum) {
+    LOG.warn(getMessage(containerData,
+        "Container data checksum updated from " + checksumToString(oldDataChecksum) + " to "
+            + checksumToString(containerData.getDataChecksum())));
+  }
+
+  /**
+   * Logged when a container is reconciled.
+   *
+   * @param containerData The container that was reconciled on this datanode.
+   * @param oldDataChecksum The old data checksum.
+   */
+  public static void logReconciled(ContainerData containerData, long oldDataChecksum, DatanodeDetails peer) {
+    if (containerData.getDataChecksum() == oldDataChecksum) {
+      LOG.info(getMessage(containerData, "Container reconciled with peer " + peer.toString() +
+          ". No change in checksum."));
+    } else {
+      LOG.warn(getMessage(containerData, "Container reconciled with peer " + peer.toString() +
+          ". Checksum updated from " + checksumToString(oldDataChecksum) + " to "
+          + checksumToString(containerData.getDataChecksum())));
+    }
+  }
+
   private static String getMessage(ContainerData containerData,
                                    String message) {
     return String.join(FIELD_SEPARATOR, getMessage(containerData), message);
@@ -159,6 +187,8 @@ public final class ContainerLogger {
         "ID=" + containerData.getContainerID(),
         "Index=" + containerData.getReplicaIndex(),
         "BCSID=" + containerData.getBlockCommitSequenceId(),
-        "State=" + containerData.getState());
+        "State=" + containerData.getState(),
+        "Volume=" + containerData.getVolume(),
+        "DataChecksum=" + checksumToString(containerData.getDataChecksum()));
   }
 }
