@@ -42,12 +42,13 @@ import org.slf4j.LoggerFactory;
  * where the disk is mounted.
  */
 public final class DiskCheckUtil {
-  private DiskCheckUtil() { }
-
   // For testing purposes, an alternate check implementation can be provided
   // to inject failures.
   private static DiskChecks impl = new DiskChecksImpl();
 
+  private DiskCheckUtil() {
+  }
+
   @VisibleForTesting
   public static void setTestImpl(DiskChecks diskChecks) {
     impl = diskChecks;
@@ -80,9 +81,11 @@ public final class DiskCheckUtil {
     default boolean checkExistence(File storageDir) {
       return true;
     }
+
     default boolean checkPermissions(File storageDir) {
       return true;
     }
+
     default boolean checkReadWrite(File storageDir, File testFileDir,
                             int numBytesToWrite) {
       return true;
