@@ -38,7 +38,6 @@ import java.io.UncheckedIOException;
 import java.nio.ByteBuffer;
 import java.nio.MappedByteBuffer;
 import java.nio.channels.FileChannel;
-import java.nio.channels.FileLock;
 import java.nio.file.NoSuchFileException;
 import java.nio.file.OpenOption;
 import java.nio.file.Path;
@@ -181,10 +180,7 @@ public final class ChunkUtils {
       FileChannel channel = null;
       try {
         channel = open(path, WRITE_OPTIONS, NO_ATTRIBUTES);
-
-        try (FileLock ignored = channel.lock()) {
-          return writeDataToChannel(channel, data, offset);
-        }
+        return writeDataToChannel(channel, data, offset);
       } catch (IOException e) {
         throw new UncheckedIOException(e);
       } finally {
@@ -365,7 +361,6 @@ public final class ChunkUtils {
     return false;
   }
 
-
   /**
    * Validates chunk data and returns a boolean value that indicates if the
    * chunk data should be overwritten.
