@@ -26,11 +26,11 @@ import java.io.IOException;
 import java.time.Duration;
 import java.time.Instant;
 import java.util.ArrayList;
-import java.util.HashSet;
+import java.util.HashMap;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.Map;
 import java.util.Objects;
-import java.util.Set;
 import java.util.stream.Collectors;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.DeletedBlocksTransaction;
@@ -40,6 +40,7 @@ import org.apache.hadoop.hdds.utils.MetadataKeyFilters.KeyPrefixFilter;
 import org.apache.hadoop.hdds.utils.db.BatchOperation;
 import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.hdds.utils.db.TableIterator;
+import org.apache.hadoop.ozone.container.checksum.ContainerChecksumTreeManager;
 import org.apache.hadoop.ozone.container.common.helpers.BlockData;
 import org.apache.hadoop.ozone.container.common.helpers.BlockDeletingServiceMetrics;
 import org.apache.hadoop.ozone.container.common.impl.BlockDeletingService;
@@ -71,10 +72,12 @@ public class BlockDeletingTask implements BackgroundTask {
   private final OzoneContainer ozoneContainer;
   private final ConfigurationSource conf;
   private Duration blockDeletingMaxLockHoldingTime;
+  private final ContainerChecksumTreeManager checksumTreeManager;
 
   public BlockDeletingTask(
       BlockDeletingService blockDeletingService,
       BlockDeletingService.ContainerBlockInfo containerBlockInfo,
+      ContainerChecksumTreeManager checksumTreeManager,
       int priority) {
     this.ozoneContainer = blockDeletingService.getOzoneContainer();
     this.metrics = blockDeletingService.getMetrics();
@@ -85,25 +88,26 @@ public class BlockDeletingTask implements BackgroundTask {
     this.containerData =
         (KeyValueContainerData) containerBlockInfo.getContainerData();
     this.blocksToDelete = containerBlockInfo.getNumBlocksToDelete();
+    this.checksumTreeManager = checksumTreeManager;
   }
 
   private static class ContainerBackgroundTaskResult
       implements BackgroundTaskResult {
-    private List<String> deletedBlockIds;
+    private final List<Long> deletedBlockIds;
 
     ContainerBackgroundTaskResult() {
       deletedBlockIds = new LinkedList<>();
     }
 
-    public void addBlockId(String blockId) {
+    public void addBlockId(Long blockId) {
       deletedBlockIds.add(blockId);
     }
 
-    public void addAll(List<String> blockIds) {
+    public void addAll(List<Long> blockIds) {
       deletedBlockIds.addAll(blockIds);
     }
 
-    public List<String> getDeletedBlocks() {
+    public List<Long> getDeletedBlocks() {
       return deletedBlockIds;
     }
 
@@ -113,7 +117,6 @@ public class BlockDeletingTask implements BackgroundTask {
     }
   }
 
-
   @Override
   public BackgroundTaskResult call() throws Exception {
     ContainerBackgroundTaskResult result =
@@ -182,20 +185,17 @@ public class BlockDeletingTask implements BackgroundTask {
 
       // # of blocks to delete is throttled
       KeyPrefixFilter filter = containerData.getDeletingBlockKeyFilter();
-      List<? extends Table.KeyValue<String, BlockData>> toDeleteBlocks =
-          blockDataTable
-              .getSequentialRangeKVs(containerData.startKeyEmpty(),
-                  (int) blocksToDelete, containerData.containerPrefix(),
-                  filter);
+      final List<Table.KeyValue<String, BlockData>> toDeleteBlocks = blockDataTable.getRangeKVs(
+          containerData.startKeyEmpty(), (int) blocksToDelete, containerData.containerPrefix(), filter, true);
       if (toDeleteBlocks.isEmpty()) {
         LOG.debug("No under deletion block found in container : {}",
             containerData.getContainerID());
         return crr;
       }
 
-      List<String> succeedBlocks = new LinkedList<>();
-      LOG.debug("Container : {}, To-Delete blocks : {}",
-          containerData.getContainerID(), toDeleteBlocks.size());
+      // Maps the key in the DB to the block metadata for blocks that were successfully deleted.
+      Map<String, BlockData> succeedDeletedBlocks = new HashMap<>();
+      LOG.debug("{}, toDeleteBlocks: {}", containerData, toDeleteBlocks.size());
 
       Handler handler = Objects.requireNonNull(ozoneContainer.getDispatcher()
           .getHandler(container.getContainerType()));
@@ -203,18 +203,18 @@ public class BlockDeletingTask implements BackgroundTask {
       long releasedBytes = 0;
       for (Table.KeyValue<String, BlockData> entry : toDeleteBlocks) {
         String blockName = entry.getKey();
+        BlockData blockData = entry.getValue();
         LOG.debug("Deleting block {}", blockName);
-        if (entry.getValue() == null) {
+        if (blockData == null) {
           LOG.warn("Missing delete block(Container = " +
               container.getContainerData().getContainerID() +
               ", Block = " + blockName);
           continue;
         }
         try {
-          handler.deleteBlock(container, entry.getValue());
-          releasedBytes += KeyValueContainerUtil.getBlockLength(
-              entry.getValue());
-          succeedBlocks.add(blockName);
+          handler.deleteBlock(container, blockData);
+          releasedBytes += KeyValueContainerUtil.getBlockLength(blockData);
+          succeedDeletedBlocks.put(blockName, blockData);
         } catch (InvalidProtocolBufferException e) {
           LOG.error("Failed to parse block info for block {}", blockName, e);
         } catch (IOException e) {
@@ -222,12 +222,17 @@ public class BlockDeletingTask implements BackgroundTask {
         }
       }
 
+      // Mark blocks as deleted in the container checksum tree.
+      // Data for these blocks does not need to be copied during container reconciliation if container replicas diverge.
+      // Do this before the delete transactions are removed from the database.
+      checksumTreeManager.addDeletedBlocks(containerData, succeedDeletedBlocks.values());
+
       // Once chunks in the blocks are deleted... remove the blockID from
       // blockDataTable.
       try (BatchOperation batch = meta.getStore().getBatchHandler()
           .initBatchOperation()) {
-        for (String entry : succeedBlocks) {
-          blockDataTable.deleteWithBatch(batch, entry);
+        for (String key: succeedDeletedBlocks.keySet()) {
+          blockDataTable.deleteWithBatch(batch, key);
         }
 
         // Handler.deleteBlock calls deleteChunk to delete all the chunks
@@ -235,7 +240,7 @@ public class BlockDeletingTask implements BackgroundTask {
         // updated with decremented used bytes during deleteChunk. This is
         // done here so that all the DB update for block delete can be
         // batched together while committing to DB.
-        int deletedBlocksCount = succeedBlocks.size();
+        int deletedBlocksCount = succeedDeletedBlocks.size();
         containerData.updateAndCommitDBCounters(meta, batch,
             deletedBlocksCount, releasedBytes);
         // Once DB update is persisted, check if there are any blocks
@@ -247,21 +252,21 @@ public class BlockDeletingTask implements BackgroundTask {
 
         // update count of pending deletion blocks, block count and used
         // bytes in in-memory container status.
-        containerData.decrPendingDeletionBlocks(deletedBlocksCount);
-        containerData.decrBlockCount(deletedBlocksCount);
-        containerData.decrBytesUsed(releasedBytes);
+        containerData.getStatistics().updateDeletion(releasedBytes, deletedBlocksCount, deletedBlocksCount);
         containerData.getVolume().decrementUsedSpace(releasedBytes);
         metrics.incrSuccessCount(deletedBlocksCount);
         metrics.incrSuccessBytes(releasedBytes);
       }
 
-      if (!succeedBlocks.isEmpty()) {
+      if (!succeedDeletedBlocks.isEmpty()) {
         LOG.debug("Container: {}, deleted blocks: {}, space reclaimed: {}, " +
                 "task elapsed time: {}ms", containerData.getContainerID(),
-            succeedBlocks.size(), releasedBytes,
+            succeedDeletedBlocks.size(), releasedBytes,
             Time.monotonicNow() - startTime);
       }
-      crr.addAll(succeedBlocks);
+      crr.addAll(succeedDeletedBlocks.values().stream()
+          .map(BlockData::getLocalID)
+          .collect(Collectors.toList()));
       return crr;
     } catch (IOException exception) {
       LOG.warn("Deletion operation was not successful for container: " +
@@ -282,9 +287,7 @@ public class BlockDeletingTask implements BackgroundTask {
     Table<Long, DeletedBlocksTransaction> deleteTxns =
         ((DeleteTransactionStore<Long>) meta.getStore())
             .getDeleteTransactionTable();
-    try (TableIterator<Long,
-        ? extends Table.KeyValue<Long, DeletedBlocksTransaction>>
-             iterator = deleteTxns.iterator()) {
+    try (TableIterator<Long, DeletedBlocksTransaction> iterator = deleteTxns.valueIterator()) {
       return deleteViaTransactionStore(
           iterator, meta,
           container, dataDir, startTime, schema2Deleter);
@@ -303,9 +306,8 @@ public class BlockDeletingTask implements BackgroundTask {
     Table<String, DeletedBlocksTransaction> deleteTxns =
         ((DeleteTransactionStore<String>) meta.getStore())
             .getDeleteTransactionTable();
-    try (TableIterator<String,
-        ? extends Table.KeyValue<String, DeletedBlocksTransaction>>
-             iterator = deleteTxns.iterator(containerData.containerPrefix())) {
+    try (TableIterator<String, DeletedBlocksTransaction> iterator
+        = deleteTxns.valueIterator(containerData.containerPrefix())) {
       return deleteViaTransactionStore(
           iterator, meta,
           container, dataDir, startTime, schema3Deleter);
@@ -313,8 +315,7 @@ public class BlockDeletingTask implements BackgroundTask {
   }
 
   private ContainerBackgroundTaskResult deleteViaTransactionStore(
-      TableIterator<?, ? extends Table.KeyValue<?, DeletedBlocksTransaction>>
-          iter, DBHandle meta, Container container, File dataDir,
+      TableIterator<?, DeletedBlocksTransaction> iter, DBHandle meta, Container container, File dataDir,
       long startTime, Deleter deleter) throws IOException {
     ContainerBackgroundTaskResult crr = new ContainerBackgroundTaskResult();
     if (!checkDataDir(dataDir)) {
@@ -332,15 +333,12 @@ public class BlockDeletingTask implements BackgroundTask {
       List<DeletedBlocksTransaction> delBlocks = new ArrayList<>();
       int numBlocks = 0;
       while (iter.hasNext() && (numBlocks < blocksToDelete)) {
-        DeletedBlocksTransaction delTx = iter.next().getValue();
+        final DeletedBlocksTransaction delTx = iter.next();
         numBlocks += delTx.getLocalIDList().size();
         delBlocks.add(delTx);
       }
       if (delBlocks.isEmpty()) {
-        LOG.info("No transaction found in container {} with pending delete " +
-                "block count {}",
-            containerData.getContainerID(),
-            containerData.getNumPendingDeletionBlocks());
+        LOG.info("Pending block deletion not found in {}: {}", containerData, containerData.getStatistics());
         // If the container was queued for delete, it had a positive
         // pending delete block count. After checking the DB there were
         // actually no delete transactions for the container, so reset the
@@ -349,8 +347,7 @@ public class BlockDeletingTask implements BackgroundTask {
         return crr;
       }
 
-      LOG.debug("Container : {}, To-Delete blocks : {}",
-          containerData.getContainerID(), delBlocks.size());
+      LOG.debug("{}, delBlocks: {}", containerData, delBlocks.size());
 
       Handler handler = Objects.requireNonNull(ozoneContainer.getDispatcher()
           .getHandler(container.getContainerType()));
@@ -363,9 +360,7 @@ public class BlockDeletingTask implements BackgroundTask {
       List<DeletedBlocksTransaction> deletedBlocksTxs =
           deleteBlocksResult.deletedBlocksTxs();
       deleteBlocksResult.deletedBlocksTxs().forEach(
-          tx -> crr.addAll(tx.getLocalIDList().stream()
-              .map(String::valueOf).collect(Collectors.toList()))
-      );
+          tx -> crr.addAll(tx.getLocalIDList()));
 
       // Once blocks are deleted... remove the blockID from blockDataTable
       // and also remove the transactions from txnTable.
@@ -398,9 +393,7 @@ public class BlockDeletingTask implements BackgroundTask {
 
         // update count of pending deletion blocks, block count and used
         // bytes in in-memory container status and used space in volume.
-        containerData.decrPendingDeletionBlocks(deletedBlocksProcessed);
-        containerData.decrBlockCount(deletedBlocksCount);
-        containerData.decrBytesUsed(releasedBytes);
+        containerData.getStatistics().updateDeletion(releasedBytes, deletedBlocksCount, deletedBlocksProcessed);
         containerData.getVolume().decrementUsedSpace(releasedBytes);
         metrics.incrSuccessCount(deletedBlocksCount);
         metrics.incrSuccessBytes(releasedBytes);
@@ -435,7 +428,7 @@ public class BlockDeletingTask implements BackgroundTask {
     Instant startTime = Instant.now();
 
     // Track deleted blocks to avoid duplicate deletion
-    Set<Long> deletedBlockSet = new HashSet<>();
+    Map<Long, BlockData> deletedBlocks = new HashMap<>();
 
     for (DeletedBlocksTransaction entry : delBlocks) {
       for (Long blkLong : entry.getLocalIDList()) {
@@ -443,7 +436,7 @@ public class BlockDeletingTask implements BackgroundTask {
         blocksProcessed++;
 
         // Check if the block has already been deleted
-        if (deletedBlockSet.contains(blkLong)) {
+        if (deletedBlocks.containsKey(blkLong)) {
           LOG.debug("Skipping duplicate deletion for block {}", blkLong);
           continue;
         }
@@ -468,7 +461,7 @@ public class BlockDeletingTask implements BackgroundTask {
           blocksDeleted++;
           deleted = true;
           // Track this block as deleted
-          deletedBlockSet.add(blkLong);
+          deletedBlocks.put(blkLong, blkInfo);
         } catch (IOException e) {
           // TODO: if deletion of certain block retries exceed the certain
           //  number of times, service should skip deleting it,
@@ -478,13 +471,8 @@ public class BlockDeletingTask implements BackgroundTask {
         }
 
         if (deleted) {
-          try {
-            bytesReleased += KeyValueContainerUtil.getBlockLength(blkInfo);
-          } catch (IOException e) {
-            // TODO: handle the bytesReleased correctly for the unexpected
-            //  exception.
-            LOG.error("Failed to get block length for block {}", blkLong, e);
-          }
+          bytesReleased += KeyValueContainerUtil.getBlockLengthTryCatch(blkInfo);
+          // TODO: handle the bytesReleased correctly for the unexpected exception.
         }
       }
       deletedBlocksTxs.add(entry);
@@ -501,6 +489,7 @@ public class BlockDeletingTask implements BackgroundTask {
         break;
       }
     }
+    checksumTreeManager.addDeletedBlocks(containerData, deletedBlocks.values());
     return new DeleteTransactionStats(blocksProcessed,
         blocksDeleted, bytesReleased, deletedBlocksTxs);
   }
