@@ -17,18 +17,11 @@
 
 package org.apache.hadoop.ozone.container.ozoneimpl;
 
-import static org.apache.hadoop.ozone.container.common.interfaces.Container.ScanResult;
-import static org.apache.hadoop.ozone.container.common.interfaces.Container.ScanResult.FailureType.DELETED_CONTAINER;
-
 import com.google.common.annotations.VisibleForTesting;
 import java.io.IOException;
-import java.time.Instant;
 import java.util.Iterator;
-import java.util.Optional;
 import org.apache.hadoop.hdfs.util.Canceler;
 import org.apache.hadoop.hdfs.util.DataTransferThrottler;
-import org.apache.hadoop.ozone.container.common.helpers.ContainerUtils;
-import org.apache.hadoop.ozone.container.common.impl.ContainerData;
 import org.apache.hadoop.ozone.container.common.interfaces.Container;
 import org.apache.hadoop.ozone.container.common.volume.HddsVolume;
 import org.slf4j.Logger;
@@ -39,7 +32,7 @@ import org.slf4j.LoggerFactory;
  */
 public class BackgroundContainerDataScanner extends
     AbstractBackgroundContainerScanner {
-  public static final Logger LOG =
+  private static final Logger LOG =
       LoggerFactory.getLogger(BackgroundContainerDataScanner.class);
 
   /**
@@ -51,7 +44,7 @@ public class BackgroundContainerDataScanner extends
   private final Canceler canceler;
   private static final String NAME_FORMAT = "ContainerDataScanner(%s)";
   private final ContainerDataScannerMetrics metrics;
-  private final long minScanGap;
+  private final ContainerScanHelper scanHelper;
 
   public BackgroundContainerDataScanner(ContainerScannerConfiguration conf,
                                         ContainerController controller,
@@ -63,12 +56,7 @@ public class BackgroundContainerDataScanner extends
     canceler = new Canceler();
     this.metrics = ContainerDataScannerMetrics.create(volume.toString());
     this.metrics.setStorageDirectory(volume.toString());
-    this.minScanGap = conf.getContainerScanMinGap();
-  }
-
-  private boolean shouldScan(Container<?> container) {
-    return container.shouldScanData() &&
-        !ContainerUtils.recentlyScanned(container, minScanGap, LOG);
+    this.scanHelper = ContainerScanHelper.withScanGap(LOG, controller, metrics, conf);
   }
 
   @Override
@@ -80,32 +68,7 @@ public class BackgroundContainerDataScanner extends
       shutdown("The volume has failed.");
       return;
     }
-
-    if (!shouldScan(c)) {
-      return;
-    }
-    ContainerData containerData = c.getContainerData();
-    long containerId = containerData.getContainerID();
-    logScanStart(containerData);
-    ScanResult result = c.scanData(throttler, canceler);
-
-    // Metrics for skipped containers should not be updated.
-    if (result.getFailureType() == DELETED_CONTAINER) {
-      LOG.error("Container [{}] has been deleted.",
-          containerId, result.getException());
-      return;
-    }
-    if (!result.isHealthy()) {
-      LOG.error("Corruption detected in container [{}]. Marking it UNHEALTHY.",
-          containerId, result.getException());
-      metrics.incNumUnHealthyContainers();
-      controller.markContainerUnhealthy(containerId, result);
-    }
-
-    metrics.incNumContainersScanned();
-    Instant now = Instant.now();
-    logScanCompleted(containerData, now);
-    controller.updateDataScanTimestamp(containerId, now);
+    scanHelper.scanData(c, throttler, canceler);
   }
 
   @Override
@@ -113,23 +76,6 @@ public class BackgroundContainerDataScanner extends
     return controller.getContainers(volume);
   }
 
-  private static void logScanStart(ContainerData containerData) {
-    if (LOG.isDebugEnabled()) {
-      Optional<Instant> scanTimestamp = containerData.lastDataScanTime();
-      Object lastScanTime = scanTimestamp.map(ts -> "at " + ts).orElse("never");
-      LOG.debug("Scanning container {}, last scanned {}",
-          containerData.getContainerID(), lastScanTime);
-    }
-  }
-
-  private static void logScanCompleted(
-      ContainerData containerData, Instant timestamp) {
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("Completed scan of container {} at {}",
-          containerData.getContainerID(), timestamp);
-    }
-  }
-
   @Override
   public synchronized void shutdown() {
     shutdown("");
