@@ -51,18 +51,16 @@ public enum CopyContainerCompression {
   SNAPPY(CompressorStreamFactory.SNAPPY_FRAMED),
   ZSTD(CompressorStreamFactory.ZSTANDARD);
 
+  private static final Logger LOG = LoggerFactory.getLogger(CopyContainerCompression.class);
+
+  private static final CopyContainerCompression DEFAULT_COMPRESSION = CopyContainerCompression.NO_COMPRESSION;
+
   private final String compressorFactoryName;
 
   CopyContainerCompression(String compressorFactoryName) {
     this.compressorFactoryName = compressorFactoryName;
   }
 
-  private static final Logger LOG =
-      LoggerFactory.getLogger(CopyContainerCompression.class);
-
-  private static final CopyContainerCompression DEFAULT_COMPRESSION =
-      CopyContainerCompression.NO_COMPRESSION;
-
   public static CopyContainerCompression getConf(ConfigurationSource conf) {
     try {
       return conf.getEnum(HDDS_CONTAINER_REPLICATION_COMPRESSION,
