@@ -30,6 +30,8 @@ import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.SendContai
 import org.apache.hadoop.hdds.security.SecurityConfig;
 import org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClient;
 import org.apache.hadoop.hdds.utils.IOUtils;
+import org.apache.hadoop.ozone.container.common.interfaces.Container;
+import org.apache.hadoop.ozone.container.ozoneimpl.ContainerController;
 import org.apache.ratis.thirdparty.io.grpc.stub.CallStreamObserver;
 import org.apache.ratis.thirdparty.io.grpc.stub.StreamObserver;
 import org.slf4j.Logger;
@@ -45,17 +47,29 @@ public class GrpcContainerUploader implements ContainerUploader {
 
   private final SecurityConfig securityConfig;
   private final CertificateClient certClient;
+  private final ContainerController containerController;
 
   public GrpcContainerUploader(
-      ConfigurationSource conf, CertificateClient certClient) {
+      ConfigurationSource conf, CertificateClient certClient, 
+      ContainerController containerController) {
     this.certClient = certClient;
+    this.containerController = containerController;
     securityConfig = new SecurityConfig(conf);
   }
 
   @Override
   public OutputStream startUpload(long containerId, DatanodeDetails target,
-      CompletableFuture<Void> callback, CopyContainerCompression compression)
-      throws IOException {
+      CompletableFuture<Void> callback, CopyContainerCompression compression) throws IOException {
+    
+    // Get container size from local datanode instead of using passed replicateSize
+    Long containerSize = null;
+    Container<?> container = containerController.getContainer(containerId);
+    if (container != null) {
+      LOG.debug("Starting upload of container {} to {} with size {}",
+          containerId, target, container.getContainerData().getBytesUsed());
+      containerSize = container.getContainerData().getBytesUsed();
+    }
+    
     GrpcReplicationClient client = createReplicationClient(target, compression);
     try {
       // gRPC runtime always provides implementation of CallStreamObserver
@@ -68,7 +82,7 @@ public class GrpcContainerUploader implements ContainerUploader {
               (CallStreamObserver<SendContainerRequest>) client.upload(
               responseObserver), responseObserver);
       return new SendContainerOutputStream(requestStream, containerId,
-          GrpcReplicationService.BUFFER_SIZE, compression) {
+          GrpcReplicationService.BUFFER_SIZE, compression, containerSize) {
         @Override
         public void close() throws IOException {
           try {
@@ -156,6 +170,7 @@ public class GrpcContainerUploader implements ContainerUploader {
       this.observer = observer;
       this.responseObserver = responseObserver;
     }
+
     @Override
     public boolean isReady() {
       if (responseObserver.isError()) {
