@@ -27,22 +27,29 @@ import org.apache.ratis.thirdparty.io.grpc.stub.CallStreamObserver;
 class SendContainerOutputStream extends GrpcOutputStream<SendContainerRequest> {
 
   private final CopyContainerCompression compression;
+  private final Long size;
 
   SendContainerOutputStream(
       CallStreamObserver<SendContainerRequest> streamObserver,
-      long containerId, int bufferSize, CopyContainerCompression compression) {
+      long containerId, int bufferSize, CopyContainerCompression compression,
+      Long size) {
     super(streamObserver, containerId, bufferSize);
     this.compression = compression;
+    this.size = size;
   }
 
   @Override
   protected void sendPart(boolean eof, int length, ByteString data) {
-    SendContainerRequest request = SendContainerRequest.newBuilder()
+    SendContainerRequest.Builder requestBuilder = SendContainerRequest.newBuilder()
         .setContainerID(getContainerId())
         .setData(data)
         .setOffset(getWrittenBytes())
-        .setCompression(compression.toProto())
-        .build();
-    getStreamObserver().onNext(request);
+        .setCompression(compression.toProto());
+    
+    // Include container size in the first request
+    if (getWrittenBytes() == 0 && size != null) {
+      requestBuilder.setSize(size);
+    }
+    getStreamObserver().onNext(requestBuilder.build());
   }
 }
