@@ -18,28 +18,35 @@
 package org.apache.hadoop.ozone.container.common;
 
 import static org.apache.hadoop.ozone.common.Storage.StorageState.INITIALIZED;
+import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.mockito.Mockito.any;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
 
+import com.google.common.collect.Lists;
+import com.google.common.collect.Maps;
 import java.io.File;
 import java.io.IOException;
 import java.net.InetSocketAddress;
 import java.util.Collections;
+import java.util.List;
 import java.util.Map;
 import java.util.Random;
 import java.util.UUID;
 import java.util.concurrent.atomic.AtomicLong;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.conf.StorageUnit;
+import org.apache.hadoop.hdds.client.BlockID;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.conf.ReconfigurationHandler;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerCommandRequestProto;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerCommandResponseProto;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerDataProto;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerType;
+import org.apache.hadoop.hdds.security.token.TokenVerifier;
 import org.apache.hadoop.hdds.utils.LegacyHadoopConfigurationSource;
 import org.apache.hadoop.hdfs.util.Canceler;
 import org.apache.hadoop.hdfs.util.DataTransferThrottler;
@@ -50,12 +57,19 @@ import org.apache.hadoop.net.NetUtils;
 import org.apache.hadoop.ozone.HddsDatanodeService;
 import org.apache.hadoop.ozone.OzoneConfigKeys;
 import org.apache.hadoop.ozone.container.ContainerTestHelper;
+import org.apache.hadoop.ozone.container.checksum.ContainerChecksumTreeManager;
+import org.apache.hadoop.ozone.container.checksum.ContainerMerkleTreeWriter;
+import org.apache.hadoop.ozone.container.common.helpers.BlockData;
+import org.apache.hadoop.ozone.container.common.helpers.ChunkInfo;
+import org.apache.hadoop.ozone.container.common.helpers.ContainerMetrics;
 import org.apache.hadoop.ozone.container.common.impl.ContainerData;
 import org.apache.hadoop.ozone.container.common.impl.ContainerImplTestUtils;
 import org.apache.hadoop.ozone.container.common.impl.ContainerLayoutVersion;
+import org.apache.hadoop.ozone.container.common.impl.ContainerSet;
+import org.apache.hadoop.ozone.container.common.impl.HddsDispatcher;
 import org.apache.hadoop.ozone.container.common.interfaces.Container;
-import org.apache.hadoop.ozone.container.common.interfaces.Container.ScanResult;
 import org.apache.hadoop.ozone.container.common.interfaces.ContainerDispatcher;
+import org.apache.hadoop.ozone.container.common.interfaces.DBHandle;
 import org.apache.hadoop.ozone.container.common.interfaces.Handler;
 import org.apache.hadoop.ozone.container.common.interfaces.VolumeChoosingPolicy;
 import org.apache.hadoop.ozone.container.common.statemachine.DatanodeConfiguration;
@@ -68,10 +82,17 @@ import org.apache.hadoop.ozone.container.common.utils.StorageVolumeUtil;
 import org.apache.hadoop.ozone.container.common.volume.HddsVolume;
 import org.apache.hadoop.ozone.container.common.volume.MutableVolumeSet;
 import org.apache.hadoop.ozone.container.common.volume.RoundRobinVolumeChoosingPolicy;
+import org.apache.hadoop.ozone.container.common.volume.VolumeChoosingPolicyFactory;
+import org.apache.hadoop.ozone.container.common.volume.VolumeSet;
 import org.apache.hadoop.ozone.container.keyvalue.KeyValueContainer;
 import org.apache.hadoop.ozone.container.keyvalue.KeyValueContainerData;
+import org.apache.hadoop.ozone.container.keyvalue.KeyValueHandler;
+import org.apache.hadoop.ozone.container.keyvalue.helpers.BlockUtils;
 import org.apache.hadoop.ozone.container.keyvalue.helpers.KeyValueContainerUtil;
 import org.apache.hadoop.ozone.container.ozoneimpl.ContainerController;
+import org.apache.hadoop.ozone.container.ozoneimpl.ContainerScanError;
+import org.apache.hadoop.ozone.container.ozoneimpl.DataScanResult;
+import org.apache.hadoop.ozone.container.ozoneimpl.MetadataScanResult;
 import org.apache.hadoop.ozone.container.ozoneimpl.OzoneContainer;
 import org.apache.hadoop.ozone.protocolPB.StorageContainerDatanodeProtocolClientSideTranslatorPB;
 import org.apache.hadoop.ozone.protocolPB.StorageContainerDatanodeProtocolPB;
@@ -83,9 +104,6 @@ import org.mockito.Mockito;
  */
 public final class ContainerTestUtils {
 
-  private ContainerTestUtils() {
-  }
-
   public static final DispatcherContext WRITE_STAGE = DispatcherContext
       .newBuilder(DispatcherContext.Op.WRITE_STATE_MACHINE_DATA)
       .setStage(DispatcherContext.WriteChunkStage.WRITE_DATA)
@@ -100,6 +118,14 @@ public final class ContainerTestUtils {
   public static final DispatcherContext COMBINED_STAGE
       = DispatcherContext.getHandleWriteChunk();
 
+  private static final ContainerDispatcher NOOP_CONTAINER_DISPATCHER = new NoopContainerDispatcher();
+
+  private static final ContainerController EMPTY_CONTAINER_CONTROLLER
+      = new ContainerController(ContainerImplTestUtils.newContainerSet(), Collections.emptyMap());
+
+  private ContainerTestUtils() {
+  }
+
   /**
    * Creates an Endpoint class for testing purpose.
    *
@@ -132,7 +158,8 @@ public final class ContainerTestUtils {
       DatanodeDetails datanodeDetails, OzoneConfiguration conf)
       throws IOException {
     StateContext context = getMockContext(datanodeDetails, conf);
-    return new OzoneContainer(datanodeDetails, conf, context);
+    VolumeChoosingPolicy volumeChoosingPolicy = VolumeChoosingPolicyFactory.getPolicy(conf);
+    return new OzoneContainer(datanodeDetails, conf, context, volumeChoosingPolicy);
   }
 
   public static StateContext getMockContext(DatanodeDetails datanodeDetails,
@@ -180,6 +207,57 @@ public final class ContainerTestUtils {
     return new KeyValueContainer(kvData, new OzoneConfiguration());
   }
 
+  public static KeyValueHandler getKeyValueHandler(ConfigurationSource config,
+      String datanodeId, ContainerSet contSet, VolumeSet volSet, ContainerMetrics metrics) {
+    return getKeyValueHandler(config, datanodeId, contSet, volSet, metrics, new ContainerChecksumTreeManager(config));
+  }
+
+  /**
+   * Constructs an instance of KeyValueHandler that can be used for testing.
+   * This instance can be used for tests that do not need an ICR sender or {@link ContainerChecksumTreeManager}.
+   */
+  public static KeyValueHandler getKeyValueHandler(ConfigurationSource config,
+      String datanodeId, ContainerSet contSet, VolumeSet volSet, ContainerMetrics metrics,
+      ContainerChecksumTreeManager checksumTreeManager) {
+    return new KeyValueHandler(config, datanodeId, contSet, volSet, metrics, c -> { }, checksumTreeManager);
+  }
+
+  /**
+   * Constructs an instance of KeyValueHandler that can be used for testing.
+   * This instance can be used for tests that do not need an ICR sender, metrics, or a
+   * {@link ContainerChecksumTreeManager}.
+   */
+  public static KeyValueHandler getKeyValueHandler(ConfigurationSource config,
+      String datanodeId, ContainerSet contSet, VolumeSet volSet) {
+    return getKeyValueHandler(config, datanodeId, contSet, volSet, ContainerMetrics.create(config));
+  }
+
+  public static KeyValueHandler getKeyValueHandler(ConfigurationSource config,
+      String datanodeId, ContainerSet contSet, VolumeSet volSet, ContainerChecksumTreeManager checksumTreeManager) {
+    return getKeyValueHandler(config, datanodeId, contSet, volSet, ContainerMetrics.create(config),
+        checksumTreeManager);
+  }
+
+  public static HddsDispatcher getHddsDispatcher(OzoneConfiguration conf,
+                                                 ContainerSet contSet,
+                                                 VolumeSet volSet,
+                                                 StateContext context) {
+    return getHddsDispatcher(conf, contSet, volSet, context, null);
+  }
+
+  public static HddsDispatcher getHddsDispatcher(OzoneConfiguration conf,
+                                                 ContainerSet contSet,
+                                                 VolumeSet volSet,
+                                                 StateContext context, TokenVerifier verifier) {
+    ContainerMetrics metrics = ContainerMetrics.create(conf);
+    Map<ContainerType, Handler> handlers = Maps.newHashMap();
+    handlers.put(ContainerType.KeyValueContainer, ContainerTestUtils.getKeyValueHandler(conf,
+        context.getParent().getDatanodeDetails().getUuidString(), contSet, volSet, metrics));
+    assertEquals(1, ContainerType.values().length, "Tests only cover KeyValueContainer type");
+    return new HddsDispatcher(
+        conf, contSet, volSet, handlers, context, metrics, verifier);
+  }
+
   public static void enableSchemaV3(OzoneConfiguration conf) {
     DatanodeConfiguration dc = conf.getObject(DatanodeConfiguration.class);
     dc.setContainerSchemaV3Enabled(true);
@@ -209,18 +287,16 @@ public final class ContainerTestUtils {
 
   public static void setupMockContainer(
       Container<ContainerData> c, boolean shouldScanData,
-      ScanResult metadataScanResult, ScanResult dataScanResult,
+      MetadataScanResult metadataScanResult, DataScanResult dataScanResult,
       AtomicLong containerIdSeq, HddsVolume vol) {
     ContainerData data = mock(ContainerData.class);
     when(data.getContainerID()).thenReturn(containerIdSeq.getAndIncrement());
     when(c.getContainerData()).thenReturn(data);
     when(c.shouldScanData()).thenReturn(shouldScanData);
-    when(c.shouldScanMetadata()).thenReturn(true);
     when(c.getContainerData().getVolume()).thenReturn(vol);
 
     try {
-      when(c.scanData(any(DataTransferThrottler.class), any(Canceler.class)))
-          .thenReturn(dataScanResult);
+      when(c.scanData(any(DataTransferThrottler.class), any(Canceler.class))).thenReturn(dataScanResult);
       Mockito.lenient().when(c.scanMetaData()).thenReturn(metadataScanResult);
     } catch (InterruptedException ex) {
       // Mockito.when invocations will not throw this exception. It is just
@@ -228,22 +304,42 @@ public final class ContainerTestUtils {
     }
   }
 
-  /**
-   * Construct an unhealthy scan result to use for testing purposes.
-   */
-  public static ScanResult getUnhealthyScanResult() {
-    return ScanResult.unhealthy(ScanResult.FailureType.CORRUPT_CHUNK,
-        new File(""),
-        new IOException("Fake corruption failure for testing"));
+  public static DataScanResult getHealthyDataScanResult() {
+    return DataScanResult.fromErrors(Collections.emptyList(), new ContainerMerkleTreeWriter());
   }
 
   /**
-   * Construct an unhealthy scan result with DELETED_CONTAINER failure type.
+   * Construct an unhealthy scan result to use for testing purposes.
    */
-  public static ScanResult getDeletedContainerResult() {
-    return ScanResult.unhealthy(ScanResult.FailureType.DELETED_CONTAINER,
-        new File(""),
-        new IOException("Fake deleted container exception"));
+  public static DataScanResult getUnhealthyDataScanResult() {
+    return DataScanResult.fromErrors(Collections.singletonList(getDataScanError()), new ContainerMerkleTreeWriter());
+  }
+
+  public static MetadataScanResult getHealthyMetadataScanResult() {
+    return MetadataScanResult.fromErrors(Collections.emptyList());
+  }
+
+  /**
+   * Construct a generic data scan error that can be used for testing.
+   */
+  public static ContainerScanError getDataScanError() {
+    return new ContainerScanError(ContainerScanError.FailureType.CORRUPT_CHUNK, new File(""),
+        new IOException("Fake data corruption failure for testing"));
+  }
+
+  /**
+   * Construct a generic metadata scan error that can be used for testing.
+   */
+  public static ContainerScanError getMetadataScanError() {
+    return new ContainerScanError(ContainerScanError.FailureType.CORRUPT_CONTAINER_FILE, new File(""),
+        new IOException("Fake metadata corruption failure for testing"));
+  }
+
+  /**
+   * Construct an unhealthy scan result to use for testing purposes.
+   */
+  public static MetadataScanResult getUnhealthyMetadataScanResult() {
+    return DataScanResult.fromErrors(Collections.singletonList(getMetadataScanError()));
   }
 
   public static KeyValueContainer addContainerToDeletedDir(
@@ -329,16 +425,10 @@ public final class ContainerTestUtils {
     }
   }
 
-  private static final ContainerDispatcher NOOP_CONTAINER_DISPATCHER
-      = new NoopContainerDispatcher();
-
   public static ContainerDispatcher getNoopContainerDispatcher() {
     return NOOP_CONTAINER_DISPATCHER;
   }
 
-  private static final ContainerController EMPTY_CONTAINER_CONTROLLER
-      = new ContainerController(ContainerImplTestUtils.newContainerSet(), Collections.emptyMap());
-
   public static ContainerController getEmptyContainerController() {
     return EMPTY_CONTAINER_CONTROLLER;
   }
@@ -361,4 +451,27 @@ public final class ContainerTestUtils {
       layoutStorage.initialize();
     }
   }
+
+  /**
+   * Creates block metadata for the given container with the specified number of blocks and chunks per block.
+   */
+  public static void createBlockMetaData(KeyValueContainerData data, int numOfBlocksPerContainer,
+                                         int numOfChunksPerBlock) throws IOException {
+    try (DBHandle metadata = BlockUtils.getDB(data, new OzoneConfiguration())) {
+      for (int j = 0; j < numOfBlocksPerContainer; j++) {
+        BlockID blockID = new BlockID(data.getContainerID(), j);
+        String blockKey = data.getBlockKey(blockID.getLocalID());
+        BlockData kd = new BlockData(blockID);
+        List<ContainerProtos.ChunkInfo> chunks = Lists.newArrayList();
+        for (int k = 0; k < numOfChunksPerBlock; k++) {
+          long dataLen = 10L;
+          ChunkInfo chunkInfo = ContainerTestHelper.getChunk(blockID.getLocalID(), k, k * dataLen, dataLen);
+          ContainerTestHelper.setDataChecksum(chunkInfo, ContainerTestHelper.getData((int) dataLen));
+          chunks.add(chunkInfo.getProtoBufMessage());
+        }
+        kd.setChunks(chunks);
+        metadata.getStore().getBlockDataTable().put(blockKey, kd);
+      }
+    }
+  }
 }
