@@ -48,7 +48,6 @@ import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.container.common.helpers.BlockData;
 import org.apache.hadoop.ozone.container.common.helpers.ChunkInfoList;
-import org.apache.hadoop.ozone.container.common.helpers.ContainerMetrics;
 import org.apache.hadoop.ozone.container.common.impl.ContainerDataYaml;
 import org.apache.hadoop.ozone.container.common.impl.ContainerSet;
 import org.apache.hadoop.ozone.container.common.interfaces.BlockIterator;
@@ -192,7 +191,7 @@ public class TestSchemaOneBackwardsCompatibility {
               refCountedDB.getStore().getDeletedBlocksTable();
 
       // Test rangeKVs.
-      List<? extends Table.KeyValue<String, ChunkInfoList>> deletedBlocks =
+      List<Table.KeyValue<String, ChunkInfoList>> deletedBlocks =
               deletedBlocksTable.getRangeKVs(cData.startKeyEmpty(), 100,
                   cData.containerPrefix());
 
@@ -276,11 +275,8 @@ public class TestSchemaOneBackwardsCompatibility {
     ContainerSet containerSet = makeContainerSet();
     VolumeSet volumeSet = new MutableVolumeSet(datanodeUuid, conf, null,
         StorageVolume.VolumeType.DATA_VOLUME, null);
-    ContainerMetrics metrics = ContainerMetrics.create(conf);
     KeyValueHandler keyValueHandler =
-        new KeyValueHandler(conf, datanodeUuid, containerSet, volumeSet,
-            metrics, c -> {
-        });
+        ContainerTestUtils.getKeyValueHandler(conf, datanodeUuid, containerSet, volumeSet);
     long initialTotalSpace = newKvData().getBytesUsed();
     long blockSpace = initialTotalSpace / TestDB.KEY_COUNT;
 
@@ -349,15 +345,12 @@ public class TestSchemaOneBackwardsCompatibility {
     ContainerSet containerSet = makeContainerSet();
     VolumeSet volumeSet = new MutableVolumeSet(datanodeUuid, conf, null,
         StorageVolume.VolumeType.DATA_VOLUME, null);
-    ContainerMetrics metrics = ContainerMetrics.create(conf);
     KeyValueHandler keyValueHandler =
-        new KeyValueHandler(conf, datanodeUuid, containerSet, volumeSet,
-            metrics, c -> {
-        });
+        ContainerTestUtils.getKeyValueHandler(conf, datanodeUuid, containerSet, volumeSet);
     KeyValueContainerData cData = newKvData();
     try (DBHandle refCountedDB = BlockUtils.getDB(cData, conf)) {
       // Read blocks that were already deleted before the upgrade.
-      List<? extends Table.KeyValue<String, ChunkInfoList>> deletedBlocks =
+      List<Table.KeyValue<String, ChunkInfoList>> deletedBlocks =
               refCountedDB.getStore().getDeletedBlocksTable()
                   .getRangeKVs(cData.startKeyEmpty(), 100,
                       cData.containerPrefix());
@@ -366,8 +359,7 @@ public class TestSchemaOneBackwardsCompatibility {
 
       for (Table.KeyValue<String, ChunkInfoList> chunkListKV: deletedBlocks) {
         preUpgradeBlocks.add(chunkListKV.getKey());
-        assertThrows(IOException.class, () -> chunkListKV.getValue(),
-            "No exception thrown when trying to retrieve old deleted blocks values as chunk lists.");
+        assertNull(chunkListKV.getValue());
       }
 
       assertEquals(TestDB.NUM_DELETED_BLOCKS, preUpgradeBlocks.size());
@@ -416,7 +408,7 @@ public class TestSchemaOneBackwardsCompatibility {
       }
 
       // Test decoding keys from the database.
-      List<? extends Table.KeyValue<String, BlockData>> blockKeyValues =
+      List<Table.KeyValue<String, BlockData>> blockKeyValues =
           blockDataTable.getRangeKVs(cData.startKeyEmpty(), 100,
               cData.containerPrefix(), cData.getUnprefixedKeyFilter());
 
@@ -462,7 +454,7 @@ public class TestSchemaOneBackwardsCompatibility {
       }
 
       // Test decoding keys from the database.
-      List<? extends Table.KeyValue<String, BlockData>> blockKeyValues =
+      List<Table.KeyValue<String, BlockData>> blockKeyValues =
           blockDataTable.getRangeKVs(cData.startKeyEmpty(), 100,
               cData.containerPrefix(), cData.getDeletingBlockKeyFilter());
 
@@ -535,7 +527,7 @@ public class TestSchemaOneBackwardsCompatibility {
       }
 
       // Test decoding keys from the database.
-      List<? extends Table.KeyValue<String, ChunkInfoList>> chunkInfoKeyValues =
+      List<Table.KeyValue<String, ChunkInfoList>> chunkInfoKeyValues =
           deletedBlocksTable.getRangeKVs(cData.startKeyEmpty(), 100,
               cData.containerPrefix());
 
@@ -608,7 +600,7 @@ public class TestSchemaOneBackwardsCompatibility {
     Yaml yaml = ContainerDataYaml.getYamlForContainerType(
             kvData.getContainerType(),
         kvData.getReplicaIndex() > 0);
-    kvData.computeAndSetChecksum(yaml);
+    kvData.computeAndSetContainerFileChecksum(yaml);
 
     KeyValueContainerUtil.parseKVContainerData(kvData, conf);
 
