@@ -26,17 +26,17 @@ import org.apache.hadoop.hdds.client.BlockID;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos;
 import org.apache.hadoop.ozone.common.Checksum;
 import org.junit.jupiter.api.Test;
-import org.junit.jupiter.api.Timeout;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
  * Tests to test block deleting service.
  */
-@Timeout(10)
 public class TestBlockData {
   static final Logger LOG = LoggerFactory.getLogger(TestBlockData.class);
 
+  private static int chunkCount = 0;
+
   static ContainerProtos.ChunkInfo buildChunkInfo(String name, long offset,
       long len) {
     return ContainerProtos.ChunkInfo.newBuilder()
@@ -64,7 +64,6 @@ public class TestBlockData {
     }
   }
 
-  private static int chunkCount = 0;
   static ContainerProtos.ChunkInfo addChunk(
       List<ContainerProtos.ChunkInfo> expected, long offset) {
     final long length = ThreadLocalRandom.current().nextLong(1000);
@@ -83,7 +82,6 @@ public class TestBlockData {
     return info.getLen();
   }
 
-
   static void removeChunk(List<ContainerProtos.ChunkInfo> expected,
       BlockData computed) {
     final int i = ThreadLocalRandom.current().nextInt(expected.size());
