@@ -17,8 +17,12 @@
 
 package org.apache.hadoop.ozone.container.common.statemachine.commandhandler;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.mockito.Mockito.any;
 import static org.mockito.Mockito.anyBoolean;
+import static org.mockito.Mockito.doAnswer;
 import static org.mockito.Mockito.eq;
 import static org.mockito.Mockito.lenient;
 import static org.mockito.Mockito.mock;
@@ -31,6 +35,11 @@ import java.io.IOException;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.List;
+import java.util.UUID;
+import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.TimeUnit;
 import java.util.stream.Collectors;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
@@ -59,7 +68,6 @@ import org.junit.jupiter.api.Test;
 public class TestClosePipelineCommandHandler {
 
   private OzoneContainer ozoneContainer;
-  private StateContext stateContext;
   private SCMConnectionManager connectionManager;
   private RaftClient raftClient;
   private GroupManagementApi raftClientGroupManager;
@@ -83,7 +91,7 @@ public class TestClosePipelineCommandHandler {
     final PipelineID pipelineID = PipelineID.randomId();
     final SCMCommand<ClosePipelineCommandProto> command =
         new ClosePipelineCommand(pipelineID);
-    stateContext = ContainerTestUtils.getMockContext(currentDatanode, conf);
+    StateContext stateContext = ContainerTestUtils.getMockContext(currentDatanode, conf);
 
     final boolean shouldDeleteRatisLogDirectory = true;
     XceiverServerRatis writeChannel = mock(XceiverServerRatis.class);
@@ -115,7 +123,7 @@ public class TestClosePipelineCommandHandler {
     final PipelineID pipelineID = PipelineID.randomId();
     final SCMCommand<ClosePipelineCommandProto> command =
         new ClosePipelineCommand(pipelineID);
-    stateContext = ContainerTestUtils.getMockContext(currentDatanode, conf);
+    StateContext stateContext = ContainerTestUtils.getMockContext(currentDatanode, conf);
 
     XceiverServerRatis writeChannel = mock(XceiverServerRatis.class);
     when(ozoneContainer.getWriteChannel()).thenReturn(writeChannel);
@@ -133,6 +141,55 @@ public class TestClosePipelineCommandHandler {
         .remove(any(), anyBoolean(), anyBoolean());
   }
 
+  @Test
+  void testPendingPipelineClose() throws IOException, InterruptedException {
+    final List<DatanodeDetails> datanodes = getDatanodes();
+    final DatanodeDetails currentDatanode = datanodes.get(0);
+    final PipelineID pipelineID = PipelineID.randomId();
+    final UUID pipelineUUID = pipelineID.getId();
+    final SCMCommand<ClosePipelineCommandProto> command1 = new ClosePipelineCommand(pipelineID);
+    final SCMCommand<ClosePipelineCommandProto> command2 = new ClosePipelineCommand(pipelineID);
+    StateContext stateContext = ContainerTestUtils.getMockContext(currentDatanode, conf);
+
+    final boolean shouldDeleteRatisLogDirectory = true;
+    XceiverServerRatis writeChannel = mock(XceiverServerRatis.class);
+    when(ozoneContainer.getWriteChannel()).thenReturn(writeChannel);
+    when(writeChannel.getShouldDeleteRatisLogDirectory()).thenReturn(shouldDeleteRatisLogDirectory);
+    when(writeChannel.isExist(pipelineID.getProtobuf())).thenReturn(true);
+    Collection<RaftPeer> raftPeers = datanodes.stream()
+        .map(RatisHelper::toRaftPeer)
+        .collect(Collectors.toList());
+    when(writeChannel.getServer()).thenReturn(mock(RaftServer.class));
+    when(writeChannel.getServer().getId()).thenReturn(RatisHelper.toRaftPeerId(currentDatanode));
+    when(writeChannel.getRaftPeersInPipeline(pipelineID)).thenReturn(raftPeers);
+
+    CountDownLatch firstCommandStarted = new CountDownLatch(1);
+    CountDownLatch secondCommandSubmitted = new CountDownLatch(1);
+
+    doAnswer(invocation -> {
+      firstCommandStarted.countDown();
+      secondCommandSubmitted.await();
+      return null;
+    }).when(writeChannel).removeGroup(pipelineID.getProtobuf());
+
+    ExecutorService singleThreadExecutor = Executors.newSingleThreadExecutor();
+
+    final ClosePipelineCommandHandler commandHandler =
+        new ClosePipelineCommandHandler((leader, tls) -> raftClient, singleThreadExecutor);
+    assertFalse(commandHandler.isPipelineCloseInProgress(pipelineUUID));
+    commandHandler.handle(command1, ozoneContainer, stateContext, connectionManager);
+    assertTrue(firstCommandStarted.await(5, TimeUnit.SECONDS));
+    commandHandler.handle(command2, ozoneContainer, stateContext, connectionManager);
+    secondCommandSubmitted.countDown();
+
+    singleThreadExecutor.shutdown();
+    assertTrue(singleThreadExecutor.awaitTermination(10, TimeUnit.SECONDS));
+    
+    // Only one command should have been processed due to duplicate prevention
+    assertEquals(1, commandHandler.getInvocationCount());
+    assertFalse(commandHandler.isPipelineCloseInProgress(pipelineUUID));
+  }
+
   private List<DatanodeDetails> getDatanodes() {
     final DatanodeDetails dnOne = MockDatanodeDetails.randomDatanodeDetails();
     final DatanodeDetails dnTwo = MockDatanodeDetails.randomDatanodeDetails();
