@@ -17,10 +17,9 @@
 
 package org.apache.hadoop.ozone.container.common.volume;
 
-import static org.apache.hadoop.hdds.HddsConfigKeys.HDDS_DATANODE_VOLUME_MIN_FREE_SPACE;
-import static org.apache.hadoop.hdds.HddsConfigKeys.HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_PERCENT;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.HDDS_DATANODE_DIR_DU_RESERVED_PERCENT;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.HDDS_DATANODE_DIR_DU_RESERVED_PERCENT_DEFAULT;
+import static org.apache.hadoop.ozone.container.common.statemachine.DatanodeConfiguration.HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_PERCENT;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotEquals;
 import static org.junit.jupiter.api.Assertions.assertThrows;
@@ -30,11 +29,11 @@ import java.nio.file.Files;
 import java.nio.file.Path;
 import java.util.UUID;
 import org.apache.hadoop.conf.StorageUnit;
-import org.apache.hadoop.hdds.HddsConfigKeys;
 import org.apache.hadoop.hdds.conf.ConfigurationException;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.fs.MockSpaceUsageCheckFactory;
 import org.apache.hadoop.hdds.scm.ScmConfigKeys;
+import org.apache.hadoop.ozone.container.common.statemachine.DatanodeConfiguration;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
@@ -69,11 +68,11 @@ public class TestReservedVolumeSpace {
     // Gets the total capacity reported by Ozone, which may be limited to less than the volume's real capacity by the
     // DU reserved configurations.
     long volumeCapacity = hddsVolume.getCurrentUsage().getCapacity();
-    VolumeUsage usage = hddsVolume.getVolumeInfo().get().getUsageForTesting();
+    VolumeUsage usage = hddsVolume.getVolumeUsage().get();
 
     // Gets the actual total capacity without accounting for DU reserved space configurations.
     long totalCapacity = usage.realUsage().getCapacity();
-    long reservedCapacity = usage.getReservedBytes();
+    long reservedCapacity = usage.getReservedInBytes();
 
     assertEquals(getExpectedDefaultReserved(hddsVolume), reservedCapacity);
     assertEquals(totalCapacity - reservedCapacity, volumeCapacity);
@@ -93,11 +92,11 @@ public class TestReservedVolumeSpace {
         HDDS_DATANODE_DIR_DU_RESERVED_PERCENT_DEFAULT);
 
     long volumeCapacity = hddsVolume.getCurrentUsage().getCapacity();
-    VolumeUsage usage = hddsVolume.getVolumeInfo().get().getUsageForTesting();
+    VolumeUsage usage = hddsVolume.getVolumeUsage().get();
 
     //Gets the actual total capacity
     long totalCapacity = usage.realUsage().getCapacity();
-    long reservedCapacity = usage.getReservedBytes();
+    long reservedCapacity = usage.getReservedInBytes();
     long reservedCalculated = (long) Math.ceil(totalCapacity * percentage);
 
     assertEquals(reservedCalculated, reservedCapacity);
@@ -117,7 +116,7 @@ public class TestReservedVolumeSpace {
         folder.toString() + ":500B");
     HddsVolume hddsVolume = volumeBuilder.conf(conf).build();
 
-    long reservedFromVolume = hddsVolume.getVolumeInfo().get()
+    long reservedFromVolume = hddsVolume.getVolumeUsage().get()
             .getReservedInBytes();
     assertEquals(500, reservedFromVolume);
   }
@@ -131,8 +130,8 @@ public class TestReservedVolumeSpace {
         temp.toString() + ":500B");
     HddsVolume hddsVolume = volumeBuilder.conf(conf).build();
 
-    VolumeUsage usage = hddsVolume.getVolumeInfo().get().getUsageForTesting();
-    long reservedFromVolume = usage.getReservedBytes();
+    VolumeUsage usage = hddsVolume.getVolumeUsage().get();
+    long reservedFromVolume = usage.getReservedInBytes();
     assertNotEquals(0, reservedFromVolume);
 
     long totalCapacity = usage.realUsage().getCapacity();
@@ -151,7 +150,7 @@ public class TestReservedVolumeSpace {
         folder.toString() + ":500C");
     HddsVolume hddsVolume1 = volumeBuilder.conf(conf1).build();
 
-    long reservedFromVolume1 = hddsVolume1.getVolumeInfo().get()
+    long reservedFromVolume1 = hddsVolume1.getVolumeUsage().get()
             .getReservedInBytes();
     assertEquals(getExpectedDefaultReserved(hddsVolume1), reservedFromVolume1);
 
@@ -161,7 +160,7 @@ public class TestReservedVolumeSpace {
     conf2.set(HDDS_DATANODE_DIR_DU_RESERVED_PERCENT, "20");
     HddsVolume hddsVolume2 = volumeBuilder.conf(conf2).build();
 
-    long reservedFromVolume2 = hddsVolume2.getVolumeInfo().get()
+    long reservedFromVolume2 = hddsVolume2.getVolumeUsage().get()
             .getReservedInBytes();
     assertEquals(getExpectedDefaultReserved(hddsVolume2), reservedFromVolume2);
   }
@@ -188,7 +187,7 @@ public class TestReservedVolumeSpace {
     conf.set(ScmConfigKeys.HDDS_DATANODE_DIR_DU_RESERVED, symlink + ":500B");
     HddsVolume hddsVolume = volumeBuilder.conf(conf).build();
 
-    long reservedFromVolume = hddsVolume.getVolumeInfo().get().getReservedInBytes();
+    long reservedFromVolume = hddsVolume.getVolumeUsage().get().getReservedInBytes();
     assertEquals(500, reservedFromVolume);
   }
 
@@ -196,26 +195,22 @@ public class TestReservedVolumeSpace {
   public void testMinFreeSpaceCalculator() throws Exception {
     OzoneConfiguration conf = new OzoneConfiguration();
     double minSpace = 100.0;
-    conf.setStorageSize(HddsConfigKeys.HDDS_DATANODE_VOLUME_MIN_FREE_SPACE,
+    conf.setStorageSize(DatanodeConfiguration.HDDS_DATANODE_VOLUME_MIN_FREE_SPACE,
         minSpace, StorageUnit.BYTES);
-    VolumeUsage.MinFreeSpaceCalculator calc = new VolumeUsage.MinFreeSpaceCalculator(conf);
     long capacity = 1000;
-    assertEquals(minSpace, calc.get(capacity));
+    assertEquals(minSpace, conf.getObject(DatanodeConfiguration.class).getMinFreeSpace(capacity));
 
     conf.setFloat(HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_PERCENT, 0.01f);
-    calc = new VolumeUsage.MinFreeSpaceCalculator(conf);
-    // default is 5GB
-    assertEquals(5L * 1024 * 1024 * 1024, calc.get(capacity));
+    // When both are set, max(minSpace, %cent), minSpace will be used
+    assertEquals(minSpace, conf.getObject(DatanodeConfiguration.class).getMinFreeSpace(capacity));
 
-    // capacity * 1% = 10
-    conf.unset(HDDS_DATANODE_VOLUME_MIN_FREE_SPACE);
-    calc = new VolumeUsage.MinFreeSpaceCalculator(conf);
-    assertEquals(10, calc.get(capacity));
+    conf.setFloat(HDDS_DATANODE_VOLUME_MIN_FREE_SPACE_PERCENT, 1f);
+    // When both are set, max(minSpace, %cent), hence %cent will be used
+    assertEquals(1000, conf.getObject(DatanodeConfiguration.class).getMinFreeSpace(capacity));
   }
 
-
   private long getExpectedDefaultReserved(HddsVolume volume) {
-    long totalCapacity = volume.getVolumeInfo().get().getUsageForTesting().realUsage().getCapacity();
+    long totalCapacity = volume.getVolumeUsage().get().realUsage().getCapacity();
     return (long) Math.ceil(totalCapacity * HDDS_DATANODE_DIR_DU_RESERVED_PERCENT_DEFAULT);
   }
 }
