@@ -34,6 +34,7 @@ import java.io.File;
 import java.nio.file.Path;
 import java.time.Duration;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.List;
 import java.util.Optional;
 import java.util.Set;
@@ -58,8 +59,7 @@ import org.apache.hadoop.ozone.container.ozoneimpl.OzoneContainer;
 import org.apache.hadoop.util.DiskChecker.DiskErrorException;
 import org.apache.hadoop.util.FakeTimer;
 import org.apache.ozone.test.GenericTestUtils;
-import org.junit.jupiter.api.TestInfo;
-import org.junit.jupiter.api.Timeout;
+import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.Arguments;
@@ -70,9 +70,8 @@ import org.slf4j.LoggerFactory;
 /**
  * Tests for {@link StorageVolumeChecker}.
  */
-@Timeout(300)
 public class TestStorageVolumeChecker {
-  public static final Logger LOG = LoggerFactory.getLogger(
+  private static final Logger LOG = LoggerFactory.getLogger(
       TestStorageVolumeChecker.class);
 
   private static final int NUM_VOLUMES = 2;
@@ -87,12 +86,8 @@ public class TestStorageVolumeChecker {
    */
   private VolumeCheckResult expectedVolumeHealth;
 
-  private ContainerLayoutVersion layoutVersion;
-
-  private void initTest(VolumeCheckResult result,
-      ContainerLayoutVersion layout) {
+  private void initTest(VolumeCheckResult result) {
     this.expectedVolumeHealth = result;
-    this.layoutVersion = layout;
     setup();
   }
 
@@ -106,13 +101,19 @@ public class TestStorageVolumeChecker {
    * Run each test case for each possible value of {@link VolumeCheckResult}.
    * Including "null" for 'throw exception'.
    */
+  private static List<VolumeCheckResult> volumeCheckResults() {
+    List<VolumeCheckResult> list = new ArrayList<>(Arrays.asList(VolumeCheckResult.values()));
+    list.add(null);
+    return list;
+  }
+
   private static List<Arguments> provideTestData() {
+    List<VolumeCheckResult> volumeCheckResults = volumeCheckResults();
     List<Arguments> values = new ArrayList<>();
     for (ContainerLayoutVersion layout : ContainerLayoutVersion.values()) {
-      for (VolumeCheckResult result : VolumeCheckResult.values()) {
+      for (VolumeCheckResult result : volumeCheckResults) {
         values.add(Arguments.of(result, layout));
       }
-      values.add(Arguments.of(null, layout));
     }
     return values;
   }
@@ -121,23 +122,18 @@ public class TestStorageVolumeChecker {
   /**
    * Test {@link StorageVolumeChecker#checkVolume} propagates the
    * check to the delegate checker.
-   *
-   * @throws Exception
    */
   @ParameterizedTest
-  @MethodSource("provideTestData")
-  public void testCheckOneVolume(
-      VolumeCheckResult checkResult, ContainerLayoutVersion layout,
-      TestInfo testInfo) throws Exception {
-    initTest(checkResult, layout);
-    LOG.info("Executing {}", testInfo.getTestMethod());
+  @MethodSource("volumeCheckResults")
+  public void testCheckOneVolume(VolumeCheckResult checkResult) throws Exception {
+    initTest(checkResult);
     final HddsVolume volume = makeVolumes(1, expectedVolumeHealth).get(0);
     final StorageVolumeChecker checker =
         new StorageVolumeChecker(new OzoneConfiguration(), new FakeTimer(), "");
     checker.setDelegateChecker(new DummyChecker());
     final AtomicLong numCallbackInvocations = new AtomicLong(0);
 
-    /**
+    /*
      * Request a check and ensure it triggered {@link HddsVolume#check}.
      */
     boolean result =
@@ -167,15 +163,11 @@ public class TestStorageVolumeChecker {
   /**
    * Test {@link StorageVolumeChecker#checkAllVolumes} propagates
    * checks for all volumes to the delegate checker.
-   *
-   * @throws Exception
    */
   @ParameterizedTest
-  @MethodSource("provideTestData")
-  public void testCheckAllVolumes(VolumeCheckResult checkResult,
-      ContainerLayoutVersion layout, TestInfo testInfo) throws Exception {
-    initTest(checkResult, layout);
-    LOG.info("Executing {}", testInfo.getTestMethod());
+  @MethodSource("volumeCheckResults")
+  public void testCheckAllVolumes(VolumeCheckResult checkResult) throws Exception {
+    initTest(checkResult);
 
     final List<HddsVolume> volumes = makeVolumes(
         NUM_VOLUMES, expectedVolumeHealth);
@@ -205,15 +197,12 @@ public class TestStorageVolumeChecker {
   /**
    * Test {@link StorageVolumeChecker#checkAllVolumes} propagates
    * checks for all volumes to the delegate checker.
-   *
-   * @throws Exception
    */
   @ParameterizedTest
   @MethodSource("provideTestData")
   public void testVolumeDeletion(VolumeCheckResult checkResult,
-      ContainerLayoutVersion layout, TestInfo testInfo) throws Exception {
-    initTest(checkResult, layout);
-    LOG.info("Executing {}", testInfo.getTestMethod());
+      ContainerLayoutVersion layout) throws Exception {
+    initTest(checkResult);
 
     DatanodeConfiguration dnConf =
         conf.getObject(DatanodeConfiguration.class);
@@ -264,6 +253,52 @@ public class TestStorageVolumeChecker {
     ozoneContainer.stop();
   }
 
+  /**
+   * Test numScansSkipped metric from VolumeInfoMetrics when volume check is skipped.
+   */
+  @Test
+  public void testNumScansSkipped() throws Exception {
+    initTest(VolumeCheckResult.HEALTHY);
+
+    final List<HddsVolume> volumes = makeVolumes(3, expectedVolumeHealth);
+
+    FakeTimer timer = new FakeTimer();
+    final StorageVolumeChecker checker =
+        new StorageVolumeChecker(new OzoneConfiguration(), timer, "");
+
+    VolumeInfoMetrics metrics1 = new VolumeInfoMetrics("test-volume-1", volumes.get(0));
+    VolumeInfoMetrics metrics2 = new VolumeInfoMetrics("test-volume-2", volumes.get(1));
+    VolumeInfoMetrics metrics3 = new VolumeInfoMetrics("test-volume-3", volumes.get(2));
+    when(volumes.get(0).getVolumeInfoStats()).thenReturn(metrics1);
+    when(volumes.get(1).getVolumeInfoStats()).thenReturn(metrics2);
+    when(volumes.get(2).getVolumeInfoStats()).thenReturn(metrics3);
+
+    checker.checkAllVolumes(volumes);
+
+    // No volume skipped on first call
+    assertEquals(0, metrics1.getNumScansSkipped());
+    assertEquals(0, metrics2.getNumScansSkipped());
+    assertEquals(0, metrics3.getNumScansSkipped());
+
+    // Second call skipped because msSinceLastCheck (8 mins) < minMsBetweenChecks (10 mins)
+    timer.advance(480_000);
+    checker.checkAllVolumes(volumes);
+
+    assertEquals(1, metrics1.getNumScansSkipped());
+    assertEquals(1, metrics2.getNumScansSkipped());
+    assertEquals(1, metrics3.getNumScansSkipped());
+
+    // Third call should not skip volumes as msSinceLastCheck (11 mins) > minMsBetweenChecks (10 mins)
+    timer.advance(180_000);
+    checker.checkAllVolumes(volumes);
+
+    assertEquals(1, metrics1.getNumScansSkipped());
+    assertEquals(1, metrics2.getNumScansSkipped());
+    assertEquals(1, metrics3.getNumScansSkipped());
+
+    checker.shutdownAndWait(0, TimeUnit.SECONDS);
+  }
+
   /**
    * A checker to wraps the result of {@link HddsVolume#check} in
    * an ImmediateFuture.
@@ -280,7 +315,7 @@ public class TestStorageVolumeChecker {
         return Optional.of(
             Futures.immediateFuture(target.check(context)));
       } catch (Exception e) {
-        LOG.info("check routine threw exception {}", e);
+        LOG.info("check routine threw exception", e);
         return Optional.of(Futures.immediateFailedFuture(e));
       }
     }
