@@ -1,13 +1,12 @@
-/**
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License.  You may obtain a copy of the License at
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -15,21 +14,23 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 package org.apache.hadoop.ozone.container.common.volume;
 
-import org.apache.hadoop.hdds.server.http.PrometheusMetricsSink;
-import org.apache.hadoop.metrics2.MetricsSystem;
-import org.apache.hadoop.metrics2.lib.DefaultMetricsSystem;
-import org.junit.jupiter.api.AfterEach;
-import org.junit.jupiter.api.Assertions;
-import org.junit.jupiter.api.BeforeEach;
-import org.junit.jupiter.api.Test;
+import static java.nio.charset.StandardCharsets.UTF_8;
+import static org.apache.hadoop.hdds.HddsConfigKeys.OZONE_DATANODE_IO_METRICS_PERCENTILES_INTERVALS_SECONDS_KEY;
+import static org.assertj.core.api.Assertions.assertThat;
 
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.OutputStreamWriter;
-
-import static java.nio.charset.StandardCharsets.UTF_8;
+import org.apache.hadoop.hdds.conf.OzoneConfiguration;
+import org.apache.hadoop.hdds.server.http.PrometheusMetricsSink;
+import org.apache.hadoop.metrics2.MetricsSystem;
+import org.apache.hadoop.metrics2.lib.DefaultMetricsSystem;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 /**
  * Test PrometheusMetricSink regarding VolumeIOStats.
@@ -54,28 +55,25 @@ public class TestVolumeIOStatsWithPrometheusSink {
 
   @Test
   public void testMultipleVolumeIOMetricsExist() throws IOException {
+    OzoneConfiguration conf = new OzoneConfiguration();
+    int[] intervals = conf.getInts(OZONE_DATANODE_IO_METRICS_PERCENTILES_INTERVALS_SECONDS_KEY);
+
     //GIVEN
     VolumeIOStats volumeIOStats1 = new VolumeIOStats("VolumeIOStat1",
-        "vol1/dir");
+        "vol1/dir", intervals);
     VolumeIOStats volumeIOStat2 = new VolumeIOStats("VolumeIOStat2",
-        "vol2/dir");
+        "vol2/dir", intervals);
 
     //WHEN
     String writtenMetrics = publishMetricsAndGetOutput();
 
     //THEN
-    Assertions.assertTrue(
-        writtenMetrics.contains("storagedirectory=\"" +
-            volumeIOStats1.getStorageDirectory() + "\""),
-        "The expected metric line is missing from prometheus" +
-            " metrics output"
-    );
-    Assertions.assertTrue(
-        writtenMetrics.contains("storagedirectory=\"" +
-            volumeIOStat2.getStorageDirectory() + "\""),
-        "The expected metric line is missing from prometheus" +
-            " metrics output"
-    );
+    assertThat(writtenMetrics)
+        .withFailMessage("The expected metric line is missing from prometheus metrics output")
+        .contains("storagedirectory=\"" + volumeIOStats1.getStorageDirectory() + "\"");
+    assertThat(writtenMetrics)
+        .withFailMessage("The expected metric line is missing from prometheus metrics output")
+        .contains("storagedirectory=\"" + volumeIOStat2.getStorageDirectory() + "\"");
   }
 
   private String publishMetricsAndGetOutput() throws IOException {
