@@ -49,10 +49,12 @@ import org.apache.ozone.test.GenericTestUtils;
 /**
  * Tests for {@link KeyValueContainerMetadataInspector}.
  */
-public class TestKeyValueContainerMetadataInspector
-    extends TestKeyValueContainerIntegrityChecks {
+public class TestKeyValueContainerMetadataInspector extends TestKeyValueContainerIntegrityChecks {
   private static final long CONTAINER_ID = 102;
 
+  static final DeletedBlocksTransactionGeneratorForTesting GENERATOR =
+      new DeletedBlocksTransactionGeneratorForTesting();
+
   @ContainerTestVersionInfo.ContainerTest
   public void testRunDisabled(ContainerTestVersionInfo versionInfo)
       throws Exception {
@@ -193,9 +195,6 @@ public class TestKeyValueContainerMetadataInspector
     }
   }
 
-  static final DeletedBlocksTransactionGeneratorForTesting GENERATOR
-      = new DeletedBlocksTransactionGeneratorForTesting();
-
   @ContainerTestVersionInfo.ContainerTest
   public void testCorrectDeleteWithTransaction(
       ContainerTestVersionInfo versionInfo) throws Exception {
@@ -413,7 +412,6 @@ public class TestKeyValueContainerMetadataInspector
     assertTrue(matchFound);
   }
 
-
   public void setDBBlockAndByteCounts(KeyValueContainerData containerData,
       long blockCount, long byteCount) throws Exception {
     setDB(containerData, blockCount, byteCount,
