@@ -17,11 +17,13 @@
 
 package org.apache.hadoop.ozone.container.keyvalue.impl;
 
+import static org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerDataProto.State.CLOSED;
 import static org.apache.hadoop.ozone.OzoneConsts.INCREMENTAL_CHUNK_LIST;
 import static org.apache.hadoop.ozone.container.keyvalue.helpers.KeyValueContainerUtil.isSameSchemaVersion;
 import static org.apache.hadoop.ozone.container.keyvalue.impl.BlockManagerImpl.FULL_CHUNK;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.mockito.ArgumentMatchers.anyList;
 import static org.mockito.ArgumentMatchers.anyLong;
 import static org.mockito.Mockito.mock;
@@ -36,10 +38,12 @@ import org.apache.hadoop.conf.StorageUnit;
 import org.apache.hadoop.hdds.client.BlockID;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos;
+import org.apache.hadoop.hdds.scm.container.common.helpers.StorageContainerException;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.container.common.helpers.BlockData;
 import org.apache.hadoop.ozone.container.common.helpers.ChunkInfo;
 import org.apache.hadoop.ozone.container.common.impl.ContainerLayoutVersion;
+import org.apache.hadoop.ozone.container.common.interfaces.DBHandle;
 import org.apache.hadoop.ozone.container.common.utils.StorageVolumeUtil;
 import org.apache.hadoop.ozone.container.common.volume.HddsVolume;
 import org.apache.hadoop.ozone.container.common.volume.MutableVolumeSet;
@@ -62,15 +66,10 @@ public class TestBlockManagerImpl {
   private Path folder;
   private OzoneConfiguration config;
   private String scmId = UUID.randomUUID().toString();
-  private VolumeSet volumeSet;
-  private RoundRobinVolumeChoosingPolicy volumeChoosingPolicy;
-  private KeyValueContainerData keyValueContainerData;
   private KeyValueContainer keyValueContainer;
   private BlockData blockData;
   private BlockData blockData1;
   private BlockManagerImpl blockManager;
-  private BlockID blockID;
-  private BlockID blockID1;
 
   private ContainerLayoutVersion layout;
   private String schemaVersion;
@@ -81,10 +80,10 @@ public class TestBlockManagerImpl {
     this.schemaVersion = versionInfo.getSchemaVersion();
     this.config = new OzoneConfiguration();
     ContainerTestVersionInfo.setTestSchemaVersion(schemaVersion, config);
-    initilaze();
+    initialize();
   }
 
-  private void initilaze() throws Exception {
+  private void initialize() throws Exception {
     UUID datanodeId = UUID.randomUUID();
     HddsVolume hddsVolume = new HddsVolume.Builder(folder.toString())
         .conf(config)
@@ -93,13 +92,13 @@ public class TestBlockManagerImpl {
     StorageVolumeUtil.checkVolume(hddsVolume, scmId, scmId, config,
         null, null);
 
-    volumeSet = mock(MutableVolumeSet.class);
+    VolumeSet volumeSet = mock(MutableVolumeSet.class);
 
-    volumeChoosingPolicy = mock(RoundRobinVolumeChoosingPolicy.class);
+    RoundRobinVolumeChoosingPolicy volumeChoosingPolicy = mock(RoundRobinVolumeChoosingPolicy.class);
     when(volumeChoosingPolicy.chooseVolume(anyList(), anyLong()))
         .thenReturn(hddsVolume);
 
-    keyValueContainerData = new KeyValueContainerData(1L,
+    KeyValueContainerData keyValueContainerData = new KeyValueContainerData(1L,
         layout,
         (long) StorageUnit.GB.toBytes(5), UUID.randomUUID().toString(),
         datanodeId.toString());
@@ -110,7 +109,7 @@ public class TestBlockManagerImpl {
     keyValueContainer.create(volumeSet, volumeChoosingPolicy, scmId);
 
     // Creating BlockData
-    blockID = new BlockID(1L, 1L);
+    BlockID blockID = new BlockID(1L, 1L);
     blockData = new BlockData(blockID);
     blockData.addMetadata(OzoneConsts.VOLUME, OzoneConsts.OZONE);
     blockData.addMetadata(OzoneConsts.OWNER,
@@ -122,13 +121,13 @@ public class TestBlockManagerImpl {
     blockData.setChunks(chunkList);
 
     // Creating BlockData
-    blockID1 = new BlockID(1L, 2L);
-    blockData1 = new BlockData(blockID1);
+    blockID = new BlockID(1L, 2L);
+    blockData1 = new BlockData(blockID);
     blockData1.addMetadata(OzoneConsts.VOLUME, OzoneConsts.OZONE);
     blockData1.addMetadata(OzoneConsts.OWNER,
         OzoneConsts.OZONE_SIMPLE_HDFS_USER);
     List<ContainerProtos.ChunkInfo> chunkList1 = new ArrayList<>();
-    ChunkInfo info1 = new ChunkInfo(String.format("%d.data.%d", blockID1
+    ChunkInfo info1 = new ChunkInfo(String.format("%d.data.%d", blockID
         .getLocalID(), 0), 0, 1024);
     chunkList1.add(info1.getProtoBufMessage());
     blockData1.setChunks(chunkList1);
@@ -196,6 +195,73 @@ public class TestBlockManagerImpl {
 
   }
 
+  @ContainerTestVersionInfo.ContainerTest
+  public void testPutBlockForClosed(ContainerTestVersionInfo versionInfo)
+      throws Exception {
+    initTest(versionInfo);
+    KeyValueContainerData containerData = keyValueContainer.getContainerData();
+    assertEquals(0, containerData.getBlockCount());
+    keyValueContainer.close();
+    assertEquals(CLOSED, keyValueContainer.getContainerState());
+
+    try (DBHandle db = BlockUtils.getDB(containerData, config)) {
+      // 1. Put Block with bcsId = 1, Overwrite BCS ID = true
+      blockData1 = createBlockData(1L, 2L, 1, 0, 2048, 1);
+      blockManager.putBlockForClosedContainer(keyValueContainer, blockData1, true);
+
+      BlockData fromGetBlockData;
+      //Check Container's bcsId
+      fromGetBlockData = blockManager.getBlock(keyValueContainer, blockData.getBlockID());
+      assertEquals(1, containerData.getBlockCount());
+      assertEquals(1, containerData.getBlockCommitSequenceId());
+      assertEquals(1, fromGetBlockData.getBlockCommitSequenceId());
+      assertEquals(1, db.getStore().getMetadataTable().get(containerData.getBcsIdKey()));
+      assertEquals(1, db.getStore().getMetadataTable().get(containerData.getBlockCountKey()));
+
+      // 2. Put Block with bcsId = 2, Overwrite BCS ID = false
+      BlockData blockData2 = createBlockData(1L, 3L, 1, 0, 2048, 2);
+      blockManager.putBlockForClosedContainer(keyValueContainer, blockData2, false);
+
+      // The block should be written, but we won't be able to read it, As expected BcsId < container's BcsId
+      // fails during block read.
+      assertThrows(StorageContainerException.class, () -> blockManager
+          .getBlock(keyValueContainer, blockData2.getBlockID()));
+      fromGetBlockData = db.getStore().getBlockDataTable().get(containerData.getBlockKey(blockData2.getLocalID()));
+      assertEquals(2, containerData.getBlockCount());
+      // BcsId should still be 1, as the BcsId is not overwritten
+      assertEquals(1, containerData.getBlockCommitSequenceId());
+      assertEquals(2, fromGetBlockData.getBlockCommitSequenceId());
+      assertEquals(2, db.getStore().getMetadataTable().get(containerData.getBlockCountKey()));
+      assertEquals(1, db.getStore().getMetadataTable().get(containerData.getBcsIdKey()));
+
+      // 3. Put Block with bcsId = 2, Overwrite BCS ID = true
+      // This should succeed as we are overwriting the BcsId, The container BcsId should be updated to 2
+      // The block count should not change.
+      blockManager.putBlockForClosedContainer(keyValueContainer, blockData2, true);
+      fromGetBlockData = blockManager.getBlock(keyValueContainer, blockData2.getBlockID());
+      assertEquals(2, containerData.getBlockCount());
+      assertEquals(2, containerData.getBlockCommitSequenceId());
+      assertEquals(2, fromGetBlockData.getBlockCommitSequenceId());
+      assertEquals(2, db.getStore().getMetadataTable().get(containerData.getBlockCountKey()));
+      assertEquals(2, db.getStore().getMetadataTable().get(containerData.getBcsIdKey()));
+
+      // 4. Put Block with bcsId = 1 < container bcsId, Overwrite BCS ID = true
+      // We are overwriting an existing block with lower bcsId than container bcsId. Container bcsId should not change
+      BlockData blockData3 = createBlockData(1L, 1L, 1, 0, 2048, 1);
+      blockManager.putBlockForClosedContainer(keyValueContainer, blockData3, true);
+      fromGetBlockData = blockManager.getBlock(keyValueContainer, blockData3.getBlockID());
+      assertEquals(3, containerData.getBlockCount());
+      assertEquals(2, containerData.getBlockCommitSequenceId());
+      assertEquals(1, fromGetBlockData.getBlockCommitSequenceId());
+      assertEquals(3, db.getStore().getMetadataTable().get(containerData.getBlockCountKey()));
+      assertEquals(2, db.getStore().getMetadataTable().get(containerData.getBcsIdKey()));
+
+      // writeChunk updates the in-memory state of the used bytes, putBlock persists the in-memory state to the DB.
+      // We are only doing putBlock without writeChunk, the used bytes should be 0.
+      assertEquals(0, db.getStore().getMetadataTable().get(containerData.getBytesUsedKey()));
+    }
+  }
+
   @ContainerTestVersionInfo.ContainerTest
   public void testListBlock(ContainerTestVersionInfo versionInfo)
       throws Exception {
@@ -207,7 +273,7 @@ public class TestBlockManagerImpl {
     assertEquals(1, listBlockData.size());
 
     for (long i = 2; i <= 10; i++) {
-      blockID = new BlockID(1L, i);
+      BlockID blockID = new BlockID(1L, i);
       blockData = new BlockData(blockID);
       blockData.addMetadata(OzoneConsts.VOLUME, OzoneConsts.OZONE);
       blockData.addMetadata(OzoneConsts.OWNER,
@@ -229,10 +295,10 @@ public class TestBlockManagerImpl {
   private BlockData createBlockData(long containerID, long blockNo,
       int chunkID, long offset, long len, long bcsID)
       throws IOException {
-    blockID1 = new BlockID(containerID, blockNo);
-    blockData = new BlockData(blockID1);
+    BlockID blockID = new BlockID(containerID, blockNo);
+    blockData = new BlockData(blockID);
     List<ContainerProtos.ChunkInfo> chunkList1 = new ArrayList<>();
-    ChunkInfo info1 = new ChunkInfo(String.format("%d_chunk_%d", blockID1
+    ChunkInfo info1 = new ChunkInfo(String.format("%d_chunk_%d", blockID
         .getLocalID(), chunkID), offset, len);
     chunkList1.add(info1.getProtoBufMessage());
     blockData.setChunks(chunkList1);
@@ -245,14 +311,14 @@ public class TestBlockManagerImpl {
   private BlockData createBlockDataWithOneFullChunk(long containerID,
       long blockNo, int chunkID, long offset, long len, long bcsID)
       throws IOException {
-    blockID1 = new BlockID(containerID, blockNo);
-    blockData = new BlockData(blockID1);
+    BlockID blockID = new BlockID(containerID, blockNo);
+    blockData = new BlockData(blockID);
     List<ContainerProtos.ChunkInfo> chunkList1 = new ArrayList<>();
-    ChunkInfo info1 = new ChunkInfo(String.format("%d_chunk_%d", blockID1
+    ChunkInfo info1 = new ChunkInfo(String.format("%d_chunk_%d", blockID
         .getLocalID(), 1), 0, 4 * 1024 * 1024);
     info1.addMetadata(FULL_CHUNK, "");
 
-    ChunkInfo info2 = new ChunkInfo(String.format("%d_chunk_%d", blockID1
+    ChunkInfo info2 = new ChunkInfo(String.format("%d_chunk_%d", blockID
         .getLocalID(), chunkID), offset, len);
     chunkList1.add(info1.getProtoBufMessage());
     chunkList1.add(info2.getProtoBufMessage());
@@ -265,13 +331,13 @@ public class TestBlockManagerImpl {
 
   private BlockData createBlockDataWithThreeFullChunks(long containerID,
       long blockNo, long bcsID) throws IOException {
-    blockID1 = new BlockID(containerID, blockNo);
-    blockData = new BlockData(blockID1);
+    BlockID blockID = new BlockID(containerID, blockNo);
+    blockData = new BlockData(blockID);
     List<ContainerProtos.ChunkInfo> chunkList1 = new ArrayList<>();
     long chunkLimit = 4 * 1024 * 1024;
     for (int i = 1; i < 4; i++) {
       ChunkInfo info1 = new ChunkInfo(
-          String.format("%d_chunk_%d", blockID1.getLocalID(), i),
+          String.format("%d_chunk_%d", blockID.getLocalID(), i),
           chunkLimit * i, chunkLimit);
       info1.addMetadata(FULL_CHUNK, "");
       chunkList1.add(info1.getProtoBufMessage());
