@@ -46,10 +46,6 @@ import org.apache.hadoop.util.StopWatch;
  */
 public final class RawErasureCoderBenchmark {
 
-  private RawErasureCoderBenchmark() {
-    // prevent instantiation
-  }
-
   // target size of input data buffer
   private static final int TARGET_BUFFER_SIZE_MB = 126;
 
@@ -61,6 +57,10 @@ public final class RawErasureCoderBenchmark {
           Arrays.asList(new DummyRawErasureCoderFactory(),
               new RSRawErasureCoderFactory()));
 
+  private RawErasureCoderBenchmark() {
+    // prevent instantiation
+  }
+
   enum CODER {
     DUMMY_CODER("Dummy coder"),
     RS_CODER("Reed-Solomon Java coder");
@@ -85,7 +85,7 @@ public final class RawErasureCoderBenchmark {
     StringBuilder sb = new StringBuilder(
         "Available coders with coderIndex:\n");
     for (CODER coder : CODER.values()) {
-      sb.append(coder.ordinal()).append(":").append(coder).append("\n");
+      sb.append(coder.ordinal()).append(':').append(coder).append('\n');
     }
     System.out.println(sb);
   }
