@@ -85,8 +85,6 @@ public abstract class BaseHttpServer {
 
   private boolean prometheusSupport;
 
-  private boolean profilerSupport;
-
   public BaseHttpServer(MutableConfigurationSource conf, String name)
       throws IOException {
     this.name = name;
@@ -152,8 +150,8 @@ public abstract class BaseHttpServer {
       prometheusSupport = addDefaultApps &&
           conf.getBoolean(HddsConfigKeys.HDDS_PROMETHEUS_ENABLED, true);
 
-      profilerSupport = addDefaultApps &&
-          conf.getBoolean(HddsConfigKeys.HDDS_PROFILER_ENABLED, false);
+      boolean profilerSupport = addDefaultApps &&
+                                    conf.getBoolean(HddsConfigKeys.HDDS_PROFILER_ENABLED, false);
 
       if (prometheusSupport) {
         prometheusMetricsSink = new PrometheusMetricsSink(name);
@@ -244,7 +242,6 @@ public abstract class BaseHttpServer {
     return builder;
   }
 
-
   /**
    * Add a servlet to BaseHttpServer.
    *
@@ -355,20 +352,19 @@ public abstract class BaseHttpServer {
     int connIdx = 0;
     if (policy.isHttpEnabled()) {
       httpAddress = httpServer.getConnectorAddress(connIdx++);
-      String realAddress = NetUtils.getHostPortString(httpAddress);
+      String realAddress = NetUtils.getHostPortString(NetUtils.getConnectAddress(httpAddress));
       conf.set(getHttpAddressKey(), realAddress);
       LOG.info("HTTP server of {} listening at http://{}", name, realAddress);
     }
 
     if (policy.isHttpsEnabled()) {
       httpsAddress = httpServer.getConnectorAddress(connIdx);
-      String realAddress = NetUtils.getHostPortString(httpsAddress);
+      String realAddress = NetUtils.getHostPortString(NetUtils.getConnectAddress(httpsAddress));
       conf.set(getHttpsAddressKey(), realAddress);
       LOG.info("HTTPS server of {} listening at https://{}", name, realAddress);
     }
   }
 
-
   public static HttpServer2.Builder loadSslConfToHttpServerBuilder(
       HttpServer2.Builder builder, ConfigurationSource sslConf) {
     return builder
@@ -412,6 +408,7 @@ public abstract class BaseHttpServer {
     }
     return password;
   }
+
   /**
    * Load HTTPS-related configuration.
    */
