@@ -31,28 +31,8 @@ import org.eclipse.jetty.util.thread.QueuedThreadPool;
  */
 @InterfaceAudience.Private
 public final class HttpServer2Metrics implements MetricsSource {
-  enum HttpServer2MetricsInfo implements MetricsInfo {
-    SERVER_NAME("HttpServer2 Metrics."),
-    HttpServerThreadCount("Number of threads in the pool."),
-    HttpServerIdleThreadCount("Number of idle threads but not reserved."),
-    HttpServerMaxThreadCount("Maximum number of threads in the pool."),
-    HttpServerThreadQueueWaitingTaskCount(
-        "The number of jobs in the queue waiting for a thread");
 
-    private final String desc;
-
-    HttpServer2MetricsInfo(String desc) {
-      this.desc = desc;
-    }
-
-    @Override
-    public String description() {
-      return desc;
-    }
-  }
-
-  public static final String SOURCE_NAME =
-      HttpServer2Metrics.class.getSimpleName();
+  public static final String SOURCE_NAME = HttpServer2Metrics.class.getSimpleName();
 
   public static final String NAME = HttpServer2Metrics.class.getSimpleName();
 
@@ -91,4 +71,24 @@ public final class HttpServer2Metrics implements MetricsSource {
     MetricsSystem ms = DefaultMetricsSystem.instance();
     ms.unregisterSource(NAME);
   }
+
+  enum HttpServer2MetricsInfo implements MetricsInfo {
+    SERVER_NAME("HttpServer2 Metrics."),
+    HttpServerThreadCount("Number of threads in the pool."),
+    HttpServerIdleThreadCount("Number of idle threads but not reserved."),
+    HttpServerMaxThreadCount("Maximum number of threads in the pool."),
+    HttpServerThreadQueueWaitingTaskCount(
+        "The number of jobs in the queue waiting for a thread");
+
+    private final String desc;
+
+    HttpServer2MetricsInfo(String desc) {
+      this.desc = desc;
+    }
+
+    @Override
+    public String description() {
+      return desc;
+    }
+  }
 }
