@@ -17,12 +17,15 @@
 
 package org.apache.hadoop.hdds.utils.db;
 
-import java.io.IOException;
+import static org.apache.ratis.util.JavaUtils.getClassSimpleName;
+
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import org.apache.hadoop.hdds.utils.CollectionUtils;
+import org.apache.hadoop.hdds.utils.db.cache.TableCache.CacheType;
 import org.apache.hadoop.hdds.utils.db.managed.ManagedColumnFamilyOptions;
 
 /**
@@ -32,6 +35,23 @@ import org.apache.hadoop.hdds.utils.db.managed.ManagedColumnFamilyOptions;
  * @param <VALUE> they type of the value.
  */
 public class DBColumnFamilyDefinition<KEY, VALUE> {
+
+  private final String tableName;
+  private final Codec<KEY> keyCodec;
+  private final Codec<VALUE> valueCodec;
+  private final String name;
+
+  private volatile ManagedColumnFamilyOptions cfOptions;
+
+  public DBColumnFamilyDefinition(String tableName, Codec<KEY> keyCodec, Codec<VALUE> valueCodec) {
+    this.tableName = Objects.requireNonNull(tableName, "tableName == null");
+    this.keyCodec = Objects.requireNonNull(keyCodec, "keyCodec == null");
+    this.valueCodec = Objects.requireNonNull(valueCodec, "valueCodec == null");
+    this.name = tableName + "-def: " + getClassSimpleName(getKeyType())
+        + " -> " + getClassSimpleName(getValueType());
+    this.cfOptions = null;
+  }
+
   public static Map<String, DBColumnFamilyDefinition<?, ?>> newUnmodifiableMap(
       DBColumnFamilyDefinition<?, ?>... families) {
     return newUnmodifiableMap(Collections.emptyMap(), families);
@@ -44,29 +64,20 @@ public class DBColumnFamilyDefinition<KEY, VALUE> {
         DBColumnFamilyDefinition::getName, existing);
   }
 
-  public static Map<String, List<DBColumnFamilyDefinition<?, ?>>>
-      newUnmodifiableMultiMap(DBColumnFamilyDefinition<?, ?>... families) {
+  public static Map<String, List<DBColumnFamilyDefinition<?, ?>>> newUnmodifiableMultiMap(
+      DBColumnFamilyDefinition<?, ?>... families
+  ) {
     return CollectionUtils.newUnmodifiableMultiMap(Arrays.asList(families),
         DBColumnFamilyDefinition::getName);
   }
 
-  private final String tableName;
-
-  private final Codec<KEY> keyCodec;
-
-  private final Codec<VALUE> valueCodec;
-
-  private volatile ManagedColumnFamilyOptions cfOptions;
-
-  public DBColumnFamilyDefinition(String tableName, Codec<KEY> keyCodec, Codec<VALUE> valueCodec) {
-    this.tableName = tableName;
-    this.keyCodec = keyCodec;
-    this.valueCodec = valueCodec;
-    this.cfOptions = null;
+  public TypedTable<KEY, VALUE> getTable(DBStore db) throws RocksDatabaseException, CodecException {
+    return db.getTable(tableName, keyCodec, valueCodec);
   }
 
-  public Table<KEY, VALUE> getTable(DBStore db) throws IOException {
-    return db.getTable(tableName, getKeyType(), getValueType());
+  public TypedTable<KEY, VALUE> getTable(DBStore db, CacheType cacheType)
+      throws RocksDatabaseException, CodecException {
+    return db.getTable(tableName, keyCodec, valueCodec, cacheType);
   }
 
   public String getName() {
@@ -96,4 +107,9 @@ public class DBColumnFamilyDefinition<KEY, VALUE> {
   public void setCfOptions(ManagedColumnFamilyOptions cfOptions) {
     this.cfOptions = cfOptions;
   }
+
+  @Override
+  public String toString() {
+    return name;
+  }
 }
