@@ -29,21 +29,17 @@ import static org.apache.hadoop.ozone.OzoneConfigKeys.OZONE_OM_DELTA_UPDATE_DATA
 import static org.rocksdb.RocksDB.DEFAULT_COLUMN_FAMILY;
 
 import com.google.common.base.Preconditions;
-import com.google.protobuf.MessageLite;
 import java.io.File;
-import java.io.IOException;
 import java.nio.file.Path;
 import java.nio.file.Paths;
-import java.util.ArrayList;
-import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Optional;
 import java.util.Set;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.hadoop.hdds.HddsConfigKeys;
-import org.apache.hadoop.hdds.StringUtils;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.StorageUnit;
 import org.apache.hadoop.hdds.utils.db.managed.ManagedColumnFamilyOptions;
@@ -52,9 +48,8 @@ import org.apache.hadoop.hdds.utils.db.managed.ManagedLogger;
 import org.apache.hadoop.hdds.utils.db.managed.ManagedRocksDB;
 import org.apache.hadoop.hdds.utils.db.managed.ManagedStatistics;
 import org.apache.hadoop.hdds.utils.db.managed.ManagedWriteOptions;
-import org.eclipse.jetty.util.StringUtil;
-import org.rocksdb.ColumnFamilyDescriptor;
 import org.rocksdb.InfoLogLevel;
+import org.rocksdb.RocksDBException;
 import org.rocksdb.StatsLevel;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -70,7 +65,7 @@ public final class DBStoreBuilder {
       LoggerFactory.getLogger(ManagedRocksDB.ORIGINAL_CLASS);
 
   public static final String DEFAULT_COLUMN_FAMILY_NAME =
-      StringUtils.bytes2String(DEFAULT_COLUMN_FAMILY);
+      org.apache.hadoop.hdds.StringUtils.bytes2String(DEFAULT_COLUMN_FAMILY);
 
   // DB PKIProfile used by ROCKDB instances.
   public static final DBProfile HDDS_DEFAULT_DB_PROFILE = DBProfile.DISK;
@@ -85,20 +80,19 @@ public final class DBStoreBuilder {
 
   private String dbname;
   private Path dbPath;
+  private Path optionsPath;
   private String dbJmxBeanNameName;
   // Maps added column family names to the column family options they were
   // added with. Value will be null if the column family was not added with
   // any options. On build, this will be replaced with defaultCfOptions.
   private Map<String, ManagedColumnFamilyOptions> cfOptions;
   private ConfigurationSource configuration;
-  private final CodecRegistry.Builder registry = CodecRegistry.newBuilder();
   private String rocksDbStat;
   // RocksDB column family write buffer size
   private long rocksDbCfWriteBufferSize;
   private RocksDBConfiguration rocksDBConfiguration;
   // Flag to indicate if the RocksDB should be opened readonly.
   private boolean openReadOnly = false;
-  private int maxFSSnapshots = 0;
   private final DBProfile defaultCfProfile;
   private boolean enableCompactionDag;
   private boolean createCheckpointDirs = true;
@@ -111,30 +105,25 @@ public final class DBStoreBuilder {
   /**
    * Create DBStoreBuilder from a generic DBDefinition.
    */
-  public static DBStore createDBStore(ConfigurationSource configuration,
-      DBDefinition definition) throws IOException {
-    return newBuilder(configuration, definition).build();
+  public static DBStore createDBStore(ConfigurationSource configuration, DBDefinition definition)
+      throws RocksDatabaseException {
+    return newBuilder(configuration, definition, null, null).build();
   }
 
-  public static DBStoreBuilder newBuilder(ConfigurationSource configuration,
-      DBDefinition definition) {
+  public static DBStoreBuilder newBuilder(ConfigurationSource conf, DBDefinition definition, File dbDir) {
+    return newBuilder(conf, definition, dbDir.getName(), dbDir.getParentFile().toPath());
+  }
 
-    DBStoreBuilder builder = newBuilder(configuration);
-    builder.applyDBDefinition(definition);
-
-    return builder;
+  public static DBStoreBuilder newBuilder(ConfigurationSource conf, DBDefinition definition,
+      String name, Path metadataDir) {
+    return newBuilder(conf).apply(definition, name, metadataDir);
   }
 
   public static DBStoreBuilder newBuilder(ConfigurationSource configuration) {
-    return newBuilder(configuration,
+    return new DBStoreBuilder(configuration,
         configuration.getObject(RocksDBConfiguration.class));
   }
 
-  public static DBStoreBuilder newBuilder(ConfigurationSource configuration,
-      RocksDBConfiguration rocksDBConfiguration) {
-    return new DBStoreBuilder(configuration, rocksDBConfiguration);
-  }
-
   private DBStoreBuilder(ConfigurationSource configuration,
       RocksDBConfiguration rocksDBConfiguration) {
     cfOptions = new HashMap<>();
@@ -151,7 +140,7 @@ public final class DBStoreBuilder {
     // Get default DBOptions and ColumnFamilyOptions from the default DB
     // profile.
     defaultCfProfile = this.configuration.getEnum(HDDS_DB_PROFILE,
-          HDDS_DEFAULT_DB_PROFILE);
+        HDDS_DEFAULT_DB_PROFILE);
     LOG.debug("Default DB profile:{}", defaultCfProfile);
 
     this.maxDbUpdatesSizeThreshold = (long) configuration.getStorageSize(
@@ -174,21 +163,28 @@ public final class DBStoreBuilder {
     return metadataDir;
   }
 
-  private void applyDBDefinition(DBDefinition definition) {
-    // Set metadata dirs.
-    File metadataDir = getDBDirPath(definition, configuration);
+  private DBStoreBuilder apply(DBDefinition definition, String name, Path metadataDir) {
+    if (name == null) {
+      name = definition.getName();
+    }
+    setName(name);
 
-    setName(definition.getName());
-    setPath(Paths.get(metadataDir.getPath()));
+    // Set metadata dirs.
+    if (metadataDir == null) {
+      metadataDir = getDBDirPath(definition, configuration).toPath();
+    }
+    setPath(metadataDir);
+    setOptionsPath(definition.getOptionsPath(configuration));
 
     // Add column family names and codecs.
-    for (DBColumnFamilyDefinition columnFamily :
-        definition.getColumnFamilies()) {
-
-      addTable(columnFamily.getName(), columnFamily.getCfOptions());
-      addCodec(columnFamily.getKeyType(), columnFamily.getKeyCodec());
-      addCodec(columnFamily.getValueType(), columnFamily.getValueCodec());
+    for (DBColumnFamilyDefinition<?, ?> columnFamily : definition.getColumnFamilies()) {
+      ManagedColumnFamilyOptions cfOptionsFromFile = getCfOptionsFromFile(columnFamily.getName());
+      if (cfOptionsFromFile == null) {
+        cfOptionsFromFile = columnFamily.getCfOptions();
+      }
+      addTable(columnFamily.getName(), cfOptionsFromFile);
     }
+    return this;
   }
 
   private void setDBOptionsProps(ManagedDBOptions dbOptions) {
@@ -207,41 +203,45 @@ public final class DBStoreBuilder {
    *
    * @return DBStore
    */
-  public DBStore build() throws IOException {
-    if (StringUtil.isBlank(dbname) || (dbPath == null)) {
+  public RDBStore build() throws RocksDatabaseException {
+    if (StringUtils.isBlank(dbname) || (dbPath == null)) {
       LOG.error("Required Parameter missing.");
-      throw new IOException("Required parameter is missing. Please make sure "
+      throw new RocksDatabaseException("Required parameter is missing. Please make sure "
           + "Path and DB name is provided.");
     }
 
     Set<TableConfig> tableConfigs = makeTableConfigs();
-
+    ManagedWriteOptions writeOptions = null;
     try {
       if (rocksDBOption == null) {
-        rocksDBOption = getDefaultDBOptions(tableConfigs);
+        rocksDBOption = getDefaultDBOptions();
       }
       setDBOptionsProps(rocksDBOption);
-      ManagedWriteOptions writeOptions = new ManagedWriteOptions();
-      writeOptions.setSync(rocksDBConfiguration.getSyncOption());
-
       File dbFile = getDBFile();
       if (!dbFile.getParentFile().exists()) {
-        throw new IOException("The DB destination directory should exist.");
+        throw new RocksDatabaseException("The DB destination directory should exist.");
       }
+      writeOptions = new ManagedWriteOptions();
+      writeOptions.setSync(rocksDBConfiguration.getSyncOption());
 
       return new RDBStore(dbFile, rocksDBOption, statistics, writeOptions, tableConfigs,
-          registry.build(), openReadOnly, maxFSSnapshots, dbJmxBeanNameName,
-          enableCompactionDag, maxDbUpdatesSizeThreshold, createCheckpointDirs,
-          configuration, enableRocksDbMetrics);
+          openReadOnly, dbJmxBeanNameName, enableCompactionDag,
+          maxDbUpdatesSizeThreshold, createCheckpointDirs, configuration,
+          enableRocksDbMetrics);
+    } catch (Exception ex) {
+      try {
+        if (writeOptions != null) {
+          writeOptions.close();
+        }
+      } catch (Exception suppressed) {
+        ex.addSuppressed(suppressed);
+      }
+      throw ex;
     } finally {
       tableConfigs.forEach(TableConfig::close);
     }
   }
 
-  public DBStoreBuilder setMaxFSSnapshots(int maxFSSnapshots) {
-    this.maxFSSnapshots = maxFSSnapshots;
-    return this;
-  }
   public DBStoreBuilder setName(String name) {
     dbname = name;
     return this;
@@ -262,15 +262,6 @@ public final class DBStoreBuilder {
     return this;
   }
 
-  public <T> DBStoreBuilder addCodec(Class<T> type, Codec<T> codec) {
-    registry.addCodec(type, codec);
-    return this;
-  }
-
-  public <T extends MessageLite> DBStoreBuilder addProto2Codec(T type) {
-    return addCodec((Class<T>)type.getClass(), Proto2Codec.get(type));
-  }
-
   public DBStoreBuilder setDBOptions(ManagedDBOptions option) {
     rocksDBOption = option;
     return this;
@@ -288,6 +279,12 @@ public final class DBStoreBuilder {
     return this;
   }
 
+  public DBStoreBuilder setOptionsPath(Path optionsPath) {
+    Preconditions.checkNotNull(optionsPath);
+    this.optionsPath = optionsPath;
+    return this;
+  }
+
   public DBStoreBuilder setOpenReadOnly(boolean readOnly) {
     this.openReadOnly = readOnly;
     return this;
@@ -307,6 +304,7 @@ public final class DBStoreBuilder {
     this.enableRocksDbMetrics = enableRocksDbMetrics;
     return this;
   }
+
   /**
    * Set the {@link ManagedDBOptions} and default
    * {@link ManagedColumnFamilyOptions} based on {@code prof}.
@@ -332,18 +330,20 @@ public final class DBStoreBuilder {
     Set<TableConfig> tableConfigs = new HashSet<>();
 
     // If default column family was not added, add it with the default options.
-    cfOptions.putIfAbsent(DEFAULT_COLUMN_FAMILY_NAME,
-            getCfOptions(rocksDbCfWriteBufferSize));
+    ManagedColumnFamilyOptions usedCFOptions = getFromFileOrDefault(DEFAULT_COLUMN_FAMILY_NAME);
+    usedCFOptions.setWriteBufferSize(rocksDbCfWriteBufferSize);
+    cfOptions.putIfAbsent(DEFAULT_COLUMN_FAMILY_NAME, usedCFOptions);
 
-    for (Map.Entry<String, ManagedColumnFamilyOptions> entry:
+    for (Map.Entry<String, ManagedColumnFamilyOptions> entry :
         cfOptions.entrySet()) {
       String name = entry.getKey();
       ManagedColumnFamilyOptions options = entry.getValue();
 
       if (options == null) {
         LOG.debug("using default column family options for table: {}", name);
-        tableConfigs.add(new TableConfig(name,
-                getCfOptions(rocksDbCfWriteBufferSize)));
+        ManagedColumnFamilyOptions fromFileOrDefault = getFromFileOrDefault(name);
+        fromFileOrDefault.setWriteBufferSize(rocksDbCfWriteBufferSize);
+        tableConfigs.add(new TableConfig(name, fromFileOrDefault));
       } else {
         tableConfigs.add(new TableConfig(name, options));
       }
@@ -357,30 +357,23 @@ public final class DBStoreBuilder {
         .orElseGet(defaultCfProfile::getColumnFamilyOptions);
   }
 
-  /**
-   * Pass true to disable auto compaction for Column Family by default.
-   * Sets Disable auto compaction flag for Default Column Family option
-   * @param defaultCFAutoCompaction
-   */
-  public DBStoreBuilder disableDefaultCFAutoCompaction(
-          boolean defaultCFAutoCompaction) {
-    ManagedColumnFamilyOptions defaultCFOptions =
-            getDefaultCfOptions();
-    defaultCFOptions.setDisableAutoCompactions(defaultCFAutoCompaction);
-    setDefaultCFOptions(defaultCFOptions);
-    return this;
+  private ManagedColumnFamilyOptions getFromFileOrDefault(String cfName) {
+    ManagedColumnFamilyOptions cfOptionsFromFile = getCfOptionsFromFile(cfName);
+    return cfOptionsFromFile != null
+        ? cfOptionsFromFile : getDefaultCfOptions();
   }
 
   /**
-   * Get default column family options, but with column family write buffer
-   * size limit overridden.
-   * @param writeBufferSize Specify column family write buffer size.
-   * @return ManagedColumnFamilyOptions
+   * Pass true to disable auto compaction for Column Family by default.
+   * Sets Disable auto compaction flag for Default Column Family option
+   *
+   * @param defaultCFAutoCompaction
    */
-  private ManagedColumnFamilyOptions getCfOptions(long writeBufferSize) {
-    ManagedColumnFamilyOptions cfOpts = getDefaultCfOptions();
-    cfOpts.setWriteBufferSize(writeBufferSize);
-    return cfOpts;
+  public DBStoreBuilder disableDefaultCFAutoCompaction(boolean defaultCFAutoCompaction) {
+    ManagedColumnFamilyOptions defaultCFOptions = getFromFileOrDefault(DEFAULT_COLUMN_FAMILY_NAME);
+    defaultCFOptions.setDisableAutoCompactions(defaultCFAutoCompaction);
+    setDefaultCFOptions(defaultCFOptions);
+    return this;
   }
 
   /**
@@ -392,19 +385,19 @@ public final class DBStoreBuilder {
    * will also have statistics added if they are not turned off in the
    * builder's {@link ConfigurationSource}.
    *
-   * @param tableConfigs Configurations for each column family, used when
-   * reading DB options from the ini file.
-   *
    * @return The {@link ManagedDBOptions} that should be used as the default
    * value for this builder if one is not specified by the caller.
    */
-  private ManagedDBOptions getDefaultDBOptions(
-      Collection<TableConfig> tableConfigs) {
-    ManagedDBOptions dbOptions = getDBOptionsFromFile(tableConfigs);
-
+  private ManagedDBOptions getDefaultDBOptions() {
+    ManagedDBOptions dbOptions = null;
+    try {
+      Path configuredPath = optionsPath != null ? optionsPath : dbPath;
+      dbOptions = DBConfigFromFile.readDBOptionsFromFile(configuredPath);
+    } catch (RocksDBException e) {
+      LOG.error("Error trying to use dbOptions from file: {}", optionsPath);
+    }
     if (dbOptions == null) {
       dbOptions = defaultCfProfile.getDBOptions();
-      LOG.debug("Using RocksDB DBOptions from default profile.");
     }
 
     // Apply logging settings.
@@ -428,49 +421,36 @@ public final class DBStoreBuilder {
   }
 
   /**
-   * Attempts to construct a {@link ManagedDBOptions} object from the
-   * configuration directory with name equal to {@code database name}.ini,
-   * where {@code database name} is the property set by
-   * {@link DBStoreBuilder#setName(String)}.
+   * Attempts to get RocksDB {@link ManagedColumnFamilyOptions} from an ini config
+   * file. If that file does not exist, the value of {@code getColumnFamilyOptions}
+   * is used instead.
+   *
+   * @return The {@link ManagedColumnFamilyOptions} that should be used as the default
+   * value for this builder if one is not specified by the caller.
    */
-  private ManagedDBOptions getDBOptionsFromFile(
-      Collection<TableConfig> tableConfigs) {
-    ManagedDBOptions option = null;
-
-    List<ColumnFamilyDescriptor> columnFamilyDescriptors = new ArrayList<>();
-
-    if (StringUtil.isNotBlank(dbname)) {
-      for (TableConfig tc : tableConfigs) {
-        columnFamilyDescriptors.add(tc.getDescriptor());
-      }
-
-      if (!columnFamilyDescriptors.isEmpty()) {
-        try {
-          option = DBConfigFromFile.readFromFile(dbname,
-              columnFamilyDescriptors);
-          if (option != null) {
-            LOG.info("Using RocksDB DBOptions from {}.ini file", dbname);
-          }
-        } catch (IOException ex) {
-          LOG.info("Unable to read RocksDB DBOptions from {}", dbname, ex);
-        } finally {
-          columnFamilyDescriptors.forEach(d -> d.getOptions().close());
-        }
-      }
+  public ManagedColumnFamilyOptions getCfOptionsFromFile(String cfName) {
+    if (Objects.isNull(defaultCfProfile)) {
+      throw new RuntimeException();
     }
-
-    return option;
+    Path configuredPath = optionsPath != null ? optionsPath : dbPath;
+    ManagedColumnFamilyOptions cfOptionsFromFile = null;
+    try {
+      cfOptionsFromFile = DBConfigFromFile.readCFOptionsFromFile(configuredPath, cfName);
+    } catch (RocksDBException e) {
+      LOG.error("Error while trying to read ColumnFamilyOptions from file: {}", configuredPath);
+    }
+    return cfOptionsFromFile;
   }
 
-  private File getDBFile() throws IOException {
+  private File getDBFile() throws RocksDatabaseException {
     if (dbPath == null) {
       LOG.error("DB path is required.");
-      throw new IOException("A Path to for DB file is needed.");
+      throw new RocksDatabaseException("A Path to for DB file is needed.");
     }
 
-    if (StringUtil.isBlank(dbname)) {
+    if (StringUtils.isBlank(dbname)) {
       LOG.error("DBName is a required.");
-      throw new IOException("A valid DB name is required.");
+      throw new RocksDatabaseException("A valid DB name is required.");
     }
     return Paths.get(dbPath.toString(), dbname).toFile();
   }
