@@ -34,6 +34,7 @@ import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
@@ -61,7 +62,6 @@ public class RDBStore implements DBStore {
       LoggerFactory.getLogger(RDBStore.class);
   private final RocksDatabase db;
   private final File dbLocation;
-  private final CodecRegistry codecRegistry;
   private RocksDBStoreMetrics metrics;
   private final RDBCheckpointManager checkPointManager;
   private final String checkpointsParentDir;
@@ -76,21 +76,19 @@ public class RDBStore implements DBStore {
   private final ManagedStatistics statistics;
 
   @SuppressWarnings("parameternumber")
-  public RDBStore(File dbFile, ManagedDBOptions dbOptions, ManagedStatistics statistics,
+  RDBStore(File dbFile, ManagedDBOptions dbOptions, ManagedStatistics statistics,
                   ManagedWriteOptions writeOptions, Set<TableConfig> families,
-                  CodecRegistry registry, boolean readOnly, int maxFSSnapshots,
+                  boolean readOnly,
                   String dbJmxBeanName, boolean enableCompactionDag,
                   long maxDbUpdatesSizeThreshold,
                   boolean createCheckpointDirs,
                   ConfigurationSource configuration,
                   boolean enableRocksDBMetrics)
-
-      throws IOException {
+      throws RocksDatabaseException {
     Preconditions.checkNotNull(dbFile, "DB file location cannot be null");
     Preconditions.checkNotNull(families);
     Preconditions.checkArgument(!families.isEmpty());
     this.maxDbUpdatesSizeThreshold = maxDbUpdatesSizeThreshold;
-    codecRegistry = registry;
     dbLocation = dbFile;
     this.dbOptions = dbOptions;
     this.statistics = statistics;
@@ -170,17 +168,13 @@ public class RDBStore implements DBStore {
       //Initialize checkpoint manager
       checkPointManager = new RDBCheckpointManager(db, dbLocation.getName());
       rdbMetrics = RDBMetrics.create();
-
-    } catch (Exception e) {
-      // Close DB and other things if got initialized.
-      close();
-      String msg = "Failed init RocksDB, db path : " + dbFile.getAbsolutePath()
-          + ", " + "exception :" + (e.getCause() == null ?
-          e.getClass().getCanonicalName() + " " + e.getMessage() :
-          e.getCause().getClass().getCanonicalName() + " " +
-              e.getCause().getMessage());
-
-      throw new IOException(msg, e);
+    } catch (IOException | RuntimeException e) {
+      try {
+        close();
+      } catch (Exception suppressed) {
+        e.addSuppressed(suppressed);
+      }
+      throw new RocksDatabaseException("Failed to create RDBStore from " + dbFile, e);
     }
 
     if (LOG.isDebugEnabled()) {
@@ -204,7 +198,6 @@ public class RDBStore implements DBStore {
     return rocksDBCheckpointDiffer;
   }
 
-
   /**
    * Returns the RocksDB's DBOptions.
    */
@@ -213,7 +206,7 @@ public class RDBStore implements DBStore {
   }
 
   @Override
-  public void compactDB() throws IOException {
+  public void compactDB() throws RocksDatabaseException {
     try (ManagedCompactRangeOptions options =
              new ManagedCompactRangeOptions()) {
       db.compactDB(options);
@@ -221,7 +214,23 @@ public class RDBStore implements DBStore {
   }
 
   @Override
-  public void close() throws IOException {
+  public void compactTable(String tableName) throws RocksDatabaseException {
+    try (ManagedCompactRangeOptions options = new ManagedCompactRangeOptions()) {
+      compactTable(tableName, options);
+    }
+  }
+
+  @Override
+  public void compactTable(String tableName, ManagedCompactRangeOptions options) throws RocksDatabaseException {
+    RocksDatabase.ColumnFamily columnFamily = db.getColumnFamily(tableName);
+    if (columnFamily == null) {
+      throw new RocksDatabaseException("Table not found: " + tableName);
+    }
+    db.compactRange(columnFamily, null, null, options);
+  }
+
+  @Override
+  public void close() {
     if (metrics != null) {
       metrics.unregister();
       metrics = null;
@@ -240,36 +249,7 @@ public class RDBStore implements DBStore {
   }
 
   @Override
-  public <K, V> void move(K key, Table<K, V> source,
-                                Table<K, V> dest) throws IOException {
-    try (BatchOperation batchOperation = initBatchOperation()) {
-
-      V value = source.get(key);
-      dest.putWithBatch(batchOperation, key, value);
-      source.deleteWithBatch(batchOperation, key);
-      commitBatchOperation(batchOperation);
-    }
-  }
-
-  @Override
-  public <K, V> void move(K key, V value, Table<K, V> source,
-                                Table<K, V> dest) throws IOException {
-    move(key, key, value, source, dest);
-  }
-
-  @Override
-  public <K, V> void move(K sourceKey, K destKey, V value,
-                                Table<K, V> source,
-                                Table<K, V> dest) throws IOException {
-    try (BatchOperation batchOperation = initBatchOperation()) {
-      dest.putWithBatch(batchOperation, destKey, value);
-      source.deleteWithBatch(batchOperation, sourceKey);
-      commitBatchOperation(batchOperation);
-    }
-  }
-
-  @Override
-  public long getEstimatedKeyCount() throws IOException {
+  public long getEstimatedKeyCount() throws RocksDatabaseException {
     return db.estimateNumKeys();
   }
 
@@ -280,42 +260,29 @@ public class RDBStore implements DBStore {
 
   @Override
   public void commitBatchOperation(BatchOperation operation)
-      throws IOException {
+      throws RocksDatabaseException {
     ((RDBBatchOperation) operation).commit(db);
   }
 
   @Override
-  public RDBTable getTable(String name) throws IOException {
+  public RDBTable getTable(String name) throws RocksDatabaseException {
     final ColumnFamily handle = db.getColumnFamily(name);
     if (handle == null) {
-      throw new IOException("No such table in this DB. TableName : " + name);
+      throw new RocksDatabaseException("No such table in this DB. TableName : " + name);
     }
     return new RDBTable(this.db, handle, rdbMetrics);
   }
 
-  @Override
-  public <K, V> TypedTable<K, V> getTable(String name,
-      Class<K> keyType, Class<V> valueType) throws IOException {
-    return new TypedTable<>(getTable(name), codecRegistry, keyType,
-        valueType);
-  }
-
   @Override
   public <K, V> TypedTable<K, V> getTable(
-      String name, Codec<K> keyCodec, Codec<V> valueCodec, TableCache.CacheType cacheType) throws IOException {
+      String name, Codec<K> keyCodec, Codec<V> valueCodec, TableCache.CacheType cacheType)
+      throws RocksDatabaseException, CodecException {
     return new TypedTable<>(getTable(name), keyCodec, valueCodec, cacheType);
   }
 
   @Override
-  public <K, V> Table<K, V> getTable(String name,
-      Class<K> keyType, Class<V> valueType,
-      TableCache.CacheType cacheType) throws IOException {
-    return new TypedTable<>(getTable(name), codecRegistry, keyType, valueType, cacheType);
-  }
-
-  @Override
-  public ArrayList<Table> listTables() {
-    ArrayList<Table> returnList = new ArrayList<>();
+  public List<Table<?, ?>> listTables() {
+    final List<Table<?, ?>> returnList = new ArrayList<>();
     for (ColumnFamily family : getColumnFamilies()) {
       returnList.add(new RDBTable(db, family, rdbMetrics));
     }
@@ -323,19 +290,19 @@ public class RDBStore implements DBStore {
   }
 
   @Override
-  public void flushDB() throws IOException {
+  public void flushDB() throws RocksDatabaseException {
     db.flush();
   }
 
   @Override
-  public void flushLog(boolean sync) throws IOException {
+  public void flushLog(boolean sync) throws RocksDatabaseException {
     // for RocksDB it is sufficient to flush the WAL as entire db can
     // be reconstructed using it.
     db.flushWal(sync);
   }
 
   @Override
-  public DBCheckpoint getCheckpoint(boolean flush) throws IOException {
+  public DBCheckpoint getCheckpoint(boolean flush) throws RocksDatabaseException {
     if (flush) {
       this.flushDB();
     }
@@ -343,14 +310,14 @@ public class RDBStore implements DBStore {
   }
 
   @Override
-  public DBCheckpoint getCheckpoint(String parentPath, boolean flush) throws IOException {
+  public DBCheckpoint getCheckpoint(String parentPath, boolean flush) throws RocksDatabaseException {
     if (flush) {
       this.flushDB();
     }
     return checkPointManager.createCheckpoint(parentPath, null);
   }
 
-  public DBCheckpoint getSnapshot(String name) throws IOException {
+  public DBCheckpoint getSnapshot(String name) throws RocksDatabaseException {
     this.flushLog(true);
     return checkPointManager.createCheckpoint(snapshotsParentDir, name);
   }
@@ -371,13 +338,13 @@ public class RDBStore implements DBStore {
 
   @Override
   public DBUpdatesWrapper getUpdatesSince(long sequenceNumber)
-      throws IOException {
+      throws SequenceNumberNotFoundException {
     return getUpdatesSince(sequenceNumber, Long.MAX_VALUE);
   }
 
   @Override
   public DBUpdatesWrapper getUpdatesSince(long sequenceNumber, long limitCount)
-      throws IOException {
+      throws SequenceNumberNotFoundException {
     if (limitCount <= 0) {
       throw new IllegalArgumentException("Illegal count for getUpdatesSince.");
     }
@@ -446,7 +413,7 @@ public class RDBStore implements DBStore {
       // Throw the exception back to Recon. Expect Recon to fall back to
       // full snapshot.
       throw e;
-    } catch (RocksDBException | IOException e) {
+    } catch (RocksDBException | RocksDatabaseException e) {
       LOG.error("Unable to get delta updates since sequenceNumber {}. "
               + "This exception will not be thrown to the client ",
           sequenceNumber, e);
@@ -474,20 +441,16 @@ public class RDBStore implements DBStore {
     return db;
   }
 
-  public String getProperty(String property) throws IOException {
+  public String getProperty(String property) throws RocksDatabaseException {
     return db.getProperty(property);
   }
 
   public String getProperty(ColumnFamily family, String property)
-      throws IOException {
+      throws RocksDatabaseException {
     return db.getProperty(family, property);
   }
 
   public RDBMetrics getMetrics() {
     return rdbMetrics;
   }
-
-  public static Logger getLogger() {
-    return LOG;
-  }
 }
