@@ -18,14 +18,13 @@
 package org.apache.hadoop.hdds.utils.db;
 
 import java.io.File;
-import java.io.IOException;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import org.apache.commons.lang3.NotImplementedException;
 import org.apache.hadoop.hdds.annotation.InterfaceStability;
-import org.apache.hadoop.hdds.utils.MetadataKeyFilters;
+import org.apache.hadoop.hdds.utils.MetadataKeyFilters.KeyPrefixFilter;
 import org.apache.hadoop.hdds.utils.TableCacheMetrics;
 import org.apache.hadoop.hdds.utils.db.cache.CacheKey;
 import org.apache.hadoop.hdds.utils.db.cache.CacheValue;
@@ -37,7 +36,7 @@ import org.apache.hadoop.hdds.utils.db.cache.CacheValue;
  * different kind of tables.
  */
 @InterfaceStability.Evolving
-public interface Table<KEY, VALUE> extends AutoCloseable {
+public interface Table<KEY, VALUE> {
 
   /**
    * Puts a key-value pair into the store.
@@ -45,7 +44,7 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
    * @param key metadata key
    * @param value metadata value
    */
-  void put(KEY key, VALUE value) throws IOException;
+  void put(KEY key, VALUE value) throws RocksDatabaseException, CodecException;
 
   /**
    * Puts a key-value pair into the store as part of a bath operation.
@@ -54,14 +53,12 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
    * @param key metadata key
    * @param value metadata value
    */
-  void putWithBatch(BatchOperation batch, KEY key, VALUE value)
-      throws IOException;
+  void putWithBatch(BatchOperation batch, KEY key, VALUE value) throws RocksDatabaseException, CodecException;
 
   /**
    * @return true if the metadata store is empty.
-   * @throws IOException on Failure
    */
-  boolean isEmpty() throws IOException;
+  boolean isEmpty() throws RocksDatabaseException;
 
   /**
    * Check if a given key exists in Metadata store.
@@ -69,9 +66,8 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
    * A lock on the key / bucket needs to be acquired before invoking this API.
    * @param key metadata key
    * @return true if the metadata store contains a key.
-   * @throws IOException on Failure
    */
-  boolean isExist(KEY key) throws IOException;
+  boolean isExist(KEY key) throws RocksDatabaseException, CodecException;
 
   /**
    * Returns the value mapped to the given key in byte array or returns null
@@ -79,10 +75,8 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
    *
    * @param key metadata key
    * @return value in byte array or null if the key is not found.
-   * @throws IOException on Failure
    */
-  VALUE get(KEY key) throws IOException;
-
+  VALUE get(KEY key) throws RocksDatabaseException, CodecException;
 
   /**
    * Skip checking cache and get the value mapped to the given key in byte
@@ -90,13 +84,11 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
    *
    * @param key metadata key
    * @return value in byte array or null if the key is not found.
-   * @throws IOException on Failure
    */
-  default VALUE getSkipCache(KEY key) throws IOException {
+  default VALUE getSkipCache(KEY key) throws RocksDatabaseException, CodecException {
     throw new NotImplementedException("getSkipCache is not implemented");
   }
 
-
   /**
    * Returns the value mapped to the given key in byte array or returns null
    * if the key is not found.
@@ -106,13 +98,11 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
    *
    * @param key metadata key
    * @return value in byte array or null if the key is not found.
-   * @throws IOException on Failure
    */
-  default VALUE getReadCopy(KEY key) throws IOException {
+  default VALUE getReadCopy(KEY key) throws RocksDatabaseException, CodecException {
     throw new NotImplementedException("getReadCopy is not implemented");
   }
 
-
   /**
    * Returns the value mapped to the given key in byte array or returns null
    * if the key is not found.
@@ -126,66 +116,106 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
    *
    * @param key metadata key
    * @return value in byte array or null if the key is not found.
-   * @throws IOException on Failure
    */
-  VALUE getIfExist(KEY key) throws IOException;
+  VALUE getIfExist(KEY key) throws RocksDatabaseException, CodecException;
 
   /**
    * Deletes a key from the metadata store.
    *
    * @param key metadata key
-   * @throws IOException on Failure
    */
-  void delete(KEY key) throws IOException;
+  void delete(KEY key) throws RocksDatabaseException, CodecException;
 
   /**
    * Deletes a key from the metadata store as part of a batch operation.
    *
    * @param batch the batch operation
    * @param key metadata key
-   * @throws IOException on Failure
    */
-  void deleteWithBatch(BatchOperation batch, KEY key) throws IOException;
+  void deleteWithBatch(BatchOperation batch, KEY key) throws CodecException;
+
+  /**
+   * Deletes a range of keys from the metadata store as part of a batch operation.
+   * @param batch Batch operation to perform the delete operation.
+   * @param beginKey start metadata key, inclusive.
+   * @param endKey end metadata key, exclusive.
+   */
+  void deleteRangeWithBatch(BatchOperation batch, KEY beginKey, KEY endKey) throws CodecException;
 
   /**
    * Deletes a range of keys from the metadata store.
    *
    * @param beginKey start metadata key
    * @param endKey end metadata key
-   * @throws IOException on Failure
    */
-  void deleteRange(KEY beginKey, KEY endKey) throws IOException;
+  void deleteRange(KEY beginKey, KEY endKey) throws RocksDatabaseException, CodecException;
+
+  /** The same as iterator(null, KEY_AND_VALUE). */
+  default KeyValueIterator<KEY, VALUE> iterator() throws RocksDatabaseException, CodecException {
+    return iterator(null, KeyValueIterator.Type.KEY_AND_VALUE);
+  }
+
+  /** The same as iterator(prefix, KEY_AND_VALUE). */
+  default KeyValueIterator<KEY, VALUE> iterator(KEY prefix) throws RocksDatabaseException, CodecException {
+    return iterator(prefix, KeyValueIterator.Type.KEY_AND_VALUE);
+  }
 
   /**
-   * Returns the iterator for this metadata store.
+   * Iterate the elements in this table.
+   * <p>
+   * Note that using a more restrictive type may improve performance
+   * since the unrequired data may not be read from the DB.
+   * <p>
+   * Note also that, when the prefix is non-empty,
+   * using a non-key type may not improve performance
+   * since it has to read keys for matching the prefix.
    *
-   * @return MetaStoreIterator
-   * @throws IOException on failure.
+   * @param prefix The prefix of the elements to be iterated.
+   * @param type Specify whether key and/or value are required.
+   * @return an iterator.
    */
-  TableIterator<KEY, ? extends KeyValue<KEY, VALUE>> iterator()
-      throws IOException;
+  KeyValueIterator<KEY, VALUE> iterator(KEY prefix, KeyValueIterator.Type type)
+      throws RocksDatabaseException, CodecException;
 
   /**
-   * Returns a prefixed iterator for this metadata store.
-   * @param prefix
-   * @return MetaStoreIterator
+   * @param prefix The prefix of the elements to be iterated.
+   * @return a key-only iterator
    */
-  TableIterator<KEY, ? extends KeyValue<KEY, VALUE>> iterator(KEY prefix)
-      throws IOException;
+  default TableIterator<KEY, KEY> keyIterator(KEY prefix) throws RocksDatabaseException, CodecException {
+    final KeyValueIterator<KEY, VALUE> i = iterator(prefix, KeyValueIterator.Type.KEY_ONLY);
+    return TableIterator.convert(i, KeyValue::getKey);
+  }
+
+  /** The same as keyIterator(null). */
+  default TableIterator<KEY, KEY> keyIterator() throws RocksDatabaseException, CodecException {
+    return keyIterator(null);
+  }
+
+  /**
+   * @param prefix The prefix of the elements to be iterated.
+   * @return a value-only iterator.
+   */
+  default TableIterator<KEY, VALUE> valueIterator(KEY prefix) throws RocksDatabaseException, CodecException {
+    final KeyValueIterator<KEY, VALUE> i = iterator(prefix, KeyValueIterator.Type.VALUE_ONLY);
+    return TableIterator.convert(i, KeyValue::getValue);
+  }
+
+  /** The same as valueIterator(null). */
+  default TableIterator<KEY, VALUE> valueIterator() throws RocksDatabaseException, CodecException {
+    return valueIterator(null);
+  }
 
   /**
    * Returns the Name of this Table.
    * @return - Table Name.
-   * @throws IOException on failure.
    */
-  String getName() throws IOException;
+  String getName();
 
   /**
    * Returns the key count of this Table.  Note the result can be inaccurate.
    * @return Estimated key count of this Table
-   * @throws IOException on failure
    */
-  long getEstimatedKeyCount() throws IOException;
+  long getEstimatedKeyCount() throws RocksDatabaseException;
 
   /**
    * Add entry to the table cache.
@@ -238,14 +268,12 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
   /**
    * Create the metrics datasource that emits table cache metrics.
    */
-  default TableCacheMetrics createCacheMetrics() throws IOException {
+  default TableCacheMetrics createCacheMetrics() throws RocksDatabaseException {
     throw new NotImplementedException("getCacheValue is not implemented");
   }
 
   /**
-   * Returns a certain range of key value pairs as a list based on a
-   * startKey or count. Further a {@link org.apache.hadoop.hdds.utils.MetadataKeyFilters.MetadataKeyFilter}
-   * can be added to * filter keys if necessary.
+   * Returns a certain range of key value pairs as a list based on a startKey or count.
    * To prevent race conditions while listing
    * entries, this implementation takes a snapshot and lists the entries from
    * the snapshot. This may, on the other hand, cause the range result slight
@@ -259,47 +287,33 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
    * The count argument is to limit number of total entries to return,
    * the value for count must be an integer greater than 0.
    * <p>
-   * This method allows to specify one or more
-   * {@link org.apache.hadoop.hdds.utils.MetadataKeyFilters.MetadataKeyFilter}
-   * to filter keys by certain condition. Once given, only the entries
-   * whose key passes all the filters will be included in the result.
+   * This method allows to specify a {@link KeyPrefixFilter} to filter keys.
+   * Once given, only the entries whose key passes all the filters will be included in the result.
    *
    * @param startKey a start key.
    * @param count max number of entries to return.
    * @param prefix fixed key schema specific prefix
-   * @param filters customized one or more
-   * {@link org.apache.hadoop.hdds.utils.MetadataKeyFilters.MetadataKeyFilter}.
+   * @param filter for filtering keys
+   * @param isSequential does it require sequential keys?
    * @return a list of entries found in the database or an empty list if the
    * startKey is invalid.
-   * @throws IOException if there are I/O errors.
    * @throws IllegalArgumentException if count is less than 0.
    */
-  List<? extends KeyValue<KEY, VALUE>> getRangeKVs(KEY startKey,
-          int count, KEY prefix,
-          MetadataKeyFilters.MetadataKeyFilter... filters)
-          throws IOException, IllegalArgumentException;
+  List<KeyValue<KEY, VALUE>> getRangeKVs(KEY startKey,
+          int count, KEY prefix, KeyPrefixFilter filter, boolean isSequential)
+          throws RocksDatabaseException, CodecException;
 
-  /**
-   * This method is very similar to {@link #getRangeKVs}, the only
-   * different is this method is supposed to return a sequential range
-   * of elements based on the filters. While iterating the elements,
-   * if it met any entry that cannot pass the filter, the iterator will stop
-   * from this point without looking for next match. If no filter is given,
-   * this method behaves just like {@link #getRangeKVs}.
-   *
-   * @param startKey a start key.
-   * @param count max number of entries to return.
-   * @param prefix fixed key schema specific prefix
-   * @param filters customized one or more
-   * {@link org.apache.hadoop.hdds.utils.MetadataKeyFilters.MetadataKeyFilter}.
-   * @return a list of entries found in the database.
-   * @throws IOException
-   * @throws IllegalArgumentException
-   */
-  List<? extends KeyValue<KEY, VALUE>> getSequentialRangeKVs(KEY startKey,
-          int count, KEY prefix,
-          MetadataKeyFilters.MetadataKeyFilter... filters)
-          throws IOException, IllegalArgumentException;
+  /** The same as getRangeKVs(startKey, count, prefix, filter, false). */
+  default List<KeyValue<KEY, VALUE>> getRangeKVs(KEY startKey, int count, KEY prefix, KeyPrefixFilter filter)
+      throws RocksDatabaseException, CodecException {
+    return getRangeKVs(startKey, count, prefix, filter, false);
+  }
+
+  /** The same as getRangeKVs(startKey, count, prefix, null). */
+  default List<KeyValue<KEY, VALUE>> getRangeKVs(KEY startKey, int count, KEY prefix)
+      throws RocksDatabaseException, CodecException {
+    return getRangeKVs(startKey, count, prefix, null);
+  }
 
   /**
    * Deletes all keys with the specified prefix from the metadata store
@@ -307,121 +321,102 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
    * @param batch
    * @param prefix
    */
-  void deleteBatchWithPrefix(BatchOperation batch, KEY prefix)
-      throws IOException;
+  void deleteBatchWithPrefix(BatchOperation batch, KEY prefix) throws RocksDatabaseException, CodecException;
 
   /**
    * Dump all key value pairs with a prefix into an external file.
    * @param externalFile
    * @param prefix
-   * @throws IOException
    */
-  void dumpToFileWithPrefix(File externalFile, KEY prefix) throws IOException;
+  void dumpToFileWithPrefix(File externalFile, KEY prefix) throws RocksDatabaseException, CodecException;
 
   /**
    * Load key value pairs from an external file created by
    * dumpToFileWithPrefix.
    * @param externalFile
-   * @throws IOException
    */
-  void loadFromFile(File externalFile) throws IOException;
+  void loadFromFile(File externalFile) throws RocksDatabaseException;
 
   /**
    * Class used to represent the key and value pair of a db entry.
    */
-  interface KeyValue<KEY, VALUE> {
+  final class KeyValue<K, V> {
+    private final K key;
+    private final V value;
+    private final int valueByteSize;
 
-    KEY getKey() throws IOException;
+    private KeyValue(K key, V value, int valueByteSize) {
+      this.key = key;
+      this.value = value;
+      this.valueByteSize = valueByteSize;
+    }
 
-    VALUE getValue() throws IOException;
+    public K getKey() {
+      return key;
+    }
 
-    default int getRawSize()  throws IOException {
-      return 0;
+    public V getValue() {
+      return value;
+    }
+
+    /** @return the value serialized byte size if it is available; otherwise, return -1. */
+    public int getValueByteSize() {
+      return value != null ? valueByteSize : -1;
+    }
+
+    @Override
+    public String toString() {
+      return "(key=" + key + ", value=" + value + ")";
+    }
+
+    @Override
+    public boolean equals(Object obj) {
+      if (this == obj) {
+        return true;
+      } else if (!(obj instanceof KeyValue)) {
+        return false;
+      }
+      final KeyValue<?, ?> that = (KeyValue<?, ?>) obj;
+      return Objects.equals(this.getKey(), that.getKey())
+          && Objects.equals(this.getValue(), that.getValue());
+    }
+
+    @Override
+    public int hashCode() {
+      return Objects.hash(getKey(), getValue());
     }
   }
 
   static <K, V> KeyValue<K, V> newKeyValue(K key, V value) {
-    return new KeyValue<K, V>() {
-      @Override
-      public K getKey() {
-        return key;
-      }
-
-      @Override
-      public V getValue() {
-        return value;
-      }
-
-      @Override
-      public String toString() {
-        return "(key=" + key + ", value=" + value + ")";
-      }
-
-      @Override
-      public boolean equals(Object obj) {
-        if (!(obj instanceof KeyValue)) {
-          return false;
-        }
-        KeyValue<?, ?> kv = (KeyValue<?, ?>) obj;
-        try {
-          return getKey().equals(kv.getKey()) && getValue().equals(kv.getValue());
-        } catch (IOException e) {
-          throw new RuntimeException(e);
-        }
-      }
-
-      @Override
-      public int hashCode() {
-        return Objects.hash(getKey(), getValue());
-      }
-    };
+    return newKeyValue(key, value, -1);
   }
 
-  static <K, V> KeyValue<K, V> newKeyValue(K key, V value, int rawSize) {
-    return new KeyValue<K, V>() {
-      @Override
-      public K getKey() {
-        return key;
-      }
-
-      @Override
-      public V getValue() {
-        return value;
-      }
-
-      @Override
-      public int getRawSize() throws IOException {
-        return rawSize;
-      }
-
-      @Override
-      public String toString() {
-        return "(key=" + key + ", value=" + value + ")";
-      }
-
-      @Override
-      public boolean equals(Object obj) {
-        if (!(obj instanceof KeyValue)) {
-          return false;
-        }
-        KeyValue<?, ?> kv = (KeyValue<?, ?>) obj;
-        try {
-          return getKey().equals(kv.getKey()) && getValue().equals(kv.getValue());
-        } catch (IOException e) {
-          throw new RuntimeException(e);
-        }
-      }
-
-      @Override
-      public int hashCode() {
-        return Objects.hash(getKey(), getValue());
-      }
-    };
+  static <K, V> KeyValue<K, V> newKeyValue(K key, V value, int valueByteSize) {
+    return new KeyValue<>(key, value, valueByteSize);
   }
 
-
   /** A {@link TableIterator} to iterate {@link KeyValue}s. */
   interface KeyValueIterator<KEY, VALUE>
       extends TableIterator<KEY, KeyValue<KEY, VALUE>> {
+
+    /** The iterator type. */
+    enum Type {
+      /** Neither read key nor value. */
+      NEITHER,
+      /** Read key only. */
+      KEY_ONLY,
+      /** Read value only. */
+      VALUE_ONLY,
+      /** Read both key and value. */
+      KEY_AND_VALUE;
+
+      boolean readKey() {
+        return (this.ordinal() & KEY_ONLY.ordinal()) != 0;
+      }
+
+      boolean readValue() {
+        return (this.ordinal() & VALUE_ONLY.ordinal()) != 0;
+      }
+    }
   }
 }
