@@ -23,7 +23,6 @@ import static org.apache.ratis.util.JavaUtils.getClassSimpleName;
 
 import com.google.common.annotations.VisibleForTesting;
 import java.io.File;
-import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Iterator;
@@ -31,7 +30,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import org.apache.hadoop.hdds.utils.IOUtils;
-import org.apache.hadoop.hdds.utils.MetadataKeyFilters;
+import org.apache.hadoop.hdds.utils.MetadataKeyFilters.KeyPrefixFilter;
 import org.apache.hadoop.hdds.utils.TableCacheMetrics;
 import org.apache.hadoop.hdds.utils.db.cache.CacheKey;
 import org.apache.hadoop.hdds.utils.db.cache.CacheResult;
@@ -68,31 +67,6 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
       = new CodecBuffer.Capacity(this, BUFFER_SIZE_DEFAULT);
   private final TableCache<KEY, VALUE> cache;
 
-  /**
-   * The same as this(rawTable, codecRegistry, keyType, valueType,
-   *                  CacheType.PARTIAL_CACHE).
-   */
-  TypedTable(RDBTable rawTable, CodecRegistry codecRegistry, Class<KEY> keyType, Class<VALUE> valueType)
-      throws IOException {
-    this(rawTable, codecRegistry, keyType, valueType, CacheType.PARTIAL_CACHE);
-  }
-
-  /**
-   * Create an TypedTable from the raw table with specified cache type.
-   *
-   * @param rawTable The underlying (untyped) table in RocksDB.
-   * @param codecRegistry To look up codecs.
-   * @param keyType The key type.
-   * @param valueType The value type.
-   * @param cacheType How to cache the entries?
-   * @throws IOException if failed to iterate the raw table.
-   */
-  TypedTable(RDBTable rawTable, CodecRegistry codecRegistry, Class<KEY> keyType, Class<VALUE> valueType,
-      CacheType cacheType) throws IOException {
-    this(rawTable, codecRegistry.getCodecFromClass(keyType), codecRegistry.getCodecFromClass(valueType),
-        cacheType);
-  }
-
   /**
    * Create an TypedTable from the raw table with specified cache type.
    *
@@ -100,10 +74,9 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
    * @param keyCodec The key codec.
    * @param valueCodec The value codec.
    * @param cacheType How to cache the entries?
-   * @throws IOException
    */
-  public TypedTable(
-      RDBTable rawTable, Codec<KEY> keyCodec, Codec<VALUE> valueCodec, CacheType cacheType) throws IOException {
+  TypedTable(RDBTable rawTable, Codec<KEY> keyCodec, Codec<VALUE> valueCodec, CacheType cacheType)
+      throws RocksDatabaseException, CodecException {
     this.rawTable = Objects.requireNonNull(rawTable, "rawTable==null");
     this.keyCodec = Objects.requireNonNull(keyCodec, "keyCodec == null");
     this.valueCodec = Objects.requireNonNull(valueCodec, "valueCodec == null");
@@ -118,7 +91,7 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
     if (cacheType == CacheType.FULL_CACHE) {
       cache = new FullTableCache<>(threadNamePrefix);
       //fill cache
-      try (TableIterator<KEY, ? extends KeyValue<KEY, VALUE>> tableIterator = iterator()) {
+      try (KeyValueIterator<KEY, VALUE> tableIterator = iterator()) {
 
         while (tableIterator.hasNext()) {
           KeyValue< KEY, VALUE > kv = tableIterator.next();
@@ -137,28 +110,28 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
     }
   }
 
-  private CodecBuffer encodeKeyCodecBuffer(KEY key) throws IOException {
+  private CodecBuffer encodeKeyCodecBuffer(KEY key) throws CodecException {
     return key == null ? null : keyCodec.toDirectCodecBuffer(key);
   }
 
-  private byte[] encodeKey(KEY key) throws IOException {
+  private byte[] encodeKey(KEY key) throws CodecException {
     return key == null ? null : keyCodec.toPersistedFormat(key);
   }
 
-  private byte[] encodeValue(VALUE value) throws IOException {
+  private byte[] encodeValue(VALUE value) throws CodecException {
     return value == null ? null : valueCodec.toPersistedFormat(value);
   }
 
-  private KEY decodeKey(byte[] key) throws IOException {
-    return key == null ? null : keyCodec.fromPersistedFormat(key);
+  private KEY decodeKey(byte[] key) throws CodecException {
+    return key != null ? keyCodec.fromPersistedFormat(key) : null;
   }
 
-  private VALUE decodeValue(byte[] value) throws IOException {
-    return value == null ? null : valueCodec.fromPersistedFormat(value);
+  private VALUE decodeValue(byte[] value) throws CodecException {
+    return value != null ? valueCodec.fromPersistedFormat(value) : null;
   }
 
   @Override
-  public void put(KEY key, VALUE value) throws IOException {
+  public void put(KEY key, VALUE value) throws RocksDatabaseException, CodecException {
     if (supportCodecBuffer) {
       try (CodecBuffer k = keyCodec.toDirectCodecBuffer(key);
            CodecBuffer v = valueCodec.toDirectCodecBuffer(value)) {
@@ -170,8 +143,7 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
   }
 
   @Override
-  public void putWithBatch(BatchOperation batch, KEY key, VALUE value)
-      throws IOException {
+  public void putWithBatch(BatchOperation batch, KEY key, VALUE value) throws RocksDatabaseException, CodecException {
     if (supportCodecBuffer) {
       CodecBuffer keyBuffer = null;
       CodecBuffer valueBuffer = null;
@@ -190,12 +162,12 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
   }
 
   @Override
-  public boolean isEmpty() throws IOException {
+  public boolean isEmpty() throws RocksDatabaseException {
     return rawTable.isEmpty();
   }
 
   @Override
-  public boolean isExist(KEY key) throws IOException {
+  public boolean isExist(KEY key) throws RocksDatabaseException, CodecException  {
 
     CacheResult<VALUE> cacheResult =
         cache.lookup(new CacheKey<>(key));
@@ -226,10 +198,9 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
    *
    * @param key metadata key
    * @return the mapped value; or null if the key is not found.
-   * @throws IOException when {@link #getFromTable(Object)} throw an exception.
    */
   @Override
-  public VALUE get(KEY key) throws IOException {
+  public VALUE get(KEY key) throws RocksDatabaseException, CodecException {
     // Here the metadata lock will guarantee that cache is not updated for same
     // key during get key.
 
@@ -251,10 +222,9 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
    *
    * @param key metadata key
    * @return value in byte array or null if the key is not found.
-   * @throws IOException on Failure
    */
   @Override
-  public VALUE getSkipCache(KEY key) throws IOException {
+  public VALUE getSkipCache(KEY key) throws RocksDatabaseException, CodecException {
     return getFromTable(key);
   }
 
@@ -275,10 +245,9 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
    * modifying the same cached object.
    * @param key metadata key
    * @return VALUE
-   * @throws IOException when {@link #getFromTable(Object)} throw an exception.
    */
   @Override
-  public VALUE getReadCopy(KEY key) throws IOException {
+  public VALUE getReadCopy(KEY key) throws RocksDatabaseException, CodecException {
     // Here the metadata lock will guarantee that cache is not updated for same
     // key during get key.
 
@@ -295,7 +264,7 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
   }
 
   @Override
-  public VALUE getIfExist(KEY key) throws IOException {
+  public VALUE getIfExist(KEY key) throws RocksDatabaseException, CodecException {
     // Here the metadata lock will guarantee that cache is not updated for same
     // key during get key.
 
@@ -321,15 +290,13 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
    *                 partial value may be written.
    * @return null if the key is not found;
    *         otherwise, return the size of the value.
-   * @throws IOException in case is an error reading from the db.
    */
-  private Integer getFromTable(CodecBuffer key, CodecBuffer outValue)
-      throws IOException {
+  private Integer getFromTable(CodecBuffer key, CodecBuffer outValue) throws RocksDatabaseException {
     return outValue.putFromSource(
         buffer -> rawTable.get(key.asReadOnlyByteBuffer(), buffer));
   }
 
-  private VALUE getFromTable(KEY key) throws IOException {
+  private VALUE getFromTable(KEY key) throws RocksDatabaseException, CodecException {
     if (supportCodecBuffer) {
       return getFromTable(key, this::getFromTable);
     } else {
@@ -343,15 +310,14 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
    * Similar to {@link #getFromTable(CodecBuffer, CodecBuffer)} except that
    * this method use {@link RDBTable#getIfExist(ByteBuffer, ByteBuffer)}.
    */
-  private Integer getFromTableIfExist(CodecBuffer key, CodecBuffer outValue)
-      throws IOException {
+  private Integer getFromTableIfExist(CodecBuffer key, CodecBuffer outValue) throws RocksDatabaseException {
     return outValue.putFromSource(
         buffer -> rawTable.getIfExist(key.asReadOnlyByteBuffer(), buffer));
   }
 
   private VALUE getFromTable(KEY key,
-      CheckedBiFunction<CodecBuffer, CodecBuffer, Integer, IOException> get)
-      throws IOException {
+      CheckedBiFunction<CodecBuffer, CodecBuffer, Integer, RocksDatabaseException> get)
+      throws RocksDatabaseException, CodecException {
     try (CodecBuffer inKey = keyCodec.toDirectCodecBuffer(key)) {
       for (; ;) {
         final Integer required;
@@ -388,7 +354,7 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
     }
   }
 
-  private VALUE getFromTableIfExist(KEY key) throws IOException {
+  private VALUE getFromTableIfExist(KEY key) throws RocksDatabaseException, CodecException {
     if (supportCodecBuffer) {
       return getFromTable(key, this::getFromTableIfExist);
     } else {
@@ -399,7 +365,7 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
   }
 
   @Override
-  public void delete(KEY key) throws IOException {
+  public void delete(KEY key) throws RocksDatabaseException, CodecException {
     if (keyCodec.supportCodecBuffer()) {
       try (CodecBuffer buffer = keyCodec.toDirectCodecBuffer(key)) {
         rawTable.delete(buffer.asReadOnlyByteBuffer());
@@ -410,37 +376,28 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
   }
 
   @Override
-  public void deleteWithBatch(BatchOperation batch, KEY key)
-      throws IOException {
+  public void deleteWithBatch(BatchOperation batch, KEY key) throws CodecException {
     rawTable.deleteWithBatch(batch, encodeKey(key));
   }
 
   @Override
-  public void deleteRange(KEY beginKey, KEY endKey) throws IOException {
+  public void deleteRangeWithBatch(BatchOperation batch, KEY beginKey, KEY endKey) throws CodecException {
+    rawTable.deleteRangeWithBatch(batch, encodeKey(beginKey), encodeKey(endKey));
+  }
+
+  @Override
+  public void deleteRange(KEY beginKey, KEY endKey) throws RocksDatabaseException, CodecException {
     rawTable.deleteRange(encodeKey(beginKey), encodeKey(endKey));
   }
 
   @Override
-  public Table.KeyValueIterator<KEY, VALUE> iterator() throws IOException {
-    return iterator(null);
-  }
-
-  @Override
-  public Table.KeyValueIterator<KEY, VALUE> iterator(KEY prefix)
-      throws IOException {
+  public KeyValueIterator<KEY, VALUE> iterator(KEY prefix, KeyValueIterator.Type type)
+      throws RocksDatabaseException, CodecException {
     if (supportCodecBuffer) {
-      final CodecBuffer prefixBuffer = encodeKeyCodecBuffer(prefix);
-      try {
-        return newCodecBufferTableIterator(rawTable.iterator(prefixBuffer));
-      } catch (Throwable t) {
-        if (prefixBuffer != null) {
-          prefixBuffer.release();
-        }
-        throw t;
-      }
+      return newCodecBufferTableIterator(prefix, type);
     } else {
       final byte[] prefixBytes = encodeKey(prefix);
-      return new TypedTableIterator(rawTable.iterator(prefixBytes));
+      return new TypedTableIterator(rawTable.iterator(prefixBytes, type));
     }
   }
 
@@ -455,19 +412,13 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
   }
 
   @Override
-  public long getEstimatedKeyCount() throws IOException {
+  public long getEstimatedKeyCount() throws RocksDatabaseException {
     if (cache.getCacheType() == CacheType.FULL_CACHE) {
       return cache.size();
     }
     return rawTable.getEstimatedKeyCount();
   }
 
-  @Override
-  public void close() throws Exception {
-    rawTable.close();
-
-  }
-
   @Override
   public void addCacheEntry(CacheKey<KEY> cacheKey,
       CacheValue<VALUE> cacheValue) {
@@ -491,60 +442,37 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
   }
 
   @Override
-  public List<TypedKeyValue> getRangeKVs(
-          KEY startKey, int count, KEY prefix,
-          MetadataKeyFilters.MetadataKeyFilter... filters)
-          throws IOException, IllegalArgumentException {
+  public List<KeyValue<KEY, VALUE>> getRangeKVs(
+      KEY startKey, int count, KEY prefix, KeyPrefixFilter filter, boolean isSequential)
+      throws RocksDatabaseException, CodecException {
+    // TODO use CodecBuffer if the key codec supports
 
     // A null start key means to start from the beginning of the table.
     // Cannot convert a null key to bytes.
     final byte[] startKeyBytes = encodeKey(startKey);
     final byte[] prefixBytes = encodeKey(prefix);
 
-    List<? extends KeyValue<byte[], byte[]>> rangeKVBytes =
-        rawTable.getRangeKVs(startKeyBytes, count, prefixBytes, filters);
-
-    List<TypedKeyValue> rangeKVs = new ArrayList<>();
-    rangeKVBytes.forEach(byteKV -> rangeKVs.add(new TypedKeyValue(byteKV)));
-
+    List<KeyValue<byte[], byte[]>> rangeKVBytes =
+        rawTable.getRangeKVs(startKeyBytes, count, prefixBytes, filter, isSequential);
+    final List<KeyValue<KEY, VALUE>> rangeKVs = new ArrayList<>();
+    for (KeyValue<byte[], byte[]> kv : rangeKVBytes) {
+      rangeKVs.add(Table.newKeyValue(decodeKey(kv.getKey()), decodeValue(kv.getValue())));
+    }
     return rangeKVs;
   }
 
   @Override
-  public List<TypedKeyValue> getSequentialRangeKVs(
-          KEY startKey, int count, KEY prefix,
-          MetadataKeyFilters.MetadataKeyFilter... filters)
-          throws IOException, IllegalArgumentException {
-
-    // A null start key means to start from the beginning of the table.
-    // Cannot convert a null key to bytes.
-    final byte[] startKeyBytes = encodeKey(startKey);
-    final byte[] prefixBytes = encodeKey(prefix);
-
-    List<? extends KeyValue<byte[], byte[]>> rangeKVBytes =
-        rawTable.getSequentialRangeKVs(startKeyBytes, count,
-            prefixBytes, filters);
-
-    List<TypedKeyValue> rangeKVs = new ArrayList<>();
-    rangeKVBytes.forEach(byteKV -> rangeKVs.add(new TypedKeyValue(byteKV)));
-
-    return rangeKVs;
-  }
-
-  @Override
-  public void deleteBatchWithPrefix(BatchOperation batch, KEY prefix)
-      throws IOException {
+  public void deleteBatchWithPrefix(BatchOperation batch, KEY prefix) throws RocksDatabaseException, CodecException {
     rawTable.deleteBatchWithPrefix(batch, encodeKey(prefix));
   }
 
   @Override
-  public void dumpToFileWithPrefix(File externalFile, KEY prefix)
-      throws IOException {
+  public void dumpToFileWithPrefix(File externalFile, KEY prefix) throws RocksDatabaseException, CodecException {
     rawTable.dumpToFileWithPrefix(externalFile, encodeKey(prefix));
   }
 
   @Override
-  public void loadFromFile(File externalFile) throws IOException {
+  public void loadFromFile(File externalFile) throws RocksDatabaseException {
     rawTable.loadFromFile(externalFile);
   }
 
@@ -558,33 +486,31 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
     return cache;
   }
 
-  /**
-   * Key value implementation for strongly typed tables.
-   */
-  public final class TypedKeyValue implements KeyValue<KEY, VALUE> {
-
-    private final KeyValue<byte[], byte[]> rawKeyValue;
-
-    private TypedKeyValue(KeyValue<byte[], byte[]> rawKeyValue) {
-      this.rawKeyValue = rawKeyValue;
+  private RawIterator<CodecBuffer> newCodecBufferTableIterator(KEY prefix, KeyValueIterator.Type type)
+      throws RocksDatabaseException, CodecException {
+    final CodecBuffer encoded = encodeKeyCodecBuffer(prefix);
+    final CodecBuffer prefixBuffer;
+    if (encoded != null && encoded.readableBytes() == 0) {
+      encoded.release();
+      prefixBuffer = null;
+    } else {
+      prefixBuffer = encoded;
     }
 
-    @Override
-    public KEY getKey() throws IOException {
-      return decodeKey(rawKeyValue.getKey());
-    }
-
-    @Override
-    public VALUE getValue() throws IOException {
-      return decodeValue(rawKeyValue.getValue());
+    try {
+      return newCodecBufferTableIterator(rawTable.iterator(prefixBuffer, type));
+    } catch (Throwable t) {
+      if (prefixBuffer != null) {
+        prefixBuffer.release();
+      }
+      throw t;
     }
   }
 
-  RawIterator<CodecBuffer> newCodecBufferTableIterator(
-      TableIterator<CodecBuffer, KeyValue<CodecBuffer, CodecBuffer>> i) {
+  private RawIterator<CodecBuffer> newCodecBufferTableIterator(KeyValueIterator<CodecBuffer, CodecBuffer> i) {
     return new RawIterator<CodecBuffer>(i) {
       @Override
-      AutoCloseSupplier<CodecBuffer> convert(KEY key) throws IOException {
+      AutoCloseSupplier<CodecBuffer> convert(KEY key) throws CodecException {
         final CodecBuffer buffer = encodeKeyCodecBuffer(key);
         return new AutoCloseSupplier<CodecBuffer>() {
           @Override
@@ -600,12 +526,13 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
       }
 
       @Override
-      KeyValue<KEY, VALUE> convert(KeyValue<CodecBuffer, CodecBuffer> raw)
-          throws IOException {
-        final int rawSize = raw.getValue().readableBytes();
-        final KEY key = keyCodec.fromCodecBuffer(raw.getKey());
-        final VALUE value = valueCodec.fromCodecBuffer(raw.getValue());
-        return Table.newKeyValue(key, value, rawSize);
+      KeyValue<KEY, VALUE> convert(KeyValue<CodecBuffer, CodecBuffer> raw) throws CodecException {
+        final CodecBuffer keyBuffer = raw.getKey();
+        final KEY key = keyBuffer != null ? keyCodec.fromCodecBuffer(keyBuffer) : null;
+
+        final CodecBuffer valueBuffer = raw.getValue();
+        return valueBuffer == null ? Table.newKeyValue(key, null)
+            : Table.newKeyValue(key, valueCodec.fromCodecBuffer(valueBuffer), valueBuffer.readableBytes());
       }
     };
   }
@@ -614,20 +541,22 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
    * Table Iterator implementation for strongly typed tables.
    */
   public class TypedTableIterator extends RawIterator<byte[]> {
-    TypedTableIterator(
-        TableIterator<byte[], KeyValue<byte[], byte[]>> rawIterator) {
+    TypedTableIterator(KeyValueIterator<byte[], byte[]> rawIterator) {
       super(rawIterator);
     }
 
     @Override
-    AutoCloseSupplier<byte[]> convert(KEY key) throws IOException {
+    AutoCloseSupplier<byte[]> convert(KEY key) throws CodecException {
       final byte[] keyArray = encodeKey(key);
       return () -> keyArray;
     }
 
     @Override
-    KeyValue<KEY, VALUE> convert(KeyValue<byte[], byte[]> raw) {
-      return new TypedKeyValue(raw);
+    KeyValue<KEY, VALUE> convert(KeyValue<byte[], byte[]> raw) throws CodecException {
+      final KEY key = decodeKey(raw.getKey());
+      final byte[] valueBytes = raw.getValue();
+      return valueBytes == null ? Table.newKeyValue(key, null)
+          : Table.newKeyValue(key, decodeValue(valueBytes), valueBytes.length);
     }
   }
 
@@ -638,21 +567,20 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
    */
   abstract class RawIterator<RAW>
       implements Table.KeyValueIterator<KEY, VALUE> {
-    private final TableIterator<RAW, KeyValue<RAW, RAW>> rawIterator;
+    private final KeyValueIterator<RAW, RAW> rawIterator;
 
-    RawIterator(TableIterator<RAW, KeyValue<RAW, RAW>> rawIterator) {
+    RawIterator(KeyValueIterator<RAW, RAW> rawIterator) {
       this.rawIterator = rawIterator;
     }
 
     /** Covert the given key to the {@link RAW} type. */
-    abstract AutoCloseSupplier<RAW> convert(KEY key) throws IOException;
+    abstract AutoCloseSupplier<RAW> convert(KEY key) throws CodecException;
 
     /**
      * Covert the given {@link Table.KeyValue}
      * from ({@link RAW}, {@link RAW}) to ({@link KEY}, {@link VALUE}).
      */
-    abstract KeyValue<KEY, VALUE> convert(KeyValue<RAW, RAW> raw)
-        throws IOException;
+    abstract KeyValue<KEY, VALUE> convert(KeyValue<RAW, RAW> raw) throws CodecException;
 
     @Override
     public void seekToFirst() {
@@ -665,7 +593,7 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
     }
 
     @Override
-    public KeyValue<KEY, VALUE> seek(KEY key) throws IOException {
+    public KeyValue<KEY, VALUE> seek(KEY key) throws RocksDatabaseException, CodecException {
       try (AutoCloseSupplier<RAW> rawKey = convert(key)) {
         final KeyValue<RAW, RAW> result = rawIterator.seek(rawKey.get());
         return result == null ? null : convert(result);
@@ -673,7 +601,7 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
     }
 
     @Override
-    public void close() throws IOException {
+    public void close() throws RocksDatabaseException {
       rawIterator.close();
     }
 
@@ -686,13 +614,13 @@ public class TypedTable<KEY, VALUE> implements Table<KEY, VALUE> {
     public KeyValue<KEY, VALUE> next() {
       try {
         return convert(rawIterator.next());
-      } catch (IOException e) {
-        throw new IllegalStateException("Failed next()", e);
+      } catch (CodecException e) {
+        throw new IllegalStateException("Failed next() in " + TypedTable.this, e);
       }
     }
 
     @Override
-    public void removeFromDB() throws IOException {
+    public void removeFromDB() throws RocksDatabaseException, CodecException {
       rawIterator.removeFromDB();
     }
   }
