@@ -50,7 +50,7 @@ import org.slf4j.LoggerFactory;
 @Evolving
 public class FullTableCache<KEY, VALUE> implements TableCache<KEY, VALUE> {
 
-  public static final Logger LOG =
+  private static final Logger LOG =
       LoggerFactory.getLogger(FullTableCache.class);
 
   private final Map<CacheKey<KEY>, CacheValue<VALUE>> cache;
@@ -62,7 +62,6 @@ public class FullTableCache<KEY, VALUE> implements TableCache<KEY, VALUE> {
 
   private final CacheStatsRecorder statsRecorder;
 
-
   public FullTableCache(String threadNamePrefix) {
     // As for full table cache only we need elements to be inserted in sorted
     // manner, so that list will be easy. But look ups have log(N) time
