@@ -46,7 +46,7 @@ import org.slf4j.LoggerFactory;
 @Evolving
 public class PartialTableCache<KEY, VALUE> implements TableCache<KEY, VALUE> {
 
-  public static final Logger LOG =
+  private static final Logger LOG =
       LoggerFactory.getLogger(PartialTableCache.class);
 
   private final Map<CacheKey<KEY>, CacheValue<VALUE>> cache;
@@ -54,7 +54,6 @@ public class PartialTableCache<KEY, VALUE> implements TableCache<KEY, VALUE> {
   private final ExecutorService executorService;
   private final CacheStatsRecorder statsRecorder;
 
-
   public PartialTableCache(String threadNamePrefix) {
     // We use concurrent Hash map for O(1) lookup for get API.
     // During list operation for partial cache we anyway merge between DB and
