@@ -42,6 +42,7 @@ import org.apache.hadoop.hdds.security.x509.certificate.utils.CertificateCodec;
 import org.apache.hadoop.hdds.security.x509.certificate.utils.SelfSignedCertificate;
 import org.apache.hadoop.security.ssl.KeyStoreTestUtil;
 import org.apache.ozone.test.GenericTestUtils;
+import org.apache.ozone.test.GenericTestUtils.LogCapturer;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.mockito.Mock;
@@ -53,7 +54,7 @@ import org.mockito.MockitoAnnotations;
 public class TestRootCaRotationPoller {
 
   private SecurityConfig secConf;
-  private GenericTestUtils.LogCapturer logCapturer;
+  private LogCapturer logCapturer;
 
   @Mock
   private SCMSecurityProtocolClientSideTranslatorPB scmSecurityClient;
@@ -64,8 +65,7 @@ public class TestRootCaRotationPoller {
     OzoneConfiguration conf = new OzoneConfiguration();
     conf.set(HDDS_X509_ROOTCA_CERTIFICATE_POLLING_INTERVAL, "PT1s");
     secConf = new SecurityConfig(conf);
-    logCapturer = GenericTestUtils.LogCapturer.captureLogs(
-        org.slf4j.LoggerFactory.getLogger(RootCaRotationPoller.class));
+    logCapturer = LogCapturer.captureLogs(RootCaRotationPoller.class);
   }
 
   @Test
