@@ -18,28 +18,26 @@
 package org.apache.hadoop.hdds.utils.db;
 
 import static org.apache.hadoop.hdds.utils.db.DBConfigFromFile.getOptionsFileNameFromDB;
+import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.io.File;
 import java.io.IOException;
-import java.nio.charset.StandardCharsets;
 import java.nio.file.Path;
 import java.nio.file.Paths;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.List;
 import org.apache.commons.io.FileUtils;
-import org.apache.hadoop.hdds.StringUtils;
+import org.apache.hadoop.hdds.utils.db.managed.ManagedColumnFamilyOptions;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
-import org.rocksdb.ColumnFamilyDescriptor;
-import org.rocksdb.ColumnFamilyOptions;
+import org.rocksdb.CompactionStyle;
 import org.rocksdb.DBOptions;
-import org.rocksdb.RocksDB;
+import org.rocksdb.RocksDBException;
 
 /**
  * DBConf tests.
@@ -63,22 +61,8 @@ public class TestDBConfigFromFile {
   }
 
   @Test
-  public void readFromFile() throws IOException {
-    final List<String> families =
-        Arrays.asList(StringUtils.bytes2String(RocksDB.DEFAULT_COLUMN_FAMILY),
-            "First", "Second", "Third",
-            "Fourth", "Fifth",
-            "Sixth");
-    final List<ColumnFamilyDescriptor> columnFamilyDescriptors =
-        new ArrayList<>();
-    for (String family : families) {
-      columnFamilyDescriptors.add(
-          new ColumnFamilyDescriptor(family.getBytes(StandardCharsets.UTF_8),
-              new ColumnFamilyOptions()));
-    }
-
-    final DBOptions options = DBConfigFromFile.readFromFile(DB_FILE,
-        columnFamilyDescriptors);
+  public void readFromFile() throws RocksDBException {
+    final DBOptions options = DBConfigFromFile.readDBOptionsFromFile(Paths.get(DB_FILE));
 
     // Some Random Values Defined in the test.db.ini, we verify that we are
     // able to get values that are defined in the test.db.ini.
@@ -89,24 +73,38 @@ public class TestDBConfigFromFile {
   }
 
   @Test
-  public void readFromFileInvalidConfig() throws IOException {
-    final List<String> families =
-        Arrays.asList(StringUtils.bytes2String(RocksDB.DEFAULT_COLUMN_FAMILY),
-            "First", "Second", "Third",
-            "Fourth", "Fifth",
-            "Sixth");
-    final List<ColumnFamilyDescriptor> columnFamilyDescriptors =
-        new ArrayList<>();
-    for (String family : families) {
-      columnFamilyDescriptors.add(
-          new ColumnFamilyDescriptor(family.getBytes(StandardCharsets.UTF_8),
-              new ColumnFamilyOptions()));
-    }
-
-    final DBOptions options = DBConfigFromFile.readFromFile("badfile.db.ini",
-        columnFamilyDescriptors);
-
+  public void readFromNonExistentFile() throws RocksDBException {
+    final DBOptions options = DBConfigFromFile.readDBOptionsFromFile(Paths.get("nonExistent.db.ini"));
     // This has to return a Null, since we have config defined for badfile.db
     assertNull(options);
   }
+
+  @Test
+  public void readFromEmptyFilePath() throws RocksDBException {
+    final DBOptions options = DBConfigFromFile.readDBOptionsFromFile(Paths.get(""));
+    // This has to return a Null, since the path is empty.
+    assertNull(options);
+  }
+
+  @Test
+  public void readFromEmptyFile() throws IOException {
+    File emptyFile = new File(Paths.get(System.getProperty(DBConfigFromFile.CONFIG_DIR)).toString(), "empty.ini");
+    assertTrue(emptyFile.createNewFile());
+    RocksDBException thrownException =
+        assertThrows(RocksDBException.class, () -> DBConfigFromFile.readDBOptionsFromFile(emptyFile.toPath()));
+    assertThat(thrownException.getMessage()).contains("A RocksDB Option file must have a single DBOptions section");
+  }
+
+  @Test
+  public void readColumnFamilyOptionsFromFile() throws RocksDBException {
+    ManagedColumnFamilyOptions managedColumnFamily = DBConfigFromFile.readCFOptionsFromFile(
+        Paths.get(DB_FILE), "default");
+    assertNotNull(managedColumnFamily);
+    assertEquals(134217728, managedColumnFamily.writeBufferSize());
+    assertEquals(6, managedColumnFamily.numLevels());
+    assertEquals(268435456, managedColumnFamily.blobFileSize());
+    assertEquals("SkipListFactory", managedColumnFamily.memTableFactoryName());
+    assertEquals(CompactionStyle.LEVEL, managedColumnFamily.compactionStyle());
+    assertEquals(16777216, managedColumnFamily.arenaBlockSize());
+  }
 }
