@@ -19,7 +19,6 @@ package org.apache.hadoop.hdds.utils.db;
 
 import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
-import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 import static org.junit.jupiter.api.Assertions.assertNotEquals;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
@@ -97,15 +96,12 @@ public class TestDBStoreBuilder {
         .build();
     // Building should succeed without error.
 
-    try (Table<byte[], byte[]> firstTable = dbStore.getTable("FIRST")) {
-      byte[] key =
-          RandomStringUtils.random(9).getBytes(StandardCharsets.UTF_8);
-      byte[] value =
-          RandomStringUtils.random(9).getBytes(StandardCharsets.UTF_8);
-      firstTable.put(key, value);
-      byte[] temp = firstTable.get(key);
-      assertArrayEquals(value, temp);
-    }
+    final Table<byte[], byte[]> firstTable = dbStore.getTable("FIRST");
+    byte[] key = RandomStringUtils.secure().next(9).getBytes(StandardCharsets.UTF_8);
+    byte[] value = RandomStringUtils.secure().next(9).getBytes(StandardCharsets.UTF_8);
+    firstTable.put(key, value);
+    byte[] temp = firstTable.get(key);
+    assertArrayEquals(value, temp);
 
     dbStore.close();
   }
@@ -119,19 +115,16 @@ public class TestDBStoreBuilder {
         .addTable("First")
         .addTable("Second")
         .build()) {
-      try (Table<byte[], byte[]> firstTable = dbStore.getTable("First")) {
-        byte[] key =
-            RandomStringUtils.random(9).getBytes(StandardCharsets.UTF_8);
-        byte[] value =
-            RandomStringUtils.random(9).getBytes(StandardCharsets.UTF_8);
-        firstTable.put(key, value);
-        byte[] temp = firstTable.get(key);
-        assertArrayEquals(value, temp);
-      }
+      final Table<byte[], byte[]> firstTable = dbStore.getTable("First");
+      byte[] key = RandomStringUtils.secure().next(9).getBytes(StandardCharsets.UTF_8);
+      byte[] value = RandomStringUtils.secure().next(9).getBytes(StandardCharsets.UTF_8);
+      firstTable.put(key, value);
+      byte[] temp = firstTable.get(key);
+      assertArrayEquals(value, temp);
 
-      try (Table secondTable = dbStore.getTable("Second")) {
-        assertTrue(secondTable.isEmpty());
-      }
+
+      final Table<byte[], byte[]> secondTable = dbStore.getTable("Second");
+      assertTrue(secondTable.isEmpty());
     }
   }
 
@@ -146,19 +139,15 @@ public class TestDBStoreBuilder {
         .addTable("Second")
         .setProfile(DBProfile.DISK)
         .build()) {
-      try (Table<byte[], byte[]> firstTable = dbStore.getTable("First")) {
-        byte[] key =
-            RandomStringUtils.random(9).getBytes(StandardCharsets.UTF_8);
-        byte[] value =
-            RandomStringUtils.random(9).getBytes(StandardCharsets.UTF_8);
-        firstTable.put(key, value);
-        byte[] temp = firstTable.get(key);
-        assertArrayEquals(value, temp);
-      }
+      Table<byte[], byte[]> firstTable = dbStore.getTable("First");
+      byte[] key = RandomStringUtils.secure().next(9).getBytes(StandardCharsets.UTF_8);
+      byte[] value = RandomStringUtils.secure().next(9).getBytes(StandardCharsets.UTF_8);
+      firstTable.put(key, value);
+      byte[] temp = firstTable.get(key);
+      assertArrayEquals(value, temp);
 
-      try (Table secondTable = dbStore.getTable("Second")) {
-        assertTrue(secondTable.isEmpty());
-      }
+      Table<byte[], byte[]> secondTable = dbStore.getTable("Second");
+      assertTrue(secondTable.isEmpty());
     }
   }
 
@@ -203,11 +192,7 @@ public class TestDBStoreBuilder {
       }
     };
 
-    try (DBStore dbStore = DBStoreBuilder.newBuilder(conf, sampleDB)
-        .setName("SampleStore").setPath(newFolder.toPath()).build()) {
-      assertInstanceOf(RDBStore.class, dbStore);
-
-      RDBStore rdbStore = (RDBStore) dbStore;
+    try (RDBStore rdbStore = DBStoreBuilder.newBuilder(conf, sampleDB, "SampleStore", newFolder.toPath()).build()) {
       Collection<RocksDatabase.ColumnFamily> cfFamilies =
           rdbStore.getColumnFamilies();
 
@@ -267,13 +252,9 @@ public class TestDBStoreBuilder {
       }
     };
 
-    try (DBStore dbStore = DBStoreBuilder.newBuilder(conf, sampleDB)
-            .setName("SampleStore")
-            .disableDefaultCFAutoCompaction(disableAutoCompaction)
-            .setPath(newFolder.toPath()).build()) {
-      assertInstanceOf(RDBStore.class, dbStore);
-
-      RDBStore rdbStore = (RDBStore) dbStore;
+    try (RDBStore rdbStore = DBStoreBuilder.newBuilder(conf, sampleDB, "SampleStore", newFolder.toPath())
+        .disableDefaultCFAutoCompaction(disableAutoCompaction)
+        .build()) {
       Collection<RocksDatabase.ColumnFamily> cfFamilies =
               rdbStore.getColumnFamilies();
 
