@@ -23,7 +23,6 @@ import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
-import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
@@ -56,14 +55,6 @@ import org.rocksdb.StatsLevel;
  * RDBStore Tests.
  */
 public class TestRDBStore {
-  public static RDBStore newRDBStore(File dbFile, ManagedDBOptions options,
-      Set<TableConfig> families,
-      long maxDbUpdatesSizeThreshold)
-      throws IOException {
-    return new RDBStore(dbFile, options, null, new ManagedWriteOptions(), families,
-        CodecRegistry.newBuilder().build(), false, 1000, null, false,
-        maxDbUpdatesSizeThreshold, true, null, true);
-  }
 
   public static final int MAX_DB_UPDATES_SIZE_THRESHOLD = 80;
   private final List<String> families =
@@ -72,20 +63,39 @@ public class TestRDBStore {
           "Fourth", "Fifth",
           "Sixth");
   private RDBStore rdbStore = null;
-  private ManagedDBOptions options = null;
+  private ManagedDBOptions options;
   private Set<TableConfig> configSet;
 
-  @BeforeEach
-  public void setUp(@TempDir File tempDir) throws Exception {
-    CodecBuffer.enableLeakDetection();
-
-    options = new ManagedDBOptions();
+  static ManagedDBOptions newManagedDBOptions() {
+    final ManagedDBOptions options = new ManagedDBOptions();
     options.setCreateIfMissing(true);
     options.setCreateMissingColumnFamilies(true);
 
     Statistics statistics = new Statistics();
     statistics.setStatsLevel(StatsLevel.ALL);
     options.setStatistics(statistics);
+    return options;
+  }
+
+  static RDBStore newRDBStore(File dbFile, ManagedDBOptions options, Set<TableConfig> families)
+      throws IOException {
+    return newRDBStore(dbFile, options, families, MAX_DB_UPDATES_SIZE_THRESHOLD);
+  }
+
+  public static RDBStore newRDBStore(File dbFile, ManagedDBOptions options,
+      Set<TableConfig> families,
+      long maxDbUpdatesSizeThreshold)
+      throws IOException {
+    return new RDBStore(dbFile, options, null, new ManagedWriteOptions(), families,
+        false, null, false,
+        maxDbUpdatesSizeThreshold, true, null, true);
+  }
+
+  @BeforeEach
+  public void setUp(@TempDir File tempDir) throws Exception {
+    CodecBuffer.enableLeakDetection();
+
+    options = newManagedDBOptions();
     configSet = new HashSet<>();
     for (String name : families) {
       TableConfig newConfig = new TableConfig(name,
@@ -106,18 +116,14 @@ public class TestRDBStore {
 
   public void insertRandomData(RDBStore dbStore, int familyIndex)
       throws IOException {
-    try (Table<byte[], byte[]> firstTable = dbStore.getTable(families.
-        get(familyIndex))) {
-      assertNotNull(firstTable, "Table cannot be null");
-      for (int x = 0; x < 100; x++) {
-        byte[] key =
-          RandomStringUtils.random(10).getBytes(StandardCharsets.UTF_8);
-        byte[] value =
-          RandomStringUtils.random(10).getBytes(StandardCharsets.UTF_8);
-        firstTable.put(key, value);
-      }
-    } catch (Exception e) {
-      throw new IOException(e);
+    Table<byte[], byte[]> firstTable = dbStore.getTable(families.get(familyIndex));
+    assertNotNull(firstTable, "Table cannot be null");
+    for (int x = 0; x < 100; x++) {
+      byte[] key =
+          RandomStringUtils.secure().next(10).getBytes(StandardCharsets.UTF_8);
+      byte[] value =
+          RandomStringUtils.secure().next(10).getBytes(StandardCharsets.UTF_8);
+      firstTable.put(key, value);
     }
   }
 
@@ -140,6 +146,23 @@ public class TestRDBStore {
 
   }
 
+  @Test
+  public void compactTable() throws Exception {
+    assertNotNull(rdbStore, "DBStore cannot be null");
+
+    for (int j = 0; j <= 20; j++) {
+      insertRandomData(rdbStore, 0);
+      rdbStore.flushDB();
+    }
+
+    int metaSizeBeforeCompact = rdbStore.getDb().getLiveFilesMetaDataSize();
+    rdbStore.compactTable(StringUtils.bytes2String(RocksDB.DEFAULT_COLUMN_FAMILY));
+    int metaSizeAfterCompact = rdbStore.getDb().getLiveFilesMetaDataSize();
+
+    assertThat(metaSizeAfterCompact).isLessThan(metaSizeBeforeCompact);
+    assertThat(metaSizeAfterCompact).isEqualTo(1);
+  }
+
   @Test
   public void close() throws Exception {
     assertNotNull(rdbStore, "DBStore cannot be null");
@@ -156,54 +179,6 @@ public class TestRDBStore {
     assertTrue(rdbStore.isClosed());
   }
 
-  @Test
-  public void moveKey() throws Exception {
-    byte[] key =
-        RandomStringUtils.random(10).getBytes(StandardCharsets.UTF_8);
-    byte[] value =
-        RandomStringUtils.random(10).getBytes(StandardCharsets.UTF_8);
-
-    try (Table firstTable = rdbStore.getTable(families.get(1))) {
-      firstTable.put(key, value);
-      try (Table<byte[], byte[]> secondTable = rdbStore
-          .getTable(families.get(2))) {
-        rdbStore.move(key, firstTable, secondTable);
-        byte[] newvalue = secondTable.get(key);
-        // Make sure we have value in the second table
-        assertNotNull(newvalue);
-        //and it is same as what we wrote to the FirstTable
-        assertArrayEquals(value, newvalue);
-      }
-      // After move this key must not exist in the first table.
-      assertNull(firstTable.get(key));
-    }
-  }
-
-  @Test
-  public void moveWithValue() throws Exception {
-    byte[] key =
-        RandomStringUtils.random(10).getBytes(StandardCharsets.UTF_8);
-    byte[] value =
-        RandomStringUtils.random(10).getBytes(StandardCharsets.UTF_8);
-
-    byte[] nextValue =
-        RandomStringUtils.random(10).getBytes(StandardCharsets.UTF_8);
-    try (Table firstTable = rdbStore.getTable(families.get(1))) {
-      firstTable.put(key, value);
-      try (Table<byte[], byte[]> secondTable = rdbStore
-          .getTable(families.get(2))) {
-        rdbStore.move(key, nextValue, firstTable, secondTable);
-        byte[] newvalue = secondTable.get(key);
-        // Make sure we have value in the second table
-        assertNotNull(newvalue);
-        //and it is not same as what we wrote to the FirstTable, and equals
-        // the new value.
-        assertArrayEquals(nextValue, newvalue);
-      }
-    }
-
-  }
-
   @Test
   public void getEstimatedKeyCount() throws Exception {
     assertNotNull(rdbStore, "DB Store cannot be null");
@@ -222,9 +197,8 @@ public class TestRDBStore {
   @Test
   public void getTable() throws Exception {
     for (String tableName : families) {
-      try (Table table = rdbStore.getTable(tableName)) {
-        assertNotNull(table, tableName + "is null");
-      }
+      Table<byte[], byte[]> table = rdbStore.getTable(tableName);
+      assertNotNull(table, tableName + "is null");
     }
     assertThrows(IOException.class,
         () -> rdbStore.getTable("ATableWithNoName"));
@@ -232,7 +206,7 @@ public class TestRDBStore {
 
   @Test
   public void listTables() throws Exception {
-    List<Table> tableList = rdbStore.listTables();
+    final List<Table<?, ?>> tableList = rdbStore.listTables();
     assertNotNull(tableList, "Table list cannot be null");
     Map<String, Table> hashTable = new HashMap<>();
 
@@ -287,48 +261,42 @@ public class TestRDBStore {
 
   @Test
   public void testGetDBUpdatesSince() throws Exception {
-
-    try (Table firstTable = rdbStore.getTable(families.get(1))) {
-      firstTable.put(
-          org.apache.commons.codec.binary.StringUtils.getBytesUtf16("Key1"),
-          org.apache.commons.codec.binary.StringUtils
-              .getBytesUtf16("Value1"));
-      firstTable.put(
-          org.apache.commons.codec.binary.StringUtils.getBytesUtf16("Key2"),
-          org.apache.commons.codec.binary.StringUtils
-              .getBytesUtf16("Value2"));
-    }
+    final Table<byte[], byte[]> firstTable = rdbStore.getTable(families.get(1));
+    firstTable.put(
+        getBytesUtf16("Key1"),
+        getBytesUtf16("Value1"));
+    firstTable.put(
+        getBytesUtf16("Key2"),
+        getBytesUtf16("Value2"));
     assertEquals(2, rdbStore.getDb().getLatestSequenceNumber());
 
     DBUpdatesWrapper dbUpdatesSince = rdbStore.getUpdatesSince(0);
     assertEquals(2, dbUpdatesSince.getData().size());
   }
 
+  static byte[] getBytesUtf16(String s) {
+    return s.getBytes(StandardCharsets.UTF_16);
+  }
+
   @Test
   public void testGetDBUpdatesSinceWithLimitCount() throws Exception {
 
-    try (Table firstTable = rdbStore.getTable(families.get(1))) {
-      firstTable.put(
-          org.apache.commons.codec.binary.StringUtils.getBytesUtf16("Key1"),
-          org.apache.commons.codec.binary.StringUtils
-              .getBytesUtf16("Value1"));
-      firstTable.put(
-          org.apache.commons.codec.binary.StringUtils.getBytesUtf16("Key2"),
-          org.apache.commons.codec.binary.StringUtils
-              .getBytesUtf16("Value2"));
-      firstTable.put(
-          org.apache.commons.codec.binary.StringUtils.getBytesUtf16("Key3"),
-          org.apache.commons.codec.binary.StringUtils
-              .getBytesUtf16("Value3"));
-      firstTable.put(
-          org.apache.commons.codec.binary.StringUtils.getBytesUtf16("Key4"),
-          org.apache.commons.codec.binary.StringUtils
-              .getBytesUtf16("Value4"));
-      firstTable.put(
-          org.apache.commons.codec.binary.StringUtils.getBytesUtf16("Key5"),
-          org.apache.commons.codec.binary.StringUtils
-              .getBytesUtf16("Value5"));
-    }
+    final Table<byte[], byte[]> firstTable = rdbStore.getTable(families.get(1));
+    firstTable.put(
+        getBytesUtf16("Key1"),
+        getBytesUtf16("Value1"));
+    firstTable.put(
+        getBytesUtf16("Key2"),
+        getBytesUtf16("Value2"));
+    firstTable.put(
+        getBytesUtf16("Key3"),
+        getBytesUtf16("Value3"));
+    firstTable.put(
+        getBytesUtf16("Key4"),
+        getBytesUtf16("Value4"));
+    firstTable.put(
+        getBytesUtf16("Key5"),
+        getBytesUtf16("Value5"));
     assertEquals(5, rdbStore.getDb().getLatestSequenceNumber());
 
     DBUpdatesWrapper dbUpdatesSince = rdbStore.getUpdatesSince(0, 5);
@@ -342,12 +310,10 @@ public class TestRDBStore {
     // Write data to current DB which has 6 column families at the time of
     // writing this test.
     for (String family : families) {
-      try (Table table = rdbStore.getTable(family)) {
-        byte[] key = family.getBytes(StandardCharsets.UTF_8);
-        byte[] value =
-            RandomStringUtils.random(10).getBytes(StandardCharsets.UTF_8);
-        table.put(key, value);
-      }
+      final Table<byte[], byte[]> table = rdbStore.getTable(family);
+      byte[] key = family.getBytes(StandardCharsets.UTF_8);
+      byte[] value = RandomStringUtils.secure().next(10).getBytes(StandardCharsets.UTF_8);
+      table.put(key, value);
     }
     // Close current DB.
     rdbStore.close();
@@ -366,21 +332,19 @@ public class TestRDBStore {
     rdbStore = newRDBStore(rdbStore.getDbLocation(), options, configSet,
         MAX_DB_UPDATES_SIZE_THRESHOLD);
     for (String family : familiesMinusOne) {
-      try (Table table = rdbStore.getTable(family)) {
-        assertNotNull(table, family + "is null");
-        Object val = table.get(family.getBytes(StandardCharsets.UTF_8));
-        assertNotNull(val);
-      }
+      final Table<byte[], byte[]> table = rdbStore.getTable(family);
+      assertNotNull(table, family + "is null");
+      Object val = table.get(family.getBytes(StandardCharsets.UTF_8));
+      assertNotNull(val);
     }
 
     // Technically the extra column family should also be open, even though
     // we do not use it.
     String extraFamily = families.get(families.size() - 1);
-    try (Table table = rdbStore.getTable(extraFamily)) {
-      assertNotNull(table, extraFamily + "is null");
-      Object val = table.get(extraFamily.getBytes(StandardCharsets.UTF_8));
-      assertNotNull(val);
-    }
+    final Table<byte[], byte[]> table = rdbStore.getTable(extraFamily);
+    assertNotNull(table, extraFamily + "is null");
+    Object val = table.get(extraFamily.getBytes(StandardCharsets.UTF_8));
+    assertNotNull(val);
   }
 
   @Test
