@@ -177,7 +177,7 @@ public class TestOzoneAuditLogger {
     assertThat(message).contains(USER);
     assertThat(message).contains(IP_ADDRESS);
     assertThat(message).contains(DummyAction.CREATE_VOLUME.name());
-    assertThat(message).contains(PARAMS.toString());
+    assertThat(message).contains(PARAMS.values());
     assertThat(message).contains(FAILURE.getStatus());
   }
 
@@ -214,7 +214,7 @@ public class TestOzoneAuditLogger {
     verifyLog(
         "ERROR | OMAudit | ? | user=john | "
             + "ip=192.168.0.1 | op=CREATE_VOLUME "
-            + "{key1=value1, key2=value2} | ret=FAILURE",
+            + "{\"key1\":\"value1\",\"key2\":\"value2\"} | ret=FAILURE",
         "org.apache.hadoop.ozone.audit."
             + "TestOzoneAuditLogger$TestException: Dummy exception message",
         "at org.apache.hadoop.ozone.audit.TestOzoneAuditLogger"
