@@ -1,11 +1,10 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- *  with the License.  You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -14,41 +13,38 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
  */
+
 package org.apache.hadoop.hdds.utils.db.managed;
 
+import jakarta.annotation.Nullable;
+import java.io.File;
+import java.time.Duration;
 import org.apache.hadoop.hdds.HddsUtils;
+import org.apache.hadoop.hdds.ratis.RatisHelper;
 import org.apache.hadoop.hdds.utils.LeakDetector;
+import org.apache.hadoop.hdds.utils.db.RocksDatabaseException;
 import org.apache.ratis.util.UncheckedAutoCloseable;
-import org.awaitility.Awaitility;
-import org.awaitility.core.ConditionTimeoutException;
 import org.rocksdb.RocksDB;
+import org.rocksdb.util.Environment;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import javax.annotation.Nullable;
-import java.io.File;
-import java.io.IOException;
-import java.time.Duration;
-import java.time.Instant;
-
 /**
  * Utilities to help assert RocksObject closures.
  */
 public final class ManagedRocksObjectUtils {
-  private ManagedRocksObjectUtils() {
-  }
 
-  public static final Logger LOG =
+  static final Logger LOG =
       LoggerFactory.getLogger(ManagedRocksObjectUtils.class);
 
-  private static final Duration POLL_DELAY_DURATION = Duration.ZERO;
   private static final Duration POLL_INTERVAL_DURATION = Duration.ofMillis(100);
 
-
   private static final LeakDetector LEAK_DETECTOR = new LeakDetector("ManagedRocksObject");
 
+  private ManagedRocksObjectUtils() {
+  }
+
   static UncheckedAutoCloseable track(AutoCloseable object) {
     ManagedRocksObjectMetrics.INSTANCE.increaseManagedObject();
     final Class<?> clazz = object.getClass();
@@ -58,12 +54,7 @@ public final class ManagedRocksObjectUtils {
 
   static void reportLeak(Class<?> clazz, String stackTrace) {
     ManagedRocksObjectMetrics.INSTANCE.increaseLeakObject();
-    String warning = String.format("%s is not closed properly", clazz.getSimpleName());
-    if (stackTrace != null && LOG.isDebugEnabled()) {
-      String debugMessage = String.format("%nStackTrace for unclosed instance: %s", stackTrace);
-      warning = warning.concat(debugMessage);
-    }
-    LOG.warn(warning);
+    HddsUtils.reportLeak(clazz, stackTrace, LOG);
   }
 
   private static @Nullable StackTraceElement[] getStackTrace() {
@@ -78,40 +69,16 @@ public final class ManagedRocksObjectUtils {
    * Wait for file to be deleted.
    * @param file File to be deleted.
    * @param maxDuration poll max duration.
-   * @param interval poll interval.
-   * @param pollDelayDuration poll delay val.
-   */
-  public static void waitForFileDelete(File file, Duration maxDuration,
-                                       Duration interval,
-                                       Duration pollDelayDuration)
-      throws IOException {
-    Instant start = Instant.now();
-    try {
-      Awaitility.with().atMost(maxDuration)
-          .pollDelay(pollDelayDuration)
-          .pollInterval(interval)
-          .await()
-          .until(() -> !file.exists());
-      LOG.info("Waited for {} milliseconds for file {} deletion.",
-          Duration.between(start, Instant.now()).toMillis(),
-          file.getAbsoluteFile());
-    } catch (ConditionTimeoutException exception) {
-      LOG.info("File: {} didn't get deleted in {} secs.",
-          file.getAbsolutePath(), maxDuration.getSeconds());
-      throw new IOException(exception);
-    }
-  }
-
-  /**
-   * Wait for file to be deleted.
-   * @param file File to be deleted.
-   * @param maxDuration poll max duration.
-   * @throws IOException in case of failure.
+   * @throws RocksDatabaseException in case of failure.
    */
   public static void waitForFileDelete(File file, Duration maxDuration)
-      throws IOException {
-    waitForFileDelete(file, maxDuration, POLL_INTERVAL_DURATION,
-        POLL_DELAY_DURATION);
+      throws RocksDatabaseException {
+    if (!RatisHelper.attemptUntilTrue(() -> !file.exists(), POLL_INTERVAL_DURATION, maxDuration)) {
+      String msg = String.format("File: %s didn't get deleted in %s secs.",
+          file.getAbsolutePath(), maxDuration.getSeconds());
+      LOG.warn(msg);
+      throw new RocksDatabaseException(msg);
+    }
   }
 
   /**
@@ -122,4 +89,11 @@ public final class ManagedRocksObjectUtils {
   public static void loadRocksDBLibrary() {
     RocksDB.loadLibrary();
   }
+
+  /**
+   * Returns RocksDB library file name.
+   */
+  public static String getRocksDBLibFileName() {
+    return Environment.getJniLibraryFileName("rocksdb");
+  }
 }
