@@ -32,6 +32,21 @@ import org.slf4j.LoggerFactory;
  */
 public class ManagedRawSSTFileReader<T> implements Closeable {
 
+  private static final Logger LOG = LoggerFactory.getLogger(ManagedRawSSTFileReader.class);
+
+  private final String fileName;
+  // Native address of pointer to the object.
+  private final long nativeHandle;
+
+  public static boolean tryLoadLibrary() {
+    try {
+      loadLibrary();
+      return true;
+    } catch (NativeLibraryNotLoadedException ignored) {
+      return false;
+    }
+  }
+
   public static boolean loadLibrary() throws NativeLibraryNotLoadedException {
     ManagedRocksObjectUtils.loadRocksDBLibrary();
     if (!NativeLibraryLoader.getInstance().loadLibrary(ROCKS_TOOLS_NATIVE_LIBRARY_NAME, Arrays.asList(
@@ -41,11 +56,6 @@ public class ManagedRawSSTFileReader<T> implements Closeable {
     return true;
   }
 
-  private final String fileName;
-  // Native address of pointer to the object.
-  private final long nativeHandle;
-  private static final Logger LOG = LoggerFactory.getLogger(ManagedRawSSTFileReader.class);
-
   public ManagedRawSSTFileReader(final ManagedOptions options, final String fileName, final int readAheadSize) {
     this.fileName = fileName;
     this.nativeHandle = this.newRawSSTFileReader(options.getNativeHandle(), fileName, readAheadSize);
@@ -66,7 +76,6 @@ public class ManagedRawSSTFileReader<T> implements Closeable {
 
   private native long newRawSSTFileReader(long optionsHandle, String filePath, int readSize);
 
-
   private native long newIterator(long handle, boolean hasFrom, long fromSliceHandle, boolean hasTo,
                                   long toSliceHandle);
 
