@@ -39,10 +39,6 @@ public final class CompactionLogEntry implements
       CompactionLogEntry::getProtobuf,
       CompactionLogEntry.class);
 
-  public static Codec<CompactionLogEntry> getCodec() {
-    return CODEC;
-  }
-
   private final long dbSequenceNumber;
   private final long compactionTime;
   private final List<CompactionFileInfo> inputFileInfoList;
@@ -62,6 +58,10 @@ public final class CompactionLogEntry implements
     this.compactionReason = compactionReason;
   }
 
+  public static Codec<CompactionLogEntry> getCodec() {
+    return CODEC;
+  }
+
   public List<CompactionFileInfo> getInputFileInfoList() {
     return inputFileInfoList;
   }
@@ -129,13 +129,23 @@ public final class CompactionLogEntry implements
         inputFileInfoList, outputFileInfoList, compactionReason);
   }
 
+  public Builder toBuilder() {
+    Builder builder = new Builder(this.getDbSequenceNumber(), this.getCompactionTime(),
+        this.getInputFileInfoList(), this.getOutputFileInfoList());
+    String reason = this.getCompactionReason();
+    if (this.getCompactionReason() != null) {
+      builder.setCompactionReason(reason);
+    }
+    return builder;
+  }
+
   /**
    * Builder of CompactionLogEntry.
    */
   public static class Builder {
     private final long dbSequenceNumber;
     private final long compactionTime;
-    private final List<CompactionFileInfo> inputFileInfoList;
+    private List<CompactionFileInfo> inputFileInfoList;
     private final List<CompactionFileInfo> outputFileInfoList;
     private String compactionReason;
 
@@ -157,6 +167,11 @@ public final class CompactionLogEntry implements
       return this;
     }
 
+    public Builder updateInputFileInfoList(List<CompactionFileInfo> fileInfoList) {
+      this.inputFileInfoList = fileInfoList;
+      return this;
+    }
+
     public CompactionLogEntry build() {
       return new CompactionLogEntry(dbSequenceNumber, compactionTime,
           inputFileInfoList, outputFileInfoList, compactionReason);
