@@ -21,6 +21,7 @@ import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.util.stream.Stream;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.CompactionFileInfoProto;
@@ -41,6 +42,12 @@ public class TestCompactionFileInfo {
             "endRange",
             "columnFamily"
         ),
+        Arguments.of("Only fileName is present.",
+            "fileName",
+            null,
+            null,
+            null
+        ),
         Arguments.of("Only fileName is present.",
             "fileName",
             null,
@@ -58,10 +65,15 @@ public class TestCompactionFileInfo {
                                                   String endRange,
                                                   String columnFamily) {
 
-    CompactionFileInfo compactionFileInfo =
-        new CompactionFileInfo.Builder(fileName).setStartRange(startRange)
-            .setEndRange(endRange).setColumnFamily(columnFamily).build();
+    CompactionFileInfo.Builder builder = new CompactionFileInfo.Builder(fileName).setStartRange(startRange)
+        .setEndRange(endRange).setColumnFamily(columnFamily);
+    CompactionFileInfo compactionFileInfo = builder.build();
     assertNotNull(compactionFileInfo);
+    CompactionFileInfo prunedCompactionFileInfo = builder.setPruned().build();
+    assertFalse(compactionFileInfo.isPruned());
+    compactionFileInfo.setPruned();
+    assertTrue(compactionFileInfo.isPruned());
+    assertTrue(prunedCompactionFileInfo.isPruned());
   }
 
   private static Stream<Arguments> compactionFileInfoInvalidScenarios() {
@@ -215,5 +227,16 @@ public class TestCompactionFileInfo {
     assertEquals(startRange, compactionFileInfo.getStartKey());
     assertEquals(endRange, compactionFileInfo.getEndKey());
     assertEquals(columnFamily, compactionFileInfo.getColumnFamily());
+    assertFalse(compactionFileInfo.isPruned());
+
+    CompactionFileInfoProto unPrunedProtobuf = builder.setPruned(false).build();
+    CompactionFileInfo unPrunedCompactionFileInfo =
+        CompactionFileInfo.getFromProtobuf(unPrunedProtobuf);
+    assertFalse(unPrunedCompactionFileInfo.isPruned());
+
+    CompactionFileInfoProto prunedProtobuf = builder.setPruned(true).build();
+    CompactionFileInfo prunedCompactionFileInfo =
+        CompactionFileInfo.getFromProtobuf(prunedProtobuf);
+    assertTrue(prunedCompactionFileInfo.isPruned());
   }
 }
