@@ -17,29 +17,29 @@
 
 package org.apache.hadoop.hdds.scm.block;
 
-import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_BLOCK_DELETION_MAX_RETRY;
-import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_BLOCK_DELETION_MAX_RETRY_DEFAULT;
+import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_BLOCK_DELETION_PER_DN_DISTRIBUTION_FACTOR;
+import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_BLOCK_DELETION_PER_DN_DISTRIBUTION_FACTOR_DEFAULT;
 import static org.apache.hadoop.hdds.scm.block.SCMDeletedBlockTransactionStatusManager.SCMDeleteBlocksCommandStatusManager.CmdStatus;
 import static org.apache.hadoop.hdds.scm.ha.SequenceIdGenerator.DEL_TXN_ID;
 
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.collect.Lists;
 import java.io.IOException;
 import java.time.Duration;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.UUID;
-import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
 import java.util.stream.Collectors;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
+import org.apache.hadoop.hdds.conf.StorageUnit;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.protocol.DatanodeID;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.CommandStatus;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerBlocksDeletionACKProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.DeletedBlocksTransaction;
+import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.command.CommandStatusReportHandler.DeleteBlockStatus;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
@@ -55,13 +55,12 @@ import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
 import org.apache.hadoop.hdds.server.events.EventHandler;
 import org.apache.hadoop.hdds.server.events.EventPublisher;
 import org.apache.hadoop.hdds.utils.db.Table;
-import org.apache.hadoop.hdds.utils.db.TableIterator;
 import org.apache.hadoop.ozone.protocol.commands.SCMCommand;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
- * A implement class of {@link DeletedBlockLog}, and it uses
+ * An implement class of {@link DeletedBlockLog}, and it uses
  * K/V db to maintain block deletion transactions between scm and datanode.
  * This is a very basic implementation, it simply scans the log and
  * memorize the position that scanned by last time, and uses this to
@@ -73,15 +72,14 @@ import org.slf4j.LoggerFactory;
 public class DeletedBlockLogImpl
     implements DeletedBlockLog, EventHandler<DeleteBlockStatus> {
 
-  public static final Logger LOG =
+  private static final Logger LOG =
       LoggerFactory.getLogger(DeletedBlockLogImpl.class);
 
-  private final int maxRetry;
   private final ContainerManager containerManager;
   private final Lock lock;
   // The access to DeletedBlocksTXTable is protected by
   // DeletedBlockLogStateManager.
-  private final DeletedBlockLogStateManager deletedBlockLogStateManager;
+  private DeletedBlockLogStateManager deletedBlockLogStateManager;
   private final SCMContext scmContext;
   private final SequenceIdGenerator sequenceIdGen;
   private final ScmBlockDeletingServiceMetrics metrics;
@@ -89,16 +87,15 @@ public class DeletedBlockLogImpl
       transactionStatusManager;
   private long scmCommandTimeoutMs = Duration.ofSeconds(300).toMillis();
 
-  private static final int LIST_ALL_FAILED_TRANSACTIONS = -1;
   private long lastProcessedTransactionId = -1;
+  private final int logAppenderQueueByteLimit;
+  private int deletionFactorPerDatanode;
 
   public DeletedBlockLogImpl(ConfigurationSource conf,
       StorageContainerManager scm,
       ContainerManager containerManager,
       DBTransactionBuffer dbTxBuffer,
       ScmBlockDeletingServiceMetrics metrics) {
-    maxRetry = conf.getInt(OZONE_SCM_BLOCK_DELETION_MAX_RETRY,
-        OZONE_SCM_BLOCK_DELETION_MAX_RETRY_DEFAULT);
     this.containerManager = containerManager;
     this.lock = new ReentrantLock();
 
@@ -116,38 +113,25 @@ public class DeletedBlockLogImpl
     this.transactionStatusManager =
         new SCMDeletedBlockTransactionStatusManager(deletedBlockLogStateManager,
             containerManager, metrics, scmCommandTimeoutMs);
+    int limit = (int) conf.getStorageSize(
+        ScmConfigKeys.OZONE_SCM_HA_RAFT_LOG_APPENDER_QUEUE_BYTE_LIMIT,
+        ScmConfigKeys.OZONE_SCM_HA_RAFT_LOG_APPENDER_QUEUE_BYTE_LIMIT_DEFAULT,
+        StorageUnit.BYTES);
+    this.logAppenderQueueByteLimit = (int) (limit * 0.9);
+    int deletionFactor = conf.getInt(OZONE_SCM_BLOCK_DELETION_PER_DN_DISTRIBUTION_FACTOR,
+        OZONE_SCM_BLOCK_DELETION_PER_DN_DISTRIBUTION_FACTOR_DEFAULT);
+    this.deletionFactorPerDatanode = deletionFactor <= 0 ? 1 : deletionFactor;
+
   }
 
-  @Override
-  public List<DeletedBlocksTransaction> getFailedTransactions(int count,
-      long startTxId) throws IOException {
-    lock.lock();
-    try {
-      final List<DeletedBlocksTransaction> failedTXs = Lists.newArrayList();
-      try (TableIterator<Long,
-          ? extends Table.KeyValue<Long, DeletedBlocksTransaction>> iter =
-               deletedBlockLogStateManager.getReadOnlyIterator()) {
-        if (count == LIST_ALL_FAILED_TRANSACTIONS) {
-          while (iter.hasNext()) {
-            DeletedBlocksTransaction delTX = iter.next().getValue();
-            if (delTX.getCount() == -1) {
-              failedTXs.add(delTX);
-            }
-          }
-        } else {
-          iter.seek(startTxId);
-          while (iter.hasNext() && failedTXs.size() < count) {
-            DeletedBlocksTransaction delTX = iter.next().getValue();
-            if (delTX.getCount() == -1 && delTX.getTxID() >= startTxId) {
-              failedTXs.add(delTX);
-            }
-          }
-        }
-      }
-      return failedTXs;
-    } finally {
-      lock.unlock();
-    }
+  @VisibleForTesting
+  void setDeletedBlockLogStateManager(DeletedBlockLogStateManager manager) {
+    this.deletedBlockLogStateManager = manager;
+  }
+
+  @VisibleForTesting
+  void setDeleteBlocksFactorPerDatanode(int deleteBlocksFactorPerDatanode) {
+    this.deletionFactorPerDatanode = deleteBlocksFactorPerDatanode;
   }
 
   /**
@@ -159,62 +143,7 @@ public class DeletedBlockLogImpl
   @Override
   public void incrementCount(List<Long> txIDs)
       throws IOException {
-    lock.lock();
-    try {
-      transactionStatusManager.incrementRetryCount(txIDs, maxRetry);
-    } finally {
-      lock.unlock();
-    }
-  }
-
-  /**
-   * {@inheritDoc}
-   *
-   */
-  @Override
-  public int resetCount(List<Long> txIDs) throws IOException {
-    final int batchSize = 1000;
-    int totalProcessed = 0;
-
-    try {
-      if (txIDs != null && !txIDs.isEmpty()) {
-        return resetRetryCount(txIDs);
-      }
-
-      // If txIDs are null or empty, fetch all failed transactions in batches
-      long startTxId = 0;
-      List<DeletedBlocksTransaction> batch;
-
-      do {
-        // Fetch the batch of failed transactions
-        batch = getFailedTransactions(batchSize, startTxId);
-        if (batch.isEmpty()) {
-          break;
-        }
-
-        List<Long> batchTxIDs = batch.stream().map(DeletedBlocksTransaction::getTxID).collect(Collectors.toList());
-        totalProcessed += resetRetryCount(new ArrayList<>(batchTxIDs));
-        // Update startTxId to continue from the last processed transaction
-        startTxId = batch.get(batch.size() - 1).getTxID() + 1;
-      } while (!batch.isEmpty());
-
-    } catch (Exception e) {
-      throw new IOException("Error during transaction reset", e);
-    }
-    return totalProcessed;
-  }
-
-  private int resetRetryCount(List<Long> txIDs) throws IOException {
-    int totalProcessed;
-    lock.lock();
-    try {
-      transactionStatusManager.resetRetryCount(txIDs);
-      totalProcessed = deletedBlockLogStateManager.resetRetryCountOfTransactionInDB(new ArrayList<>(
-          txIDs));
-    } finally {
-      lock.unlock();
-    }
-    return totalProcessed;
+    transactionStatusManager.incrementRetryCount(txIDs);
   }
 
   private DeletedBlocksTransaction constructNewTransaction(
@@ -231,18 +160,15 @@ public class DeletedBlockLogImpl
   public int getNumOfValidTransactions() throws IOException {
     lock.lock();
     try {
-      final AtomicInteger num = new AtomicInteger(0);
-      try (TableIterator<Long,
-          ? extends Table.KeyValue<Long, DeletedBlocksTransaction>> iter =
+      int count = 0;
+      try (Table.KeyValueIterator<Long, DeletedBlocksTransaction> iter =
                deletedBlockLogStateManager.getReadOnlyIterator()) {
         while (iter.hasNext()) {
-          DeletedBlocksTransaction delTX = iter.next().getValue();
-          if (delTX.getCount() > -1) {
-            num.incrementAndGet();
-          }
+          iter.next();
+          count++;
         }
       }
-      return num.get();
+      return count;
     } finally {
       lock.unlock();
     }
@@ -285,43 +211,61 @@ public class DeletedBlockLogImpl
     lock.lock();
     try {
       ArrayList<DeletedBlocksTransaction> txsToBeAdded = new ArrayList<>();
+      long currentBatchSizeBytes = 0;
       for (Map.Entry< Long, List< Long > > entry :
           containerBlocksMap.entrySet()) {
         long nextTXID = sequenceIdGen.getNextId(DEL_TXN_ID);
         DeletedBlocksTransaction tx = constructNewTransaction(nextTXID,
             entry.getKey(), entry.getValue());
         txsToBeAdded.add(tx);
-      }
+        long txSize = tx.getSerializedSize();
+        currentBatchSizeBytes += txSize;
 
-      deletedBlockLogStateManager.addTransactionsToDB(txsToBeAdded);
-      metrics.incrBlockDeletionTransactionCreated(txsToBeAdded.size());
+        if (currentBatchSizeBytes >= logAppenderQueueByteLimit) {
+          deletedBlockLogStateManager.addTransactionsToDB(txsToBeAdded);
+          metrics.incrBlockDeletionTransactionCreated(txsToBeAdded.size());
+          txsToBeAdded.clear();
+          currentBatchSizeBytes = 0;
+        }
+      }
+      if (!txsToBeAdded.isEmpty()) {
+        deletedBlockLogStateManager.addTransactionsToDB(txsToBeAdded);
+        metrics.incrBlockDeletionTransactionCreated(txsToBeAdded.size());
+      }
     } finally {
       lock.unlock();
     }
   }
 
   @Override
-  public void close() throws IOException {
+  public void close() {
   }
 
   private void getTransaction(DeletedBlocksTransaction tx,
       DatanodeDeletedBlockTransactions transactions,
-      Set<DatanodeDetails> dnList, Set<ContainerReplica> replicas,
-      Map<UUID, Map<Long, CmdStatus>> commandStatus) {
-    DeletedBlocksTransaction updatedTxn =
-        DeletedBlocksTransaction.newBuilder(tx)
-            .setCount(transactionStatusManager.getOrDefaultRetryCount(
-              tx.getTxID(), 0))
-            .build();
+      Set<ContainerReplica> replicas,
+      Map<DatanodeID, Map<Long, CmdStatus>> commandStatus,
+      int maxDeleteBlocksPerDatanode) {
+    // Ensure all DNs for this transaction are below their max block limit.
+    if (!replicas.stream().allMatch(replica -> {
+      final DatanodeID datanodeID = replica.getDatanodeDetails().getID();
+      return transactions.getNumberOfBlocksForDatanode(datanodeID) < maxDeleteBlocksPerDatanode;
+    })) {
+      return;
+    }
 
+    boolean flag = false;
     for (ContainerReplica replica : replicas) {
-      DatanodeDetails details = replica.getDatanodeDetails();
+      final DatanodeID datanodeID = replica.getDatanodeDetails().getID();
       if (!transactionStatusManager.isDuplication(
-          details, updatedTxn.getTxID(), commandStatus)) {
-        transactions.addTransactionToDN(details.getUuid(), updatedTxn);
-        metrics.incrProcessedTransaction();
+          datanodeID, tx.getTxID(), commandStatus)) {
+        transactions.addTransactionToDN(datanodeID, tx);
+        flag = true;
       }
     }
+    if (flag) {
+      metrics.incrProcessedTransaction();
+    }
   }
 
   private Boolean checkInadequateReplica(Set<ContainerReplica> replicas,
@@ -348,7 +292,7 @@ public class DeletedBlockLogImpl
       if (!dnList.contains(datanodeDetails)) {
         DatanodeDetails dnDetail = replica.getDatanodeDetails();
         LOG.debug("Skip Container = {}, because DN = {} is not in dnList.",
-            containerId, dnDetail.getUuid());
+            containerId, dnDetail);
         return true;
       }
     }
@@ -368,8 +312,7 @@ public class DeletedBlockLogImpl
           scmCommandTimeoutMs);
       DatanodeDeletedBlockTransactions transactions =
           new DatanodeDeletedBlockTransactions();
-      try (TableIterator<Long,
-          ? extends Table.KeyValue<Long, DeletedBlocksTransaction>> iter =
+      try (Table.KeyValueIterator<Long, DeletedBlocksTransaction> iter =
                deletedBlockLogStateManager.getReadOnlyIterator()) {
         if (lastProcessedTransactionId != -1) {
           iter.seek(lastProcessedTransactionId);
@@ -401,13 +344,19 @@ public class DeletedBlockLogImpl
 
         // Get the CmdStatus status of the aggregation, so that the current
         // status of the specified transaction can be found faster
-        Map<UUID, Map<Long, CmdStatus>> commandStatus =
+        final Map<DatanodeID, Map<Long, CmdStatus>> commandStatus =
             getSCMDeletedBlockTransactionStatusManager()
                 .getCommandStatusByTxId(dnList.stream().
-                map(DatanodeDetails::getUuid).collect(Collectors.toSet()));
+                map(DatanodeDetails::getID).collect(Collectors.toSet()));
         ArrayList<Long> txIDs = new ArrayList<>();
         metrics.setNumBlockDeletionTransactionDataNodes(dnList.size());
         Table.KeyValue<Long, DeletedBlocksTransaction> keyValue = null;
+
+        int factor = dnList.size() / deletionFactorPerDatanode;
+        int maxDeleteBlocksPerDatanode = (factor > 0)
+            ? Math.min(blockDeletionLimit, blockDeletionLimit / factor)
+            : blockDeletionLimit;
+
         // Here takes block replica count as the threshold to avoid the case
         // that part of replicas committed the TXN and recorded in the
         // SCMDeletedBlockTransactionStatusManager, while they are counted
@@ -417,25 +366,23 @@ public class DeletedBlockLogImpl
           keyValue = iter.next();
           DeletedBlocksTransaction txn = keyValue.getValue();
           final ContainerID id = ContainerID.valueOf(txn.getContainerID());
+          final ContainerInfo container = containerManager.getContainer(id);
           try {
             // HDDS-7126. When container is under replicated, it is possible
             // that container is deleted, but transactions are not deleted.
-            if (containerManager.getContainer(id).isDeleted()) {
-              LOG.warn("Container: {} was deleted for the " +
-                  "transaction: {}.", id, txn);
+            if (container.isDeleted()) {
+              LOG.warn("Container: {} was deleted for the transaction: {}.", id, txn);
               txIDs.add(txn.getTxID());
-            } else if (txn.getCount() > -1 && txn.getCount() <= maxRetry
-                && !containerManager.getContainer(id).isOpen()) {
+            } else if (!container.isOpen()) {
               Set<ContainerReplica> replicas = containerManager
                   .getContainerReplicas(
                       ContainerID.valueOf(txn.getContainerID()));
-              if (checkInadequateReplica(replicas, txn, dnList)) {
+              if (!checkInadequateReplica(replicas, txn, dnList)) {
+                getTransaction(txn, transactions, replicas, commandStatus, maxDeleteBlocksPerDatanode);
+              } else {
                 metrics.incrSkippedTransaction();
-                continue;
               }
-              getTransaction(
-                  txn, transactions, dnList, replicas, commandStatus);
-            } else if (txn.getCount() >= maxRetry || containerManager.getContainer(id).isOpen()) {
+            } else if (containerManager.getContainer(id).isOpen()) {
               metrics.incrSkippedTransaction();
             }
           } catch (ContainerNotFoundException ex) {
@@ -482,14 +429,19 @@ public class DeletedBlockLogImpl
   }
 
   @Override
-  public void recordTransactionCreated(UUID dnId, long scmCmdId,
+  public void recordTransactionCreated(DatanodeID dnId, long scmCmdId,
       Set<Long> dnTxSet) {
     getSCMDeletedBlockTransactionStatusManager()
         .recordTransactionCreated(dnId, scmCmdId, dnTxSet);
   }
 
   @Override
-  public void onDatanodeDead(UUID dnId) {
+  public int getTransactionToDNsCommitMapSize() {
+    return getSCMDeletedBlockTransactionStatusManager().getTransactionToDNsCommitMapSize();
+  }
+
+  @Override
+  public void onDatanodeDead(DatanodeID dnId) {
     getSCMDeletedBlockTransactionStatusManager().onDatanodeDead(dnId);
   }
 
@@ -507,7 +459,7 @@ public class DeletedBlockLogImpl
     }
 
     DatanodeDetails details = deleteBlockStatus.getDatanodeDetails();
-    UUID dnId = details.getUuid();
+    DatanodeID dnId = details.getID();
     for (CommandStatus commandStatus : deleteBlockStatus.getCmdStatus()) {
       CommandStatus.Status status = commandStatus.getStatus();
       lock.lock();
