@@ -27,7 +27,6 @@ import java.io.IOException;
 import java.time.Duration;
 import java.time.Instant;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -35,10 +34,11 @@ import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.UUID;
+import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.stream.Collectors;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.protocol.DatanodeID;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.CommandStatus;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerBlocksDeletionACKProto.DeleteBlockTransactionResult;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
@@ -55,10 +55,10 @@ import org.slf4j.LoggerFactory;
  * DeletedBlockTransaction sent to the DN.
  */
 public class SCMDeletedBlockTransactionStatusManager {
-  public static final Logger LOG =
+  private static final Logger LOG =
       LoggerFactory.getLogger(SCMDeletedBlockTransactionStatusManager.class);
   // Maps txId to set of DNs which are successful in committing the transaction
-  private final Map<Long, Set<UUID>> transactionToDNsCommitMap;
+  private final Map<Long, Set<DatanodeID>> transactionToDNsCommitMap;
   // Maps txId to its retry counts;
   private final Map<Long, Integer> transactionToRetryCountMap;
   // The access to DeletedBlocksTXTable is protected by
@@ -90,7 +90,7 @@ public class SCMDeletedBlockTransactionStatusManager {
     this.transactionToDNsCommitMap = new ConcurrentHashMap<>();
     this.transactionToRetryCountMap = new ConcurrentHashMap<>();
     this.scmDeleteBlocksCommandStatusManager =
-        new SCMDeleteBlocksCommandStatusManager();
+        new SCMDeleteBlocksCommandStatusManager(metrics);
   }
 
   /**
@@ -98,16 +98,18 @@ public class SCMDeletedBlockTransactionStatusManager {
    * on DeleteBlocksCommand.
    */
   protected static class SCMDeleteBlocksCommandStatusManager {
-    public static final Logger LOG =
+    private static final Logger LOG =
         LoggerFactory.getLogger(SCMDeleteBlocksCommandStatusManager.class);
-    private final Map<UUID, Map<Long, CmdStatusData>> scmCmdStatusRecord;
+    private final Map<DatanodeID, Map<Long, CmdStatusData>> scmCmdStatusRecord;
 
     private static final CmdStatus DEFAULT_STATUS = TO_BE_SENT;
-    private static final Set<CmdStatus> STATUSES_REQUIRING_TIMEOUT =
-        new HashSet<>(Arrays.asList(SENT));
+    private static final Set<CmdStatus> STATUSES_REQUIRING_TIMEOUT = Collections.singleton(SENT);
 
-    public SCMDeleteBlocksCommandStatusManager() {
+    private ScmBlockDeletingServiceMetrics metrics;
+
+    public SCMDeleteBlocksCommandStatusManager(ScmBlockDeletingServiceMetrics metrics) {
       this.scmCmdStatusRecord = new ConcurrentHashMap<>();
+      this.metrics = metrics;
     }
 
     /**
@@ -128,14 +130,13 @@ public class SCMDeletedBlockTransactionStatusManager {
     }
 
     protected static final class CmdStatusData {
-      private final UUID dnId;
+      private final DatanodeID dnId;
       private final long scmCmdId;
       private final Set<Long> deletedBlocksTxIds;
       private Instant updateTime;
       private CmdStatus status;
 
-      private CmdStatusData(
-          UUID dnId, long scmTxID, Set<Long> deletedBlocksTxIds) {
+      private CmdStatusData(DatanodeID dnId, long scmTxID, Set<Long> deletedBlocksTxIds) {
         this.dnId = dnId;
         this.scmCmdId = scmTxID;
         this.deletedBlocksTxIds = deletedBlocksTxIds;
@@ -146,7 +147,7 @@ public class SCMDeletedBlockTransactionStatusManager {
         return Collections.unmodifiableSet(deletedBlocksTxIds);
       }
 
-      public UUID getDnId() {
+      DatanodeID getDnId() {
         return dnId;
       }
 
@@ -180,7 +181,7 @@ public class SCMDeletedBlockTransactionStatusManager {
     }
 
     protected static CmdStatusData createScmCmdStatusData(
-        UUID dnId, long scmCmdId, Set<Long> deletedBlocksTxIds) {
+        DatanodeID dnId, long scmCmdId, Set<Long> deletedBlocksTxIds) {
       return new CmdStatusData(dnId, scmCmdId, deletedBlocksTxIds);
     }
 
@@ -190,22 +191,22 @@ public class SCMDeletedBlockTransactionStatusManager {
           new ConcurrentHashMap<>()).put(statusData.getScmCmdId(), statusData);
     }
 
-    protected void onSent(UUID dnId, long scmCmdId) {
+    void onSent(DatanodeID dnId, long scmCmdId) {
       updateStatus(dnId, scmCmdId, CommandStatus.Status.PENDING);
     }
 
-    protected void onDatanodeDead(UUID dnId) {
+    void onDatanodeDead(DatanodeID dnId) {
       LOG.info("Clean SCMCommand record for Datanode: {}", dnId);
       scmCmdStatusRecord.remove(dnId);
     }
 
-    protected void updateStatusByDNCommandStatus(UUID dnId, long scmCmdId,
+    void updateStatusByDNCommandStatus(DatanodeID dnId, long scmCmdId,
         CommandStatus.Status newState) {
       updateStatus(dnId, scmCmdId, newState);
     }
 
     protected void cleanAllTimeoutSCMCommand(long timeoutMs) {
-      for (UUID dnId : scmCmdStatusRecord.keySet()) {
+      for (DatanodeID dnId : scmCmdStatusRecord.keySet()) {
         for (CmdStatus status : STATUSES_REQUIRING_TIMEOUT) {
           removeTimeoutScmCommand(
               dnId, getScmCommandIds(dnId, status), timeoutMs);
@@ -213,14 +214,14 @@ public class SCMDeletedBlockTransactionStatusManager {
       }
     }
 
-    public void cleanTimeoutSCMCommand(UUID dnId, long timeoutMs) {
+    void cleanTimeoutSCMCommand(DatanodeID dnId, long timeoutMs) {
       for (CmdStatus status : STATUSES_REQUIRING_TIMEOUT) {
         removeTimeoutScmCommand(
             dnId, getScmCommandIds(dnId, status), timeoutMs);
       }
     }
 
-    private Set<Long> getScmCommandIds(UUID dnId, CmdStatus status) {
+    private Set<Long> getScmCommandIds(DatanodeID dnId, CmdStatus status) {
       Set<Long> scmCmdIds = new HashSet<>();
       Map<Long, CmdStatusData> record = scmCmdStatusRecord.get(dnId);
       if (record == null) {
@@ -234,7 +235,7 @@ public class SCMDeletedBlockTransactionStatusManager {
       return scmCmdIds;
     }
 
-    private Instant getUpdateTime(UUID dnId, long scmCmdId) {
+    private Instant getUpdateTime(DatanodeID dnId, long scmCmdId) {
       Map<Long, CmdStatusData> record = scmCmdStatusRecord.get(dnId);
       if (record == null || record.get(scmCmdId) == null) {
         return null;
@@ -242,7 +243,7 @@ public class SCMDeletedBlockTransactionStatusManager {
       return record.get(scmCmdId).getUpdateTime();
     }
 
-    private void updateStatus(UUID dnId, long scmCmdId,
+    private void updateStatus(DatanodeID dnId, long scmCmdId,
         CommandStatus.Status newStatus) {
       Map<Long, CmdStatusData> recordForDn = scmCmdStatusRecord.get(dnId);
       if (recordForDn == null) {
@@ -309,7 +310,7 @@ public class SCMDeletedBlockTransactionStatusManager {
       }
     }
 
-    private void removeTimeoutScmCommand(UUID dnId,
+    private void removeTimeoutScmCommand(DatanodeID dnId,
         Set<Long> scmCmdIds, long timeoutMs) {
       Instant now = Instant.now();
       for (Long scmCmdId : scmCmdIds) {
@@ -317,13 +318,14 @@ public class SCMDeletedBlockTransactionStatusManager {
         if (updateTime != null &&
             Duration.between(updateTime, now).toMillis() > timeoutMs) {
           CmdStatusData state = removeScmCommand(dnId, scmCmdId);
+          metrics.incrDNCommandsTimeout(dnId, 1);
           LOG.warn("SCM BlockDeletionCommand {} for Datanode: {} was removed after {}ms without update",
               state, dnId, timeoutMs);
         }
       }
     }
 
-    private CmdStatusData removeScmCommand(UUID dnId, long scmCmdId) {
+    private CmdStatusData removeScmCommand(DatanodeID dnId, long scmCmdId) {
       Map<Long, CmdStatusData> record = scmCmdStatusRecord.get(dnId);
       if (record == null || record.get(scmCmdId) == null) {
         return null;
@@ -333,12 +335,10 @@ public class SCMDeletedBlockTransactionStatusManager {
       return statusData;
     }
 
-    public Map<UUID, Map<Long, CmdStatus>> getCommandStatusByTxId(
-        Set<UUID> dnIds) {
-      Map<UUID, Map<Long, CmdStatus>> result =
-          new HashMap<>(scmCmdStatusRecord.size());
+    Map<DatanodeID, Map<Long, CmdStatus>> getCommandStatusByTxId(Set<DatanodeID> dnIds) {
+      final Map<DatanodeID, Map<Long, CmdStatus>> result = new HashMap<>(scmCmdStatusRecord.size());
 
-      for (UUID dnId : dnIds) {
+      for (DatanodeID dnId : dnIds) {
         Map<Long, CmdStatusData> record = scmCmdStatusRecord.get(dnId);
         if (record == null) {
           continue;
@@ -361,56 +361,26 @@ public class SCMDeletedBlockTransactionStatusManager {
     }
 
     @VisibleForTesting
-    Map<UUID, Map<Long, CmdStatusData>> getScmCmdStatusRecord() {
+    Map<DatanodeID, Map<Long, CmdStatusData>> getScmCmdStatusRecord() {
       return scmCmdStatusRecord;
     }
   }
 
-  public void incrementRetryCount(List<Long> txIDs, long maxRetry)
-      throws IOException {
-    ArrayList<Long> txIDsToUpdate = new ArrayList<>();
-    for (Long txID : txIDs) {
-      int currentCount =
-          transactionToRetryCountMap.getOrDefault(txID, 0);
-      if (currentCount > maxRetry) {
-        continue;
-      } else {
-        currentCount += 1;
-        if (currentCount > maxRetry) {
-          txIDsToUpdate.add(txID);
-        }
-        transactionToRetryCountMap.put(txID, currentCount);
-      }
-    }
-
-    if (!txIDsToUpdate.isEmpty()) {
-      deletedBlockLogStateManager
-          .increaseRetryCountOfTransactionInDB(txIDsToUpdate);
-    }
-  }
-
-  public void resetRetryCount(List<Long> txIDs) throws IOException {
-    for (Long txID: txIDs) {
-      transactionToRetryCountMap.computeIfPresent(txID, (key, value) -> 0);
-    }
-  }
-
-  public int getOrDefaultRetryCount(long txID, int defaultValue) {
-    return transactionToRetryCountMap.getOrDefault(txID, defaultValue);
+  public void incrementRetryCount(List<Long> txIDs) {
+    CompletableFuture.runAsync(() ->
+        txIDs.forEach(tx ->
+            transactionToRetryCountMap.compute(tx, (k, v) -> (v == null) ? 1 : v + 1)));
   }
 
   public void onSent(DatanodeDetails dnId, SCMCommand<?> scmCommand) {
-    scmDeleteBlocksCommandStatusManager.onSent(
-        dnId.getUuid(), scmCommand.getId());
+    scmDeleteBlocksCommandStatusManager.onSent(dnId.getID(), scmCommand.getId());
   }
 
-  public Map<UUID, Map<Long, CmdStatus>> getCommandStatusByTxId(
-      Set<UUID> dnIds) {
+  Map<DatanodeID, Map<Long, CmdStatus>> getCommandStatusByTxId(Set<DatanodeID> dnIds) {
     return scmDeleteBlocksCommandStatusManager.getCommandStatusByTxId(dnIds);
   }
 
-  public void recordTransactionCreated(
-      UUID dnId, long scmCmdId, Set<Long> dnTxSet) {
+  void recordTransactionCreated(DatanodeID dnId, long scmCmdId, Set<Long> dnTxSet) {
     scmDeleteBlocksCommandStatusManager.recordScmCommand(
         SCMDeleteBlocksCommandStatusManager
             .createScmCmdStatusData(dnId, scmCmdId, dnTxSet));
@@ -428,21 +398,19 @@ public class SCMDeletedBlockTransactionStatusManager {
     scmDeleteBlocksCommandStatusManager.cleanAllTimeoutSCMCommand(timeoutMs);
   }
 
-  public void onDatanodeDead(UUID dnId) {
+  void onDatanodeDead(DatanodeID dnId) {
     scmDeleteBlocksCommandStatusManager.onDatanodeDead(dnId);
   }
 
-  public boolean isDuplication(DatanodeDetails dnDetail, long tx,
-      Map<UUID, Map<Long, CmdStatus>> commandStatus) {
-    if (alreadyExecuted(dnDetail.getUuid(), tx)) {
+  boolean isDuplication(DatanodeID datanodeID, long tx, Map<DatanodeID, Map<Long, CmdStatus>> commandStatus) {
+    if (alreadyExecuted(datanodeID, tx)) {
       return true;
     }
-    return inProcessing(dnDetail.getUuid(), tx, commandStatus);
+    return inProcessing(datanodeID, tx, commandStatus);
   }
 
-  public boolean alreadyExecuted(UUID dnId, long txId) {
-    Set<UUID> dnsWithTransactionCommitted =
-        transactionToDNsCommitMap.get(txId);
+  private boolean alreadyExecuted(DatanodeID dnId, long txId) {
+    final Set<DatanodeID> dnsWithTransactionCommitted = transactionToDNsCommitMap.get(txId);
     return dnsWithTransactionCommitted != null && dnsWithTransactionCommitted
         .contains(dnId);
   }
@@ -455,22 +423,20 @@ public class SCMDeletedBlockTransactionStatusManager {
    * @param dnId - ID of datanode which acknowledges the delete block command.
    */
   @VisibleForTesting
-  public void commitTransactions(
-      List<DeleteBlockTransactionResult> transactionResults, UUID dnId) {
+  public void commitTransactions(List<DeleteBlockTransactionResult> transactionResults, DatanodeID dnId) {
 
     ArrayList<Long> txIDsToBeDeleted = new ArrayList<>();
-    Set<UUID> dnsWithCommittedTxn;
     for (DeleteBlockTransactionResult transactionResult :
         transactionResults) {
       if (isTransactionFailed(transactionResult)) {
-        metrics.incrBlockDeletionTransactionFailure();
+        metrics.incrBlockDeletionTransactionFailureOnDatanodes();
         continue;
       }
       try {
-        metrics.incrBlockDeletionTransactionSuccess();
+        metrics.incrBlockDeletionTransactionSuccessOnDatanodes();
         long txID = transactionResult.getTxID();
         // set of dns which have successfully committed transaction txId.
-        dnsWithCommittedTxn = transactionToDNsCommitMap.get(txID);
+        final Set<DatanodeID> dnsWithCommittedTxn = transactionToDNsCommitMap.get(txID);
         final ContainerID containerId = ContainerID.valueOf(
             transactionResult.getContainerID());
         if (dnsWithCommittedTxn == null) {
@@ -494,9 +460,9 @@ public class SCMDeletedBlockTransactionStatusManager {
         // the nodes returned in the pipeline match the replication factor.
         if (min(replicas.size(), dnsWithCommittedTxn.size())
             >= container.getReplicationConfig().getRequiredNodes()) {
-          List<UUID> containerDns = replicas.stream()
+          final List<DatanodeID> containerDns = replicas.stream()
               .map(ContainerReplica::getDatanodeDetails)
-              .map(DatanodeDetails::getUuid)
+              .map(DatanodeDetails::getID)
               .collect(Collectors.toList());
           if (dnsWithCommittedTxn.containsAll(containerDns)) {
             transactionToDNsCommitMap.remove(txID);
@@ -526,34 +492,29 @@ public class SCMDeletedBlockTransactionStatusManager {
   }
 
   @VisibleForTesting
-  public void commitSCMCommandStatus(List<CommandStatus> deleteBlockStatus,
-      UUID dnId) {
+  void commitSCMCommandStatus(List<CommandStatus> deleteBlockStatus, DatanodeID dnId) {
     processSCMCommandStatus(deleteBlockStatus, dnId);
     scmDeleteBlocksCommandStatusManager.
         cleanTimeoutSCMCommand(dnId, scmCommandTimeoutMs);
   }
 
-  private boolean inProcessing(UUID dnId, long deletedBlocksTxId,
-      Map<UUID, Map<Long, CmdStatus>> commandStatus) {
+  static boolean inProcessing(DatanodeID dnId, long deletedBlocksTxId,
+      Map<DatanodeID, Map<Long, CmdStatus>> commandStatus) {
     Map<Long, CmdStatus> deletedBlocksTxStatus = commandStatus.get(dnId);
     return deletedBlocksTxStatus != null &&
         deletedBlocksTxStatus.get(deletedBlocksTxId) != null;
   }
 
-  private void processSCMCommandStatus(List<CommandStatus> deleteBlockStatus,
-      UUID dnID) {
-    Map<Long, CommandStatus> lastStatus = new HashMap<>();
+  private void processSCMCommandStatus(List<CommandStatus> deleteBlockStatus, DatanodeID dnID) {
     Map<Long, CommandStatus.Status> summary = new HashMap<>();
-
     // The CommandStatus is ordered in the report. So we can focus only on the
     // last status in the command report.
     deleteBlockStatus.forEach(cmdStatus -> {
-      lastStatus.put(cmdStatus.getCmdId(), cmdStatus);
       summary.put(cmdStatus.getCmdId(), cmdStatus.getStatus());
     });
     LOG.debug("CommandStatus {} from Datanode: {} ", summary, dnID);
-    for (Map.Entry<Long, CommandStatus> entry : lastStatus.entrySet()) {
-      CommandStatus.Status status = entry.getValue().getStatus();
+    for (Map.Entry<Long, CommandStatus.Status> entry : summary.entrySet()) {
+      final CommandStatus.Status status = entry.getValue();
       scmDeleteBlocksCommandStatusManager.updateStatusByDNCommandStatus(
           dnID, entry.getKey(), status);
     }
@@ -572,4 +533,8 @@ public class SCMDeletedBlockTransactionStatusManager {
     }
     return false;
   }
+
+  public int getTransactionToDNsCommitMapSize() {
+    return transactionToDNsCommitMap.size();
+  }
 }
