@@ -1,13 +1,12 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License.  You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -18,15 +17,14 @@
 
 package org.apache.hadoop.hdds.scm.container.replication;
 
+import java.util.List;
+import java.util.Set;
+import java.util.stream.Collectors;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.scm.ContainerPlacementStatus;
 import org.apache.hadoop.hdds.scm.PlacementPolicy;
 import org.apache.hadoop.hdds.scm.container.ContainerReplica;
 
-import java.util.List;
-import java.util.Set;
-import java.util.stream.Collectors;
-
 /**
  * This class holds some common methods that will be shared among
  * different kinds of implementation of OverReplicationHandler.
@@ -48,11 +46,10 @@ public abstract class AbstractOverReplicationHandler
    * @param replica the replica to be removed
    */
   public boolean isPlacementStatusActuallyEqualAfterRemove(
+      ContainerPlacementStatus currentCPS,
       final Set<ContainerReplica> replicas,
       final ContainerReplica replica,
       final int replicationFactor) {
-    ContainerPlacementStatus currentCPS =
-        getPlacementStatus(replicas, replicationFactor);
     replicas.remove(replica);
     ContainerPlacementStatus newCPS =
         getPlacementStatus(replicas, replicationFactor);
@@ -78,7 +75,7 @@ public abstract class AbstractOverReplicationHandler
    * @param replicationFactor Expected Replication Factor of the containe
    * @return ContainerPlacementStatus indicating if the policy is met or not
    */
-  private ContainerPlacementStatus getPlacementStatus(
+  protected ContainerPlacementStatus getPlacementStatus(
       Set<ContainerReplica> replicas, int replicationFactor) {
     List<DatanodeDetails> replicaDns = replicas.stream()
         .map(ContainerReplica::getDatanodeDetails)
