@@ -60,12 +60,11 @@ import org.slf4j.LoggerFactory;
  */
 public final class  HASecurityUtils {
 
+  private static final Logger LOG = LoggerFactory.getLogger(HASecurityUtils.class);
+
   private HASecurityUtils() {
   }
 
-  public static final Logger LOG =
-      LoggerFactory.getLogger(HASecurityUtils.class);
-
   /**
    * Initialize Security which generates public, private key pair and get SCM
    * signed certificate and persist to local disk.
