@@ -72,6 +72,8 @@ import org.slf4j.LoggerFactory;
  */
 public class DatanodeAdminMonitorImpl implements DatanodeAdminMonitor {
 
+  private static final Logger LOG = LoggerFactory.getLogger(DatanodeAdminMonitorImpl.class);
+
   private EventPublisher eventQueue;
   private NodeManager nodeManager;
   private ReplicationManager replicationManager;
@@ -86,6 +88,12 @@ public class DatanodeAdminMonitorImpl implements DatanodeAdminMonitor {
   private long unClosedContainers = 0;
   private long underReplicatedContainers = 0;
 
+  private Map<String, ContainerStateInWorkflow> containerStateByHost;
+
+  // The number of containers for each of under replicated and unhealthy
+  // that will be logged in detail each time a node is checked.
+  private final int containerDetailsLoggingLimit;
+
   /**
    * Inner class for snapshot of Datanode ContainerState in
    * Decommissioning and Maintenance mode workflow.
@@ -130,14 +138,6 @@ public class DatanodeAdminMonitorImpl implements DatanodeAdminMonitor {
     }
   }
 
-  private Map<String, ContainerStateInWorkflow> containerStateByHost;
-
-  private static final Logger LOG =
-      LoggerFactory.getLogger(DatanodeAdminMonitorImpl.class);
-  // The number of containers for each of under replicated and unhealthy
-  // that will be logged in detail each time a node is checked.
-  private final int containerDetailsLoggingLimit;
-
   public DatanodeAdminMonitorImpl(
       OzoneConfiguration conf,
       EventPublisher eventQueue,
@@ -499,7 +499,7 @@ public class DatanodeAdminMonitorImpl implements DatanodeAdminMonitor {
     sb.append(replicas.stream()
         .map(Object::toString)
         .collect(Collectors.joining(",")));
-    sb.append("}");
+    sb.append('}');
     return sb.toString();
   }
 
