@@ -1,13 +1,12 @@
-/**
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License.  You may obtain a copy of the License at
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -19,14 +18,14 @@
 package org.apache.hadoop.hdds.scm.pipeline.choose.algorithms;
 
 import com.google.common.annotations.VisibleForTesting;
+import java.lang.reflect.Constructor;
 import org.apache.hadoop.hdds.scm.PipelineChoosePolicy;
 import org.apache.hadoop.hdds.scm.ScmConfig;
 import org.apache.hadoop.hdds.scm.exceptions.SCMException;
+import org.apache.hadoop.hdds.scm.node.NodeManager;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import java.lang.reflect.Constructor;
-
 /**
  * A factory to create pipeline choose policy instance based on configuration
  * property {@link ScmConfig}.
@@ -48,14 +47,14 @@ public final class PipelineChoosePolicyFactory {
   private PipelineChoosePolicyFactory() {
   }
 
-  public static PipelineChoosePolicy getPolicy(
+  public static PipelineChoosePolicy getPolicy(final NodeManager nodeManager,
       ScmConfig scmConfig, boolean forEC) throws SCMException {
     Class<? extends PipelineChoosePolicy> policyClass = null;
     String policyName = forEC ? scmConfig.getECPipelineChoosePolicyName() :
         scmConfig.getPipelineChoosePolicyName();
     try {
       policyClass = getClass(policyName, PipelineChoosePolicy.class);
-      return createPipelineChoosePolicyFromClass(policyClass);
+      return createPipelineChoosePolicyFromClass(nodeManager, policyClass);
     } catch (Exception e) {
       Class<? extends PipelineChoosePolicy> defaultPolicy = forEC ?
           OZONE_SCM_EC_PIPELINE_CHOOSE_POLICY_IMPL_DEFAULT :
@@ -64,13 +63,14 @@ public final class PipelineChoosePolicyFactory {
         LOG.error("Met an exception while create pipeline choose policy "
             + "for the given class {}. Fallback to the default pipeline "
             + " choose policy {}", policyName, defaultPolicy, e);
-        return createPipelineChoosePolicyFromClass(defaultPolicy);
+        return createPipelineChoosePolicyFromClass(nodeManager, defaultPolicy);
       }
       throw e;
     }
   }
 
   private static PipelineChoosePolicy createPipelineChoosePolicyFromClass(
+      final NodeManager nodeManager,
       Class<? extends PipelineChoosePolicy> policyClass) throws SCMException {
     Constructor<? extends PipelineChoosePolicy> constructor;
     try {
@@ -86,7 +86,7 @@ public final class PipelineChoosePolicyFactory {
     }
 
     try {
-      return constructor.newInstance();
+      return constructor.newInstance().init(nodeManager);
     } catch (Exception e) {
       throw new RuntimeException("Failed to instantiate class " +
           policyClass.getCanonicalName() + " for " + e.getMessage());
