@@ -1,43 +1,40 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with this
- * work for additional information regarding copyright ownership.  The ASF
- * licenses this file to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- * <p>
- * http://www.apache.org/licenses/LICENSE-2.0
- * <p>
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
  * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
- * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
- * License for the specific language governing permissions and limitations under
- * the License.
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
+
 package org.apache.hadoop.hdds.scm.protocol;
 
+import com.google.protobuf.ProtocolMessageEnum;
+import com.google.protobuf.RpcController;
+import com.google.protobuf.ServiceException;
 import java.io.IOException;
 import java.util.List;
-
 import org.apache.hadoop.hdds.protocol.SCMSecurityProtocol;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetAllRootCaCertificatesResponseProto;
+import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetCertRequestProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetCertResponseProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetCertResponseProto.ResponseCode;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetCertificateRequestProto;
-import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetCrlsRequestProto;
-import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetCrlsResponseProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetDataNodeCertRequestProto;
-import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetCertRequestProto;
-import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetLatestCrlIdRequestProto;
-import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetLatestCrlIdResponseProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetOMCertRequestProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetSCMCertRequestProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMListCertificateRequestProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMListCertificateResponseProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMRemoveExpiredCertificatesResponseProto;
-import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMRevokeCertificatesRequestProto;
-import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMRevokeCertificatesResponseProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMSecurityRequest;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMSecurityResponse;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.Status;
@@ -45,13 +42,8 @@ import org.apache.hadoop.hdds.protocolPB.SCMSecurityProtocolPB;
 import org.apache.hadoop.hdds.scm.ha.RatisUtil;
 import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
 import org.apache.hadoop.hdds.security.exception.SCMSecurityException;
-import org.apache.hadoop.hdds.security.x509.crl.CRLInfo;
 import org.apache.hadoop.hdds.server.OzoneProtocolMessageDispatcher;
 import org.apache.hadoop.hdds.utils.ProtocolMessageMetrics;
-
-import com.google.protobuf.ProtocolMessageEnum;
-import com.google.protobuf.RpcController;
-import com.google.protobuf.ServiceException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -68,6 +60,7 @@ public class SCMSecurityProtocolServerSideTranslatorPB
 
   private final SCMSecurityProtocol impl;
   private final StorageContainerManager scm;
+  private static final String ROLE_TYPE = "SCM";
 
   private OzoneProtocolMessageDispatcher<SCMSecurityRequest,
       SCMSecurityResponse, ProtocolMessageEnum>
@@ -89,7 +82,7 @@ public class SCMSecurityProtocolServerSideTranslatorPB
     if (!scm.checkLeader()) {
       RatisUtil.checkRatisException(
           scm.getScmHAManager().getRatisServer().triggerNotLeaderException(),
-          scm.getSecurityProtocolRpcPort(), scm.getScmId());
+          scm.getSecurityProtocolRpcPort(), scm.getScmId(), scm.getHostname(), ROLE_TYPE);
     }
     return dispatcher.processRequest(request, this::processRequest,
         request.getCmdType(), request.getTraceID());
@@ -130,21 +123,12 @@ public class SCMSecurityProtocolServerSideTranslatorPB
         return scmSecurityResponse.setListCertificateResponseProto(
             listCACertificate()).build();
       case GetCrls:
-        return SCMSecurityResponse.newBuilder()
-            .setCmdType(request.getCmdType())
-            .setGetCrlsResponseProto(getCrls(request.getGetCrlsRequest()))
-            .build();
       case GetLatestCrlId:
-        return SCMSecurityResponse.newBuilder()
-            .setCmdType(request.getCmdType())
-            .setGetLatestCrlIdResponseProto(getLatestCrlId(
-                request.getGetLatestCrlIdRequest()))
-            .build();
       case RevokeCertificates:
-        return SCMSecurityResponse.newBuilder()
-            .setCmdType(request.getCmdType())
-            .setRevokeCertificatesResponseProto(revokeCertificates(
-                request.getRevokeCertificatesRequest()))
+        return scmSecurityResponse
+            .setStatus(Status.INTERNAL_ERROR)
+            .setMessage("Unsupported operation.")
+            .setSuccess(false)
             .build();
       case GetCert:
         return scmSecurityResponse.setGetCertResponseProto(
@@ -166,7 +150,7 @@ public class SCMSecurityProtocolServerSideTranslatorPB
       }
     } catch (IOException e) {
       RatisUtil.checkRatisException(e, scm.getSecurityProtocolRpcPort(),
-          scm.getScmId());
+          scm.getScmId(), scm.getHostname(), ROLE_TYPE);
       scmSecurityResponse.setSuccess(false);
       scmSecurityResponse.setStatus(exceptionToResponseStatus(e));
       // If actual cause is set in SCMSecurityException, set message with
@@ -326,7 +310,7 @@ public class SCMSecurityProtocolServerSideTranslatorPB
   public SCMListCertificateResponseProto listCertificate(
       SCMListCertificateRequestProto request) throws IOException {
     List<String> certs = impl.listCertificate(request.getRole(),
-        request.getStartCertId(), request.getCount(), request.getIsRevoked());
+        request.getStartCertId(), request.getCount());
 
     SCMListCertificateResponseProto.Builder builder =
         SCMListCertificateResponseProto
@@ -338,40 +322,6 @@ public class SCMSecurityProtocolServerSideTranslatorPB
 
   }
 
-  public SCMGetCrlsResponseProto getCrls(
-      SCMGetCrlsRequestProto request) throws IOException {
-    List<CRLInfo> crls = impl.getCrls(request.getCrlIdList());
-    SCMGetCrlsResponseProto.Builder builder =
-        SCMGetCrlsResponseProto.newBuilder();
-    for (CRLInfo crl : crls) {
-      try {
-        builder.addCrlInfos(crl.getProtobuf());
-      } catch (SCMSecurityException e) {
-        LOG.error("Fail in parsing CRL info", e);
-        throw new SCMSecurityException("Fail in parsing CRL info", e);
-      }
-    }
-    return builder.build();
-  }
-
-  public SCMGetLatestCrlIdResponseProto getLatestCrlId(
-      SCMGetLatestCrlIdRequestProto request) throws IOException {
-    SCMGetLatestCrlIdResponseProto.Builder builder =
-        SCMGetLatestCrlIdResponseProto
-            .newBuilder().
-            setCrlId(impl.getLatestCrlId());
-    return builder.build();
-  }
-
-  public SCMRevokeCertificatesResponseProto revokeCertificates(
-      SCMRevokeCertificatesRequestProto request) throws IOException {
-    SCMRevokeCertificatesResponseProto.Builder builder =
-        SCMRevokeCertificatesResponseProto.newBuilder().setCrlId(
-            impl.revokeCertificates(request.getCertIdsList(),
-                request.getReason().getNumber(), request.getRevokeTime()));
-    return builder.build();
-  }
-
   public SCMGetCertResponseProto getRootCACertificate() throws IOException {
     if (scm.getScmStorageConfig().checkPrimarySCMIdInitialized()) {
       throw createNotHAException();
