@@ -18,10 +18,13 @@
 package org.apache.hadoop.hdds.scm.safemode;
 
 import java.util.HashSet;
-import java.util.UUID;
+import java.util.Set;
 import org.apache.hadoop.hdds.HddsConfigKeys;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
+import org.apache.hadoop.hdds.protocol.DatanodeID;
 import org.apache.hadoop.hdds.scm.events.SCMEvents;
+import org.apache.hadoop.hdds.scm.node.NodeManager;
+import org.apache.hadoop.hdds.scm.node.NodeStatus;
 import org.apache.hadoop.hdds.scm.server.SCMDatanodeProtocolServer.NodeRegistrationContainerReport;
 import org.apache.hadoop.hdds.server.events.EventQueue;
 import org.apache.hadoop.hdds.server.events.TypedEvent;
@@ -33,22 +36,23 @@ import org.apache.hadoop.hdds.server.events.TypedEvent;
 public class DataNodeSafeModeRule extends
     SafeModeExitRule<NodeRegistrationContainerReport> {
 
-  private static final String NAME = "DataNodeSafeModeRule";
-
   // Min DataNodes required to exit safe mode.
   private int requiredDns;
   private int registeredDns = 0;
   // Set to track registered DataNodes.
-  private HashSet<UUID> registeredDnSet;
+  private final Set<DatanodeID> registeredDnSet;
+  private NodeManager nodeManager;
 
   public DataNodeSafeModeRule(EventQueue eventQueue,
       ConfigurationSource conf,
+      NodeManager nodeManager,
       SCMSafeModeManager manager) {
-    super(manager, NAME, eventQueue);
+    super(manager, eventQueue);
     requiredDns = conf.getInt(
         HddsConfigKeys.HDDS_SCM_SAFEMODE_MIN_DATANODE,
         HddsConfigKeys.HDDS_SCM_SAFEMODE_MIN_DATANODE_DEFAULT);
     registeredDnSet = new HashSet<>(requiredDns * 2);
+    this.nodeManager = nodeManager;
   }
 
   @Override
@@ -58,13 +62,16 @@ public class DataNodeSafeModeRule extends
 
   @Override
   protected boolean validate() {
-    return registeredDns >= requiredDns;
+    if (validateBasedOnReportProcessing()) {
+      return registeredDns >= requiredDns;
+    }
+    return nodeManager.getNodes(NodeStatus.inServiceHealthy()).size() >= requiredDns;
   }
 
   @Override
   protected void process(NodeRegistrationContainerReport reportsProto) {
 
-    registeredDnSet.add(reportsProto.getDatanodeDetails().getUuid());
+    registeredDnSet.add(reportsProto.getDatanodeDetails().getID());
     registeredDns = registeredDnSet.size();
 
     if (scmInSafeMode()) {
@@ -87,7 +94,6 @@ public class DataNodeSafeModeRule extends
             this.registeredDns, this.requiredDns);
   }
 
-
   @Override
   public void refresh(boolean forceRefresh) {
     // Do nothing.
