@@ -48,7 +48,6 @@ public class OneReplicaPipelineSafeModeRule extends
 
   private static final Logger LOG =
       LoggerFactory.getLogger(OneReplicaPipelineSafeModeRule.class);
-  private static final String NAME = "AtleastOneDatanodeReportedRule";
 
   private int thresholdCount;
   private final Set<PipelineID> reportedPipelineIDSet = new HashSet<>();
@@ -57,10 +56,9 @@ public class OneReplicaPipelineSafeModeRule extends
   private PipelineManager pipelineManager;
   private final double pipelinePercent;
 
-
   public OneReplicaPipelineSafeModeRule(EventQueue eventQueue, PipelineManager pipelineManager,
       SCMSafeModeManager safeModeManager, ConfigurationSource configuration) {
-    super(safeModeManager, NAME, eventQueue);
+    super(safeModeManager, eventQueue);
 
     pipelinePercent =
         configuration.getDouble(
