@@ -17,6 +17,7 @@
 
 package org.apache.hadoop.hdds.scm.safemode;
 
+import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.metrics2.MetricsSystem;
 import org.apache.hadoop.metrics2.annotation.Metric;
 import org.apache.hadoop.metrics2.lib.DefaultMetricsSystem;
@@ -25,12 +26,13 @@ import org.apache.hadoop.metrics2.lib.MutableGaugeLong;
 
 /**
  * This class is used for maintaining SafeMode metric information, which can
- * be used for monitoring during SCM startup when SCM is still in SafeMode.
+ * be used for monitoring during SCM startup when SCM is still in SafeMode.<p>
+ * The metrics from this class are valid iff
+ * {@link org.apache.hadoop.hdds.HddsConfigKeys#HDDS_SCM_SAFEMODE_ENABLED} is
+ * set to true and the SCM is still in SafeMode.
  */
 public class SafeModeMetrics {
-  private static final String SOURCE_NAME =
-      SafeModeMetrics.class.getSimpleName();
-
+  private static final String SOURCE_NAME = SafeModeMetrics.class.getSimpleName();
 
   // These all values will be set to some values when safemode is enabled.
   private @Metric MutableGaugeLong
@@ -42,8 +44,7 @@ public class SafeModeMetrics {
   private @Metric MutableCounterLong
       currentContainersWithECDataReplicaReportedCount;
 
-  // When hdds.scm.safemode.pipeline-availability.check is set then only
-  // below metrics will have some values, otherwise they will be zero.
+  // Pipeline metrics for safemode
   private @Metric MutableGaugeLong numHealthyPipelinesThreshold;
   private @Metric MutableCounterLong currentHealthyPipelinesCount;
   private @Metric MutableGaugeLong
@@ -52,10 +53,8 @@ public class SafeModeMetrics {
       currentPipelinesWithAtleastOneReplicaReportedCount;
 
   public static SafeModeMetrics create() {
-    MetricsSystem ms = DefaultMetricsSystem.instance();
-    return ms.register(SOURCE_NAME,
-        "SCM Safemode Metrics",
-        new SafeModeMetrics());
+    final MetricsSystem ms = DefaultMetricsSystem.instance();
+    return ms.register(SOURCE_NAME, "SCM Safemode Metrics", new SafeModeMetrics());
   }
 
   public void setNumHealthyPipelinesThreshold(long val) {
@@ -74,12 +73,17 @@ public class SafeModeMetrics {
     this.currentPipelinesWithAtleastOneReplicaReportedCount.incr();
   }
 
-  public void setNumContainerWithOneReplicaReportedThreshold(long val) {
-    this.numContainerWithOneReplicaReportedThreshold.set(val);
-  }
-
-  public void setNumContainerWithECDataReplicaReportedThreshold(long val) {
-    this.numContainerWithECDataReplicaReportedThreshold.set(val);
+  public void setNumContainerReportedThreshold(HddsProtos.ReplicationType type, long val) {
+    switch (type) {
+    case RATIS:
+      this.numContainerWithOneReplicaReportedThreshold.set(val);
+      break;
+    case EC:
+      this.numContainerWithECDataReplicaReportedThreshold.set(val);
+      break;
+    default:
+      throw new IllegalArgumentException("Unsupported replication type: " + type);
+    }
   }
 
   public void incCurrentContainersWithOneReplicaReportedCount() {
@@ -119,7 +123,6 @@ public class SafeModeMetrics {
     return currentContainersWithOneReplicaReportedCount;
   }
 
-
   public void unRegister() {
     MetricsSystem ms = DefaultMetricsSystem.instance();
     ms.unregisterSource(SOURCE_NAME);
