@@ -41,7 +41,7 @@ import org.slf4j.LoggerFactory;
  */
 public class RootCARotationHandlerImpl implements RootCARotationHandler {
 
-  public static final Logger LOG =
+  private static final Logger LOG =
       LoggerFactory.getLogger(RootCARotationHandlerImpl.class);
 
   private final StorageContainerManager scm;
