@@ -23,6 +23,7 @@ import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProt
 import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.SCMCommandProto.Type.deleteBlocksCommand;
 import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.SCMCommandProto.Type.deleteContainerCommand;
 import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.SCMCommandProto.Type.finalizeNewLayoutVersionCommand;
+import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.SCMCommandProto.Type.reconcileContainerCommand;
 import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.SCMCommandProto.Type.reconstructECContainersCommand;
 import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.SCMCommandProto.Type.refreshVolumeUsageInfo;
 import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.SCMCommandProto.Type.replicateContainerCommand;
@@ -52,7 +53,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.OptionalLong;
 import java.util.concurrent.TimeoutException;
-import org.apache.hadoop.fs.CommonConfigurationKeys;
+import org.apache.hadoop.fs.CommonConfigurationKeysPublic;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
@@ -95,6 +96,7 @@ import org.apache.hadoop.ozone.protocol.commands.CreatePipelineCommand;
 import org.apache.hadoop.ozone.protocol.commands.DeleteBlocksCommand;
 import org.apache.hadoop.ozone.protocol.commands.DeleteContainerCommand;
 import org.apache.hadoop.ozone.protocol.commands.FinalizeNewLayoutVersionCommand;
+import org.apache.hadoop.ozone.protocol.commands.ReconcileContainerCommand;
 import org.apache.hadoop.ozone.protocol.commands.RefreshVolumeUsageCommand;
 import org.apache.hadoop.ozone.protocol.commands.RegisteredCommand;
 import org.apache.hadoop.ozone.protocol.commands.ReplicateContainerCommand;
@@ -181,7 +183,7 @@ public class SCMDatanodeProtocolServer implements
             conf, getDatanodeAddressKey(), datanodeRpcAddr,
             datanodeRpcServer);
 
-    if (conf.getBoolean(CommonConfigurationKeys.HADOOP_SECURITY_AUTHORIZATION,
+    if (conf.getBoolean(CommonConfigurationKeysPublic.HADOOP_SECURITY_AUTHORIZATION,
         false)) {
       datanodeRpcServer.refreshServiceAcl(conf, getPolicyProvider());
     }
@@ -276,14 +278,14 @@ public class SCMDatanodeProtocolServer implements
 
   private String constructCommandAuditMap(List<SCMCommandProto> cmds) {
     StringBuilder auditMap = new StringBuilder();
-    auditMap.append("[");
+    auditMap.append('[');
     for (SCMCommandProto cmd : cmds) {
       if (cmd.getCommandType().equals(deleteBlocksCommand)) {
         auditMap.append("commandType: ").append(cmd.getCommandType());
         auditMap.append(" deleteTransactionsCount: ")
             .append(cmd.getDeleteBlocksCommandProto().getDeletedBlocksTransactionsCount());
         auditMap.append(" cmdID: ").append(cmd.getDeleteBlocksCommandProto().getCmdId());
-        auditMap.append(" encodedToken: \"").append(cmd.getEncodedToken()).append("\"");
+        auditMap.append(" encodedToken: \"").append(cmd.getEncodedToken()).append('"');
         auditMap.append(" deadlineMsSinceEpoch: ").append(cmd.getDeadlineMsSinceEpoch());
       } else {
         auditMap.append(TextFormat.shortDebugString(cmd));
@@ -294,7 +296,7 @@ public class SCMDatanodeProtocolServer implements
     if (len > 2) {
       auditMap.delete(len - 2, len);
     }
-    auditMap.append("]");
+    auditMap.append(']');
     return auditMap.toString();
   }
 
@@ -430,6 +432,12 @@ public class SCMDatanodeProtocolServer implements
           .setRefreshVolumeUsageCommandProto(
               ((RefreshVolumeUsageCommand)cmd).getProto())
           .build();
+    case reconcileContainerCommand:
+      return builder
+          .setCommandType(reconcileContainerCommand)
+          .setReconcileContainerCommandProto(
+              ((ReconcileContainerCommand)cmd).getProto())
+          .build();
 
     default:
       throw new IllegalArgumentException("Scm command " +
@@ -437,7 +445,6 @@ public class SCMDatanodeProtocolServer implements
     }
   }
 
-
   public void join() throws InterruptedException {
     LOG.trace("Join RPC server for DataNodes");
     datanodeRpcServer.join();
