@@ -1,13 +1,12 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License.  You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -21,6 +20,10 @@ package org.apache.hadoop.hdds.scm.server.upgrade;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.util.concurrent.ThreadFactoryBuilder;
+import java.io.IOException;
+import java.util.Collections;
+import java.util.concurrent.Executors;
+import java.util.concurrent.ThreadFactory;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.scm.ha.SCMContext;
 import org.apache.hadoop.hdds.scm.ha.SCMHAManager;
@@ -31,17 +34,12 @@ import org.apache.hadoop.hdds.upgrade.HDDSLayoutVersionManager;
 import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.ozone.upgrade.BasicUpgradeFinalizer;
 import org.apache.hadoop.ozone.upgrade.DefaultUpgradeFinalizationExecutor;
+import org.apache.hadoop.ozone.upgrade.UpgradeFinalization;
 import org.apache.hadoop.ozone.upgrade.UpgradeFinalizationExecutor;
-import org.apache.hadoop.ozone.upgrade.UpgradeFinalizer;
 import org.apache.ratis.util.ExitUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import java.io.IOException;
-import java.util.Collections;
-import java.util.concurrent.Executors;
-import java.util.concurrent.ThreadFactory;
-
 /**
  * Class to initiate SCM finalization and query its progress.
  */
@@ -109,7 +107,7 @@ public class FinalizationManagerImpl implements FinalizationManager {
   }
 
   @Override
-  public UpgradeFinalizer.StatusAndMessages finalizeUpgrade(
+  public UpgradeFinalization.StatusAndMessages finalizeUpgrade(
       String upgradeClientID)
       throws IOException {
     Preconditions.checkNotNull(context, "Cannot finalize upgrade without " +
@@ -118,11 +116,11 @@ public class FinalizationManagerImpl implements FinalizationManager {
   }
 
   @Override
-  public UpgradeFinalizer.StatusAndMessages queryUpgradeFinalizationProgress(
+  public UpgradeFinalization.StatusAndMessages queryUpgradeFinalizationProgress(
       String upgradeClientID, boolean takeover, boolean readonly
   ) throws IOException {
     if (readonly) {
-      return new UpgradeFinalizer.StatusAndMessages(
+      return new UpgradeFinalization.StatusAndMessages(
           upgradeFinalizer.getStatus(), Collections.emptyList());
     }
     return upgradeFinalizer.reportStatus(upgradeClientID, takeover);
@@ -183,7 +181,6 @@ public class FinalizationManagerImpl implements FinalizationManager {
   /**
    * Builds a {@link FinalizationManagerImpl}.
    */
-  @SuppressWarnings("checkstyle:hiddenfield")
   public static class Builder {
     private OzoneConfiguration conf;
     private HDDSLayoutVersionManager versionManager;
@@ -196,14 +193,14 @@ public class FinalizationManagerImpl implements FinalizationManager {
       executor = new DefaultUpgradeFinalizationExecutor<>();
     }
 
-    public Builder setConfiguration(OzoneConfiguration conf) {
-      this.conf = conf;
+    public Builder setConfiguration(OzoneConfiguration configuration) {
+      this.conf = configuration;
       return this;
     }
 
     public Builder setLayoutVersionManager(
-        HDDSLayoutVersionManager versionManager) {
-      this.versionManager = versionManager;
+        HDDSLayoutVersionManager layoutVersionManager) {
+      this.versionManager = layoutVersionManager;
       return this;
     }
 
@@ -212,8 +209,8 @@ public class FinalizationManagerImpl implements FinalizationManager {
       return this;
     }
 
-    public Builder setHAManager(SCMHAManager scmHAManager) {
-      this.scmHAManager = scmHAManager;
+    public Builder setHAManager(SCMHAManager haManager) {
+      this.scmHAManager = haManager;
       return this;
     }
 
@@ -224,8 +221,8 @@ public class FinalizationManagerImpl implements FinalizationManager {
     }
 
     public Builder setFinalizationExecutor(
-        UpgradeFinalizationExecutor<SCMUpgradeFinalizationContext> executor) {
-      this.executor = executor;
+        UpgradeFinalizationExecutor<SCMUpgradeFinalizationContext> finalizationExecutor) {
+      this.executor = finalizationExecutor;
       return this;
     }
 
