@@ -23,6 +23,7 @@ import static org.apache.hadoop.hdds.scm.HddsTestUtils.getContainerReports;
 import static org.apache.hadoop.hdds.scm.HddsTestUtils.getECContainer;
 import static org.apache.hadoop.hdds.scm.HddsTestUtils.getReplicas;
 import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
 import static org.mockito.ArgumentMatchers.eq;
 import static org.mockito.Mockito.any;
@@ -42,6 +43,7 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
 import java.util.concurrent.TimeoutException;
 import java.util.stream.Collectors;
@@ -60,7 +62,6 @@ import org.apache.hadoop.hdds.scm.events.SCMEvents;
 import org.apache.hadoop.hdds.scm.ha.SCMHAManager;
 import org.apache.hadoop.hdds.scm.ha.SCMHAManagerStub;
 import org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition;
-import org.apache.hadoop.hdds.scm.node.NodeManager;
 import org.apache.hadoop.hdds.scm.node.NodeStatus;
 import org.apache.hadoop.hdds.scm.node.states.NodeNotFoundException;
 import org.apache.hadoop.hdds.scm.pipeline.MockPipelineManager;
@@ -89,14 +90,13 @@ import org.junit.jupiter.params.provider.ValueSource;
  */
 public class TestContainerReportHandler {
 
-  private NodeManager nodeManager;
+  private MockNodeManager nodeManager;
   private ContainerManager containerManager;
   private ContainerStateManager containerStateManager;
   private EventPublisher publisher;
   @TempDir
   private File testDir;
   private DBStore dbStore;
-  private SCMHAManager scmhaManager;
   private PipelineManager pipelineManager;
 
   @BeforeEach
@@ -105,7 +105,7 @@ public class TestContainerReportHandler {
     nodeManager = new MockNodeManager(true, 10);
     containerManager = mock(ContainerManager.class);
     dbStore = DBStoreBuilder.createDBStore(conf, SCMDBDefinition.get());
-    scmhaManager = SCMHAManagerStub.getInstance(true);
+    SCMHAManager scmhaManager = SCMHAManagerStub.getInstance(true);
     pipelineManager =
         new MockPipelineManager(dbStore, scmhaManager, nodeManager);
     containerStateManager = ContainerStateManagerImpl.newBuilder()
@@ -163,7 +163,6 @@ public class TestContainerReportHandler {
 
   @AfterEach
   public void tearDown() throws Exception {
-    containerStateManager.close();
     if (dbStore != null) {
       dbStore.close();
     }
@@ -304,7 +303,6 @@ public class TestContainerReportHandler {
         datanodeOne, datanodeTwo, datanodeThree)
         .forEach(containerStateManager::updateContainerReplica);
 
-
     // SCM expects both containerOne and containerTwo to be in all the three
     // datanodes datanodeOne, datanodeTwo and datanodeThree
 
@@ -361,7 +359,6 @@ public class TestContainerReportHandler {
         datanodeOne, datanodeTwo, datanodeThree)
         .forEach(containerStateManager::updateContainerReplica);
 
-
     // SCM expects both containerOne and containerTwo to be in all the three
     // datanodes datanodeOne, datanodeTwo and datanodeThree
 
@@ -380,7 +377,6 @@ public class TestContainerReportHandler {
         containerOne.containerID()).size());
   }
 
-
   @Test
   public void testClosingToClosed() throws NodeNotFoundException, IOException,
       TimeoutException {
@@ -1180,6 +1176,153 @@ public class TestContainerReportHandler {
     assertEquals(1, containerManager.getContainerReplicas(containerOne.containerID()).size());
   }
 
+  @Test
+  public void testWithNoContainerDataChecksum() throws Exception {
+    final ContainerReportHandler reportHandler = new ContainerReportHandler(nodeManager, containerManager);
+
+    final int numNodes = 3;
+    List<DatanodeDetails> datanodes = nodeManager.getNodes(NodeStatus.inServiceHealthy()).stream()
+        .limit(numNodes)
+        .collect(Collectors.toList());
+
+    // Create a container and put one replica on each datanode.
+    final ContainerInfo container = getContainer(LifeCycleState.CLOSED);
+    ContainerID contID = container.containerID();
+    final Set<ContainerID> containerIDSet = Stream.of(contID).collect(Collectors.toSet());
+
+    for (DatanodeDetails dn: datanodes) {
+      nodeManager.setContainers(dn, containerIDSet);
+    }
+
+    containerStateManager.addContainer(container.getProtobuf());
+
+    getReplicas(contID, ContainerReplicaProto.State.CLOSED, 0, datanodes)
+        .forEach(containerStateManager::updateContainerReplica);
+
+    // Container manager should now be aware of 3 replicas of each container.
+    assertEquals(numNodes, containerManager.getContainerReplicas(contID).size());
+
+    // All replicas should start with an empty data checksum in SCM.
+    boolean contOneDataChecksumsEmpty = containerManager.getContainerReplicas(contID).stream()
+        .allMatch(r -> r.getDataChecksum() == 0);
+    assertTrue(contOneDataChecksumsEmpty, "Replicas of container one should not yet have any data checksums.");
+
+    // Send a report to SCM from one datanode that still does not have a data checksum.
+    int numReportsSent = 0;
+    for (DatanodeDetails dn: datanodes) {
+      final ContainerReportsProto dnReportProto = getContainerReportsProto(
+          contID, ContainerReplicaProto.State.CLOSED, dn.getUuidString());
+      final ContainerReportFromDatanode dnReport = new ContainerReportFromDatanode(dn, dnReportProto);
+      reportHandler.onMessage(dnReport, publisher);
+      numReportsSent++;
+    }
+    assertEquals(numNodes, numReportsSent);
+
+    // Regardless of which datanode sent the report, none of them have checksums, so all replica's data checksums
+    // should remain empty.
+    boolean containerDataChecksumEmpty = containerManager.getContainerReplicas(contID).stream()
+        .allMatch(r -> r.getDataChecksum() == 0);
+    assertTrue(containerDataChecksumEmpty, "Replicas of the container should not have any data checksums.");
+  }
+
+  @Test
+  public void testWithContainerDataChecksum() throws Exception {
+    final ContainerReportHandler reportHandler = new ContainerReportHandler(nodeManager, containerManager);
+
+    final int numNodes = 3;
+    List<DatanodeDetails> datanodes = nodeManager.getNodes(NodeStatus.inServiceHealthy()).stream()
+        .limit(numNodes)
+        .collect(Collectors.toList());
+
+    // Create a container and put one replica on each datanode.
+    final ContainerInfo container = getContainer(LifeCycleState.CLOSED);
+    ContainerID contID = container.containerID();
+    final Set<ContainerID> containerIDSet = Stream.of(contID).collect(Collectors.toSet());
+
+    for (DatanodeDetails dn: datanodes) {
+      nodeManager.setContainers(dn, containerIDSet);
+    }
+
+    containerStateManager.addContainer(container.getProtobuf());
+
+    getReplicas(contID, ContainerReplicaProto.State.CLOSED, 0, datanodes)
+        .forEach(containerStateManager::updateContainerReplica);
+
+    // Container manager should now be aware of 3 replicas of each container.
+    assertEquals(numNodes, containerManager.getContainerReplicas(contID).size());
+
+    // All replicas should start with an empty data checksum in SCM.
+    boolean dataChecksumsEmpty = containerManager.getContainerReplicas(contID).stream()
+        .allMatch(r -> r.getDataChecksum() == 0);
+    assertTrue(dataChecksumsEmpty, "Replicas of container one should not yet have any data checksums.");
+
+    // For each datanode, send a container report with a mismatched checksum.
+    for (DatanodeDetails dn: datanodes) {
+      ContainerReportsProto dnReportProto = getContainerReportsProto(
+          contID, ContainerReplicaProto.State.CLOSED, dn.getUuidString());
+      ContainerReplicaProto replicaWithChecksum = dnReportProto.getReports(0).toBuilder()
+          .setDataChecksum(createUniqueDataChecksumForReplica(contID, dn.getUuidString()))
+          .build();
+      ContainerReportsProto reportWithChecksum = dnReportProto.toBuilder()
+          .clearReports()
+          .addReports(replicaWithChecksum)
+          .build();
+      final ContainerReportFromDatanode dnReport = new ContainerReportFromDatanode(dn, reportWithChecksum);
+      reportHandler.onMessage(dnReport, publisher);
+    }
+
+    // All the replicas should have different checksums.
+    // Since the containers don't have any data in this test, different checksums are based on container ID and
+    // datanode ID.
+    int numReplicasChecked = 0;
+    for (ContainerReplica replica: containerManager.getContainerReplicas(contID)) {
+      long expectedChecksum = createUniqueDataChecksumForReplica(contID, replica.getDatanodeDetails().getUuidString());
+      assertEquals(expectedChecksum, replica.getDataChecksum());
+      numReplicasChecked++;
+    }
+    assertEquals(numNodes, numReplicasChecked);
+
+    // For each datanode, send a container report with a matching checksum.
+    // This simulates reconciliation running.
+    for (DatanodeDetails dn: datanodes) {
+      ContainerReportsProto dnReportProto = getContainerReportsProto(
+          contID, ContainerReplicaProto.State.CLOSED, dn.getUuidString());
+      ContainerReplicaProto replicaWithChecksum = dnReportProto.getReports(0).toBuilder()
+          .setDataChecksum(createMatchingDataChecksumForReplica(contID))
+          .build();
+      ContainerReportsProto reportWithChecksum = dnReportProto.toBuilder()
+          .clearReports()
+          .addReports(replicaWithChecksum)
+          .build();
+      final ContainerReportFromDatanode dnReport = new ContainerReportFromDatanode(dn, reportWithChecksum);
+      reportHandler.onMessage(dnReport, publisher);
+    }
+
+    // All the replicas should now have matching checksums.
+    // Since the containers don't have any data in this test, the matching checksums are based on container ID only.
+    numReplicasChecked = 0;
+    for (ContainerReplica replica: containerManager.getContainerReplicas(contID)) {
+      long expectedChecksum = createMatchingDataChecksumForReplica(contID);
+      assertEquals(expectedChecksum, replica.getDataChecksum());
+      numReplicasChecked++;
+    }
+    assertEquals(numNodes, numReplicasChecked);
+  }
+
+  /**
+   * Generates a placeholder data checksum for testing that is specific to a container replica.
+   */
+  protected static long createUniqueDataChecksumForReplica(ContainerID containerID, String datanodeID) {
+    return (datanodeID + containerID).hashCode();
+  }
+
+  /**
+   * Generates a placeholder data checksum for testing that is the same for all container replicas.
+   */
+  protected static long createMatchingDataChecksumForReplica(ContainerID containerID) {
+    return Objects.hashCode(containerID);
+  }
+
   private ContainerReportFromDatanode getContainerReportFromDatanode(
       ContainerID containerId, ContainerReplicaProto.State state,
       DatanodeDetails dn, long bytesUsed, long keyCount) {
@@ -1232,7 +1375,6 @@ public class TestContainerReportHandler {
         2000000000L, 100000000L, bcsId, replicaIndex);
   }
 
-
   protected static ContainerReportsProto getContainerReportsProto(
       final ContainerID containerId, final ContainerReplicaProto.State state,
       final String originNodeId, final long usedBytes, final long keyCount,
@@ -1253,7 +1395,6 @@ public class TestContainerReportHandler {
             .setContainerID(containerId.getId())
             .setState(state)
             .setOriginNodeId(originNodeId)
-            .setFinalhash("e16cc9d6024365750ed8dbd194ea46d2")
             .setSize(5368709120L)
             .setUsed(usedBytes)
             .setKeyCount(keyCount)
@@ -1268,5 +1409,4 @@ public class TestContainerReportHandler {
             .build();
     return crBuilder.addReports(replicaProto).build();
   }
-
 }
