@@ -74,7 +74,7 @@ public class TestContainerBalancerDatanodeNodeLimit {
 
   @BeforeAll
   public static void setup() {
-    GenericTestUtils.setLogLevel(ContainerBalancerTask.LOG, Level.DEBUG);
+    GenericTestUtils.setLogLevel(ContainerBalancerTask.class, Level.DEBUG);
   }
 
   private static Stream<Arguments> createMockedSCMs() {
@@ -427,7 +427,6 @@ public class TestContainerBalancerDatanodeNodeLimit {
     }
   }
 
-
   @ParameterizedTest(name = "MockedSCM #{index}: {0}")
   @MethodSource("createMockedSCMs")
   public void selectedContainerShouldNotAlreadyHaveBeenSelected(@Nonnull MockedSCM mockedSCM)
@@ -554,7 +553,7 @@ public class TestContainerBalancerDatanodeNodeLimit {
     when(mockedSCM.getMoveManager()
         .move(any(ContainerID.class), any(DatanodeDetails.class), any(DatanodeDetails.class)))
         .thenReturn(genCompletableFutureWithException(1))
-        .thenThrow(new ContainerNotFoundException("Test Container not found"))
+        .thenThrow(ContainerNotFoundException.newInstanceForTesting())
         .thenReturn(future);
 
     ContainerBalancerTask task = mockedSCM.startBalancerTask(config);
