@@ -248,7 +248,7 @@ public final class TestableCluster {
         .setContainerID(containerID)
         .setContainerState(ContainerReplicaProto.State.CLOSED)
         .setDatanodeDetails(datanodeDetails)
-        .setOriginNodeId(datanodeDetails.getUuid())
+        .setOriginNodeId(datanodeDetails.getID())
         .setSequenceId(1000L)
         .setBytesUsed(usedBytes)
         .build();
