@@ -97,8 +97,8 @@ public class TestSCMContainerPlacementRackScatter {
 
   private void updateStorageInDatanode(int dnIndex, long used, long remaining) {
     StorageReportProto storage = HddsTestUtils.createStorageReport(
-            dnInfos.get(dnIndex).getUuid(),
-            "/data1-" + dnInfos.get(dnIndex).getUuidString(),
+            dnInfos.get(dnIndex).getID(),
+            "/data1-" + dnInfos.get(dnIndex).getID(),
             STORAGE_CAPACITY, used, remaining, null);
     dnInfos.get(dnIndex).updateStorageReports(
             new ArrayList<>(Arrays.asList(storage)));
@@ -183,11 +183,11 @@ public class TestSCMContainerPlacementRackScatter {
         UpgradeUtils.defaultLayoutVersionProto());
 
     StorageReportProto storage1 = HddsTestUtils.createStorageReport(
-        datanodeInfo.getUuid(), "/data1-" + datanodeInfo.getUuidString(),
+        datanodeInfo.getID(), "/data1-" + datanodeInfo.getID(),
         STORAGE_CAPACITY, 0, 100L, null);
     MetadataStorageReportProto metaStorage1 =
         HddsTestUtils.createMetadataStorageReport(
-            "/metadata1-" + datanodeInfo.getUuidString(),
+            "/metadata1-" + datanodeInfo.getID(),
             STORAGE_CAPACITY, 0, 100L, null);
     datanodeInfo.updateStorageReports(
         new ArrayList<>(Collections.singletonList(storage1)));
@@ -199,40 +199,40 @@ public class TestSCMContainerPlacementRackScatter {
   private void createMocksAndUpdateStorageReports(int datanodeCount) {
     if (datanodeCount > 4) {
       StorageReportProto storage2 = HddsTestUtils.createStorageReport(
-          dnInfos.get(2).getUuid(),
-          "/data1-" + datanodes.get(2).getUuidString(),
+          dnInfos.get(2).getID(),
+          "/data1-" + datanodes.get(2).getID(),
           STORAGE_CAPACITY, 90L, 10L, null);
       dnInfos.get(2).updateStorageReports(
           new ArrayList<>(Arrays.asList(storage2)));
       StorageReportProto storage3 = HddsTestUtils.createStorageReport(
-          dnInfos.get(3).getUuid(),
-          "/data1-" + dnInfos.get(3).getUuidString(),
+          dnInfos.get(3).getID(),
+          "/data1-" + dnInfos.get(3).getID(),
           STORAGE_CAPACITY, 80L, 20L, null);
       dnInfos.get(3).updateStorageReports(
           new ArrayList<>(Arrays.asList(storage3)));
       StorageReportProto storage4 = HddsTestUtils.createStorageReport(
-          dnInfos.get(4).getUuid(),
-          "/data1-" + dnInfos.get(4).getUuidString(),
+          dnInfos.get(4).getID(),
+          "/data1-" + dnInfos.get(4).getID(),
           STORAGE_CAPACITY, 70L, 30L, null);
       dnInfos.get(4).updateStorageReports(
           new ArrayList<>(Arrays.asList(storage4)));
     } else if (datanodeCount > 3) {
       StorageReportProto storage2 = HddsTestUtils.createStorageReport(
-          dnInfos.get(2).getUuid(),
-          "/data1-" + dnInfos.get(2).getUuidString(),
+          dnInfos.get(2).getID(),
+          "/data1-" + dnInfos.get(2).getID(),
           STORAGE_CAPACITY, 90L, 10L, null);
       dnInfos.get(2).updateStorageReports(
           new ArrayList<>(Arrays.asList(storage2)));
       StorageReportProto storage3 = HddsTestUtils.createStorageReport(
-          dnInfos.get(3).getUuid(),
-          "/data1-" + dnInfos.get(3).getUuidString(),
+          dnInfos.get(3).getID(),
+          "/data1-" + dnInfos.get(3).getID(),
           STORAGE_CAPACITY, 80L, 20L, null);
       dnInfos.get(3).updateStorageReports(
           new ArrayList<>(Arrays.asList(storage3)));
     } else if (datanodeCount > 2) {
       StorageReportProto storage2 = HddsTestUtils.createStorageReport(
-          dnInfos.get(2).getUuid(),
-          "/data1-" + dnInfos.get(2).getUuidString(),
+          dnInfos.get(2).getID(),
+          "/data1-" + dnInfos.get(2).getID(),
           STORAGE_CAPACITY, 84L, 16L, null);
       dnInfos.get(2).updateStorageReports(
           new ArrayList<>(Arrays.asList(storage2)));
@@ -243,7 +243,7 @@ public class TestSCMContainerPlacementRackScatter {
     when(nodeManager.getNodes(NodeStatus.inServiceHealthy()))
         .thenReturn(new ArrayList<>(datanodes));
     for (DatanodeInfo dn: dnInfos) {
-      when(nodeManager.getNodeByUuid(dn.getUuid()))
+      when(nodeManager.getNode(dn.getID()))
           .thenReturn(dn);
     }
     when(nodeManager.getClusterNetworkTopologyMap())
@@ -493,11 +493,11 @@ public class TestSCMContainerPlacementRackScatter {
           UpgradeUtils.defaultLayoutVersionProto());
 
       StorageReportProto storage1 = HddsTestUtils.createStorageReport(
-          dnInfo.getUuid(), "/data1-" + dnInfo.getUuidString(),
+          dnInfo.getID(), "/data1-" + dnInfo.getID(),
           STORAGE_CAPACITY, 0, 100L, null);
       MetadataStorageReportProto metaStorage1 =
           HddsTestUtils.createMetadataStorageReport(
-          "/metadata1-" + dnInfo.getUuidString(),
+          "/metadata1-" + dnInfo.getID(),
           STORAGE_CAPACITY, 0, 100L, null);
       dnInfo.updateStorageReports(
           new ArrayList<>(Arrays.asList(storage1)));
@@ -511,7 +511,7 @@ public class TestSCMContainerPlacementRackScatter {
     assertEquals(dataList.size(), StringUtils.countMatches(
         clusterMap.toString(), NetConstants.DEFAULT_RACK));
     for (DatanodeInfo dn: dnInfoList) {
-      when(nodeManager.getNodeByUuid(dn.getUuid()))
+      when(nodeManager.getNode(dn.getID()))
           .thenReturn(dn);
     }
 
@@ -826,7 +826,7 @@ public class TestSCMContainerPlacementRackScatter {
     setup(datanodeCount);
 
     // DN 5 is out of service
-    dnInfos.get(5).setNodeStatus(new NodeStatus(DECOMMISSIONED, HEALTHY));
+    dnInfos.get(5).setNodeStatus(NodeStatus.valueOf(DECOMMISSIONED, HEALTHY));
 
     // SCM should have detected that DN 5 is dead
     cluster.remove(datanodes.get(5));
@@ -893,7 +893,6 @@ public class TestSCMContainerPlacementRackScatter {
     assertEquals(1, chosenNodes.size());
   }
 
-
   private int getRackSize(List<DatanodeDetails>... datanodeDetails) {
     Set<Node> racks = new HashSet<>();
     for (List<DatanodeDetails> list : datanodeDetails) {
