@@ -20,6 +20,7 @@ package org.apache.hadoop.hdds.scm.container.replication;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeOperationalState.IN_SERVICE;
 import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerReplicaProto.State.CLOSED;
 import static org.apache.hadoop.hdds.scm.exceptions.SCMException.ResultCodes.FAILED_TO_FIND_SUITABLE_NODE;
+import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.mockito.Mockito.any;
 import static org.mockito.Mockito.anyBoolean;
 import static org.mockito.Mockito.anyInt;
@@ -31,18 +32,20 @@ import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
-import java.util.UUID;
 import java.util.concurrent.atomic.AtomicBoolean;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
+import org.apache.hadoop.hdds.conf.StorageUnit;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.protocol.DatanodeID;
 import org.apache.hadoop.hdds.protocol.MockDatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerReplicaProto;
 import org.apache.hadoop.hdds.scm.ContainerPlacementStatus;
 import org.apache.hadoop.hdds.scm.PlacementPolicy;
 import org.apache.hadoop.hdds.scm.SCMCommonPlacementPolicy;
+import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
 import org.apache.hadoop.hdds.scm.container.ContainerReplica;
@@ -51,6 +54,7 @@ import org.apache.hadoop.hdds.scm.container.placement.algorithms.ContainerPlacem
 import org.apache.hadoop.hdds.scm.exceptions.SCMException;
 import org.apache.hadoop.hdds.scm.net.Node;
 import org.apache.hadoop.hdds.scm.node.NodeManager;
+import org.apache.hadoop.hdds.utils.HddsServerUtil;
 import org.apache.hadoop.ozone.protocol.commands.DeleteContainerCommand;
 import org.apache.hadoop.ozone.protocol.commands.ReconstructECContainersCommand;
 import org.apache.hadoop.ozone.protocol.commands.ReplicateContainerCommand;
@@ -117,7 +121,7 @@ public final class ReplicationTestUtil {
       DatanodeDetails dn = MockDatanodeDetails.randomDatanodeDetails();
       replicas.add(createContainerReplica(containerID, i, IN_SERVICE,
           replicaState, keyCount, bytesUsed,
-          dn, dn.getUuid()));
+          dn, dn.getID()));
     }
     return replicas;
   }
@@ -126,7 +130,7 @@ public final class ReplicationTestUtil {
       ContainerID containerID, ContainerReplicaProto.State replicaState,
       int... indexes) {
     Set<ContainerReplica> replicas = new HashSet<>();
-    UUID originNodeId = MockDatanodeDetails.randomDatanodeDetails().getUuid();
+    final DatanodeID originNodeId = DatanodeID.randomID();
     for (int i : indexes) {
       replicas.add(createContainerReplica(
           containerID, i, IN_SERVICE, replicaState, 123L, 1234L,
@@ -134,6 +138,7 @@ public final class ReplicationTestUtil {
     }
     return replicas;
   }
+
   public static ContainerReplica createEmptyContainerReplica(ContainerID containerID,
       int replicaIndex, HddsProtos.NodeOperationalState opState,
       ContainerReplicaProto.State replicaState) {
@@ -141,14 +146,14 @@ public final class ReplicationTestUtil {
         = MockDatanodeDetails.randomDatanodeDetails();
     return createContainerReplica(containerID, replicaIndex, opState,
         replicaState, 0L, 0L,
-        datanodeDetails, datanodeDetails.getUuid());
+        datanodeDetails, datanodeDetails.getID());
   }
 
   public static Set<ContainerReplica> createReplicasWithOriginAndOpState(
       ContainerID containerID, ContainerReplicaProto.State replicaState,
-      Pair<UUID, HddsProtos.NodeOperationalState>... nodes) {
+      Pair<DatanodeID, HddsProtos.NodeOperationalState>... nodes) {
     Set<ContainerReplica> replicas = new HashSet<>();
-    for (Pair<UUID, HddsProtos.NodeOperationalState> i : nodes) {
+    for (Pair<DatanodeID, HddsProtos.NodeOperationalState> i : nodes) {
       replicas.add(createContainerReplica(
           containerID, 0, i.getRight(), replicaState, 123L, 1234L,
           MockDatanodeDetails.randomDatanodeDetails(), i.getLeft()));
@@ -163,7 +168,7 @@ public final class ReplicationTestUtil {
         = MockDatanodeDetails.randomDatanodeDetails();
     return createContainerReplica(containerID, replicaIndex, opState,
         replicaState, 123L, 1234L,
-        datanodeDetails, datanodeDetails.getUuid());
+        datanodeDetails, datanodeDetails.getID());
   }
 
   public static ContainerReplica createContainerReplica(ContainerID containerID,
@@ -173,14 +178,14 @@ public final class ReplicationTestUtil {
         = MockDatanodeDetails.randomDatanodeDetails();
     return createContainerReplica(containerID, replicaIndex, opState,
         replicaState, 123L, 1234L,
-        datanodeDetails, datanodeDetails.getUuid(), seqId);
+        datanodeDetails, datanodeDetails.getID(), seqId);
   }
 
   @SuppressWarnings("checkstyle:ParameterNumber")
   public static ContainerReplica createContainerReplica(ContainerID containerID,
       int replicaIndex, HddsProtos.NodeOperationalState opState,
       ContainerReplicaProto.State replicaState, long keyCount, long bytesUsed,
-      DatanodeDetails datanodeDetails, UUID originNodeId) {
+      DatanodeDetails datanodeDetails, DatanodeID originNodeId) {
     ContainerReplica.ContainerReplicaBuilder builder
         = ContainerReplica.newBuilder();
     datanodeDetails.setPersistedOpState(opState);
@@ -200,7 +205,7 @@ public final class ReplicationTestUtil {
   public static ContainerReplica createContainerReplica(ContainerID containerID,
       int replicaIndex, HddsProtos.NodeOperationalState opState,
       ContainerReplicaProto.State replicaState, long keyCount, long bytesUsed,
-      DatanodeDetails datanodeDetails, UUID originNodeId, long seqId) {
+      DatanodeDetails datanodeDetails, DatanodeID originNodeId, long seqId) {
     ContainerReplica.ContainerReplicaBuilder builder
         = ContainerReplica.newBuilder();
     datanodeDetails.setPersistedOpState(opState);
@@ -328,6 +333,9 @@ public final class ReplicationTestUtil {
               List<DatanodeDetails> favoredNodes, int nodesRequiredToChoose,
               long metadataSizeRequired, long dataSizeRequired)
               throws SCMException {
+        long containerSize = (long) conf.getStorageSize(ScmConfigKeys.OZONE_SCM_CONTAINER_SIZE,
+            ScmConfigKeys.OZONE_SCM_CONTAINER_SIZE_DEFAULT, StorageUnit.BYTES);
+        assertEquals(HddsServerUtil.requiredReplicationSpace(containerSize), dataSizeRequired);
         if (nodesRequiredToChoose > 1) {
           throw new IllegalArgumentException("Only one node is allowed");
         }
@@ -356,6 +364,9 @@ public final class ReplicationTestUtil {
               List<DatanodeDetails> favoredNodes, int nodesRequiredToChoose,
               long metadataSizeRequired, long dataSizeRequired)
               throws SCMException {
+        long containerSize = (long) conf.getStorageSize(ScmConfigKeys.OZONE_SCM_CONTAINER_SIZE,
+            ScmConfigKeys.OZONE_SCM_CONTAINER_SIZE_DEFAULT, StorageUnit.BYTES);
+        assertEquals(HddsServerUtil.requiredReplicationSpace(containerSize), dataSizeRequired);
         throw new SCMException("No nodes available",
                 FAILED_TO_FIND_SUITABLE_NODE);
       }
@@ -383,6 +394,9 @@ public final class ReplicationTestUtil {
           List<DatanodeDetails> favoredNodes, int nodesRequiredToChoose,
           long metadataSizeRequired, long dataSizeRequired)
           throws SCMException {
+        long containerSize = (long) conf.getStorageSize(ScmConfigKeys.OZONE_SCM_CONTAINER_SIZE,
+            ScmConfigKeys.OZONE_SCM_CONTAINER_SIZE_DEFAULT, StorageUnit.BYTES);
+        assertEquals(HddsServerUtil.requiredReplicationSpace(containerSize), dataSizeRequired);
         if (nodesRequiredToChoose >= throwWhenThisOrMoreNodesRequested) {
           throw new SCMException("No nodes available",
               FAILED_TO_FIND_SUITABLE_NODE);
