@@ -106,6 +106,7 @@ public class TestRatisUnderReplicationHandler {
             ReplicationManagerConfiguration.class));
     metrics = ReplicationManagerMetrics.create(replicationManager);
     when(replicationManager.getMetrics()).thenReturn(metrics);
+    when(replicationManager.getContainerReplicaPendingOps()).thenReturn(mock(ContainerReplicaPendingOps.class));
 
     /*
       Return NodeStatus with NodeOperationalState as specified in
@@ -115,7 +116,7 @@ public class TestRatisUnderReplicationHandler {
         replicationManager.getNodeStatus(any(DatanodeDetails.class)))
         .thenAnswer(invocationOnMock -> {
           DatanodeDetails dn = invocationOnMock.getArgument(0);
-          return new NodeStatus(dn.getPersistedOpState(),
+          return NodeStatus.valueOf(dn.getPersistedOpState(),
               HddsProtos.NodeState.HEALTHY);
         });
 
