@@ -48,6 +48,7 @@ import static org.mockito.Mockito.times;
 import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.when;
 
+import com.google.common.collect.Lists;
 import com.google.protobuf.Proto2Utils;
 import java.io.IOException;
 import java.time.Instant;
@@ -60,7 +61,8 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.UUID;
+import java.util.concurrent.TimeoutException;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.function.Function;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
@@ -68,6 +70,7 @@ import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.protocol.DatanodeID;
 import org.apache.hadoop.hdds.protocol.MockDatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerReplicaProto;
@@ -96,7 +99,7 @@ import org.apache.hadoop.ozone.protocol.commands.DeleteContainerCommand;
 import org.apache.hadoop.ozone.protocol.commands.ReconstructECContainersCommand;
 import org.apache.hadoop.ozone.protocol.commands.ReplicateContainerCommand;
 import org.apache.hadoop.ozone.protocol.commands.SCMCommand;
-import org.apache.hadoop.util.Lists;
+import org.apache.ozone.test.GenericTestUtils;
 import org.apache.ozone.test.TestClock;
 import org.apache.ratis.protocol.exceptions.NotLeaderException;
 import org.junit.jupiter.api.AfterEach;
@@ -104,6 +107,7 @@ import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.EnumSource;
+import org.junit.jupiter.params.provider.ValueSource;
 import org.mockito.ArgumentCaptor;
 
 /**
@@ -127,7 +131,7 @@ public class TestReplicationManager {
   private ReplicationConfig repConfig;
   private ReplicationManagerReport repReport;
   private ReplicationQueue repQueue;
-  private Set<Pair<UUID, SCMCommand<?>>> commandsSent;
+  private Set<Pair<DatanodeID, SCMCommand<?>>> commandsSent;
 
   @BeforeEach
   public void setup() throws IOException {
@@ -197,6 +201,7 @@ public class TestReplicationManager {
 
   private ReplicationManager createReplicationManager() throws IOException {
     return new ReplicationManager(
+        configuration.getObject(ReplicationManager.ReplicationManagerConfiguration.class),
         configuration,
         containerManager,
         ratisPlacementPolicy,
@@ -222,7 +227,7 @@ public class TestReplicationManager {
   @Test
   public void testPendingOpsClearedWhenStarting() {
     containerReplicaPendingOps.scheduleAddReplica(ContainerID.valueOf(1),
-        MockDatanodeDetails.randomDatanodeDetails(), 1, null, Integer.MAX_VALUE);
+        MockDatanodeDetails.randomDatanodeDetails(), 1, null, Integer.MAX_VALUE, 5L, clock.millis());
     containerReplicaPendingOps.scheduleDeleteReplica(ContainerID.valueOf(2),
         MockDatanodeDetails.randomDatanodeDetails(), 1, null, Integer.MAX_VALUE);
     assertEquals(1, containerReplicaPendingOps
@@ -369,7 +374,7 @@ public class TestReplicationManager {
     Set<ContainerReplica> replicas =
         createReplicasWithSameOrigin(container.containerID(),
             ContainerReplicaProto.State.QUASI_CLOSED, 0, 0, 0);
-    UUID origin = replicas.iterator().next().getOriginDatanodeId();
+    final DatanodeID origin = replicas.iterator().next().getOriginDatanodeId();
     ContainerReplica unhealthy =
         createContainerReplica(container.containerID(), 0, IN_SERVICE,
             ContainerReplicaProto.State.UNHEALTHY, 1, 123,
@@ -391,7 +396,7 @@ public class TestReplicationManager {
     handler.processAndSendCommands(replicas, Collections.emptyList(),
             repQueue.dequeueOverReplicatedContainer(), 2);
     assertTrue(commandsSent.iterator().hasNext());
-    assertEquals(unhealthy.getDatanodeDetails().getUuid(),
+    assertEquals(unhealthy.getDatanodeDetails().getID(),
         commandsSent.iterator().next().getKey());
     assertEquals(SCMCommandProto.Type.deleteContainerCommand,
         commandsSent.iterator().next().getValue().getType());
@@ -614,7 +619,7 @@ public class TestReplicationManager {
     addReplicas(container, ContainerReplicaProto.State.CLOSED, 1, 2, 3, 4);
     containerReplicaPendingOps.scheduleAddReplica(container.containerID(),
         MockDatanodeDetails.randomDatanodeDetails(), 5, null,
-        clock.millis() + 10000);
+        clock.millis() + 10000, 5L, clock.millis());
 
     replicationManager.processContainer(
         container, repQueue, repReport);
@@ -656,7 +661,7 @@ public class TestReplicationManager {
 
     ContainerReplica replica  = createContainerReplica(container.containerID(),
         1, IN_SERVICE, ContainerReplicaProto.State.CLOSED,
-        0, 0, MockDatanodeDetails.randomDatanodeDetails(), UUID.randomUUID());
+        0, 0, MockDatanodeDetails.randomDatanodeDetails(), DatanodeID.randomID());
 
     storeContainerAndReplicas(container, Collections.singleton(replica));
 
@@ -869,12 +874,12 @@ public class TestReplicationManager {
     // a delete command should also have been sent for UNHEALTHY replica of
     // index 1
     assertEquals(1, commandsSent.size());
-    Pair<UUID, SCMCommand<?>> command = commandsSent.iterator().next();
+    Pair<DatanodeID, SCMCommand<?>> command = commandsSent.iterator().next();
     assertEquals(SCMCommandProto.Type.deleteContainerCommand,
         command.getValue().getType());
     DeleteContainerCommand deleteCommand =
         (DeleteContainerCommand) command.getValue();
-    assertEquals(unhealthyReplica1.getDatanodeDetails().getUuid(),
+    assertEquals(unhealthyReplica1.getDatanodeDetails().getID(),
         command.getKey());
     assertEquals(container.containerID(),
         ContainerID.valueOf(deleteCommand.getContainerID()));
@@ -1313,15 +1318,14 @@ public class TestReplicationManager {
 
     ArgumentCaptor<SCMCommand<?>> command =
         ArgumentCaptor.forClass(SCMCommand.class);
-    ArgumentCaptor<UUID> targetUUID =
-        ArgumentCaptor.forClass(UUID.class);
+    ArgumentCaptor<DatanodeID> targetUUID = ArgumentCaptor.forClass(DatanodeID.class);
     verify(nodeManager).addDatanodeCommand(targetUUID.capture(), command.capture());
 
     ReplicateContainerCommand sentCommand =
         (ReplicateContainerCommand)command.getValue();
     assertEquals(datanodeDeadline, sentCommand.getDeadline());
     assertEquals(LOW, sentCommand.getPriority());
-    assertEquals(src.getUuid(), targetUUID.getValue());
+    assertEquals(src.getID(), targetUUID.getValue());
     assertEquals(target, sentCommand.getTargetDatanode());
   }
 
@@ -1376,8 +1380,8 @@ public class TestReplicationManager {
         container, new ArrayList<>(sourceNodes), destination, replicaIndex);
 
     assertEquals(1, commandsSent.size());
-    Pair<UUID, SCMCommand<?>> cmdWithTarget = commandsSent.iterator().next();
-    assertEquals(expectedTarget.getUuid(), cmdWithTarget.getLeft());
+    Pair<DatanodeID, SCMCommand<?>> cmdWithTarget = commandsSent.iterator().next();
+    assertEquals(expectedTarget.getID(), cmdWithTarget.getLeft());
     assertEquals(ReplicateContainerCommand.class,
         cmdWithTarget.getRight().getClass());
     ReplicateContainerCommand cmd =
@@ -1438,8 +1442,8 @@ public class TestReplicationManager {
     replicationManager.sendThrottledReconstructionCommand(container, command);
 
     assertEquals(1, commandsSent.size());
-    Pair<UUID, SCMCommand<?>> cmd = commandsSent.iterator().next();
-    assertEquals(cmdTarget.getUuid(), cmd.getLeft());
+    Pair<DatanodeID, SCMCommand<?>> cmd = commandsSent.iterator().next();
+    assertEquals(cmdTarget.getID(), cmd.getLeft());
     assertEquals(0, replicationManager.getMetrics()
         .getEcReconstructionCmdsDeferredTotal());
   }
@@ -1642,12 +1646,69 @@ public class TestReplicationManager {
 
     replicationManager.opCompleted(delOp, ContainerID.valueOf(1L), true);
     assertEquals(1, commandsSent.size());
-    Pair<UUID, SCMCommand<?>> sentCommand = commandsSent.iterator().next();
+    Pair<DatanodeID, SCMCommand<?>> sentCommand = commandsSent.iterator().next();
     // The target should be DN2 and the deadline should have been updated from the value set in commandDeadline above
-    assertEquals(dn2.getUuid(), sentCommand.getLeft());
+    assertEquals(dn2.getID(), sentCommand.getLeft());
     assertNotEquals(commandDeadline, sentCommand.getRight().getDeadline());
   }
 
+  @ParameterizedTest
+  @ValueSource(booleans = {true, false})
+  public void testNotifyNodeStateChangeWakesUpThread(boolean queueIsEmpty) 
+      throws IOException, InterruptedException, ReflectiveOperationException, TimeoutException {
+
+    AtomicBoolean processAllCalled = new AtomicBoolean(false);
+    ReplicationQueue queue = mock(ReplicationQueue.class);
+    when(queue.isEmpty()).thenReturn(queueIsEmpty);
+    final ReplicationManager customRM = new ReplicationManager(
+        configuration.getObject(ReplicationManager.ReplicationManagerConfiguration.class),
+        configuration,
+        containerManager,
+        ratisPlacementPolicy,
+        ecPlacementPolicy,
+        eventPublisher,
+        scmContext,
+        nodeManager,
+        clock,
+        containerReplicaPendingOps) {
+          @Override
+          public ReplicationQueue getQueue() {
+            return queue;
+          }
+
+          @Override
+          public synchronized void processAll() {
+            processAllCalled.set(true);
+          }
+        };
+
+    customRM.notifyStatusChanged();
+    customRM.start();
+
+    // wait for the thread become TIMED_WAITING
+    GenericTestUtils.waitFor(
+        () -> customRM.isThreadWaiting(),
+        100,
+        1000);
+
+    // The processAll method will be called when the ReplicationManager's run
+    // method is executed by the replicationMonitor thread.
+    assertTrue(processAllCalled.get());
+    processAllCalled.set(false);
+
+    assertThat(customRM.notifyNodeStateChange()).isEqualTo(queueIsEmpty);
+
+    GenericTestUtils.waitFor(
+        () -> customRM.isThreadWaiting(),
+        100,
+        1000);
+
+    // If the queue is empty, the processAll method should have been called
+    assertEquals(processAllCalled.get(), queueIsEmpty);
+
+    customRM.stop();
+  }
+
   @SafeVarargs
   private final Set<ContainerReplica>  addReplicas(ContainerInfo container,
       ContainerReplicaProto.State replicaState,
