@@ -1,23 +1,44 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- *  with the License.  You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
- *  Unless required by applicable law or agreed to in writing, software
- *  distributed under the License is distributed on an "AS IS" BASIS,
- *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- *  See the License for the specific language governing permissions and
- *  limitations under the License.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package org.apache.hadoop.hdds.scm.container.replication.health;
 
+import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleEvent.CLOSE;
+import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleEvent.QUASI_CLOSE;
+import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleState.CLOSED;
+import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleState.CLOSING;
+import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType.EC;
+import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType.RATIS;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.mockito.Mockito.any;
+import static org.mockito.Mockito.anyBoolean;
+import static org.mockito.Mockito.clearInvocations;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.never;
+import static org.mockito.Mockito.times;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.when;
+
+import java.time.Duration;
+import java.util.Collections;
+import java.util.HashSet;
+import java.util.Set;
+import java.util.stream.Stream;
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
@@ -33,27 +54,11 @@ import org.apache.hadoop.hdds.scm.container.replication.ContainerCheckRequest;
 import org.apache.hadoop.hdds.scm.container.replication.ReplicationManager;
 import org.apache.hadoop.hdds.scm.container.replication.ReplicationTestUtil;
 import org.apache.ozone.test.TestClock;
-import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.MethodSource;
 import org.mockito.ArgumentCaptor;
-import org.mockito.Mockito;
-
-import java.time.Duration;
-import java.util.Collections;
-import java.util.HashSet;
-import java.util.Set;
-import java.util.stream.Stream;
-
-import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleEvent.CLOSE;
-import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleEvent.QUASI_CLOSE;
-import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleState.CLOSED;
-import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleState.CLOSING;
-import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType.EC;
-import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType.RATIS;
-import static org.mockito.Mockito.never;
 
 /**
  * Tests for {@link ClosingContainerHandler}.
@@ -74,9 +79,8 @@ public class TestClosingContainerHandler {
 
   @BeforeEach
   public void setup() {
-    replicationManager = Mockito.mock(ReplicationManager.class);
-    Mockito.when(replicationManager.getConfig())
-        .thenReturn(rmConf);
+    replicationManager = mock(ReplicationManager.class);
+    when(replicationManager.getConfig()).thenReturn(rmConf);
     subject = new ClosingContainerHandler(replicationManager, clock);
   }
 
@@ -203,7 +207,7 @@ public class TestClosingContainerHandler {
 
     assertAndVerify(readRequest, true, 0);
     assertAndVerify(request, true, 3);
-    report.getStats().forEach((k, v) -> Assertions.assertEquals(0L, v));
+    report.getStats().forEach((k, v) -> assertEquals(0L, v));
   }
 
   @Test
@@ -222,9 +226,9 @@ public class TestClosingContainerHandler {
     assertAndVerify(request, true, 0);
     report.getStats().forEach((k, v) -> {
       if (k.equals("MISSING")) {
-        Assertions.assertEquals(1L, v);
+        assertEquals(1L, v);
       } else {
-        Assertions.assertEquals(0L, v);
+        assertEquals(0L, v);
       }
     });
   }
@@ -256,14 +260,14 @@ public class TestClosingContainerHandler {
     Duration notEnoughTime = replicationInterval.multipliedBy(3).plusMillis(1);
     clock.fastForward(notEnoughTime);
     assertAndVerify(request, true, 0);
-    Mockito.verify(replicationManager, never())
+    verify(replicationManager, never())
         .updateContainerState(containerInfo.containerID(), CLOSE);
 
     // wait time has elapsed (3x + 2x + a bit)
     Duration moreTime = replicationInterval.multipliedBy(2);
     clock.fastForward(moreTime);
     assertAndVerify(request, true, 0);
-    Mockito.verify(replicationManager, Mockito.times(1))
+    verify(replicationManager, times(1))
         .updateContainerState(containerInfo.containerID(), CLOSE);
   }
 
@@ -285,10 +289,10 @@ public class TestClosingContainerHandler {
         .build();
     subject.handle(request);
 
-    Mockito.verify(replicationManager, Mockito.times(1))
+    verify(replicationManager, times(1))
         .updateContainerState(containerInfo.containerID(), QUASI_CLOSE);
 
-    Mockito.clearInvocations(replicationManager);
+    clearInvocations(replicationManager);
 
     // Now add an open container. This time, the container should not move to
     // quasi-closed, and a close should be sent for the open replica.
@@ -297,7 +301,7 @@ public class TestClosingContainerHandler {
             ContainerReplicaProto.State.OPEN, 0));
 
     assertAndVerify(request, true, 1);
-    Mockito.verify(replicationManager, Mockito.times(0))
+    verify(replicationManager, times(0))
         .updateContainerState(containerInfo.containerID(), QUASI_CLOSE);
   }
 
@@ -319,10 +323,10 @@ public class TestClosingContainerHandler {
         .build();
     subject.handle(request);
 
-    Mockito.verify(replicationManager, Mockito.times(1))
+    verify(replicationManager, times(1))
         .updateContainerState(containerInfo.containerID(), CLOSE);
 
-    Mockito.clearInvocations(replicationManager);
+    clearInvocations(replicationManager);
 
     // Now add an open container. This time, the container should not move to
     // quasi-closed, and a close should be sent for the open replica.
@@ -331,7 +335,7 @@ public class TestClosingContainerHandler {
             ContainerReplicaProto.State.OPEN, 1));
 
     assertAndVerify(request, true, 1);
-    Mockito.verify(replicationManager, Mockito.times(0))
+    verify(replicationManager, times(0))
         .updateContainerState(containerInfo.containerID(), CLOSE);
   }
 
@@ -379,19 +383,19 @@ public class TestClosingContainerHandler {
 
     ArgumentCaptor<Boolean> forceCaptor =
         ArgumentCaptor.forClass(Boolean.class);
-    Assertions.assertTrue(subject.handle(request));
-    Mockito.verify(replicationManager, Mockito.times(replicas))
-        .sendCloseContainerReplicaCommand(Mockito.any(ContainerInfo.class),
-            Mockito.any(DatanodeDetails.class), forceCaptor.capture());
+    assertTrue(subject.handle(request));
+    verify(replicationManager, times(replicas))
+        .sendCloseContainerReplicaCommand(any(ContainerInfo.class),
+            any(DatanodeDetails.class), forceCaptor.capture());
     forceCaptor.getAllValues()
-        .forEach(f -> Assertions.assertEquals(force, f));
+        .forEach(f -> assertEquals(force, f));
   }
 
   private void assertAndVerify(ContainerCheckRequest request,
       boolean assertion, int times) {
-    Assertions.assertEquals(assertion, subject.handle(request));
-    Mockito.verify(replicationManager, Mockito.times(times))
-        .sendCloseContainerReplicaCommand(Mockito.any(ContainerInfo.class),
-            Mockito.any(DatanodeDetails.class), Mockito.anyBoolean());
+    assertEquals(assertion, subject.handle(request));
+    verify(replicationManager, times(times))
+        .sendCloseContainerReplicaCommand(any(ContainerInfo.class),
+            any(DatanodeDetails.class), anyBoolean());
   }
 }
