@@ -17,7 +17,6 @@
 
 package org.apache.hadoop.hdds.scm.metadata;
 
-import java.io.IOException;
 import java.nio.charset.StandardCharsets;
 import java.security.cert.CertificateException;
 import java.security.cert.X509Certificate;
@@ -48,24 +47,14 @@ public final class OldX509CertificateCodecForTesting
   }
 
   @Override
-  public byte[] toPersistedFormat(X509Certificate object) throws IOException {
-    try {
-      return CertificateCodec.getPEMEncodedString(object)
-          .getBytes(StandardCharsets.UTF_8);
-    } catch (SCMSecurityException exp) {
-      throw new IOException(exp);
-    }
+  public byte[] toPersistedFormatImpl(X509Certificate object) throws SCMSecurityException {
+    return CertificateCodec.getPEMEncodedString(object)
+        .getBytes(StandardCharsets.UTF_8);
   }
 
   @Override
-  public X509Certificate fromPersistedFormat(byte[] rawData)
-      throws IOException {
-    try {
-      String s = new String(rawData, StandardCharsets.UTF_8);
-      return CertificateCodec.getX509Certificate(s);
-    } catch (CertificateException exp) {
-      throw new IOException(exp);
-    }
+  public X509Certificate fromPersistedFormatImpl(byte[] rawData) throws CertificateException {
+    return CertificateCodec.getX509Certificate(rawData);
   }
 
   @Override
