@@ -22,7 +22,7 @@ import static org.junit.jupiter.api.Assertions.assertEquals;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
-import java.util.UUID;
+import org.apache.hadoop.hdds.protocol.DatanodeID;
 import org.apache.hadoop.hdds.protocol.MockDatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos;
@@ -58,35 +58,35 @@ public class TestCommandQueue {
         .ReplicationCommandPriority.LOW);
 
 
-    UUID datanode1UUID = UUID.randomUUID();
-    UUID datanode2UUID = UUID.randomUUID();
+    DatanodeID datanode1ID = DatanodeID.randomID();
+    DatanodeID datanode2ID = DatanodeID.randomID();
 
-    commandQueue.addCommand(datanode1UUID, closeContainerCommand);
-    commandQueue.addCommand(datanode1UUID, closeContainerCommand);
-    commandQueue.addCommand(datanode1UUID, createPipelineCommand);
-    commandQueue.addCommand(datanode1UUID, replicationCommand);
-    commandQueue.addCommand(datanode1UUID, lowReplicationCommand);
+    commandQueue.addCommand(datanode1ID, closeContainerCommand);
+    commandQueue.addCommand(datanode1ID, closeContainerCommand);
+    commandQueue.addCommand(datanode1ID, createPipelineCommand);
+    commandQueue.addCommand(datanode1ID, replicationCommand);
+    commandQueue.addCommand(datanode1ID, lowReplicationCommand);
 
 
-    commandQueue.addCommand(datanode2UUID, closeContainerCommand);
-    commandQueue.addCommand(datanode2UUID, createPipelineCommand);
-    commandQueue.addCommand(datanode2UUID, createPipelineCommand);
+    commandQueue.addCommand(datanode2ID, closeContainerCommand);
+    commandQueue.addCommand(datanode2ID, createPipelineCommand);
+    commandQueue.addCommand(datanode2ID, createPipelineCommand);
 
     // Check zero returned for unknown DN
     assertEquals(0, commandQueue.getDatanodeCommandCount(
-        UUID.randomUUID(), SCMCommandProto.Type.closeContainerCommand));
+        DatanodeID.randomID(), SCMCommandProto.Type.closeContainerCommand));
 
     assertEquals(2, commandQueue.getDatanodeCommandCount(
-        datanode1UUID, SCMCommandProto.Type.closeContainerCommand));
+        datanode1ID, SCMCommandProto.Type.closeContainerCommand));
     assertEquals(1, commandQueue.getDatanodeCommandCount(
-        datanode1UUID, SCMCommandProto.Type.createPipelineCommand));
+        datanode1ID, SCMCommandProto.Type.createPipelineCommand));
     assertEquals(0, commandQueue.getDatanodeCommandCount(
-        datanode1UUID, SCMCommandProto.Type.closePipelineCommand));
+        datanode1ID, SCMCommandProto.Type.closePipelineCommand));
     assertEquals(1, commandQueue.getDatanodeCommandCount(
-        datanode1UUID, SCMCommandProto.Type.replicateContainerCommand));
+        datanode1ID, SCMCommandProto.Type.replicateContainerCommand));
 
     Map<SCMCommandProto.Type, Integer> commandSummary =
-        commandQueue.getDatanodeCommandSummary(datanode1UUID);
+        commandQueue.getDatanodeCommandSummary(datanode1ID);
     assertEquals(3, commandSummary.size());
     assertEquals(Integer.valueOf(2),
         commandSummary.get(SCMCommandProto.Type.closeContainerCommand));
@@ -96,34 +96,34 @@ public class TestCommandQueue {
         commandSummary.get(SCMCommandProto.Type.replicateContainerCommand));
 
     assertEquals(1, commandQueue.getDatanodeCommandCount(
-        datanode2UUID, SCMCommandProto.Type.closeContainerCommand));
+        datanode2ID, SCMCommandProto.Type.closeContainerCommand));
     assertEquals(2, commandQueue.getDatanodeCommandCount(
-        datanode2UUID, SCMCommandProto.Type.createPipelineCommand));
+        datanode2ID, SCMCommandProto.Type.createPipelineCommand));
 
     // Ensure the counts are cleared when the commands are retrieved
-    List<SCMCommand<?>> cmds = commandQueue.getCommand(datanode1UUID);
+    List<SCMCommand<?>> cmds = commandQueue.getCommand(datanode1ID);
     assertEquals(5, cmds.size());
 
     assertEquals(0, commandQueue.getDatanodeCommandCount(
-        datanode1UUID, SCMCommandProto.Type.closeContainerCommand));
+        datanode1ID, SCMCommandProto.Type.closeContainerCommand));
     assertEquals(0, commandQueue.getDatanodeCommandCount(
-        datanode1UUID, SCMCommandProto.Type.createPipelineCommand));
+        datanode1ID, SCMCommandProto.Type.createPipelineCommand));
     assertEquals(0, commandQueue.getDatanodeCommandCount(
-        datanode1UUID, SCMCommandProto.Type.closePipelineCommand));
+        datanode1ID, SCMCommandProto.Type.closePipelineCommand));
     assertEquals(0, commandQueue.getDatanodeCommandCount(
-        datanode1UUID, SCMCommandProto.Type.replicateContainerCommand));
+        datanode1ID, SCMCommandProto.Type.replicateContainerCommand));
 
     assertEquals(1, commandQueue.getDatanodeCommandCount(
-        datanode2UUID, SCMCommandProto.Type.closeContainerCommand));
+        datanode2ID, SCMCommandProto.Type.closeContainerCommand));
     assertEquals(2, commandQueue.getDatanodeCommandCount(
-        datanode2UUID, SCMCommandProto.Type.createPipelineCommand));
+        datanode2ID, SCMCommandProto.Type.createPipelineCommand));
 
     // Ensure the commands are zeroed when the queue is cleared
     commandQueue.clear();
     assertEquals(0, commandQueue.getDatanodeCommandCount(
-        datanode2UUID, SCMCommandProto.Type.closeContainerCommand));
+        datanode2ID, SCMCommandProto.Type.closeContainerCommand));
     assertEquals(0, commandQueue.getDatanodeCommandCount(
-        datanode2UUID, SCMCommandProto.Type.createPipelineCommand));
+        datanode2ID, SCMCommandProto.Type.createPipelineCommand));
   }
 
 }
