@@ -67,8 +67,7 @@ import org.mockito.Mockito;
 public class TestNodeDecommissionManager {
 
   private NodeDecommissionManager decom;
-  private StorageContainerManager scm;
-  private NodeManager nodeManager;
+  private SCMNodeManager nodeManager;
   private ContainerManager containerManager;
   private OzoneConfiguration conf;
   private static int id = 1;
@@ -77,8 +76,8 @@ public class TestNodeDecommissionManager {
   void setup(@TempDir File dir) throws Exception {
     conf = new OzoneConfiguration();
     conf.set(HddsConfigKeys.OZONE_METADATA_DIRS, dir.getAbsolutePath());
-    scm = HddsTestUtils.getScm(conf);
-    nodeManager = scm.getScmNodeManager();
+    StorageContainerManager scm = HddsTestUtils.getScm(conf);
+    nodeManager = (SCMNodeManager) scm.getScmNodeManager();
     containerManager = mock(ContainerManager.class);
     decom = new NodeDecommissionManager(conf, nodeManager, containerManager,
         SCMContext.emptyContext(), new EventQueue(), null);
@@ -87,6 +86,10 @@ public class TestNodeDecommissionManager {
             (String) invocation.getArguments()[1]));
   }
 
+  void setContainers(DatanodeDetails datanode, Set<ContainerID> containers) throws NodeNotFoundException {
+    ScmNodeTestUtil.setContainers(nodeManager, datanode, containers);
+  }
+
   private ContainerInfo createMockContainer(ReplicationConfig rep, String owner) {
     ContainerInfo.Builder builder = new ContainerInfo.Builder()
         .setReplicationConfig(rep)
@@ -97,6 +100,7 @@ public class TestNodeDecommissionManager {
     id++;
     return builder.build();
   }
+
   private ContainerInfo getMockContainer(ReplicationConfig rep, ContainerID conId) {
     ContainerInfo.Builder builder = new ContainerInfo.Builder()
         .setReplicationConfig(rep)
@@ -426,7 +430,7 @@ public class TestNodeDecommissionManager {
     }
 
     for (DatanodeDetails dn  : nodeManager.getAllNodes().subList(0, 3)) {
-      nodeManager.setContainers(dn, idsRatis);
+      setContainers(dn, idsRatis);
     }
 
     error = decom.decommissionNodes(Arrays.asList(dns.get(1).getIpAddress(),
@@ -480,7 +484,7 @@ public class TestNodeDecommissionManager {
     }
 
     for (DatanodeDetails dn  : nodeManager.getAllNodes()) {
-      nodeManager.setContainers(dn, idsEC);
+      setContainers(dn, idsEC);
     }
 
     error = decom.decommissionNodes(Arrays.asList(dns.get(1).getIpAddress()), false);
@@ -529,10 +533,10 @@ public class TestNodeDecommissionManager {
         });
 
     for (DatanodeDetails dn  : nodeManager.getAllNodes().subList(0, 3)) {
-      nodeManager.setContainers(dn, idsRatis);
+      setContainers(dn, idsRatis);
     }
     for (DatanodeDetails dn  : nodeManager.getAllNodes()) {
-      nodeManager.setContainers(dn, idsEC);
+      setContainers(dn, idsEC);
     }
 
     error = decom.decommissionNodes(Arrays.asList(dns.get(1).getIpAddress()), false);
@@ -573,7 +577,7 @@ public class TestNodeDecommissionManager {
     }
 
     for (DatanodeDetails dn  : nodeManager.getAllNodes().subList(0, 3)) {
-      nodeManager.setContainers(dn, idsRatis);
+      setContainers(dn, idsRatis);
     }
 
     // decommission one node successfully
@@ -608,7 +612,7 @@ public class TestNodeDecommissionManager {
       idsRatis.add(container.containerID());
     }
 
-    nodeManager = mock(NodeManager.class);
+    nodeManager = mock(SCMNodeManager.class);
     decom = new NodeDecommissionManager(conf, nodeManager, containerManager,
         SCMContext.emptyContext(), new EventQueue(), null);
     when(containerManager.getContainer(any(ContainerID.class)))
@@ -669,7 +673,7 @@ public class TestNodeDecommissionManager {
       idsRatis.add(container.containerID());
     }
     for (DatanodeDetails dn  : nodeManager.getAllNodes().subList(0, 3)) {
-      nodeManager.setContainers(dn, idsRatis);
+      setContainers(dn, idsRatis);
     }
 
     decom.setMaintenanceConfigs(2, 1); // default config
@@ -769,7 +773,7 @@ public class TestNodeDecommissionManager {
       idsEC.add(container.containerID());
     }
     for (DatanodeDetails dn  : nodeManager.getAllNodes()) {
-      nodeManager.setContainers(dn, idsEC);
+      setContainers(dn, idsEC);
     }
 
     decom.setMaintenanceConfigs(2, 1); // default config
@@ -852,10 +856,10 @@ public class TestNodeDecommissionManager {
               (ContainerID)invocation.getArguments()[0]);
         });
     for (DatanodeDetails dn  : nodeManager.getAllNodes().subList(0, 3)) {
-      nodeManager.setContainers(dn, idsRatis);
+      setContainers(dn, idsRatis);
     }
     for (DatanodeDetails dn  : nodeManager.getAllNodes()) {
-      nodeManager.setContainers(dn, idsEC);
+      setContainers(dn, idsEC);
     }
 
     decom.setMaintenanceConfigs(2, 1); // default config
@@ -926,7 +930,7 @@ public class TestNodeDecommissionManager {
       idsRatis.add(container.containerID());
     }
     for (DatanodeDetails dn  : nodeManager.getAllNodes().subList(0, 3)) {
-      nodeManager.setContainers(dn, idsRatis);
+      setContainers(dn, idsRatis);
     }
 
     // put 2 nodes into maintenance successfully
@@ -966,7 +970,7 @@ public class TestNodeDecommissionManager {
       idsRatis.add(container.containerID());
     }
 
-    nodeManager = mock(NodeManager.class);
+    nodeManager = mock(SCMNodeManager.class);
     decom = new NodeDecommissionManager(conf, nodeManager, containerManager,
         SCMContext.emptyContext(), new EventQueue(), null);
     when(containerManager.getContainer(any(ContainerID.class)))
@@ -1044,7 +1048,7 @@ public class TestNodeDecommissionManager {
     }
     for (DatanodeDetails datanode : dns) {
       if (datanode.equals(dn)) {
-        return new NodeStatus(datanode.getPersistedOpState(), HddsProtos.NodeState.HEALTHY);
+        return NodeStatus.valueOf(datanode.getPersistedOpState(), HddsProtos.NodeState.HEALTHY);
       }
     }
     return null;
