@@ -18,10 +18,12 @@
 package org.apache.hadoop.hdds.scm.node.states;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNull;
 
 import java.util.List;
 import java.util.UUID;
 import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.atomic.AtomicReference;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.DatanodeID;
 import org.apache.hadoop.hdds.protocol.MockDatanodeDetails;
@@ -39,9 +41,20 @@ import org.junit.jupiter.api.Test;
  * NodeStateManager.
  */
 public class TestNodeStateMap {
+  private static final NodeStatus DECOMMISSIONING_HEALTHY_999 = NodeStatus.valueOf(
+      NodeOperationalState.DECOMMISSIONING, NodeState.HEALTHY, 999);
 
+  private final DatanodeDetails dn = generateDatanode();
   private NodeStateMap map;
 
+  void addNode(NodeStatus status) throws NodeAlreadyExistsException {
+    addNode(dn, status);
+  }
+
+  void addNode(DatanodeDetails datanode, NodeStatus status) throws NodeAlreadyExistsException {
+    map.addNode(new DatanodeInfo(datanode, status, null));
+  }
+
   @BeforeEach
   public void setUp() {
     map = new NodeStateMap();
@@ -54,59 +67,76 @@ public class TestNodeStateMap {
   @Test
   public void testNodeCanBeAddedAndRetrieved()
       throws NodeAlreadyExistsException, NodeNotFoundException {
-    DatanodeDetails dn = generateDatanode();
     NodeStatus status = NodeStatus.inServiceHealthy();
-    map.addNode(dn, status, null);
+    addNode(status);
     assertEquals(dn, map.getNodeInfo(dn.getID()));
     assertEquals(status, map.getNodeStatus(dn.getID()));
   }
 
-  @Test
-  public void testNodeHealthStateCanBeUpdated()
-      throws NodeAlreadyExistsException, NodeNotFoundException {
-    DatanodeDetails dn = generateDatanode();
-    NodeStatus status = NodeStatus.inServiceHealthy();
-    map.addNode(dn, status, null);
+  private void runTestUpdateHealth(NodeStatus original, NodeState newHealth) throws Exception {
+    addNode(original);
+    final NodeStatus returned = map.updateNodeHealthState(dn.getID(), newHealth);
 
-    NodeStatus expectedStatus = NodeStatus.inServiceStale();
-    NodeStatus returnedStatus =
-        map.updateNodeHealthState(dn.getID(), expectedStatus.getHealth());
-    assertEquals(expectedStatus, returnedStatus);
-    assertEquals(returnedStatus, map.getNodeStatus(dn.getID()));
+    final NodeStatus expected = NodeStatus.valueOf(
+        original.getOperationalState(), newHealth, original.getOpStateExpiryEpochSeconds());
+    assertEquals(expected, returned);
+    assertEquals(returned, map.getNodeStatus(dn.getID()));
+  }
+
+  @Test
+  public void testUpdateHealthyToStale() throws Exception {
+    runTestUpdateHealth(NodeStatus.inServiceHealthy(), NodeState.STALE);
+  }
+
+  @Test
+  public void testUpdateDecommissioningHealthyToStale() throws Exception {
+    runTestUpdateHealth(DECOMMISSIONING_HEALTHY_999, NodeState.STALE);
   }
 
   @Test
   public void testNodeOperationalStateCanBeUpdated()
       throws NodeAlreadyExistsException, NodeNotFoundException {
-    DatanodeDetails dn = generateDatanode();
     NodeStatus status = NodeStatus.inServiceHealthy();
-    map.addNode(dn, status, null);
+    addNode(status);
 
-    NodeStatus expectedStatus = new NodeStatus(
-        NodeOperationalState.DECOMMISSIONING,
-        NodeState.HEALTHY, 999);
+    NodeStatus expectedStatus = DECOMMISSIONING_HEALTHY_999;
     NodeStatus returnedStatus = map.updateNodeOperationalState(
-        dn.getID(), expectedStatus.getOperationalState(), 999);
+        dn.getID(), expectedStatus.getOperationalState(), expectedStatus.getOpStateExpiryEpochSeconds());
     assertEquals(expectedStatus, returnedStatus);
     assertEquals(returnedStatus, map.getNodeStatus(dn.getID()));
-    assertEquals(999, returnedStatus.getOpStateExpiryEpochSeconds());
+  }
+
+  @Test
+  public void testGetNodeNonZeroExpiry() throws Exception {
+    runTestGetNode(123);
   }
 
   @Test
   public void testGetNodeMethodsReturnCorrectCountsAndStates()
       throws NodeAlreadyExistsException {
+    runTestGetNode(0);
+  }
+
+  private void runTestGetNode(long opExpiryEpochSeconds)
+      throws NodeAlreadyExistsException {
     // Add one node for all possible states
     int nodeCount = 0;
     for (NodeOperationalState op : NodeOperationalState.values()) {
       for (NodeState health : NodeState.values()) {
-        addRandomNodeWithState(op, health);
+        addRandomNodeWithState(op, health, opExpiryEpochSeconds);
         nodeCount++;
       }
     }
-    NodeStatus requestedState = NodeStatus.inServiceStale();
+    final NodeStatus requestedState = NodeStatus.valueOf(
+        NodeOperationalState.IN_SERVICE, NodeState.STALE, opExpiryEpochSeconds);
     List<DatanodeInfo> nodes = map.getDatanodeInfos(requestedState);
     assertEquals(1, nodes.size());
     assertEquals(1, map.getNodeCount(requestedState));
+
+    List<DatanodeInfo> nodes2 = map.getDatanodeInfos(
+        requestedState.getOperationalState(), requestedState.getHealth());
+    assertEquals(1, nodes2.size());
+
     assertEquals(nodeCount, map.getTotalNodeCount());
     assertEquals(nodeCount, map.getNodeCount());
     assertEquals(nodeCount, map.getAllDatanodeInfos().size());
@@ -129,7 +159,7 @@ public class TestNodeStateMap {
     final DatanodeDetails datanodeDetails =
         MockDatanodeDetails.randomDatanodeDetails();
 
-    map.addNode(datanodeDetails, NodeStatus.inServiceHealthy(), null);
+    addNode(datanodeDetails, NodeStatus.inServiceHealthy());
 
     DatanodeID id = datanodeDetails.getID();
 
@@ -140,13 +170,14 @@ public class TestNodeStateMap {
     CountDownLatch elementRemoved = new CountDownLatch(1);
     CountDownLatch loopStarted = new CountDownLatch(1);
 
+    final AtomicReference<Exception> failure = new AtomicReference<>();
     new Thread(() -> {
       try {
         loopStarted.await();
         map.removeContainer(id, ContainerID.valueOf(1L));
         elementRemoved.countDown();
       } catch (Exception e) {
-        e.printStackTrace();
+        failure.set(e);
       }
 
     }).start();
@@ -160,23 +191,15 @@ public class TestNodeStateMap {
       first = false;
       System.out.println(key);
     }
-  }
-
-  private void addNodeWithState(
-      DatanodeDetails dn,
-      NodeOperationalState opState, NodeState health
-  )
-      throws NodeAlreadyExistsException {
-    NodeStatus status = new NodeStatus(opState, health);
-    map.addNode(dn, status, null);
+    assertNull(failure.get());
   }
 
   private void addRandomNodeWithState(
-      NodeOperationalState opState, NodeState health
-  )
-      throws NodeAlreadyExistsException {
-    DatanodeDetails dn = generateDatanode();
-    addNodeWithState(dn, opState, health);
+      NodeOperationalState opState, NodeState health,
+      long opExpiryEpochSeconds) throws NodeAlreadyExistsException {
+    DatanodeDetails random = generateDatanode();
+    NodeStatus status = NodeStatus.valueOf(opState, health, opExpiryEpochSeconds);
+    addNode(random, status);
   }
 
   private DatanodeDetails generateDatanode() {
