@@ -236,17 +236,12 @@ public class MockPipelineManager implements PipelineManager {
   }
 
   @Override
-  public void closePipeline(final Pipeline pipeline, final boolean onTimeout)
+  public void closePipeline(final PipelineID pipelineId)
       throws IOException {
-    stateManager.updatePipelineState(pipeline.getId().getProtobuf(),
+    stateManager.updatePipelineState(pipelineId.getProtobuf(),
         HddsProtos.PipelineState.PIPELINE_CLOSED);
   }
 
-  @Override
-  public void closePipeline(PipelineID pipelineID) throws IOException {
-
-  }
-
   @Override
   public void deletePipeline(PipelineID pipelineID) throws IOException {
 
@@ -354,4 +349,9 @@ public class MockPipelineManager implements PipelineManager {
   public boolean isPipelineCreationFrozen() {
     return false;
   }
+
+  @Override
+  public boolean hasEnoughSpace(Pipeline pipeline, long containerSize) {
+    return false;
+  }
 }
