@@ -17,7 +17,7 @@
 
 package org.apache.hadoop.hdds.scm.pipeline;
 
-import static org.apache.commons.collections.CollectionUtils.intersection;
+import static org.apache.commons.collections4.CollectionUtils.intersection;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_DATANODE_PIPELINE_LIMIT;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_DATANODE_RATIS_VOLUME_FREE_SPACE_MIN;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_CONTAINER_SIZE;
@@ -35,7 +35,6 @@ import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
-import java.util.UUID;
 import java.util.concurrent.TimeoutException;
 import java.util.stream.Collectors;
 import org.apache.hadoop.hdds.HddsConfigKeys;
@@ -43,6 +42,7 @@ import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.protocol.DatanodeID;
 import org.apache.hadoop.hdds.protocol.MockDatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor;
@@ -64,6 +64,8 @@ import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.Assumptions;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.CsvSource;
 
 /**
  * Test for {@link RatisPipelineProvider}.
@@ -79,6 +81,7 @@ public class TestRatisPipelineProvider {
   @TempDir
   private File testDir;
   private DBStore dbStore;
+  private int nodeCount = 10;
 
   public void init(int maxPipelinePerNode) throws Exception {
     init(maxPipelinePerNode, new OzoneConfiguration());
@@ -92,7 +95,7 @@ public class TestRatisPipelineProvider {
   public void init(int maxPipelinePerNode, OzoneConfiguration conf, File dir) throws Exception {
     conf.set(HddsConfigKeys.OZONE_METADATA_DIRS, dir.getAbsolutePath());
     dbStore = DBStoreBuilder.createDBStore(conf, SCMDBDefinition.get());
-    nodeManager = new MockNodeManager(true, 10);
+    nodeManager = new MockNodeManager(true, nodeCount);
     nodeManager.setNumPipelinePerDatanode(maxPipelinePerNode);
     SCMHAManager scmhaManager = SCMHAManagerStub.getInstance(true);
     conf.setInt(ScmConfigKeys.OZONE_DATANODE_PIPELINE_LIMIT,
@@ -109,6 +112,7 @@ public class TestRatisPipelineProvider {
 
   @AfterEach
   void cleanup() throws Exception {
+    nodeCount = 10;
     if (dbStore != null) {
       dbStore.close();
     }
@@ -164,9 +168,9 @@ public class TestRatisPipelineProvider {
     createPipelineAndAssertions(HddsProtos.ReplicationFactor.ONE);
   }
 
-  private List<DatanodeDetails> createListOfNodes(int nodeCount) {
+  private List<DatanodeDetails> createListOfNodes(int count) {
     List<DatanodeDetails> nodes = new ArrayList<>();
-    for (int i = 0; i < nodeCount; i++) {
+    for (int i = 0; i < count; i++) {
       nodes.add(MockDatanodeDetails.randomDatanodeDetails());
     }
     return nodes;
@@ -361,6 +365,33 @@ public class TestRatisPipelineProvider {
     }
   }
 
+  @ParameterizedTest
+  @CsvSource({ "1, 3", "2, 6"})
+  public void testCreatePipelineThrowErrorWithDataNodeLimit(int limit, int pipelineCount) throws Exception {
+    // increasing node count to avoid intermittent failures due to unhealthy nodes.
+    nodeCount = 13;
+    init(limit, new OzoneConfiguration(), testDir);
+
+    // Create pipelines up to the limit (3 for limit=1, 6 for limit=2).
+    for (int i = 0; i < pipelineCount; i++) {
+      stateManager.addPipeline(
+          provider.create(RatisReplicationConfig.getInstance(ReplicationFactor.THREE),
+              new ArrayList<>(), new ArrayList<>()).getProtobufMessage(ClientVersion.CURRENT_VERSION)
+      );
+    }
+
+    // Verify that creating an additional pipeline throws an exception.
+    SCMException exception = assertThrows(SCMException.class, () ->
+        provider.create(RatisReplicationConfig.getInstance(ReplicationFactor.THREE),
+            new ArrayList<>(), new ArrayList<>())
+    );
+
+    // Validate exception message.
+    String expectedError = String.format(
+        "Cannot create pipeline as it would exceed the limit per datanode: %d replicationConfig: RATIS/THREE", limit);
+    assertEquals(expectedError, exception.getMessage());
+  }
+
   private void addPipeline(
       List<DatanodeDetails> dns,
       Pipeline.PipelineState open, ReplicationConfig replicationConfig)
@@ -388,7 +419,7 @@ public class TestRatisPipelineProvider {
           .setContainerState(StorageContainerDatanodeProtocolProtos
               .ContainerReplicaProto.State.CLOSED)
           .setKeyCount(1)
-          .setOriginNodeId(UUID.randomUUID())
+          .setOriginNodeId(DatanodeID.randomID())
           .setSequenceId(1)
           .setReplicaIndex(0)
           .setDatanodeDetails(dn)
