@@ -39,8 +39,6 @@ import org.junit.jupiter.api.Test;
  */
 public class TestPipelineChoosePolicyFactory {
 
-  private OzoneConfiguration conf;
-
   private ScmConfig scmConfig;
 
   private NodeManager nodeManager;
@@ -48,7 +46,7 @@ public class TestPipelineChoosePolicyFactory {
   @BeforeEach
   public void setup() {
     //initialize network topology instance
-    conf = new OzoneConfiguration();
+    OzoneConfiguration conf = new OzoneConfiguration();
     scmConfig = conf.getObject(ScmConfig.class);
     nodeManager = new MockNodeManager(true, 5);
   }
@@ -77,7 +75,6 @@ public class TestPipelineChoosePolicyFactory {
     assertSame(DummyGoodImpl.class, policy.getClass());
   }
 
-
   /**
    * A dummy pipeline choose policy implementation for test with an invalid
    * constructor.
@@ -118,7 +115,6 @@ public class TestPipelineChoosePolicyFactory {
     }
   }
 
-
   @Test
   public void testConstructorNotFound() throws SCMException {
     // set a policy class which does't have the right constructor implemented
