@@ -69,6 +69,7 @@ import org.apache.hadoop.hdds.security.x509.certificate.utils.CertificateCodec;
 import org.apache.hadoop.hdds.security.x509.certificate.utils.SelfSignedCertificate;
 import org.apache.hadoop.security.ssl.KeyStoreTestUtil;
 import org.apache.ozone.test.GenericTestUtils;
+import org.apache.ozone.test.GenericTestUtils.LogCapturer;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
@@ -84,12 +85,7 @@ public class TestRootCARotationManager {
   private RootCARotationManager rootCARotationManager;
   private StorageContainerManager scm;
   private SCMCertificateClient scmCertClient;
-  private SCMServiceManager scmServiceManager;
-  private SCMHAManager scmhaManager;
   private SCMContext scmContext;
-  private SequenceIdGenerator sequenceIdGenerator;
-  private SCMStorageConfig scmStorageConfig;
-  private SCMSecurityProtocolServer scmSecurityProtocolServer;
   private RootCARotationHandlerImpl handler;
   private StatefulServiceStateManager statefulServiceStateManager;
   @TempDir
@@ -111,14 +107,14 @@ public class TestRootCARotationManager {
     securityConfig = new SecurityConfig(ozoneConfig);
     scmCertClient = new SCMCertificateClient(securityConfig, null, scmID, cID,
         certID.toString(), "localhost");
-    scmServiceManager = new SCMServiceManager();
+    SCMServiceManager scmServiceManager = new SCMServiceManager();
     scmContext = mock(SCMContext.class);
-    scmhaManager = mock(SCMHAManager.class);
-    sequenceIdGenerator = mock(SequenceIdGenerator.class);
-    scmStorageConfig = new SCMStorageConfig(ozoneConfig);
+    SCMHAManager scmhaManager = mock(SCMHAManager.class);
+    SequenceIdGenerator sequenceIdGenerator = mock(SequenceIdGenerator.class);
+    SCMStorageConfig scmStorageConfig = new SCMStorageConfig(ozoneConfig);
     scmStorageConfig.setScmId(scmID);
     scmStorageConfig.setClusterId(cID);
-    scmSecurityProtocolServer = mock(SCMSecurityProtocolServer.class);
+    SCMSecurityProtocolServer scmSecurityProtocolServer = mock(SCMSecurityProtocolServer.class);
     handler = mock(RootCARotationHandlerImpl.class);
     statefulServiceStateManager = mock(StatefulServiceStateManager.class);
     when(scmContext.isLeader()).thenReturn(true);
@@ -195,9 +191,8 @@ public class TestRootCARotationManager {
 
     rootCARotationManager = new RootCARotationManager(scm);
     rootCARotationManager.setRootCARotationHandler(handler);
-    GenericTestUtils.LogCapturer logs =
-        GenericTestUtils.LogCapturer.captureLogs(RootCARotationManager.LOG);
-    GenericTestUtils.setLogLevel(RootCARotationManager.LOG, INFO);
+    LogCapturer logs = LogCapturer.captureLogs(RootCARotationManager.class);
+    GenericTestUtils.setLogLevel(RootCARotationManager.class, INFO);
     rootCARotationManager.start();
     rootCARotationManager.notifyStatusChanged();
 
@@ -228,9 +223,8 @@ public class TestRootCARotationManager {
 
     rootCARotationManager = new RootCARotationManager(scm);
     rootCARotationManager.setRootCARotationHandler(handler);
-    GenericTestUtils.LogCapturer logs =
-        GenericTestUtils.LogCapturer.captureLogs(RootCARotationManager.LOG);
-    GenericTestUtils.setLogLevel(RootCARotationManager.LOG, INFO);
+    LogCapturer logs = LogCapturer.captureLogs(RootCARotationManager.class);
+    GenericTestUtils.setLogLevel(RootCARotationManager.class, INFO);
     rootCARotationManager.start();
     rootCARotationManager.notifyStatusChanged();
 
@@ -262,9 +256,8 @@ public class TestRootCARotationManager {
     certCodec.writeCertificate(cert);
     rootCARotationManager = new RootCARotationManager(scm);
     rootCARotationManager.setRootCARotationHandler(handler);
-    GenericTestUtils.LogCapturer logs =
-        GenericTestUtils.LogCapturer.captureLogs(RootCARotationManager.LOG);
-    GenericTestUtils.setLogLevel(RootCARotationManager.LOG, INFO);
+    LogCapturer logs = LogCapturer.captureLogs(RootCARotationManager.class);
+    GenericTestUtils.setLogLevel(RootCARotationManager.class, INFO);
     rootCARotationManager.start();
     rootCARotationManager.notifyStatusChanged();
 
