@@ -1454,12 +1454,11 @@ public class OzoneBucket extends WithMetadata {
         keyInfo.getModificationTime(),
         keyInfo.getReplicationConfig(),
         metadata,
-        keyInfo.isFile(),
+        status.isFile(),
         keyInfo.getOwnerName(),
         Collections.emptyMap());
   }
 
-
   /**
    * An Iterator to iterate over {@link OzoneKey} list.
    *
