@@ -1,13 +1,12 @@
-/**
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License.  You may obtain a copy of the License at
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -59,14 +58,13 @@ public final class TenantArgs {
    *
    * @return Builder
    */
-  public static TenantArgs.Builder newBuilder() {
-    return new TenantArgs.Builder();
+  public static Builder newBuilder() {
+    return new Builder();
   }
 
   /**
    * Builder for TenantArgs.
    */
-  @SuppressWarnings("checkstyle:hiddenfield")
   public static class Builder {
     private String volumeName;
     private boolean forceCreationWhenVolumeExists;
@@ -77,12 +75,12 @@ public final class TenantArgs {
     public Builder() {
     }
 
-    public TenantArgs.Builder setVolumeName(String volumeName) {
+    public Builder setVolumeName(String volumeName) {
       this.volumeName = volumeName;
       return this;
     }
 
-    public TenantArgs.Builder setForceCreationWhenVolumeExists(
+    public Builder setForceCreationWhenVolumeExists(
         boolean forceCreationWhenVolumeExists) {
       this.forceCreationWhenVolumeExists = forceCreationWhenVolumeExists;
       return this;
