@@ -55,6 +55,9 @@ import org.slf4j.LoggerFactory;
  */
 public final class ECKeyOutputStream extends KeyOutputStream
     implements KeyMetadataAware {
+
+  private static final Logger LOG = LoggerFactory.getLogger(KeyOutputStream.class);
+
   private OzoneClientConfig config;
   private ECChunkBuffers ecChunkBufferCache;
   private final BlockingQueue<ECChunkBuffers> ecStripeQueue;
@@ -75,14 +78,6 @@ public final class ECKeyOutputStream extends KeyOutputStream
    */
   private boolean atomicKeyCreation;
 
-  private enum StripeWriteStatus {
-    SUCCESS,
-    FAILED
-  }
-
-  public static final Logger LOG =
-      LoggerFactory.getLogger(KeyOutputStream.class);
-
   private volatile boolean closed;
   private volatile boolean closing;
   // how much of data is actually written yet to underlying stream
@@ -643,6 +638,7 @@ public final class ECKeyOutputStream extends KeyOutputStream
 
   private static class CheckpointDummyStripe extends ECChunkBuffers {
     private final long version;
+
     CheckpointDummyStripe(long version) {
       super();
       this.version = version;
@@ -729,4 +725,9 @@ public final class ECKeyOutputStream extends KeyOutputStream
       }
     }
   }
+
+  private enum StripeWriteStatus {
+    SUCCESS,
+    FAILED
+  }
 }
