@@ -58,18 +58,10 @@ import org.slf4j.LoggerFactory;
 public class KeyDataStreamOutput extends AbstractDataStreamOutput
     implements KeyMetadataAware {
 
-  private OzoneClientConfig config;
-
-  /**
-   * Defines stream action while calling handleFlushOrClose.
-   */
-  enum StreamAction {
-    FLUSH, HSYNC, CLOSE, FULL
-  }
-
-  public static final Logger LOG =
+  private static final Logger LOG =
       LoggerFactory.getLogger(KeyDataStreamOutput.class);
 
+  private OzoneClientConfig config;
   private boolean closed;
 
   // how much of data is actually written yet to underlying stream
@@ -520,7 +512,6 @@ public class KeyDataStreamOutput extends AbstractDataStreamOutput
       return this;
     }
 
-
     public Builder setReplicationConfig(ReplicationConfig replConfig) {
       this.replicationConfig = replConfig;
       return this;
@@ -561,4 +552,11 @@ public class KeyDataStreamOutput extends AbstractDataStreamOutput
               + blockDataStreamOutputEntryPool.getKeyName());
     }
   }
+
+  /**
+   * Defines stream action while calling handleFlushOrClose.
+   */
+  enum StreamAction {
+    FLUSH, HSYNC, CLOSE, FULL
+  }
 }
